package com.qmth.distributed.print.business.templete.execute;

import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.qmth.boot.tools.excel.ExcelWriter;
import com.qmth.boot.tools.excel.enums.ExcelType;
import com.qmth.distributed.print.business.bean.dto.SubjectiveStructDto;
import com.qmth.distributed.print.business.bean.dto.importFile.BasicExamStudentImport;
import com.qmth.distributed.print.business.templete.importData.AsyncImportTaskTemplete;
import com.qmth.distributed.print.business.templete.importData.SyncImportTaskTemplate;
import com.qmth.distributed.print.business.templete.service.ImportLogicService;
import com.qmth.distributed.print.business.templete.service.TaskLogicService;
import com.qmth.teachcloud.common.bean.examRule.CodeNameEnableValue;
import com.qmth.teachcloud.common.bean.vo.FilePathVo;
import com.qmth.teachcloud.common.contant.SpringContextHolder;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.MarkQuestion;
import com.qmth.teachcloud.common.entity.TBTask;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.enums.TaskResultEnum;
import com.qmth.teachcloud.common.enums.TaskStatusEnum;
import com.qmth.teachcloud.common.enums.UploadFileEnum;
import com.qmth.teachcloud.common.service.FileUploadService;
import com.qmth.teachcloud.common.service.TBTaskService;
import com.qmth.teachcloud.common.util.Result;
import com.qmth.teachcloud.common.util.ResultUtil;
import com.qmth.teachcloud.mark.entity.MarkPaper;
import com.qmth.teachcloud.mark.entity.MarkUserQuestion;
import com.qmth.teachcloud.mark.service.MarkPaperService;
import com.qmth.teachcloud.mark.service.MarkQuestionService;
import com.qmth.teachcloud.mark.service.MarkUserQuestionService;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.text.MessageFormat;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Description: 试卷主观题结构导入同步任务
 * @Author: CaoZixuan
 * @Date: 2024/04/09
 */
@Service
public class AsyncSubjectiveStructImportService extends AsyncImportTaskTemplete {

    public static final String OBJ_TITLE = "主观题结构导入";

    @Resource
    private MarkQuestionService markQuestionService;
    @Resource
    private MarkUserQuestionService markUserQuestionService;
    @Resource
    private MarkPaperService markPaperService;
    @Resource
    private FileUploadService fileUploadService;

    @Override
    public Result importTask(Map<String, Object> map) throws Exception {
        TBTask tbTask = (TBTask) map.get(SystemConstant.TASK);
        StringJoiner stringJoinerSummary = new StringJoiner("\n").add(
                MessageFormat.format("{0}{1}{2}", DateUtil.format(new Date(), SystemConstant.DEFAULT_DATE_PATTERN),
                        BEGIN_TITLE, OBJ_TITLE));
        tbTask.setStatus(TaskStatusEnum.RUNNING);
        TBTaskService tbTaskService = SpringContextHolder.getBean(TBTaskService.class);
        tbTaskService.updateById(tbTask);
        try (InputStream inputStream = super.getUploadFileInputStream(tbTask)) {
            map.put("inputStream", inputStream);
            ImportLogicService importLogicService = SpringContextHolder.getBean(ImportLogicService.class);
            // 执行导入考务数据
            Map<String, Object> result = importLogicService.executeImportSubjectiveStructLogic(map);
            boolean hasError = Boolean.parseBoolean(String.valueOf(result.get(SystemConstant.HAS_ERROR_DATA)));

            if (hasError) {
                SystemConstant.addSummary(stringJoinerSummary, "有异常数据，开始生成错误文件");
                tbTask.setResult(TaskResultEnum.ERROR);
                // 有异常数据
                List<SubjectiveStructDto> errorDataList = JSON.parseArray(JSON.toJSONString(result.get(SystemConstant.ERROR_DATA_LIST)), SubjectiveStructDto.class);
                try {
                    File excelFileTemp = SystemConstant.getFileTempVar(SystemConstant.EXCEL_PREFIX);
                    this.createErrorExcelFile(errorDataList, excelFileTemp);
                    String fileName = SystemConstant.getNanoId() + SystemConstant.EXCEL_PREFIX;
                    FilePathVo filePathVo = fileUploadService.uploadFile(excelFileTemp, UploadFileEnum.FILE, fileName);
                    tbTask.setErrorFilePath(JSON.toJSONString(filePathVo));
                    SystemConstant.addSummary(stringJoinerSummary, "错误文件生成生成，请到\"导入结果查询\"中，下载错误文件");
                } catch (Exception e) {
                    SystemConstant.addSummary(stringJoinerSummary, "错误文件生成失败，" + e.getMessage());
                }
            } else {
                SystemConstant.addSummary(stringJoinerSummary, "数据校验通过，开始写入试卷结构数据库");
                List<MarkQuestion> markQuestions = JSON.parseArray(JSON.toJSONString(result.get(SystemConstant.DATASOURCE)), MarkQuestion.class);
                markQuestionService.saveOrUpdateBatch(markQuestions);
                SystemConstant.addSummary(stringJoinerSummary, "写入试卷结构数据库完成，开始写入用户-结构数据库");

                // 更新扫描阅卷数据
                List<MarkUserQuestion> markUserQuestions = JSON.parseArray(JSON.toJSONString(result.get(SystemConstant.DATASOURCE1)), MarkUserQuestion.class);
                markUserQuestionService.saveOrUpdateBatch(markUserQuestions);
                // 更改试卷结构状态为已提交
                Set<String> paperNumberSet = markQuestions.stream().map(MarkQuestion::getPaperNumber).collect(Collectors.toSet());
                if (CollectionUtils.isNotEmpty(paperNumberSet)) {
                    for (String paperNumber : paperNumberSet) {
                        markPaperService.updateQuestionStatus(tbTask.getExamId(), paperNumber);
                        markPaperService.updateGroupStatus(tbTask.getExamId(), paperNumber);
                    }
                }
                SystemConstant.addSummary(stringJoinerSummary, "写入用户-结构数据库完成，导入结束");
                tbTask.setResult(TaskResultEnum.SUCCESS);

                stringJoinerSummary.add(MessageFormat.format("{0}{1}{2}{3}{4}{5}{6}{7}",
                        DateUtil.format(new Date(), SystemConstant.DEFAULT_DATE_PATTERN), FINISH_TITLE, markQuestions.size(),
                        FINISH_TOTAL_SIZE, markQuestions.size(), FINISH_SUCCESS_SIZE, 0, FINISH_ERROR_SIZE));
            }
        } catch (Exception e) {
            tbTask.setResult(TaskResultEnum.ERROR);
            stringJoinerSummary.add(MessageFormat.format("{0}{1}{2}{3}", DateUtil.format(new Date(), SystemConstant.DEFAULT_DATE_PATTERN), EXCEPTION_TITLE, EXCEPTION_DATA, e.getMessage()));
        } finally {
            tbTask.setStatus(TaskStatusEnum.FINISH);
            tbTask.setSummary(stringJoinerSummary.toString());
            tbTaskService.updateById(tbTask);
        }
        return ResultUtil.ok();
    }

    private void createErrorExcelFile(List<SubjectiveStructDto> list, File excelFileTemp) {
        try {
            FileOutputStream outputStream = new FileOutputStream(excelFileTemp);
            ExcelWriter writer = ExcelWriter.create(ExcelType.XLSX);
            writer.writeObjects("主观题导入数据", null, SubjectiveStructDto.class, list.listIterator());
            writer.output(outputStream);
            outputStream.flush();
            outputStream.close();
        } catch (Exception e) {

        }
    }
}
