package com.qmth.teachcloud.data.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.qmth.distributed.print.business.bean.params.SysAdminSetParam;
import com.qmth.distributed.print.business.enums.RequiredFieldsEnum;
import com.qmth.distributed.print.business.service.BasicExamStudentService;
import com.qmth.distributed.print.business.service.BasicMessageService;
import com.qmth.distributed.print.business.service.ExamTaskService;
import com.qmth.teachcloud.common.bean.examRule.CodeNameEnableDisabledValue;
import com.qmth.teachcloud.common.bean.sync.*;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.BasicSchool;
import com.qmth.teachcloud.common.entity.SysConfig;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.enums.MessageEnum;
import com.qmth.teachcloud.common.service.BasicCourseService;
import com.qmth.teachcloud.common.service.CommonCacheService;
import com.qmth.teachcloud.common.service.SysOrgService;
import com.qmth.teachcloud.common.service.SysUserService;
import com.qmth.teachcloud.common.util.ConvertUtil;
import com.qmth.teachcloud.data.DataUtil;
import com.qmth.teachcloud.data.dto.SyncDataResult;
import com.qmth.teachcloud.data.entity.TSyncData;
import com.qmth.teachcloud.common.bean.sync.TSyncDataStudent;
import com.qmth.teachcloud.data.enums.DataType;
import com.qmth.teachcloud.data.service.DataService;
import com.qmth.teachcloud.data.service.TSyncDataLogService;
import com.qmth.teachcloud.data.service.TSyncDataStudentService;
import com.qmth.teachcloud.data.util.JdbcUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.StringJoiner;
import java.util.concurrent.atomic.AtomicInteger;

@Service
public class DataServiceImpl implements DataService {

    @Resource
    private DataUtil dataUtil;
    @Resource
    private TSyncDataLogService tSyncDataLogService;
    @Resource
    private TSyncDataStudentService tSyncDataStudentService;
    @Resource
    private BasicCourseService basicCourseService;
    @Resource
    private BasicExamStudentService basicExamStudentService;
    @Resource
    private CommonCacheService commonCacheService;
    @Resource
    private SysOrgService sysOrgService;
    @Resource
    private SysUserService sysUserService;
    @Resource
    private ExamTaskService examTaskService;
    @Resource
    private BasicMessageService basicMessageService;

    @Override
    public boolean testConnect(SysAdminSetParam sysAdminSetParam) {
        try {
            JdbcTemplate jdbcTemplate = JdbcUtil.getJdbcTemplate(dataUtil.createDataSourceMap(sysAdminSetParam.getParam()));
            String sql = "select 1 from dual";
            String object = jdbcTemplate.queryForObject(sql, String.class);
            if (StringUtils.isNotBlank(object)) {
                return true;
            } else {
                return false;
            }
        } catch (Exception e) {
            throw ExceptionResultEnum.ERROR.exception(e.getMessage());
        }
    }

    @Override
    public void syncData(Long schoolId, Long examId, TSyncData syncData) {
        JdbcTemplate jdbcTemplate = JdbcUtil.getJdbcTemplate(dataUtil.createDataSourceMap(schoolId));
        String dataRange = syncData.getDataType();

        SysConfig datasourceType = commonCacheService.addSysConfigCache(schoolId, SystemConstant.DATA_DATASOURCE_TYPE);
        if (datasourceType == null || StringUtils.isBlank(datasourceType.getConfigValue())) {
            throw ExceptionResultEnum.ERROR.exception("未设置数据库类型");
        }

        boolean sendSms = false;
        StringJoiner stringJoiner = new StringJoiner(";");
        // 机构同步
        if (dataRange.contains(DataType.A.name())) {
            SyncDataResult syncDataResult = syncOrg(jdbcTemplate, schoolId, examId, datasourceType.getConfigValue().toString());
            stringJoiner.add(DataType.A.name() + ":" + syncDataResult.getCount());
            sendSms = StringUtils.isNotBlank(syncDataResult.getErrMsg()) ? true : sendSms;
        }
        // 用户同步
        if (dataRange.contains(DataType.B.name())) {
            SyncDataResult syncDataResult = syncUser(jdbcTemplate, schoolId, examId, datasourceType.getConfigValue().toString());
            stringJoiner.add(DataType.B.name() + ":" + syncDataResult.getCount());
            sendSms = StringUtils.isNotBlank(syncDataResult.getErrMsg()) ? true : sendSms;
        }
        // 课程同步
        if (dataRange.contains(DataType.C.name())) {
            SyncDataResult syncDataResult = syncCourse(jdbcTemplate, schoolId, examId, datasourceType.getConfigValue().toString());
            stringJoiner.add(DataType.C.name() + ":" + syncDataResult.getCount());
            sendSms = StringUtils.isNotBlank(syncDataResult.getErrMsg()) ? true : sendSms;
        }
        // 考生同步
        if (dataRange.contains(DataType.D.name())) {
            SyncDataResult syncDataResult = syncStudent(jdbcTemplate, schoolId, examId, datasourceType.getConfigValue().toString());
            stringJoiner.add(DataType.D.name() + ":" + syncDataResult.getCount());
            sendSms = StringUtils.isNotBlank(syncDataResult.getErrMsg()) ? true : sendSms;
        }
        // 命题任务同步
        if (dataRange.contains(DataType.E.name())) {
            SyncDataResult syncDataResult = syncExamTask(jdbcTemplate, schoolId, examId, datasourceType.getConfigValue().toString());
            stringJoiner.add(DataType.E.name() + ":" + syncDataResult.getCount());
            sendSms = StringUtils.isNotBlank(syncDataResult.getErrMsg()) ? true : sendSms;
        }

        syncData.setDetail(stringJoiner.toString());

        // 有一个失败，发送短信
        if (sendSms && StringUtils.isNotBlank(syncData.getPhoneNumber())) {
            BasicSchool basicSchool = commonCacheService.schoolCache(schoolId);
            Map<String, String> variableParams = new HashMap<>();
            variableParams.put("schoolName", basicSchool.getName());
            for (String phoneNumber : syncData.getPhoneNumber().split(",")) {
                basicMessageService.saveMessageSendLog(schoolId, null, null, phoneNumber, JSON.toJSONString(variableParams), MessageEnum.NOTICE_SYNC_DATA);
            }
        }
    }

    private SyncDataResult syncOrg(JdbcTemplate jdbcTemplate, Long schoolId, Long examId, String datasourceType) {
        long startTime = System.currentTimeMillis();
        List<OrgDataVo> dataVoList = dataUtil.listOrg(jdbcTemplate, DataType.A, datasourceType);
        StringJoiner errorMsg = new StringJoiner("\n");
        AtomicInteger i = new AtomicInteger(0);
        for (OrgDataVo orgDataVo : dataVoList) {
            try {
                sysOrgService.saveOrgFromSync(schoolId, orgDataVo);
                i.getAndIncrement();
            } catch (Exception e) {
                errorMsg.add(e.getMessage());
            }
        }
        long endTime = System.currentTimeMillis();
        tSyncDataLogService.saveLog(schoolId, examId, DataType.A.name(), startTime, endTime, dataVoList.size(), i.get(), errorMsg.toString());
        return new SyncDataResult(i.get(), errorMsg.toString());
    }

    private SyncDataResult syncUser(JdbcTemplate jdbcTemplate, Long schoolId, Long examId, String datasourceType) {
        long startTime = System.currentTimeMillis();
        List<UserDataVo> dataVoList = dataUtil.listUser(jdbcTemplate, DataType.B, datasourceType);
        StringJoiner errorMsg = new StringJoiner("\n");
        AtomicInteger i = new AtomicInteger(0);
        for (UserDataVo userDataVo : dataVoList) {
            try {
                sysUserService.saveUserFromSync(schoolId, userDataVo);
                i.getAndIncrement();
            } catch (Exception e) {
                errorMsg.add(e.getMessage());
            }
        }
        long endTime = System.currentTimeMillis();
        tSyncDataLogService.saveLog(schoolId, examId, DataType.B.name(), startTime, endTime, dataVoList.size(), i.get(), errorMsg.toString());
        return new SyncDataResult(i.get(), errorMsg.toString());
    }

    private SyncDataResult syncCourse(JdbcTemplate jdbcTemplate, Long schoolId, Long examId, String datasourceType) {
        long startTime = System.currentTimeMillis();
        List<CourseDataVo> dataVoList = dataUtil.listCourse(jdbcTemplate, DataType.C, datasourceType);
        StringJoiner errorMsg = new StringJoiner("\n");
        AtomicInteger i = new AtomicInteger(0);
        for (CourseDataVo courseDataVo : dataVoList) {
            try {
                basicCourseService.saveBasicCourseFormSync(schoolId, courseDataVo);
                i.getAndIncrement();
            } catch (Exception e) {
                errorMsg.add(e.getMessage());
            }
        }
        long endTime = System.currentTimeMillis();
        tSyncDataLogService.saveLog(schoolId, examId, DataType.C.name(), startTime, endTime, dataVoList.size(), i.get(), errorMsg.toString());
        return new SyncDataResult(i.get(), errorMsg.toString());
    }

    private SyncDataResult syncStudent(JdbcTemplate jdbcTemplate, Long schoolId, Long examId, String datasourceType) {
        long startTime = System.currentTimeMillis();
        List<TSyncDataStudent> dataVoList = dataUtil.listExamStudent(jdbcTemplate, DataType.D, datasourceType);
        Map<String, Object> map = new HashMap<>();
        int successCount = 0;
        String errMsg = null;
        try {
            for (TSyncDataStudent tSyncDataStudent : dataVoList) {
                tSyncDataStudent.setId(SystemConstant.getDbUuid());
                tSyncDataStudent.setSchoolId(schoolId);
                tSyncDataStudent.setExamId(examId);

                // 解析考试日期和考试时间
                if (StringUtils.isNoneBlank(tSyncDataStudent.getExamDate(), tSyncDataStudent.getExamTime())) {
                    try {
                        Map<String, Object> timeMap = ConvertUtil.analyzeExamTime(tSyncDataStudent.getExamDate(), tSyncDataStudent.getExamTime());
                        tSyncDataStudent.setExamStartTime(Long.valueOf(String.valueOf(timeMap.get("startTime"))));
                        tSyncDataStudent.setExamEndTime(Long.valueOf(String.valueOf(timeMap.get("endTime"))));
                    } catch (Exception e) {
                    }
                }
            }
            tSyncDataStudentService.remove(new UpdateWrapper<TSyncDataStudent>().lambda().eq(TSyncDataStudent::getSchoolId, schoolId).eq(TSyncDataStudent::getExamId, examId));
            tSyncDataStudentService.saveBatch(dataVoList);
            map = basicExamStudentService.saveBasicExamStudentFormSync(schoolId, examId, dataVoList);
        } catch (Exception e) {
            errMsg = e.getMessage();
        } finally {
            successCount = map.containsKey("count") ? Integer.parseInt(map.get("count").toString()) : 0;
            errMsg = errMsg == null && map.containsKey("errMsg") ? map.get("errMsg").toString() : null;
            long endTime = System.currentTimeMillis();
            tSyncDataLogService.saveLog(schoolId, examId, DataType.D.name(), startTime, endTime, dataVoList.size(), successCount, errMsg);
        }
        return new SyncDataResult(successCount, errMsg);
    }

    private SyncDataResult syncExamTask(JdbcTemplate jdbcTemplate, Long schoolId, Long examId, String datasourceType) {
        long startTime = System.currentTimeMillis();
        List<ExamTaskDataVo> dataVoList = dataUtil.listExamTask(jdbcTemplate, DataType.E, datasourceType);
        StringJoiner errorMsg = new StringJoiner("\n");
        AtomicInteger i = new AtomicInteger(0);
        for (ExamTaskDataVo examTaskDataVo : dataVoList) {
            try {
                examTaskService.saveExamTaskFormSync(schoolId, examId, examTaskDataVo);
                i.getAndIncrement();
            } catch (Exception e) {
                errorMsg.add(e.getMessage());
            }
        }
        long endTime = System.currentTimeMillis();
        tSyncDataLogService.saveLog(schoolId, examId, DataType.E.name(), startTime, endTime, dataVoList.size(), i.get(), errorMsg.toString());
        return new SyncDataResult(i.get(), errorMsg.toString());
    }
}
