package com.qmth.teachcloud.mark.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.boot.tools.excel.ExcelWriter;
import com.qmth.boot.tools.excel.enums.ExcelType;
import com.qmth.boot.tools.io.ZipReader;
import com.qmth.teachcloud.common.bean.dto.DataPermissionRule;
import com.qmth.teachcloud.common.bean.dto.mark.MarkSettingDto;
import com.qmth.teachcloud.common.bean.params.mark.setting.MarkPaperSettingConfig;
import com.qmth.teachcloud.common.bean.params.mark.setting.MarkPaperSettingList;
import com.qmth.teachcloud.common.bean.params.mark.setting.MarkPaperSettingParam;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.BasicCourse;
import com.qmth.teachcloud.common.entity.BasicExam;
import com.qmth.teachcloud.common.entity.MarkQuestion;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.enums.ObjectivePolicy;
import com.qmth.teachcloud.common.enums.UploadFileEnum;
import com.qmth.teachcloud.common.enums.mark.MarkMode;
import com.qmth.teachcloud.common.enums.mark.MarkPaperStatus;
import com.qmth.teachcloud.common.enums.mark.SubjectiveStatus;
import com.qmth.teachcloud.common.service.BasicCourseService;
import com.qmth.teachcloud.common.service.BasicRoleDataPermissionService;
import com.qmth.teachcloud.common.service.TeachcloudCommonService;
import com.qmth.teachcloud.common.util.FileUtil;
import com.qmth.teachcloud.common.util.ServletUtil;
import com.qmth.teachcloud.mark.bean.answercard.CardFile;
import com.qmth.teachcloud.mark.bean.archivescore.ArchiveScoreQuery;
import com.qmth.teachcloud.mark.bean.archivescore.ArchiveScoreVo;
import com.qmth.teachcloud.mark.bean.cardJson.CourseCard;
import com.qmth.teachcloud.mark.bean.cardJson.Description;
import com.qmth.teachcloud.mark.bean.cardJson.DescriptionCourse;
import com.qmth.teachcloud.mark.bean.document.ArchivePaperQuery;
import com.qmth.teachcloud.mark.bean.document.ArchivePaperVo;
import com.qmth.teachcloud.mark.bean.vo.parseCard.Struct;
import com.qmth.teachcloud.mark.dto.ArchiveScoreExportDto;
import com.qmth.teachcloud.mark.dto.mark.MarkPaperFileDto;
import com.qmth.teachcloud.mark.dto.mark.MarkQuestionAnswerVo;
import com.qmth.teachcloud.mark.dto.mark.score.CheckScoreListDto;
import com.qmth.teachcloud.mark.dto.mark.score.MarkPaperPackageDto;
import com.qmth.teachcloud.mark.dto.mark.score.SettingDto;
import com.qmth.teachcloud.mark.entity.*;
import com.qmth.teachcloud.mark.enums.CardSource;
import com.qmth.teachcloud.mark.enums.MarkTaskStatus;
import com.qmth.teachcloud.mark.enums.OmrTaskStatus;
import com.qmth.teachcloud.mark.enums.PaperTypeCheckStatus;
import com.qmth.teachcloud.mark.mapper.MarkPaperMapper;
import com.qmth.teachcloud.mark.service.*;
import com.qmth.teachcloud.mark.utils.Calculator;
import com.qmth.teachcloud.mark.utils.CardParseUtils;
import com.qmth.teachcloud.mark.utils.FileStoreUtils;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.io.FileUtils;
import org.apache.commons.io.FilenameUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.net.URLEncoder;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * <p>
 * 考试科目表 服务实现类
 * </p>
 *
 * @author xf
 * @since 2023-09-22
 */
@Service
public class MarkPaperServiceImpl extends ServiceImpl<MarkPaperMapper, MarkPaper> implements MarkPaperService {

    @Resource
    private ScanPackageService scanPackageService;
    @Resource
    private ScanOmrTaskService scanOmrTaskService;
    @Resource
    private ScanAnswerCardService scanAnswerCardService;
    @Resource
    private TeachcloudCommonService teachcloudCommonService;
    @Resource
    private BasicRoleDataPermissionService basicRoleDataPermissionService;
    @Resource
    private MarkQuestionService markQuestionService;
    @Resource
    private MarkQuestionAnswerService markQuestionAnswerService;
    @Resource
    private MarkStudentService markStudentService;
    @Resource
    private MarkTaskService markTaskService;
    @Resource
    private MarkUserPaperService markUserPaperService;
    @Resource
    private MarkDocumentService markDocumentService;
    @Resource
    private BasicCourseService basicCourseService;
    @Resource
    private FileStoreUtils fileStoreUtils;
    @Resource
    private MarkFileService markFileService;

    @Override
    public IPage<MarkSettingDto> listPaperSetting(Long examId, Long openCollegeId, Long courseId, String paperNumber, Boolean groupStatus, Integer pageNumber, Integer pageSize) {
        Page<MarkSettingDto> page = new Page<>(pageNumber, pageSize);
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        DataPermissionRule dpr = basicRoleDataPermissionService.findDataPermission(sysUser.getSchoolId(), sysUser.getId(), "/api/admin/mark/setting/list");
        IPage<MarkSettingDto> markSettingDtoIPage = this.baseMapper.listPaperSetting(page, sysUser.getSchoolId(), examId, openCollegeId, courseId, paperNumber, groupStatus, dpr);
        for (MarkSettingDto record : markSettingDtoIPage.getRecords()) {
            if (Objects.nonNull(record.getMarkMode())) {
                record.setMarkModeDisplay(record.getMarkMode().getName());
            }
        }
        return markSettingDtoIPage;
    }

    @Override
    public MarkPaper getByExamIdAndPaperNumber(Long examId, String paperNumber) {
        QueryWrapper<MarkPaper> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(MarkPaper::getExamId, examId)
                .eq(MarkPaper::getPaperNumber, paperNumber);
        return this.getOne(queryWrapper);
    }

    @Transactional
    @Override
    public void savePaperSetting(MarkPaper markPaper) {
        MarkPaper markPaperOld = this.getByExamIdAndPaperNumber(markPaper.getExamId(), markPaper.getPaperNumber());
        if (markPaperOld == null) {
            throw ExceptionResultEnum.ERROR.exception("评卷设置数据不存在");
        }
        // 评卷是否结束
        if (MarkPaperStatus.FINISH.equals(markPaperOld.getStatus())) {
            throw ExceptionResultEnum.ERROR.exception("科目已结束评卷，无法修改参数");
        }
        UpdateWrapper<MarkPaper> updateWrapper = new UpdateWrapper<>();
        LambdaUpdateWrapper<MarkPaper> lambda = updateWrapper.lambda();
        lambda.set(MarkPaper::getMarkMode, markPaper.getMarkMode())
                .set(MarkPaper::getForceMode, !MarkMode.UNLIMITED.equals(markPaper.getMarkMode()))
                .set(MarkPaper::getMarkStartTime, markPaper.getMarkStartTime())
                .set(MarkPaper::getMarkEndTime, markPaper.getMarkEndTime())
                .set(MarkPaper::getPassScore, markPaper.getPassScore())
                .set(MarkPaper::getExcellentScore, markPaper.getExcellentScore())
                .set(MarkPaper::getSheetView, markPaper.getSheetView())
                .set(MarkPaper::getShowObjectScore, markPaper.getShowObjectScore())
                .set(MarkPaper::getAutoScroll, markPaper.getAutoScroll())
                .set(MarkPaper::getDoubleMark, markPaper.getDoubleMark())
                .set(MarkPaper::getMergeMarker, markPaper.getMergeMarker())
                .eq(MarkPaper::getExamId, markPaper.getExamId())
                .eq(MarkPaper::getPaperNumber, markPaper.getPaperNumber());
        if (StringUtils.isNotBlank(markPaper.getSheetConfig())) {
            lambda.set(MarkPaper::getSheetConfig, markPaper.getSheetConfig());
        }
        // 取消双评时，校验题目是否有设置双评
        if (markPaper.getDoubleMark() != null && !markPaper.getDoubleMark()) {
            List<MarkQuestion> markQuestionList = markQuestionService.listByExamIdAndPaperNumberAndObjective(markPaper.getExamId(), markPaper.getPaperNumber(), false);
            long count = markQuestionList.stream().filter(m -> m.getDoubleRate() != null && m.getDoubleRate() > 0).count();
            if (count > 0) {
                throw ExceptionResultEnum.ERROR.exception("部分题目开启了双评，【是否开启双评】不能取消");
            }
        }

        // 更新评卷员模式
        markUserPaperService.updateModeBatch(markPaper.getExamId(), markPaper.getPaperNumber(), MarkMode.UNLIMITED.equals(markPaper.getMarkMode()) ? MarkMode.TRACK : markPaper.getMarkMode());
        this.update(updateWrapper);
    }

    @Transactional
    @Override
    public void saveBatchPaperSetting(MarkPaperSettingParam markPaperSettingParam) {
        List<MarkPaperSettingList> list = markPaperSettingParam.getList();
        if (CollectionUtils.isEmpty(list)) {
            throw ExceptionResultEnum.ERROR.exception("请选择要设置的数据");
        }

        MarkPaperSettingConfig config = markPaperSettingParam.getConfig();
        if (config == null) {
            throw ExceptionResultEnum.ERROR.exception("未找到参数值，请先设置");
        }

        for (MarkPaperSettingList markPaperSettingList : list) {
            MarkPaper markPaperOld = this.getByExamIdAndPaperNumber(markPaperSettingList.getExamId(), markPaperSettingList.getPaperNumber());
            if (markPaperOld == null) {
                throw ExceptionResultEnum.ERROR.exception("评卷设置数据不存在");
            }
            // 评卷是否结束
            if (MarkPaperStatus.FINISH.equals(markPaperOld.getStatus())) {
                throw ExceptionResultEnum.ERROR.exception("科目已结束评卷，无法修改参数");
            }

            UpdateWrapper<MarkPaper> updateWrapper = new UpdateWrapper<>();
            LambdaUpdateWrapper<MarkPaper> lambda = updateWrapper.lambda();
            lambda.set(MarkPaper::getMarkMode, config.getMarkMode())
                    .set(MarkPaper::getForceMode, !MarkMode.UNLIMITED.equals(config.getMarkMode()))
                    .set(MarkPaper::getMarkStartTime, config.getMarkStartTime())
                    .set(MarkPaper::getMarkEndTime, config.getMarkEndTime())
                    .set(MarkPaper::getPassScore, config.getPassScore())
                    .set(MarkPaper::getExcellentScore, config.getExcellentScore())
                    .set(MarkPaper::getShowObjectScore, config.getShowObjectScore())
                    .set(MarkPaper::getAutoScroll, config.getAutoScroll())
                    .set(MarkPaper::getDoubleMark, config.getDoubleMark())
                    .set(MarkPaper::getMergeMarker, config.getMergeMarker())
                    .eq(MarkPaper::getExamId, markPaperSettingList.getExamId())
                    .eq(MarkPaper::getPaperNumber, markPaperSettingList.getPaperNumber());
            // 更新评卷员模式
            markUserPaperService.updateModeBatch(markPaperSettingList.getExamId(), markPaperSettingList.getPaperNumber(), MarkMode.UNLIMITED.equals(config.getMarkMode()) ? MarkMode.TRACK : config.getMarkMode());
            this.update(updateWrapper);
        }
    }

    @Override
    public Boolean finishPaper(BasicExam basicExam, List<String> paperNumbers, MarkPaperStatus status) {
        Long examId = basicExam.getId();
        StringJoiner stringJoiner = new StringJoiner(";");
        for (String paperNumber : paperNumbers) {
            MarkPaper markPaper = this.getByExamIdAndPaperNumber(examId, paperNumber);
            if (MarkPaperStatus.FINISH.equals(status)) {
                BasicCourse basicCourse = basicCourseService.getById(markPaper.getCourseId());
                String courseInfo = String.format("%s[%s]，试卷编号%s，", basicCourse.getName(), basicCourse.getCode(), markPaper.getPaperNumber());
                // 客观题标答是否全部设置
                List<MarkQuestionAnswerVo> list = new ArrayList<>();
                for (String paperType : markPaper.getPaperTypeList()) {
                    list.addAll(markQuestionService.listQuestionAnswerByExamIdAndPaperNumberAndPaperType(examId, paperNumber, paperType, true));
                }
                if (CollectionUtils.isNotEmpty(list)) {
                    if (list.stream().filter(m -> StringUtils.isBlank(m.getAnswer())).count() > 0) {
                        stringJoiner.add(courseInfo + "客观题标答未设置，无法结束评卷");
                        continue;
                    }
                }

                // 主观题是否全部分组
                List<MarkQuestion> markQuestionSubjectiveList = markQuestionService.listByExamIdAndPaperNumberAndObjective(examId, paperNumber, false);
                // 没有主观题，不校验考生评卷
                if (CollectionUtils.isNotEmpty(markQuestionSubjectiveList)) {
                    if (markTaskService.countByExamIdAndPaperNumberAndStatusIn(examId, paperNumber, Arrays.asList(MarkTaskStatus.WAITING, MarkTaskStatus.WAIT_ARBITRATE, MarkTaskStatus.REJECTED, MarkTaskStatus.PROBLEM)) > 0) {
                        stringJoiner.add(courseInfo + "考生未全部评完，无法结束评卷");
                        continue;
                    }
                    // 未缺考、未违纪且已上传图片的考生全部评完
                    else if (markStudentService.countByExamIdAndPaperNumberAndMarkStatus(examId, paperNumber, SubjectiveStatus.UNMARK) > 0) {
                        stringJoiner.add(courseInfo + "考生正在统分中，无法结束评卷");
                        continue;
                    }
                    // 未全部扫描，不能结束
                    if (markStudentService.countUnexistByExamIdAndPaperNumber(examId, paperNumber) > 0) {
                        stringJoiner.add(courseInfo + "有考生未扫描未核对，请在扫描客户端进行确认操作后再结束阅卷");
                        continue;
                    }
                    // 识别缺考未确认完，不能结束
                    if (markStudentService.countOmrAbsentStudent(examId, paperNumber, false) > 0) {
                        stringJoiner.add(courseInfo + "识别缺考记录未核对，请在扫描客户端进行确认操作后再结束阅卷");
                        continue;
                    }
                    // 人工绑定未做完，不能结束
                    if (markStudentService.getAssignedCount(examId, false, markPaper.getCourseId(), markPaper.getCoursePaperId(), MarkPaperStatus.FORMAL, null) > 0) {
                        stringJoiner.add(courseInfo + "人工绑定记录未核对，请在扫描客户端进行确认操作后再结束阅卷");
                        continue;
                    }
                    // 卷型检查未做完，不能结束
                    if (markStudentService.getPaperTypeCheckCount(examId, markPaper.getCourseId(), markPaper.getCoursePaperId(), PaperTypeCheckStatus.WAITING, null) > 0) {
                        stringJoiner.add(courseInfo + "卷型检查记录未核对，请在扫描客户端进行确认操作后再结束阅卷");
                        continue;
                    }
                    // 客观题检查未做完，不能结束
                    if (scanOmrTaskService.getFinishStudentCountByExamAndUserId(examId, markPaper.getCourseId(), markPaper.getCoursePaperId(), OmrTaskStatus.WAITING.name(), null) > 0) {
                        stringJoiner.add(courseInfo + "客观题存在识别嫌疑需要人工确认，请在扫描客户端进行确认操作后再结束阅卷");
                        continue;
                    }
                }

                // 结束评卷时，文档进行归档
                markDocumentService.initMarkDocument(basicExam, markPaper);
            }

            UpdateWrapper<MarkPaper> updateWrapper = new UpdateWrapper<>();
            updateWrapper.lambda().set(MarkPaper::getStatus, status)
                    .eq(MarkPaper::getExamId, examId)
                    .eq(MarkPaper::getPaperNumber, paperNumber);
            this.update(updateWrapper);
        }

        if (StringUtils.isNotBlank(stringJoiner.toString())) {
            throw ExceptionResultEnum.ERROR.exception(stringJoiner.toString());
        }

        return true;
    }

    /**
     * 校验试卷
     *
     * @param examId
     * @param paperNumber
     * @param teachClassName
     * @return
     */
    @Override
    public Boolean finishPaper(Long examId, String paperNumber, String teachClassName) {
        MarkPaper markPaper = this.getByExamIdAndPaperNumber(examId, paperNumber);
        Objects.requireNonNull(markPaper, "未找到评卷信息");
        BasicCourse basicCourse = basicCourseService.getById(markPaper.getCourseId());
        String courseInfo = String.format("%s[%s]，试卷编号%s，", basicCourse.getName(), basicCourse.getCode(), markPaper.getPaperNumber());
        // 客观题标答是否全部设置
        for (String paperType : markPaper.getPaperTypeList()) {
            List<MarkQuestionAnswerVo> markQuestionObjectiveList = markQuestionService.listQuestionAnswerByExamIdAndPaperNumberAndPaperType(examId, paperNumber, paperType, true);
            if (CollectionUtils.isNotEmpty(markQuestionObjectiveList)) {
                if (markQuestionObjectiveList.stream().filter(m -> StringUtils.isBlank(m.getAnswer())).count() > 0) {
                    throw ExceptionResultEnum.ERROR.exception(courseInfo + "客观题标答未设置，无法结束评卷");
                }
            }
        }


        // 主观题是否全部分组
        List<MarkQuestion> markQuestionSubjectiveList = markQuestionService.listByExamIdAndPaperNumberAndObjective(examId, paperNumber, false);
        // 没有主观题，不校验考生评卷
        if (CollectionUtils.isNotEmpty(markQuestionSubjectiveList)) {
            // 识别缺考未确认完，不能结束
            if (markStudentService.countOmrAbsentStudent(examId, paperNumber, false, teachClassName) > 0) {
                throw ExceptionResultEnum.ERROR.exception(courseInfo + "请登录扫描客户端完成缺考检查待处理任务");
            }
            // 人工绑定未做完，不能结束
            if (markStudentService.getAssignedCount(examId, false, markPaper.getCourseId(), markPaper.getCoursePaperId(), MarkPaperStatus.FORMAL, null, teachClassName) > 0) {
                throw ExceptionResultEnum.ERROR.exception(courseInfo + "请登录扫描客户端完成人工绑定检查待处理任务");
            }
            // 客观题检查未做完，不能结束
            if (scanOmrTaskService.getFinishStudentCountByExamAndUserId(examId, markPaper.getCourseId(), markPaper.getCoursePaperId(), OmrTaskStatus.WAITING.name(), null, teachClassName) > 0) {
                throw ExceptionResultEnum.ERROR.exception(courseInfo + "请登录扫描客户端完成客观题检查待处理任务");
            }
        }
        return true;
    }

    @Override
    public List<MarkPaper> listQualityMarkPaperByStatus(MarkPaperStatus status, int uploadCount) {
        return this.baseMapper.listQualityMarkPaperByStatus(status.name(), uploadCount);
    }

    @Override
    public int getCourseCount(Long examId, Long courseId, String coursePaperId, MarkPaperStatus status, DataPermissionRule dpr) {
        return baseMapper.getCourseCount(examId, courseId, coursePaperId, status.name(), dpr);
    }

    @Override
    public int getPaperNumberCount(Long examId, Long courseId, String coursePaperId, MarkPaperStatus status, DataPermissionRule dpr) {
        return baseMapper.getPaperNumberCount(examId, courseId, coursePaperId, status.name(), dpr);
    }

    @Override
    public IPage<CheckScoreListDto> listStudentScoreList(Long examId, Long openCollegeId, Long courseId, String paperNumber, Integer pageNumber, Integer pageSize) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        Page<CheckScoreListDto> page = new Page<>(pageNumber, pageSize);
        DataPermissionRule dpr = basicRoleDataPermissionService.findDataPermission(sysUser.getSchoolId(), sysUser.getId(), ServletUtil.getRequest().getServletPath());
        return this.baseMapper.listStudentScoreList(page, examId, openCollegeId, courseId, paperNumber, dpr);
    }

    @Override
    public void updateUploadCount(Long examId, String paperNumber, int countUploaded) {
        UpdateWrapper<MarkPaper> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(MarkPaper::getUploadCount, countUploaded)
                .eq(MarkPaper::getExamId, examId)
                .eq(MarkPaper::getPaperNumber, paperNumber);
        this.update(updateWrapper);
    }

    @Override
    public IPage<MarkPaperPackageDto> listPackage(Long examId, String paperNumber, String packageCode, Integer pageNumber, Integer pageSize) {
        MarkPaper markPaper = this.getByExamIdAndPaperNumber(examId, paperNumber);
        Page<MarkPaperPackageDto> page = new Page<>(pageNumber, pageSize);
        IPage<MarkPaperPackageDto> markPaperPackageDtoIPage = this.baseMapper.listPackage(page, examId, markPaper.getCoursePaperId(), packageCode);
        for (MarkPaperPackageDto packageDto : markPaperPackageDtoIPage.getRecords()) {
            List<ScanPackage> scanPackageList = scanPackageService.listByExamIdAndCoursePaperIdAndPackageCode(examId, markPaper.getCoursePaperId(), packageDto.getPackageCode());
            List<String> urls = scanPackageList.stream().map(m -> teachcloudCommonService.filePreview(m.getPath())).collect(Collectors.toList());
            packageDto.setCount(urls.size());
            packageDto.setUrls(urls);
        }
        return markPaperPackageDtoIPage;
    }

    @Override
    public MarkPaper getByExamIdAndCoursePaperId(Long examId, String coursePaperId) {
        QueryWrapper<MarkPaper> wrapper = new QueryWrapper<>();
        wrapper.lambda().eq(MarkPaper::getExamId, examId)
                .eq(MarkPaper::getCoursePaperId, coursePaperId);
        wrapper.last("LIMIT 1");
        return this.getOne(wrapper);
    }

    @Override
    public List<MarkPaper> listByExamId(Long examId, MarkPaperStatus status, DataPermissionRule dpr) {
        return this.baseMapper.listByExamId(examId, status.name(), dpr);
    }

    @Override
    public int countByPropositionTeacherId(boolean status) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        DataPermissionRule dpr = basicRoleDataPermissionService.findDataPermission(schoolId, sysUser.getId(),
                "/api/admin/mark/setting/list");
        List<MarkSettingDto> markSettingDtoList = this.baseMapper.listPaperSetting(schoolId, null, null, null, null,
                status, dpr);
        return CollectionUtils.isEmpty(markSettingDtoList) ? 0 : markSettingDtoList.size();
    }

    @Override
    public void updateStudentCountByExamIdAndPaperNumber(Long examId, String paperNumber) {
        long count = markStudentService.countByExamIdAndPaperNumber(examId, paperNumber);
        UpdateWrapper<MarkPaper> updateWrapper = new UpdateWrapper<>();
        LambdaUpdateWrapper<MarkPaper> lambda = updateWrapper.lambda();
        lambda.set(MarkPaper::getStudentCount, count)
                .eq(MarkPaper::getExamId, examId)
                .eq(MarkPaper::getPaperNumber, paperNumber);
        this.update(updateWrapper);
    }

    @Override
    public void updateStudentCountByExamIdAndPaperNumberAndSerialNumber(Long examId, String paperNumber, Integer serialNumber) {
        long count = markStudentService.countByExamIdAndPaperNumberAndSerialNumber(examId, paperNumber, serialNumber);
        UpdateWrapper<MarkPaper> updateWrapper = new UpdateWrapper<>();
        LambdaUpdateWrapper<MarkPaper> lambda = updateWrapper.lambda();
        lambda.set(MarkPaper::getStudentCount, count)
                .eq(MarkPaper::getExamId, examId)
                .eq(MarkPaper::getPaperNumber, paperNumber)
                .eq(MarkPaper::getSerialNumber, serialNumber);
        this.update(updateWrapper);
    }

    @Override
    public SettingDto getSetting(Long examId, String paperNumber) {
        SettingDto settingDto = new SettingDto();
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        settingDto.setExamId(examId);
        settingDto.setPaperNumber(paperNumber);
        settingDto.setUserName(sysUser.getRealName());
        MarkPaper markPaper = this.getByExamIdAndPaperNumber(examId, paperNumber);
        if (markPaper != null) {
            settingDto.setCourseCode(markPaper.getCourseCode());
            settingDto.setCourseName(markPaper.getCourseName());
            if (StringUtils.isNotBlank(markPaper.getPaperFilePath())) {
                List<MarkPaperFileDto> markPaperFileDtos = new ArrayList<>();
                if (markPaper != null && StringUtils.isNotBlank(markPaper.getPaperFilePath())) {
                    markPaperFileDtos = JSON.parseArray(markPaper.getPaperFilePath(), MarkPaperFileDto.class);
                    for (MarkPaperFileDto markPaperFileDto : markPaperFileDtos) {
                        markPaperFileDto.setUrl(teachcloudCommonService.filePreview(JSON.toJSONString(markPaperFileDto.getFilePathVo())));
                        markPaperFileDto.setFilePathVo(null);
                    }
                }
                settingDto.setPaperUrl(markPaperFileDtos);
            }
            if (StringUtils.isNotBlank(markPaper.getAnswerFilePath())) {
                List<MarkPaperFileDto> markPaperFileDtos = new ArrayList<>();
                if (markPaper != null && StringUtils.isNotBlank(markPaper.getAnswerFilePath())) {
                    markPaperFileDtos = JSON.parseArray(markPaper.getAnswerFilePath(), MarkPaperFileDto.class);
                    for (MarkPaperFileDto markPaperFileDto : markPaperFileDtos) {
                        markPaperFileDto.setUrl(teachcloudCommonService.filePreview(JSON.toJSONString(markPaperFileDto.getFilePathVo())));
                        markPaperFileDto.setFilePathVo(null);
                    }
                }
                settingDto.setAnswerUrl(markPaperFileDtos);
            }
        }
        return settingDto;
    }

    @Override
    public IPage<ArchiveScoreVo> scoreList(ArchiveScoreQuery query) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        Page<ArchiveScoreVo> page = new Page<>(query.getPageNumber(), query.getPageSize());
        DataPermissionRule dpr = basicRoleDataPermissionService.findDataPermission(sysUser.getSchoolId(), sysUser.getId(), ServletUtil.getRequest().getServletPath());
        IPage<ArchiveScoreVo> ret = this.baseMapper.scoreList(page, sysUser.getSchoolId(), query, dpr);
        if (CollectionUtils.isNotEmpty(ret.getRecords())) {
            for (ArchiveScoreVo vo : ret.getRecords()) {
                double total = vo.getStudentCount() - vo.getAbsentCount();
                vo.setPassRate(Calculator.divide2String(Calculator.multiply(vo.getPassCount(), 100), total, 2));
                vo.setExcellentRate(Calculator.divide2String(Calculator.multiply(vo.getExcellentCount(), 100), total, 2));
                vo.setAvgScore(Calculator.round(vo.getAvgScore(), 2));
                vo.setMinScore(Calculator.round(vo.getMinScore(), 2));
                vo.setMaxScore(Calculator.round(vo.getMaxScore(), 2));
            }
        }
        return ret;
    }

    @Override
    public void deleteByExamIdAndPaperNumber(Long examId, String paperNumber) {
        UpdateWrapper<MarkPaper> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().eq(MarkPaper::getExamId, examId)
                .eq(MarkPaper::getPaperNumber, paperNumber);
        this.remove(updateWrapper);
    }

    @Override
    public void updateAbsentCount(Long examId, String paperNumber, int absentCount) {
        UpdateWrapper<MarkPaper> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(MarkPaper::getAbsentCount, absentCount)
                .eq(MarkPaper::getExamId, examId)
                .eq(MarkPaper::getPaperNumber, paperNumber);
        this.update(updateWrapper);
    }

    @Override
    public IPage<ArchivePaperVo> documentList(ArchivePaperQuery query) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        Page<ArchiveScoreVo> page = new Page<>(query.getPageNumber(), query.getPageSize());
        DataPermissionRule dpr = basicRoleDataPermissionService.findDataPermission(sysUser.getSchoolId(), sysUser.getId(), ServletUtil.getRequest().getServletPath());
        return this.baseMapper.documentList(page, query, dpr);
    }

    @Override
    public List<MarkPaper> findMarkPaperListByExamId(Long examId) {
        return this.baseMapper.findMarkPaperListByExamId(examId);
    }

    @Transactional
    @Override
    public void importCardJsonData(Long examId, MultipartFile file, String md5) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        File parentDirVar = null;
        try {
            String fileName = FilenameUtils.getName(file.getOriginalFilename());

            if (!fileName.endsWith(SystemConstant.ZIP_PREFIX)) {
                throw ExceptionResultEnum.ERROR.exception("数据包只能上传后缀为[.zip]的文件");
            }

            String fileMd5 = DigestUtils.md5Hex(file.getBytes());
            if (!Objects.equals(fileMd5, md5)) {
                throw ExceptionResultEnum.MD5_EQUALS_FALSE.exception();
            }

            // 数据包文件上传
            parentDirVar = SystemConstant.getFileTempDirVar(SystemConstant.ZIP_PREFIX);
            FileUtils.copyToFile(file.getInputStream(), parentDirVar);
            ZipReader zipReader = new ZipReader(parentDirVar);
            String zipRoot = "cards";
            String descriptionJson = "description.json";

            // 读取指定文件内容description.json
            String descriptionContent = SystemConstant.readContent(zipReader.read(zipRoot + "/" + descriptionJson));
            if (StringUtils.isBlank(descriptionContent)) {
                throw ExceptionResultEnum.ERROR.exception("数据包description.json文件内容为空");
            }
            // 解析description.json内容
            List<Description> descriptionList = JSON.parseArray(descriptionContent, Description.class);
            List<CourseCard> courseCardList = new ArrayList<>();
            if (CollectionUtils.isNotEmpty(descriptionList)) {
                for (Description description : descriptionList) {
                    for (DescriptionCourse course : description.getCourses()) {
                        courseCardList.add(new CourseCard(description.getCode(), course.getCode(), course.getName()));
                    }
                }
            }
            if (CollectionUtils.isEmpty(courseCardList)) {
                throw ExceptionResultEnum.ERROR.exception("没有可导入文件");
            }

            // 相同课程代码，取第一个题卡
            Map<String, CourseCard> courseCardMap = courseCardList.stream().collect(Collectors.toMap(m -> m.getCourseCode(), Function.identity(), (v1, v2) -> v1));

            File tempFile = SystemConstant.getFileTempDirVar(System.currentTimeMillis() + File.separator + SystemConstant.getNanoId(), SystemConstant.TEMP_PREFIX);
            String rootPath = tempFile.getParent();
            File jsonFile = null;
            for (Map.Entry<String, CourseCard> entry : courseCardMap.entrySet()) {
                String courseCode = entry.getKey();
                CourseCard courseCard = entry.getValue();
                String jsonFileName = zipRoot + "/" + courseCard.getCode() + SystemConstant.JSON_PREFIX;
                String jsonContent = SystemConstant.readContent(zipReader.read(jsonFileName));

                // 解析试卷结构，结构为空，跳过导入
                List<Struct> structList = CardParseUtils.parseCardContent(jsonContent);
                if (CollectionUtils.isEmpty(structList)) {
                    continue;
                }

                List<BasicCourse> basicCourseList = basicCourseService.listBySchoolIdAndCode(schoolId, courseCode);
                // 课程代码不存在，跳过导入
                if (CollectionUtils.isEmpty(basicCourseList)) {
                    continue;
                }
                // 课程代码一个对应多个开课学院，报错，文档里数据全部不写入
                else if (basicCourseList.size() > 1) {
                    throw ExceptionResultEnum.ERROR.exception(String.format("课程[%s(%s)]存在多个开课学院", courseCard.getCourseName(), courseCard.getCourseCode()));
                }
                BasicCourse basicCourse = basicCourseList.get(0);

                List<MarkPaper> markPaperList = this.listByExamIdAndCourseIdForModel4(examId, basicCourse.getId());
                if (CollectionUtils.isEmpty(markPaperList)) {
                    continue;
                } else if (markPaperList.size() > 1) {
                    throw ExceptionResultEnum.ERROR.exception(String.format("课程[%s(%s)]存在多个试卷", courseCard.getCourseName(), courseCard.getCourseCode()));
                }
                MarkPaper markPaper = markPaperList.get(0);
                // 结束阅卷，跳过导入
                if (MarkPaperStatus.FINISH.equals(markPaper.getStatus())) {
                    continue;
                }

                List<ScanAnswerCard> scanAnswerCardList = scanAnswerCardService.listByExamIdAndPaperNumber(examId, markPaper.getPaperNumber());
                // 已创建自定义卡格式，跳过导入
                if (scanAnswerCardList.stream().filter(m -> CardSource.CLIENT.equals(m.getSource())).count() > 1) {
                    continue;
                }
                // 电子卡格式且已适配，跳过导入
                Optional<ScanAnswerCard> optional = scanAnswerCardList.stream().filter(m -> CardSource.WEB.equals(m.getSource())).findFirst();
                ScanAnswerCard scanAnswerCard = null;
                if (optional.isPresent()) {
                    scanAnswerCard = optional.get();
                    if (StringUtils.isNotBlank(scanAnswerCard.getAdapteUri())) {
                        continue;
                    }
                }

                // 主观题已分组，跳过导入
//                List<MarkQuestion> markQuestionList = markQuestionService.listByExamIdAndPaperNumberAndObjective(examId, markPaper.getPaperNumber(), false);
//                if (CollectionUtils.isNotEmpty(markQuestionList) && markQuestionList.stream().filter(m -> m.getGroupNumber() != null).count() > 1) {
//                    continue;
//                }

                // 上传电子卡格式
                String jsonPath = rootPath + File.separator + SystemConstant.getDbUuid() + SystemConstant.JSON_PREFIX;
                CardFile cardFile = CardFile.parse(jsonContent);
                jsonFile = SystemConstant.createJsonFile(jsonPath, cardFile.output());
                Integer number = scanAnswerCardService.findMaxCardNumberByExamId(examId) + 1;
                String filePathName = markFileService.getAnswerCardUri(examId, markPaper.getCoursePaperId(), number);
                String jsonMd5 = DigestUtils.md5Hex(new FileInputStream(jsonFile));
                String uploadPath = fileStoreUtils.uploadFile(new FileInputStream(jsonFile), jsonMd5, UploadFileEnum.CARD, filePathName);
                if (StringUtils.isNotBlank(uploadPath)) {
                    // 删除试卷结构
                    markQuestionService.deleteByExamIdAndPaperNumber(examId, markPaper.getPaperNumber());
                    // 保存试卷结构
                    List<MarkQuestion> markQuestions = new ArrayList<>();
                    List<MarkQuestionAnswer> markQuestionAnswers = new ArrayList<>();
                    for (Struct struct : structList) {
                        MarkQuestion markQuestion = new MarkQuestion();
                        markQuestion.setId(SystemConstant.getDbUuid());
                        markQuestion.setExamId(examId);
                        markQuestion.setPaperNumber(markPaper.getPaperNumber());
                        markQuestion.setObjective(struct.getObjective());
                        markQuestion.setMainNumber(struct.getMainNumber());
                        markQuestion.setSubNumber(struct.getSubNumber());
                        markQuestion.setMainTitle(struct.getMainTitle());
                        markQuestion.setOptionCount(struct.getOptionCount());
                        markQuestion.setQuestionType(struct.getType());
                        markQuestion.setPaperIndex(struct.getPaperIndex());
                        markQuestion.setPageIndex(struct.getPageIndex());
                        if (struct.getScore() != null) {
                            markQuestion.setTotalScore(struct.getScore());
                            // 间隔分（整数默认1，小数默认0.5）
                            if (!markQuestion.getObjective()) {
                                markQuestion.setIntervalScore(markQuestion.getTotalScore() % 1 > 0 ? 0.5 : 1);
                            }
                        }
                        markQuestions.add(markQuestion);

                        // 保存客观题数据
                        for (String paperType : markPaper.getPaperTypeList()) {
                            MarkQuestionAnswer markQuestionAnswer = new MarkQuestionAnswer(examId, markPaper.getPaperNumber(), paperType, struct.getMainNumber(), struct.getSubNumber());
                            if (StringUtils.isNotBlank(struct.getAnswer())) {
                                // 客观题有标答时，给分策略默认为全对给分
                                markQuestionAnswer.setAnswer(struct.getAnswer());
                                markQuestionAnswer.setObjectivePolicy(ObjectivePolicy.NONE);
                            }
                            markQuestionAnswers.add(markQuestionAnswer);
                        }
                    }
                    markQuestionService.saveBatch(markQuestions);
                    markQuestionAnswerService.saveBatch(markQuestionAnswers);
                    // 删除卡格式
                    scanAnswerCardService.deleteByExamIdAndPaperNumber(examId, markPaper.getPaperNumber());
                    // 保存
                    scanAnswerCard = new ScanAnswerCard();
                    scanAnswerCard.setId(SystemConstant.getDbUuid());
                    scanAnswerCard.setExamId(examId);
                    scanAnswerCard.setPaperNumber(markPaper.getPaperNumber());
                    scanAnswerCard.setCoursePaperId(markPaper.getCoursePaperId());
                    scanAnswerCard.setPaperType(markPaper.getPaperType());
                    scanAnswerCard.setCardId(null);
                    scanAnswerCard.setNumber(scanAnswerCardService.findMaxCardNumberByExamId(examId) + 1);
                    scanAnswerCard.setSource(CardSource.WEB);
                    scanAnswerCard.setUri(uploadPath);
                    scanAnswerCard.setMd5(jsonMd5);
                    scanAnswerCard.setNeedAdapte(true);
                    scanAnswerCard.setSinglePage(false);// 默认false
                    scanAnswerCard.setContent(jsonContent);
                    // 题卡张数
                    int paperCount = CardParseUtils.calcPaperCount(jsonContent, true);
                    scanAnswerCard.setPaperCount(paperCount);
                    scanAnswerCardService.save(scanAnswerCard);

                    // 状态改为未提交
                    UpdateWrapper<MarkPaper> updateWrapper = new UpdateWrapper<>();
                    updateWrapper.lambda().set(MarkPaper::getQuestionStatus, false)
                            .eq(MarkPaper::getExamId, examId)
                            .eq(MarkPaper::getPaperNumber, markPaper.getPaperNumber());
                    this.update(updateWrapper);
                }
            }
        } catch (Exception e) {
            throw ExceptionResultEnum.ERROR.exception("上传失败：" + e.getMessage());
        } finally {
//            try {
//                FileUtils.forceDeleteOnExit(jsonFile);
//            } catch (IOException e) {
//            }
            if (parentDirVar != null) {
                FileUtil.deleteFile(parentDirVar);
            }
        }
    }

    @Override
    public List<MarkPaper> listByExamIdAndCourseIdForModel4(Long examId, Long courseId) {
        QueryWrapper<MarkPaper> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(MarkPaper::getExamId, examId)
                .eq(MarkPaper::getCourseId, courseId);
        return this.list(queryWrapper);
    }

    @Override
    public void scoreListExport(ArchiveScoreQuery query, HttpServletResponse response) throws IOException {
        IPage<ArchiveScoreVo> page = this.scoreList(query);
        List<ArchiveScoreExportDto> dtoList = new ArrayList<>();
        for (ArchiveScoreVo record : page.getRecords()) {
            ArchiveScoreExportDto dto = new ArchiveScoreExportDto();
            BeanUtils.copyProperties(record, dto);
            dto.setCourseInfo(record.getCourseName() + "(" + record.getCourseCode() + ")");
            if (SystemConstant.strNotNull(dto.getPassRate())) {
                dto.setPassRate(dto.getPassRate() + "%");
            }
            if (SystemConstant.strNotNull(dto.getExcellentRate())) {
                dto.setExcellentRate(dto.getExcellentRate() + "%");
            }
            dtoList.add(dto);
        }
        try {
            log.debug("导出Excel开始...");
            response.setHeader("Content-Disposition",
                    "inline;filename=" + URLEncoder.encode("成绩导出", SystemConstant.CHARSET_NAME) + ".xlsx");
            response.setContentType("application/vnd.ms-excel");
            ServletOutputStream outputStream = response.getOutputStream();
            ExcelWriter writer = ExcelWriter.create(ExcelType.XLSX);
            writer.writeObjects("总量分析", null, ArchiveScoreExportDto.class, dtoList.listIterator());
            writer.output(response.getOutputStream());
            outputStream.flush();
            outputStream.close();
            log.debug("导出Excel结束");
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    @Override
    public void updateQuestionStatus(Long examId, String paperNumber) {
        List<MarkQuestion> markQuestionList = markQuestionService.listByExamIdAndPaperNumberAndObjective(examId, paperNumber, false);
        boolean status = false;
        if (CollectionUtils.isNotEmpty(markQuestionList)) {
            status = markQuestionList.stream().filter(m -> m.getTotalScore() == null || m.getIntervalScore() == null).count() == 0;
        } else {
            markQuestionList = markQuestionService.listByExamIdAndPaperNumberAndObjective(examId, paperNumber, true);
            status = CollectionUtils.isNotEmpty(markQuestionList);
        }
        UpdateWrapper<MarkPaper> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(MarkPaper::getQuestionStatus, status)
                .eq(MarkPaper::getExamId, examId)
                .eq(MarkPaper::getPaperNumber, paperNumber);
        this.update(updateWrapper);
    }

    @Override
    public void updateGroupStatus(Long examId, String paperNumber) {
        int count = markQuestionService.countUnBindMarkerQuestion(examId, paperNumber);
        UpdateWrapper<MarkPaper> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(MarkPaper::getGroupStatus, count == 0)
                .eq(MarkPaper::getExamId, examId)
                .eq(MarkPaper::getPaperNumber, paperNumber);
        this.update(updateWrapper);
    }
}
