package com.qmth.teachcloud.mark.service.impl;

import com.alibaba.fastjson.JSON;
import com.qmth.teachcloud.common.bean.dto.mark.PictureConfig;
import com.qmth.teachcloud.common.bean.marking.MarkConfigItem;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.MarkQuestion;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.enums.mark.MarkProblemStatus;
import com.qmth.teachcloud.common.service.SysUserService;
import com.qmth.teachcloud.common.service.TeachcloudCommonService;
import com.qmth.teachcloud.mark.dto.mark.MarkStudentVo;
import com.qmth.teachcloud.mark.dto.mark.manage.*;
import com.qmth.teachcloud.mark.dto.mark.score.SheetUrlDto;
import com.qmth.teachcloud.mark.dto.mark.score.StudentObjectiveAnswerDto;
import com.qmth.teachcloud.mark.entity.*;
import com.qmth.teachcloud.mark.enums.MarkTaskStatus;
import com.qmth.teachcloud.mark.service.*;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

@Service
public class TaskServiceImpl implements TaskService {

    @Resource
    private MarkPaperService markPaperService;
    @Resource
    private MarkTaskService markTaskService;
    @Resource
    private MarkStudentService markStudentService;
    @Resource
    private MarkQuestionService markQuestionService;
    @Resource
    private MarkSubjectiveScoreService markSubjectiveScoreService;
    @Resource
    private TeachcloudCommonService teachcloudCommonService;
    @Resource
    private MarkProblemHistoryService markProblemHistoryService;
    @Resource
    private SysUserService sysUserService;
    @Resource
    private MarkArchiveStudentService markArchiveStudentService;

    @Override
    public Task build(MarkArbitrateHistory markArbitrateHistory, MarkQuestion markQuestion) {
        MarkStudent markStudent = markStudentService.getById(markArbitrateHistory.getStudentId());
        Task task = new Task();
        task.setArbitrateId(markArbitrateHistory.getId());
        task.setStudentId(markArbitrateHistory.getStudentId());
        task.setSecretNumber(markArbitrateHistory.getSecretNumber());
        if (StringUtils.isNotBlank(markStudent.getObjectiveScoreList())) {
            task.setObjectiveScore(markStudent.getObjectiveScore());
        }
        if (StringUtils.isNotBlank(markStudent.getSheetPath())) {
            task.setSheetUrls(teachcloudCommonService.filePreview(markStudent.getSheetPathList(), SystemConstant.MARK_JPG_EXPIRE_MINUTE));
        }
        if (StringUtils.isNotBlank(markQuestion.getPicList())) {
            task.setSliceConfig(JSON.parseArray(markQuestion.getPicList(), MarkConfigItem.class));
        }
        task.setQuestionList(buildArbitrateStep(markQuestion, markArbitrateHistory));
        return task;
    }

    @Override
    public Task build(Long studentId) {
        MarkStudentVo markStudent = markStudentService.getMarkStudentVoByStudentId(studentId);
        Task task = new Task();
        if (markStudent != null) {
            task.setExamId(markStudent.getExamId());
            task.setStudentId(markStudent.getId());
            task.setSecretNumber(markStudent.getSecretNumber());
            task.setStudentCode(markStudent.getStudentCode());
            task.setStudentName(markStudent.getStudentName());
            task.setCourseCode(markStudent.getCourseCode());
            task.setCourseName(markStudent.getCourseName());
            task.setPaperNumber(markStudent.getPaperNumber());

            // 原图
            task.setSheetUrls(teachcloudCommonService.fileMarkPreview(markStudent.getSheetPathList()));

            task.setQuestionList(buildHeaderQuestionList(markStudent));
            task.setObjectiveScore(StringUtils.isNotBlank(markStudent.getObjectiveScoreList()) ? markStudent.getObjectiveScore() : null);
            task.setMarkerScore(markStudent.getTotalScore());
            return task;
        }
        return task;
    }

    @Override
    public TaskArchive buildArchive(Long studentId) {
        MarkStudentVo markStudent = markStudentService.getMarkStudentVoByStudentId(studentId);
        TaskArchive task = new TaskArchive();
        if (markStudent != null) {
            MarkPaper markPaper = markPaperService.getByExamIdAndPaperNumber(markStudent.getExamId(), markStudent.getPaperNumber());
            if (markPaper.getArchive()) {
                MarkArchiveStudent markArchiveStudent = markArchiveStudentService.getById(studentId);
                if (markArchiveStudent != null) {
                    task.setExamId(markArchiveStudent.getExamId());
                    task.setPaperNumber(markArchiveStudent.getPaperNumber());
                    task.setStudentId(studentId);
                    task.setSecretNumber(markStudent.getSecretNumber());
                    task.setStudentName(markStudent.getStudentName());
                    task.setStudentCode(markStudent.getStudentCode());
                    task.setPaperType(markStudent.getPaperType());
                    task.setCourseCode(markStudent.getCourseCode());
                    task.setCourseName(markStudent.getCourseName());
                    task.setObjectiveScore(StringUtils.isNotBlank(markStudent.getObjectiveScoreList()) ? markStudent.getObjectiveScore() : null);
                    task.setSubjectiveScore(StringUtils.isNotBlank(markStudent.getSubjectiveScoreList()) ? markStudent.getSubjectiveScore() : null);
                    // 原图
                    String sheetUrls = markArchiveStudent.getSheetUrls();
                    if (StringUtils.isNotBlank(sheetUrls)) {
                        List<SheetUrlDto> sheetUrlDtos = JSON.parseArray(sheetUrls, SheetUrlDto.class);
                        for (SheetUrlDto sheetUrlDto : sheetUrlDtos) {
                            sheetUrlDto.setUrl(teachcloudCommonService.filePreview(sheetUrlDto.getUrl()));
                        }
                        task.setSheetUrls(sheetUrlDtos);
                    }
                    // 主观题
                    task.setSubjectiveQuestions(StringUtils.isNotBlank(markArchiveStudent.getSubjectiveQuestions()) ? JSON.parseArray(markArchiveStudent.getSubjectiveQuestions(), TaskQuestion.class) : null);
                    // 客观题
                    task.setObjectiveQuestions(StringUtils.isNotBlank(markArchiveStudent.getObjectiveQuestions()) ? JSON.parseArray(markArchiveStudent.getObjectiveQuestions(), StudentObjectiveAnswerDto.class) : null);
                    // 题卡卡格式
                    task.setCardContent(markStudentService.getCardContent(markStudent));
                }
            } else {
                task.setExamId(markStudent.getExamId());
                task.setPaperNumber(markStudent.getPaperNumber());
                task.setStudentId(markStudent.getId());
                task.setSecretNumber(markStudent.getSecretNumber());
                task.setStudentName(markStudent.getStudentName());
                task.setStudentCode(markStudent.getStudentCode());
                task.setPaperType(markStudent.getPaperType());
                task.setCourseCode(markStudent.getCourseCode());
                task.setCourseName(markStudent.getCourseName());
                task.setObjectiveScore(StringUtils.isNotBlank(markStudent.getObjectiveScoreList()) ? markStudent.getObjectiveScore() : null);
                task.setSubjectiveScore(StringUtils.isNotBlank(markStudent.getSubjectiveScoreList()) ? markStudent.getSubjectiveScore() : null);
                // 原图
                task.setSheetUrls(markStudentService.buildSheetUrls(studentId));
                // 主观题
                task.setSubjectiveQuestions(buildHeaderQuestionList(markStudent));
                // 客观题
                task.setObjectiveQuestions(markStudentService.buildObjectiveQuestion(markStudent));
                // 题卡卡格式
                task.setCardContent(markStudentService.getCardContent(markStudent));
            }
            return task;
        }
        return task;
    }

    private List<TaskQuestion> buildArbitrateStep(MarkQuestion markQuestion, MarkArbitrateHistory markArbitrateHistory) {
        List<TaskQuestion> list = new LinkedList<>();

        List<MarkTask> markTaskList = markTaskService.listByStudentIdAndQuestionId(markArbitrateHistory.getStudentId(), markArbitrateHistory.getQuestionId());
        TaskQuestion step = buildStep(markQuestion);
        if (markArbitrateHistory != null) {
            step.setSelfMark(true);
            if (markArbitrateHistory.getTotalScore() != null) {
                step.setScore(markArbitrateHistory.getTotalScore());
            }
            step.setMarkerTime(markArbitrateHistory.getUpdateTime());
            if (markArbitrateHistory.getTotalScore() != null) {
                step.setMarkerScore(markArbitrateHistory.getTotalScore());
            }
            // 增加阅卷轨迹列表获取
            List<TrackDTO> tracks = new ArrayList<>();
            for (MarkTask markTask : markTaskList) {
                tracks.addAll(markTask.listHeaderTrack());
            }
            step.setHeaderTrackList(tracks);
        }
        list.add(step);
        return list;
    }

    private TaskQuestion buildStep(MarkQuestion question) {
        TaskQuestion step = new TaskQuestion();
        step.setMainNumber(question.getMainNumber());
        step.setSubNumber(question.getSubNumber());
        step.setQuestionType(question.getQuestionType());
        step.setPicList(StringUtils.isNotBlank(question.getPicList()) ? JSON.parseArray(question.getPicList(), PictureConfig.class) : null);
        step.setQuestionId(question.getId());
        step.setTitle(question.getMainTitle());
        step.setDefaultScore(0d);
        step.setMaxScore(question.getTotalScore());
        step.setMinScore(0d);
        step.setIntervalScore(question.getIntervalScore());
        step.setUncalculate(false);
        return step;
    }

    @Override
    public Task build(Long userId, List<MarkTask> markTaskList) {
        Long markStudentId = markTaskList.get(0).getStudentId();
        MarkStudent student = markStudentService.getById(markStudentId);
        Task task = new Task();
        task.setSecretNumber(student.getSecretNumber());
        task.setPaperNumber(student.getPaperNumber());
        task.setStudentId(markStudentId);
        task.setStudentCode(student.getStudentCode());
        task.setPaperType(student.getPaperType());
        task.setPaperType(student.getPaperType());
        task.setSheetUrls(teachcloudCommonService.fileMarkPreview(student.getSheetPathList()));
        task.setSliceConfig(buildPic(markTaskList));
        MarkPaper markPaper = markPaperService.getByExamIdAndPaperNumber(student.getExamId(), student.getPaperNumber());
        task.setObjectiveScore(markPaper.getShowObjectScore() ? student.getObjectiveScore() : null);
        // 题目数据
        task.setQuestionList(buildQuestionList(userId, markTaskList));
        task.setMarkerScore(calcTotalScore(task.getQuestionList()));
        return task;
    }

    private Double calcTotalScore(List<TaskQuestion> questionList) {
        // 问题卷，待仲裁，已仲裁-回评中列表分值应该显示空
        List<MarkTaskStatus> markTaskStatuses = Arrays.asList(MarkTaskStatus.WAIT_ARBITRATE, MarkTaskStatus.ARBITRATED, MarkTaskStatus.PROBLEM);
        long count = questionList.stream().filter(m -> markTaskStatuses.contains(m.getStatus())).count();
        if (questionList.size() == count) {
            return null;
        }
        return questionList.stream().filter(m -> m.getMarkerScore() != null && MarkTaskStatus.MARKED.equals(m.getStatus())).mapToDouble(m -> m.getMarkerScore()).sum();
    }

    private List<MarkConfigItem> buildPic(List<MarkTask> markTaskList) {
        List<MarkConfigItem> markConfigItems = new ArrayList<>();
        Integer mainNumber = null;
        for (MarkTask t : markTaskList) {
            MarkQuestion markQuestion = markQuestionService.getById(t.getQuestionId());
            if (markQuestion.getQuestionType() == 4) {
                // 填空题，只显示一次
                if (mainNumber == null || !markQuestion.getMainNumber().equals(mainNumber)) {
                    markConfigItems.addAll(markQuestion.getPictureConfigList());
                }
                mainNumber = markQuestion.getMainNumber();
            } else {
                markConfigItems.addAll(markQuestion.getPictureConfigList());
            }
        }
        return markConfigItems;
    }

    private List<TaskQuestion> buildQuestionList(Long userId, List<MarkTask> markTaskList) {
        List<TaskQuestion> taskQuestions = new ArrayList<>();
        for (MarkTask t : markTaskList) {
            MarkQuestion markQuestion = markQuestionService.getById(t.getQuestionId());
            TaskQuestion tq = buildStep(markQuestion);
            tq.setTaskId(t.getId());
            tq.setStatus(t.getStatus());
            tq.setMarkerTrackList(t.listMarkerTrack());
            tq.setMarkerTagList(t.listMarkerTag());
            tq.setHeaderTrackList(t.listHeaderTrack());
            tq.setHeaderTagList(t.listHeaderTag());
            tq.setMarkerScore(t.getHeaderScore() != null ? t.getHeaderScore() : t.getMarkerScore());
            tq.setSelfMark(t.getUserId() == null || t.getUserId().equals(userId));
            tq.setRejected(t.getStatus().equals(MarkTaskStatus.REJECTED));
            tq.setPrevious(t.getStatus().equals(MarkTaskStatus.MARKED));
            tq.setProblem(t.getStatus().equals(MarkTaskStatus.PROBLEM));
            if (tq.isProblem()) {
                MarkProblemHistory markProblemHistory = markProblemHistoryService.findByTaskIdAndStatus(t.getId(), MarkProblemStatus.WAITING);
                if (markProblemHistory != null) {
                    tq.setProblemType(markProblemHistory.getType());
                    tq.setProblemRemark(markProblemHistory.getRemark());
                }
            }
            taskQuestions.add(tq);
        }
        return taskQuestions;
    }

    private List<TaskQuestion> buildHeaderQuestionList(MarkStudent markStudent) {
        Long examId = markStudent.getExamId();
        String paperNumber = markStudent.getPaperNumber();
        List<TaskQuestion> taskQuestions = new ArrayList<>();
        List<MarkQuestion> sList = markQuestionService.listByExamIdAndPaperNumberAndObjective(examId, paperNumber, false);
        Map<Long, MarkQuestion> questionMap = sList.stream().collect(Collectors.toMap(MarkQuestion::getId, Function.identity()));
        List<MarkSubjectiveScore> scoreList = markSubjectiveScoreService.listByStudentId(markStudent.getId());
        for (int i = 0; i < scoreList.size(); i++) {
            MarkSubjectiveScore markSubjectiveScore = scoreList.get(i);
            MarkQuestion question = questionMap.get(markSubjectiveScore.getQuestionId());
            TaskQuestion step = buildStep(question);
            // 管理员复核时，默认全部为自己评
            step.setSelfMark(true);
            Double score = scoreList.get(i).getScore();
            if (!scoreList.isEmpty() && scoreList.size() == sList.size()) {
                step.setMarkerScore(score);
            }
            // 增加阅卷轨迹列表获取
            List<MarkTask> markTaskList = markTaskService.listByStudentIdAndQuestionId(markStudent.getId(), question.getId());
            // 不管单评还是多评显示所有评卷员给分轨迹
            for (MarkTask markTask : markTaskList) {
                for (TrackDTO track : markTask.listMarkerTrack()) {
                    SysUser user = sysUserService.getByUserId(markTask.getUserId());
                    track.setUserId(markTask.getUserId());
                    track.setUserName(user.getRealName() + "(" + user.getLoginName() + ")");
                    step.addMarkTrack(track);
                }

                if (CollectionUtils.isNotEmpty(markTask.listHeaderTrack()) && markTask.getTaskNumber() == 1) {
                    for (TrackDTO trackDTO : markTask.listHeaderTrack()) {
                        SysUser sysUser = sysUserService.getByUserId(markTask.getHeaderId());
                        if (sysUser != null) {
                            trackDTO.setUserId(sysUser.getId());
                            trackDTO.setUserName(sysUser.getRealName() + "(" + sysUser.getLoginName() + ")");
                        }
                        trackDTO.setHeaderType(markTask.getStatus());
                        step.addHeadTrack(trackDTO);
                    }
                }

                // 普通模式无轨迹数据。单独处理评卷员每题得分
                if (CollectionUtils.isEmpty(step.getMarkerTrackList()) && markTask.getMarkerScore() != null) {
                    // 评卷员
                    MarkerScoreDTO markerScoreDTO = new MarkerScoreDTO();
                    markerScoreDTO.setUserId(markTask.getUserId());
                    SysUser markerUser = sysUserService.getByUserId(markTask.getUserId());
                    if (markerUser != null) {
                        markerScoreDTO.setLoginName(markerUser.getLoginName());
                        markerScoreDTO.setUserName(markerUser.getRealName());
                    }
                    markerScoreDTO.setScore(score);
                    markerScoreDTO.setHeader(false);
                    step.addMarkerList(markerScoreDTO);

                    // 科组长
                    if (CollectionUtils.isEmpty(markTask.listHeaderTrack()) && markTask.getTaskNumber() == 1) {
                        if (!markTask.getMarkerScore().equals(score)) {
                            MarkerScoreDTO headerScoreDTO = new MarkerScoreDTO();
                            headerScoreDTO.setUserId(markTask.getHeaderId());
                            SysUser headerUser = sysUserService.getByUserId(markTask.getHeaderId());
                            if (headerUser != null) {
                                headerScoreDTO.setLoginName(headerUser.getLoginName());
                                headerScoreDTO.setUserName(headerUser.getRealName());
                            }
                            headerScoreDTO.setScore(score);
                            headerScoreDTO.setHeader(true);
                            headerScoreDTO.setHeaderType(markTask.getStatus());
                            step.addMarkerList(headerScoreDTO);
                        }
                    }
                }
            }
            taskQuestions.add(step);
        }
        return taskQuestions;
    }
}
