package com.qmth.teachcloud.task.job.service.impl;

import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;

import javax.annotation.Resource;

import org.apache.commons.collections4.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.qmth.boot.redis.uid.RedisMachineService;
import com.qmth.distributed.print.business.bean.dto.initMarkData.ExamDetailCourseInitMarkDto;
import com.qmth.distributed.print.business.entity.*;
import com.qmth.distributed.print.business.enums.ExamDetailStatusEnum;
import com.qmth.distributed.print.business.service.*;
import com.qmth.distributed.print.business.templete.execute.AsyncCreatePdfTemplateService;
import com.qmth.distributed.print.business.util.ExamTaskUtil;
import com.qmth.teachcloud.common.bean.dto.MqDto;
import com.qmth.teachcloud.common.bean.vo.PaperInfoVo;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.BasicCourse;
import com.qmth.teachcloud.common.entity.MarkQuestion;
import com.qmth.teachcloud.common.enums.PushTypeEnum;
import com.qmth.teachcloud.common.enums.TaskResultEnum;
import com.qmth.teachcloud.common.enums.TaskStatusEnum;
import com.qmth.teachcloud.common.enums.mark.MarkPaperStatus;
import com.qmth.teachcloud.common.service.BasicCourseService;
import com.qmth.teachcloud.common.util.DateDisposeUtils;
import com.qmth.teachcloud.common.util.RedisUtil;
import com.qmth.teachcloud.data.service.TSyncDataService;
import com.qmth.teachcloud.mark.entity.MarkPaper;
import com.qmth.teachcloud.mark.entity.MarkStudent;
import com.qmth.teachcloud.mark.entity.MarkUserQuestion;
import com.qmth.teachcloud.mark.enums.LockType;
import com.qmth.teachcloud.mark.lock.LockService;
import com.qmth.teachcloud.mark.service.*;
import com.qmth.teachcloud.mark.utils.TaskLockUtil;
import com.qmth.teachcloud.task.job.service.JobService;

/**
 * @Description: job service impl
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2021/3/31
 */
@Service
public class JobServiceImpl implements JobService {
    private final static Logger log = LoggerFactory.getLogger(JobServiceImpl.class);

    @Resource
    SmsSendService smsSendService;
    @Resource
    BasicMessageService basicMessageService;
    @Resource
    private ExamTaskDetailService examTaskDetailService;
    @Resource
    MarkPaperService markPaperService;
    @Resource
    MarkQuestionService markQuestionService;
    @Resource
    MarkUserQuestionService markUserQuestionService;
    @Resource
    MarkService markService;
    @Resource
    ExamDetailService examDetailService;
    @Resource
    PrintFinishService printFinishService;
    @Resource
    RedisUtil redisUtil;
    @Resource
    RedisMachineService redisMachineService;
    @Resource
    TBSyncTaskService tbSyncTaskService;
    @Resource
    TBTaskPdfService tbTaskPdfService;
    @Resource
    private BasicCourseService basicCourseService;
    @Resource
    AsyncCreatePdfTemplateService asyncCreatePdfTemplateService;
    @Resource
    LockService lockService;

    @Resource
    private ExamTaskService examTaskService;
    @Resource
    private BasicCardRuleService basicCardRuleService;
    @Resource
    private TSyncDataService tSyncDataService;

    @Resource
    MarkStudentService markStudentService;

    @Override
    public void sendSmsExpireTask() {
        smsSendService.sendSmsExpireTask();
    }

    @Override
    public void sendSmsOverdueTask() {
        smsSendService.sendSmsOverdueTask();
    }

    @Override
    public void resendSmsTask() {
        basicMessageService.resendSmsTask();
    }

    @Override
    public void machineHeart() {
        redisUtil.set(SystemConstant.TASK_MACHINE_ID + redisMachineService.getMachineId(), redisMachineService.getMachineId(), 30, TimeUnit.SECONDS);
    }

    @Override
    public void updateMarkerQuality() {
        log.info("start auto-update marker");
        try {
            List<MarkPaper> markPaperList = markPaperService.listQualityMarkPaperByStatus(MarkPaperStatus.FORMAL, 0);
            for (MarkPaper markPaper : markPaperList) {
                List<MarkQuestion> markQuestionList = markQuestionService.listByExamIdAndPaperNumberAndObjective(markPaper.getExamId(), markPaper.getPaperNumber(), false);
                for (MarkQuestion markQuestion : markQuestionList) {
                    List<MarkUserQuestion> markUserGroups = markUserQuestionService.listByExamIdAndPaperNumberAndQuestionId(markPaper.getExamId(), markPaper.getPaperNumber(), markQuestion.getId());
                    for (MarkUserQuestion marker : markUserGroups) {
                        try {
                            if (markService.needUpdateQuality(marker, 30)) {
                                log.info("start auto-update markerId: " + marker.getId());
                                markService.updateQuality(marker);
                            }
                        } catch (Exception e) {
                            log.error("update marker quality error for markerId=" + marker.getId(), e);
                        }
                    }
                }
            }
        } catch (Exception e) {
            log.error("auto-update marker error", e);
        } finally {
            log.info("finish auto-update marker");
        }
    }

    @Override
    public void updateBuildMarkTask() {
        log.info("start auto-create library");
        try {
            List<MarkPaper> markPaperList = markPaperService.listQualityMarkPaperByStatus(MarkPaperStatus.FORMAL, 0);
            for (MarkPaper markPaper : markPaperList) {
                List<MarkQuestion> markQuestionList = markQuestionService.listByExamIdAndPaperNumberAndObjective(markPaper.getExamId(), markPaper.getPaperNumber(), false);
                if (CollectionUtils.isEmpty(markQuestionList) || markQuestionList.stream().filter(m -> m.getTotalScore() == null || m.getIntervalScore() == null).count() > 0) {
                    continue;
                }
                markService.buildMarkTask(markPaper);
                basicMessageService.smsMarkTask(markPaper.getExamId(), markPaper.getCourseId(), markPaper.getPaperNumber());
            }
        } catch (Exception e) {
            log.error("auto-create library error", e);
        } finally {
            log.info("finish auto-create library");
        }
    }

    @Override
    public void initMarkData() {
        log.info("开始同步数据：" + DateDisposeUtils.dateTimeNow(DateDisposeUtils.YYYY_MM_DD_HH_MM_SS));
        // 结束阅卷的课程，不同步，3.3.0版本更新前的历史数据，统一不同步
        long startTime = DateDisposeUtils.parseDate("2024-04-22 00:00:00").getTime();
        List<ExamDetailCourseInitMarkDto> examDetailCourseInitMarkDtoList = examDetailService.listPrintFinishExamDetailCourse(ExamDetailStatusEnum.FINISH.name(), startTime);
        for (ExamDetailCourseInitMarkDto dto : examDetailCourseInitMarkDtoList) {
            try {
                lockService.waitlock(LockType.INIT_MARK_DATA, dto.getExamId(), dto.getPaperNumber(), dto.getSerialNumber());
                BasicCourse basicCourse = basicCourseService.getById(dto.getCourseId());
                ExamTask examTask = examTaskService.getByExamIdAndPaperNumber(dto.getExamId(), dto.getPaperNumber());
                BasicCardRule basicCardRule = basicCardRuleService.getById(examTask.getCardRuleId());
                if (basicCourse == null) {
                    continue;
                }
                TBSyncTask tbSyncTask = tbSyncTaskService.saveTask(dto.getSchoolId(), dto.getSemesterId(), dto.getExamId(), dto.getCourseId(), dto.getPaperNumber(), dto.getPaperType(), PushTypeEnum.INIT_MARK_DATA, "");
                try {
                    ExamTaskDetail examTaskDetail = examTaskDetailService.getByExamIdAndPaperNumberAndSerialNumber(dto.getExamId(), dto.getPaperNumber(), dto.getSerialNumber());
                    List<PaperInfoVo> paperInfoVos = ExamTaskUtil.parsePaperAttachmentPath(examTaskDetail.getPaperAttachmentIds(), examTaskDetail.getPaperType());
                    printFinishService.insertMarkPaper(dto, paperInfoVos);
                    printFinishService.insertMarkPaperPackageCode(dto);
                    printFinishService.insertMarkStudent(dto, basicCardRule);
                    printFinishService.insertScanAnswerCardAndMarkQuestion(dto);
                    tbSyncTask.setResult(TaskResultEnum.SUCCESS);
                } catch (Exception e) {
                    tbSyncTask.setResult(TaskResultEnum.ERROR);
                    tbSyncTask.setSummary(e.getMessage());
                    log.info("结束同步失败：" + DateDisposeUtils.dateTimeNow(DateDisposeUtils.YYYY_MM_DD_HH_MM_SS));
                } finally {
                    tbSyncTask.setStatus(TaskStatusEnum.FINISH);
                    tbSyncTaskService.updateById(tbSyncTask);
                    log.info("结束同步数据：" + DateDisposeUtils.dateTimeNow(DateDisposeUtils.YYYY_MM_DD_HH_MM_SS));
                }
            } catch (Exception e) {
                log.error("init mark data error", e);
            } finally {
                lockService.unlock(LockType.INIT_MARK_DATA, dto.getExamId(), dto.getPaperNumber(), dto.getSerialNumber());
            }
        }
    }

    @Override
    public void clearTimeoutTask() {
        try {
            long timeoutMinute = 60L;
            TaskLockUtil.clearTimeoutTask(timeoutMinute * 60 * 1000);
        } catch (Exception e) {
            log.error("CronCleanTask error", e);
        }
    }

    @Override
    public void createPdfTask() throws InterruptedException {
        List<TBTaskPdf> tbTaskPdfList = tbTaskPdfService.listWaitingTask();
        if (CollectionUtils.isNotEmpty(tbTaskPdfList)) {
            int cpuNum = Runtime.getRuntime().availableProcessors();
            ExecutorService executor = Executors.newFixedThreadPool(cpuNum);
            for (TBTaskPdf tbTaskPdf : tbTaskPdfList) {
                int count = tbTaskPdfService.updateStatus(TaskStatusEnum.RUNNING, TaskStatusEnum.INIT, tbTaskPdf.getId());
                if (count == 1) {
                    executor.submit(() -> {
                        asyncCreatePdfTemplateService.createPdf(tbTaskPdf, null);
                    });
                }
            }
            executor.shutdown();
            while (!executor.awaitTermination(1, TimeUnit.MINUTES)) {
                log.info("线程池没有关闭");
            }
            log.info("线程池已经关闭");
        }
        //补救running状态
        List<TBTaskPdf> tbTaskPdfRunningList = tbTaskPdfService.listRunningTimeOutTask(TaskStatusEnum.RUNNING);
        if (CollectionUtils.isNotEmpty(tbTaskPdfRunningList)) {
            for (TBTaskPdf tbTaskPdf : tbTaskPdfRunningList) {
                tbTaskPdfService.updateStatus(TaskStatusEnum.INIT, TaskStatusEnum.RUNNING, tbTaskPdf.getId());
            }
        }
    }

    @Override
    public void sentDataSync(String schoolId, String examId) {
        tSyncDataService.syncData(Long.valueOf(schoolId), Long.valueOf(examId));
    }

    /**
     * 统分定时任务
     */
    @Override
    public void markScoreCalculate() {
        Integer pageNumber = 0;
        Integer pageSize = 500;
        Page<MarkStudent> page = new Page<>(pageNumber, pageSize);
        List<MarkStudent> markStudentList = markStudentService.findNeedScoreCalculateStudent(page).getRecords();
        while (CollectionUtils.isNotEmpty(markStudentList)) {
            for (MarkStudent m : markStudentList) {
                try {
                    markService.checkStudentSubjective(m.getId(), m.getExamId(), m.getPaperNumber(), m.getVersion(), true);
                } catch (Exception e) {
                    log.error(SystemConstant.LOG_ERROR, e);
                }
            }
            pageNumber++;
            page = new Page<>(pageNumber, pageSize);
            markStudentList = markStudentService.findNeedScoreCalculateStudent(page).getRecords();
        }
    }

    /**
     * 组装job
     *
     * @param redisKey
     */
    @Override
    public void assembleJob(String redisKey) {
        Long size = redisUtil.getHashSize(redisKey);
        if (Objects.nonNull(size) && size.longValue() > 0) {
            log.info("redisKey:{}缓冲区的消息数为:{}", redisKey, size);
            Map map = redisUtil.getHashEntries(redisKey);
            map.forEach((k, v) -> {
                MqDto mqDto = (MqDto) v;
                redisUtil.sendMessage(mqDto.getTopic(), mqDto);
            });
        }
    }
}
