USE teachcloud_db;

-- ----------------------------
-- Table structure for basic_attachment
-- ----------------------------
CREATE TABLE IF NOT EXISTS `basic_attachment` (
  `id` bigint NOT NULL COMMENT '主键',
  `school_id` bigint DEFAULT NULL COMMENT '学校id',
  `org_id` bigint DEFAULT NULL COMMENT '机构id',
  `name` varchar(100) NOT NULL COMMENT '文件名',
  `type` varchar(10) NOT NULL COMMENT '文件类型',
  `size` int NOT NULL COMMENT '单位（KB）',
  `md5` varchar(32) NOT NULL COMMENT '文件md5值',
  `path` varchar(2000) NOT NULL COMMENT '存储路径',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  `obj_id` bigint DEFAULT NULL COMMENT '关联业务id',
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='所有附件记录表';

-- ----------------------------
-- Table structure for basic_campus
-- ----------------------------
CREATE TABLE IF NOT EXISTS `basic_campus` (
  `id` bigint NOT NULL COMMENT '校区id',
  `school_id` bigint NOT NULL COMMENT '学校id',
  `campus_name` varchar(64) DEFAULT NULL COMMENT '校区名称',
  `campus_code` varchar(64) DEFAULT NULL COMMENT '校区编码',
  `print_house_id` bigint DEFAULT NULL COMMENT '印刷室id',
  `enable` tinyint DEFAULT '1' COMMENT '是否可用',
  `org_id` bigint DEFAULT NULL COMMENT '机构id',
  `create_id` bigint DEFAULT NULL COMMENT '创建人id',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人id',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  PRIMARY KEY (`id`) USING BTREE,
  UNIQUE KEY `campus_schoolId_code_idx` (`school_id`,`campus_code`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='基础校区表';

-- ----------------------------
-- Table structure for basic_card_rule
-- ----------------------------
CREATE TABLE IF NOT EXISTS `basic_card_rule` (
  `id` bigint NOT NULL COMMENT '主键',
  `school_id` bigint NOT NULL COMMENT '学校id',
  `org_id` bigint DEFAULT NULL COMMENT '机构id',
  `name` varchar(100) NOT NULL COMMENT '题卡规则名称',
  `exam_number_style` varchar(5) NOT NULL COMMENT 'PRINT-印刷条码，PASTE-粘贴条码，FILL-卡号填涂',
  `exam_number_digit` int DEFAULT NULL COMMENT '考号位数',
  `paper_type` varchar(5) NOT NULL COMMENT 'PRINT-印刷条码，FILL-卡号填涂',
  `exam_absent` tinyint NOT NULL COMMENT '0-禁用，1-启用',
  `write_sign` tinyint NOT NULL COMMENT '0-禁用，1-启用',
  `discipline` tinyint(1) DEFAULT NULL COMMENT '违纪填涂（1：是，0：否）',
  `undertaking_enable` tinyint(1) DEFAULT NULL COMMENT '启用考生承诺书',
  `undertaking_body` varchar(300) DEFAULT NULL COMMENT '考生承诺书内容',
  `default_option_number` int DEFAULT '4' COMMENT '选择题默认选项个数',
  `contains_letter` TINYINT(1) NULL DEFAULT 0 COMMENT '学号是否包含字母',
  `relation_list` VARCHAR(500) NULL COMMENT '学号字母所在位置坐标',
  `required_fields` text NOT NULL COMMENT '必选字段',
  `extend_fields` text NOT NULL COMMENT '扩展字段',
  `fill_fields` varchar(500) DEFAULT NULL COMMENT '变量信息填充字段',
  `title_rule` varchar(200) NOT NULL COMMENT '题卡标题规则',
  `attention` text NOT NULL COMMENT '注意事项',
  `objective_attention` text COMMENT '客观题注意事项',
  `subjective_attention` text COMMENT '主观题注意事项',
  `enable` tinyint NOT NULL COMMENT '0-禁用，1-启用',
  `remark` varchar(200) DEFAULT NULL COMMENT '备注',
  `create_id` bigint DEFAULT NULL COMMENT '创建人id',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  `fill_number` tinyint DEFAULT NULL COMMENT '卡号填涂位数',
  `first_level_subheading` varchar(200) DEFAULT NULL COMMENT '一级副标题',
  `second_level_subheading` varchar(200) DEFAULT NULL COMMENT '二级副标题',
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='题卡规则';

-- ----------------------------
-- Table structure for basic_clazz
-- ----------------------------
CREATE TABLE IF NOT EXISTS `basic_clazz` (
  `id` bigint NOT NULL,
  `school_id` bigint NOT NULL COMMENT '学校id',
  `major_id` bigint NOT NULL COMMENT '专业id',
  `clazz_code` varchar(50) DEFAULT NULL COMMENT '班级编号',
  `short_code` varchar(10) DEFAULT NULL COMMENT '简码',
  `clazz_name` varchar(50) DEFAULT NULL COMMENT '班级名称',
  `enable` tinyint(1) DEFAULT NULL COMMENT '是否可用',
  `org_id` bigint DEFAULT NULL COMMENT '权限机构id',
  `create_id` bigint DEFAULT NULL COMMENT '创建人id',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人id',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  PRIMARY KEY (`id`) USING BTREE,
  UNIQUE KEY `clazz_schoolId_name_idx` (`school_id`,`major_id`,`clazz_name`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='基础班级表';

-- ----------------------------
-- Table structure for basic_college
-- ----------------------------
CREATE TABLE IF NOT EXISTS `basic_college` (
  `id` bigint NOT NULL,
  `school_id` bigint NOT NULL,
  `college_name` varchar(50) NOT NULL COMMENT '学院名称',
  `org_id` bigint NOT NULL COMMENT '数据权限机构id',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  PRIMARY KEY (`id`) USING BTREE,
  UNIQUE KEY `college_schoolId_name_idx` (`school_id`,`college_name`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='学生学院表';

-- ----------------------------
-- Table structure for basic_course
-- ----------------------------
CREATE TABLE IF NOT EXISTS `basic_course` (
  `id` bigint NOT NULL COMMENT '主键',
  `school_id` bigint NOT NULL COMMENT '学校id',
  `org_id` bigint DEFAULT NULL COMMENT '机构id',
  `code` varchar(100) NOT NULL COMMENT '课程编码',
  `short_code` varchar(10) DEFAULT NULL COMMENT '简码',
  `name` varchar(50) NOT NULL COMMENT '课程名称',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '修改人',
  `update_time` bigint DEFAULT NULL COMMENT '修改时间',
  `teaching_room_id` bigint DEFAULT NULL COMMENT '教研室id',
  `enable` tinyint DEFAULT NULL,
  `historic_name` varchar(100) DEFAULT NULL COMMENT '历史课程名称',
  PRIMARY KEY (`id`) USING BTREE,
  UNIQUE KEY `uq_1` (`school_id`,`teaching_room_id`,`code`) USING BTREE,
  KEY `index1` (`teaching_room_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='课程表';

-- ----------------------------
-- Table structure for basic_exam
-- ----------------------------
CREATE TABLE IF NOT EXISTS `basic_exam` (
  `id` bigint NOT NULL,
  `school_id` bigint NOT NULL,
  `org_id` bigint NOT NULL,
  `semester_id` bigint DEFAULT NULL COMMENT '学期ID',
  `short_code` varchar(10) DEFAULT NULL COMMENT '简码',
  `name` varchar(100) DEFAULT NULL COMMENT '考试名称',
  `category` varchar(45) DEFAULT NULL COMMENT '考试类型（ONLINE：线上，OFFLINE：线下）',
  `exam_model` varchar(45) DEFAULT NULL COMMENT '业务模式（MODEL1：电子交卷环节需要提交考务数据，MODEL2：电子交卷环节不需要提交考务数据，只输入印刷份数即可，MODEL3：电子交卷环节不需要提交考务数据）',
  `review` tinyint DEFAULT NULL COMMENT '是否启用入库审核：0-禁用，1-启用',
  `print_method` varchar(6) DEFAULT NULL COMMENT 'AUTO-自动，MANUAL-手动',
  `enable` tinyint(1) NOT NULL DEFAULT '1' COMMENT '1启用/0禁用',
  `create_id` bigint DEFAULT NULL,
  `create_time` bigint DEFAULT NULL,
  `update_id` bigint DEFAULT NULL,
  `update_time` bigint DEFAULT NULL,
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='考试表';

-- ----------------------------
-- Table structure for basic_exam_rule
-- ----------------------------
CREATE TABLE IF NOT EXISTS `basic_exam_rule` (
  `id` bigint NOT NULL COMMENT '主键',
  `school_id` bigint NOT NULL COMMENT '学校id',
  `org_id` bigint DEFAULT NULL COMMENT '机构id',
  `required_fields` text COMMENT '必选字段',
  `extend_fields` text NOT NULL COMMENT '扩展字段',
  `exam_number_digit` int DEFAULT NULL COMMENT '考号位数',
  `sign_scope` text COMMENT '签到表字段可选范围',
  `package_scope` text COMMENT '卷袋贴字段可选范围',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='考务规则';

-- ----------------------------
-- Table structure for basic_exam_student
-- ----------------------------
CREATE TABLE IF NOT EXISTS `basic_exam_student` (
  `id` bigint NOT NULL COMMENT 'id',
  `school_id` bigint NOT NULL COMMENT '学校id',
  `semester_id` bigint NOT NULL COMMENT '学期id',
  `exam_id` bigint NOT NULL COMMENT '考试id',
  `course_id` bigint DEFAULT NULL COMMENT '课程ID（basic_course表ID）',
  `course_code` varchar(20) DEFAULT NULL COMMENT '课程编号',
  `clazz_id` bigint DEFAULT NULL COMMENT '教学班id（basic_teach_clazz）',
  `paper_number` varchar(100) DEFAULT NULL COMMENT '试卷编号',
  `paper_type` varchar(10) DEFAULT NULL COMMENT '卷型',
  `student_name` varchar(50) NOT NULL COMMENT '姓名',
  `student_code` varchar(50) NOT NULL COMMENT '学号',
  `site_number` varchar(10) DEFAULT NULL COMMENT '座位号',
  `college_name` varchar(100) DEFAULT NULL COMMENT '学院',
  `major_name` varchar(100) DEFAULT NULL COMMENT '专业',
  `teach_class_name` varchar(100) DEFAULT NULL,
  `class_name` varchar(100) DEFAULT NULL,
  `status` varchar(1) DEFAULT 'N' COMMENT '状态，默认N-正常，D-缓考，F-免考，M-缺考，B-违纪',
  `teacher_id` bigint DEFAULT NULL COMMENT '任课教师id',
  `exam_start_time` bigint DEFAULT NULL COMMENT '考试开始时间',
  `exam_end_time` bigint DEFAULT NULL COMMENT '考试结束时间',
  `exam_place` varchar(50) DEFAULT NULL COMMENT '考点',
  `exam_room` varchar(50) DEFAULT NULL COMMENT '考场',
  `required_fields` mediumtext COMMENT '基础字段',
  `extend_fields` mediumtext COMMENT '扩展字段',
  `enable` TINYINT(1) NULL DEFAULT 1 COMMENT '是否启用',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  PRIMARY KEY (`id`),
  UNIQUE KEY `basic_exam_student_unique` (`school_id`,`exam_id`,`course_code`,`student_code`),
  KEY `index1` (`exam_id`,`course_id`,`paper_number`),
  KEY `index2` (`exam_id`,`paper_number`,`student_code`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='考生字典表';

-- ----------------------------
-- Table structure for basic_major
-- ----------------------------
CREATE TABLE IF NOT EXISTS `basic_major` (
  `id` bigint NOT NULL COMMENT '主键',
  `school_id` bigint NOT NULL COMMENT '学校id',
  `org_id` bigint NOT NULL COMMENT '机构id',
  `belong_org_id` bigint NOT NULL COMMENT '所属机构id',
  `name` varchar(100) NOT NULL COMMENT '专业名称',
  `code` varchar(100) DEFAULT NULL COMMENT '专业编码',
  `enable` tinyint DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
  `create_id` bigint DEFAULT NULL COMMENT '创建人id',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人id',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  PRIMARY KEY (`id`) USING BTREE,
  UNIQUE KEY `major_schoolId_name_idx` (`school_id`,`name`,`belong_org_id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='专业字典表';

-- ----------------------------
-- Table structure for basic_message
-- ----------------------------
CREATE TABLE IF NOT EXISTS `basic_message` (
  `id` bigint NOT NULL COMMENT '主键',
  `school_id` bigint DEFAULT NULL COMMENT '学校id',
  `org_id` bigint DEFAULT NULL COMMENT '机构id',
  `user_id` bigint DEFAULT NULL COMMENT '消息接收人用户',
  `user_name` varchar(100) DEFAULT NULL COMMENT '用户名称',
  `mobile_number` varchar(11) DEFAULT NULL COMMENT '电话号码',
  `paper_number` varchar(100) DEFAULT NULL COMMENT '试卷编号',
  `course_code` varchar(50) DEFAULT NULL COMMENT '课程代码',
  `message_type` varchar(45) DEFAULT NULL COMMENT '消息类型',
  `business_operate` varchar(45) NOT NULL COMMENT '业务操作',
  `business_id` bigint DEFAULT NULL COMMENT '业务id',
  `template_code` varchar(45) DEFAULT NULL COMMENT '消息模板代码',
  `variable_params` text COMMENT '变量参数内容',
  `template_content` mediumtext COMMENT '模板内容',
  `send_status` varchar(45) DEFAULT NULL COMMENT '消息发送状态',
  `send_result` varchar(2000) DEFAULT NULL COMMENT '消息发送结果',
  `remark` text COMMENT '备注',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  `resend_count` int DEFAULT NULL COMMENT '重试次数',
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci;

-- ----------------------------
-- Table structure for basic_operation_log
-- ----------------------------
CREATE TABLE IF NOT EXISTS `basic_operation_log` (
  `id` bigint NOT NULL COMMENT 'id',
  `school_id` bigint DEFAULT NULL COMMENT '学校id',
  `user_id` bigint NOT NULL COMMENT '用户对象id',
  `ip` varchar(32) NOT NULL COMMENT 'ip地址',
  `privilege_id` bigint DEFAULT NULL COMMENT '权限菜单id',
  `operation_type` varchar(32) NOT NULL COMMENT '自定义记录日志操作类型（LOGIN("用户登录"),LOGOUT("用户登出"),ADD("新增"),DELETE("删除"),UPDATE("修改"),IMPORT("导入"),EXPORT("导出"),UN_KNOW("未知")）',
  `detail` mediumtext COMMENT '日志详情',
  `function_name` varchar(100) DEFAULT NULL COMMENT '功能名称',
  `url` varchar(64) DEFAULT NULL COMMENT '请求地址url',
  `args` mediumtext COMMENT '参数',
  `run_status` varchar(32) DEFAULT NULL COMMENT '方法运行状态',
  `create_time` bigint DEFAULT NULL COMMENT '日志记录时间',
  PRIMARY KEY (`id`) USING BTREE,
  KEY `operation_log_idx` (`school_id`,`user_id`,`privilege_id`,`operation_type`,`create_time`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='基础日志记录表';

-- ----------------------------
-- Table structure for basic_print_config
-- ----------------------------
CREATE TABLE IF NOT EXISTS `basic_print_config` (
  `id` bigint NOT NULL,
  `school_id` bigint NOT NULL COMMENT '学校ID',
  `org_id` bigint NOT NULL COMMENT '机构id',
  `exam_id` bigint DEFAULT NULL COMMENT '考试ID',
  `enable` tinyint(1) NOT NULL DEFAULT '1' COMMENT '1启用/0禁用',
  `card_rule_id` bigint DEFAULT NULL COMMENT '题卡规则（-1代表通卡）',
  `open_ab` TINYINT(1) NULL DEFAULT 0 COMMENT '是否开启AB卷',
  `print_content` varchar(255) DEFAULT NULL COMMENT '试卷、题卡',
  `backup_method` varchar(10) DEFAULT NULL COMMENT '试卷题卡备份方式（ROOM按考场）',
  `backup_count` double DEFAULT NULL COMMENT '试卷、题卡备份数量（0-1为按比例，大于等于1为按数量）',
  `draw_rule` varchar(10) DEFAULT NULL COMMENT '抽卷规则（ONE只抽一次，CIRCLE循环抽取）',
  `variable_content` mediumtext COMMENT '变量印品参数json',
  `ordinary_content` mediumtext COMMENT '普通印品参数json',
  `create_id` bigint DEFAULT NULL,
  `create_time` bigint DEFAULT NULL,
  `update_id` bigint DEFAULT NULL,
  `update_time` bigint DEFAULT NULL,
  `review` tinyint DEFAULT NULL COMMENT '是否启用入库审核：0-禁用，1-启用',
  `audit_content` varchar(45) DEFAULT NULL COMMENT '[PAPER,CARD]',
  `print_method` varchar(6) DEFAULT NULL COMMENT 'AUTO-自动，MANUAL-手动',
  `output_file_type` VARCHAR(100) DEFAULT NULL COMMENT '输出文件类型',
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='印品配置表';

-- ----------------------------
-- Table structure for basic_professional
-- ----------------------------
CREATE TABLE IF NOT EXISTS `basic_professional` (
  `id` bigint NOT NULL,
  `school_id` bigint DEFAULT NULL COMMENT '学校ID',
  `name` varchar(80) DEFAULT NULL COMMENT '专业名称',
  `org_id` bigint DEFAULT NULL COMMENT '所属学院ID',
  `create_id` bigint DEFAULT NULL COMMENT '创建人ID',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人ID',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='专业表';

-- ----------------------------
-- Table structure for basic_role_data_permission
-- ----------------------------
CREATE TABLE IF NOT EXISTS `basic_role_data_permission` (
  `id` bigint NOT NULL COMMENT 'id',
  `school_id` bigint DEFAULT NULL COMMENT '学校id',
  `role_id` bigint NOT NULL COMMENT '角色id',
  `privilege_id` bigint NOT NULL COMMENT '权限id',
  `data_permission_type` varchar(45) NOT NULL COMMENT '数据权限类型',
  `org_scope` varchar(45) DEFAULT NULL COMMENT '数据权限机构权限组',
  `org_id` bigint DEFAULT NULL COMMENT '创建人机构id',
  `create_id` bigint DEFAULT NULL COMMENT '创建人id',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人id',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  PRIMARY KEY (`id`) USING BTREE,
  KEY `role_data_permission_idx` (`role_id`,`privilege_id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='角色数据权限表';

-- ----------------------------
-- Records of basic_role_data_permission
-- ----------------------------
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (548156557615955968, NULL, 2, 4, 'ALL', NULL, NULL, 1, 1721631507601, NULL, 1721631507601);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (548156557615955969, NULL, 2, 5, 'ALL', NULL, NULL, 1, 1721631507605, NULL, 1721631507605);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (548156557615955970, NULL, 2, 6, 'ALL', NULL, NULL, 1, 1721631507605, NULL, 1721631507605);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (548156557615955971, NULL, 2, 8, 'ALL', NULL, NULL, 1, 1721631507606, NULL, 1721631507606);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (548156557615955972, NULL, 2, 9, 'ALL', NULL, NULL, 1, 1721631507606, NULL, 1721631507606);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (548156557615955973, NULL, 2, 624, 'ALL', NULL, NULL, 1, 1721631507607, NULL, 1721631507607);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (548156557615955974, NULL, 2, 11, 'ALL', NULL, NULL, 1, 1721631507608, NULL, 1721631507608);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (548156557615955975, NULL, 2, 402, 'ALL', NULL, NULL, 1, 1721631507608, NULL, 1721631507608);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (548156557615955976, NULL, 2, 403, 'ALL', NULL, NULL, 1, 1721631507609, NULL, 1721631507609);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (548156557615955977, NULL, 2, 510, 'ALL', NULL, NULL, 1, 1721631507609, NULL, 1721631507609);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (548156557615955978, NULL, 2, 609, 'ALL', NULL, NULL, 1, 1721631507609, NULL, 1721631507609);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (548156557615955979, NULL, 2, 515, 'ALL', NULL, NULL, 1, 1721631507611, NULL, 1721631507611);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (548156557615955980, NULL, 2, 13, 'ALL', NULL, NULL, 1, 1721631507613, NULL, 1721631507613);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (548156557615955981, NULL, 2, 1100, 'ALL', NULL, NULL, 1, 1721631507615, NULL, 1721631507615);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (548156557615955982, NULL, 2, 113, 'ALL', NULL, NULL, 1, 1721631507616, NULL, 1721631507616);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (548156557615955983, NULL, 2, 115, 'ALL', NULL, NULL, 1, 1721631507616, NULL, 1721631507616);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (548156557615955984, NULL, 2, 521, 'ALL', NULL, NULL, 1, 1721631507617, NULL, 1721631507617);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (548156557615955985, NULL, 2, 549, 'ALL', NULL, NULL, 1, 1721631507617, NULL, 1721631507617);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (548156557615955986, NULL, 2, 653, 'ALL', NULL, NULL, 1, 1721631507618, NULL, 1721631507618);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (548156557615955987, NULL, 2, 782, 'ALL', NULL, NULL, 1, 1721631507619, NULL, 1721631507619);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (548156557615955988, NULL, 2, 2088, 'ALL', NULL, NULL, 1, 1721631507619, NULL, 1721631507619);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (548156557615955989, NULL, 2, 2089, 'ALL', NULL, NULL, 1, 1721631507620, NULL, 1721631507620);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (548156557615955990, NULL, 2, 576, 'ALL', NULL, NULL, 1, 1721631507620, NULL, 1721631507620);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (548156557620150272, NULL, 2, 39, 'ALL', NULL, NULL, 1, 1721631507621, NULL, 1721631507621);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (548156557620150273, NULL, 2, 42, 'ALL', NULL, NULL, 1, 1721631507622, NULL, 1721631507622);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (548156557620150274, NULL, 2, 43, 'ALL', NULL, NULL, 1, 1721631507623, NULL, 1721631507623);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (548156557620150275, NULL, 2, 44, 'ALL', NULL, NULL, 1, 1721631507623, NULL, 1721631507623);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (548156557620150276, NULL, 2, 45, 'ALL', NULL, NULL, 1, 1721631507624, NULL, 1721631507624);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (548156557620150277, NULL, 2, 47, 'ALL', NULL, NULL, 1, 1721631507624, NULL, 1721631507624);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (548156557620150278, NULL, 2, 491, 'ALL', NULL, NULL, 1, 1721631507625, NULL, 1721631507625);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (548156557620150279, NULL, 2, 897, 'ALL', NULL, NULL, 1, 1721631507626, NULL, 1721631507626);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (548156557620150280, NULL, 2, 917, 'ALL', NULL, NULL, 1, 1721631507626, NULL, 1721631507626);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (548156557620150281, NULL, 2, 946, 'ALL', NULL, NULL, 1, 1721631507627, NULL, 1721631507627);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (548156557620150282, NULL, 2, 487, 'ALL', NULL, NULL, 1, 1721631507627, NULL, 1721631507627);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (548156557620150283, NULL, 2, 577, 'ALL', NULL, NULL, 1, 1721631507628, NULL, 1721631507628);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (548156557620150284, NULL, 2, 708, 'ALL', NULL, NULL, 1, 1721631507629, NULL, 1721631507629);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (548156557620150285, NULL, 2, 681, 'ALL', NULL, NULL, 1, 1721631507629, NULL, 1721631507629);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (548156557620150286, NULL, 2, 970, 'ALL', NULL, NULL, 1, 1721631507629, NULL, 1721631507629);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (548156557620150287, NULL, 2, 2036, 'ALL', NULL, NULL, 1, 1721631507630, NULL, 1721631507630);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (548156557620150288, NULL, 2, 3000, 'ALL', NULL, NULL, 1, 1721631507631, NULL, 1721631507631);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (548156557620150289, NULL, 2, 3001, 'ALL', NULL, NULL, 1, 1721631507631, NULL, 1721631507631);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (548156557620150290, NULL, 2, 2070, 'ALL', NULL, NULL, 1, 1721631507632, NULL, 1721631507632);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (548156557620150291, NULL, 2, 2002, 'ALL', NULL, NULL, 1, 1721631507632, NULL, 1721631507632);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (548156557620150292, NULL, 2, 2080, 'ALL', NULL, NULL, 1, 1721631507633, NULL, 1721631507633);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (548156557620150293, NULL, 2, 2001, 'ALL', NULL, NULL, 1, 1721631507633, NULL, 1721631507633);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (548156828068872192, NULL, 3, 576, 'SELF', NULL, NULL, 1, 1721631572074, NULL, 1721631572074);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (548156828068872193, NULL, 3, 491, 'SELF', NULL, NULL, 1, 1721631572077, NULL, 1721631572077);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (548156828068872194, NULL, 3, 897, 'SELF', NULL, NULL, 1, 1721631572078, NULL, 1721631572078);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (548156828068872195, NULL, 3, 917, 'SELF', NULL, NULL, 1, 1721631572078, NULL, 1721631572078);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (548156828068872196, NULL, 3, 946, 'SELF', NULL, NULL, 1, 1721631572079, NULL, 1721631572079);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (548156828068872197, NULL, 3, 487, 'SELF', NULL, NULL, 1, 1721631572079, NULL, 1721631572079);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (548156828068872198, NULL, 3, 577, 'SELF', NULL, NULL, 1, 1721631572079, NULL, 1721631572079);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (548156828068872199, NULL, 3, 708, 'SELF', NULL, NULL, 1, 1721631572080, NULL, 1721631572080);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (548156828068872200, NULL, 3, 970, 'SELF', NULL, NULL, 1, 1721631572080, NULL, 1721631572080);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (548156828068872201, NULL, 3, 2036, 'SELF_COURSE', NULL, NULL, 1, 1721631572080, NULL, 1721631572080);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (548156828068872202, NULL, 3, 3000, 'SELF', NULL, NULL, 1, 1721631572081, NULL, 1721631572081);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (548156828068872203, NULL, 3, 3001, 'SELF', NULL, NULL, 1, 1721631572081, NULL, 1721631572081);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (548156828068872204, NULL, 3, 2070, 'ALL', NULL, NULL, 1, 1721631572081, NULL, 1721631572081);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (548156828068872205, NULL, 3, 2002, 'SELF', NULL, NULL, 1, 1721631572082, NULL, 1721631572082);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (548156828068872206, NULL, 3, 2080, 'ALL', NULL, NULL, 1, 1721631572082, NULL, 1721631572082);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (548156828068872207, NULL, 3, 2001, 'SELF', NULL, NULL, 1, 1721631572082, NULL, 1721631572082);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (548156890568196096, NULL, 4, 576, 'SELF_ORG_BELOW', NULL, NULL, 1, 1721631586974, NULL, 1721631586974);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (548156890568196097, NULL, 4, 39, 'SELF_ORG_BELOW', NULL, NULL, 1, 1721631586976, NULL, 1721631586976);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (548156890568196098, NULL, 4, 42, 'SELF_ORG_BELOW', NULL, NULL, 1, 1721631586976, NULL, 1721631586976);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (548156974190034944, NULL, 6, 970, 'SELF_ORG_BELOW', NULL, NULL, 1, 1721631606912, NULL, 1721631606912);
INSERT INTO `basic_role_data_permission` (`id`, `school_id`, `role_id`, `privilege_id`, `data_permission_type`, `org_scope`, `org_id`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (548157012974764032, NULL, 7, 491, 'SELF', NULL, NULL, 1, 1721631616159, NULL, 1721631616159);

-- ----------------------------
-- Table structure for basic_school
-- ----------------------------
CREATE TABLE IF NOT EXISTS `basic_school` (
  `id` bigint NOT NULL COMMENT '主键',
  `code` varchar(20) NOT NULL COMMENT '学校编码',
  `domain_name` varchar(20) DEFAULT NULL COMMENT '域名',
  `name` varchar(100) DEFAULT NULL COMMENT '学校名称',
  `enable` tinyint NOT NULL DEFAULT '1' COMMENT '是否启用，false：停用，true：启用',
  `access_key` varchar(50) DEFAULT NULL COMMENT '访问key',
  `access_secret` varchar(50) DEFAULT NULL COMMENT '访问秘钥',
  `remark` text COMMENT '备注',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  `logo` mediumtext COMMENT '学校logo',
  `init_password` varchar(100) DEFAULT NULL COMMENT '该学校用户的初始密码，默认为Qmth87863577',
  `default_school` tinyint(1) DEFAULT '0',
  `has_paper_number` tinyint(1) DEFAULT '0' COMMENT '该学校是否有固定试卷编号',
  `background_image` varchar(500) DEFAULT NULL COMMENT '学校首页背景图',
  `exam_task_instr` varchar(500) DEFAULT NULL COMMENT '广药命题界面提示信息',
  PRIMARY KEY (`id`) USING BTREE,
  UNIQUE KEY `code` (`code`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='学校表';

-- ----------------------------
-- Table structure for basic_semester
-- ----------------------------
CREATE TABLE IF NOT EXISTS `basic_semester` (
  `id` bigint NOT NULL COMMENT '主键',
  `school_id` bigint NOT NULL COMMENT '学校id',
  `name` varchar(100) NOT NULL COMMENT '学期名称',
  `code` varchar(100) DEFAULT NULL COMMENT '学期编码',
  `start_time` bigint DEFAULT NULL COMMENT '开始日期',
  `end_time` bigint DEFAULT NULL COMMENT '结束日期',
  `org_id` bigint DEFAULT NULL COMMENT '机构id',
  `enable` tinyint DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
  `create_id` bigint DEFAULT NULL COMMENT '创建人id',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人id',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  PRIMARY KEY (`id`) USING BTREE,
  UNIQUE KEY `semester_schoolId_name_idx` (`school_id`,`name`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='学期字典表';

-- ----------------------------
-- Table structure for basic_student
-- ----------------------------
CREATE TABLE IF NOT EXISTS `basic_student` (
  `id` bigint NOT NULL COMMENT '学生id',
  `school_id` bigint NOT NULL COMMENT '学校id',
  `student_name` varchar(100) NOT NULL COMMENT '学生姓名',
  `student_code` varchar(100) NOT NULL COMMENT '学生编号',
  `phone_number` varchar(11) DEFAULT NULL COMMENT '电话号码',
  `belong_org_id` bigint DEFAULT NULL COMMENT '所属院系',
  `major_id` bigint DEFAULT NULL COMMENT '专业id',
  `clazz_id` bigint DEFAULT NULL COMMENT '班级主键',
  `enable` tinyint DEFAULT '1' COMMENT '可用状态（1：可用，0：不可用）',
  `org_id` bigint DEFAULT NULL COMMENT '可操作人机构id',
  `create_id` bigint DEFAULT NULL COMMENT '创建人id',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人id',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  PRIMARY KEY (`id`) USING BTREE,
  UNIQUE KEY `student_schoolId_code_orgId_idx` (`school_id`,`student_code`,`belong_org_id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='学校id';

-- ----------------------------
-- Table structure for basic_teach_clazz
-- ----------------------------
CREATE TABLE IF NOT EXISTS `basic_teach_clazz` (
  `id` bigint NOT NULL COMMENT 'id',
  `school_id` bigint NOT NULL COMMENT '学校id',
  `exam_id` bigint NOT NULL COMMENT '考试id',
  `course_id` bigint DEFAULT NULL COMMENT '课程ID（basic_course表ID）',
  `course_code` varchar(20) DEFAULT NULL COMMENT '课程编号',
  `clazz_name` varchar(50) NOT NULL COMMENT '教学班名称',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  PRIMARY KEY (`id`),
  UNIQUE KEY `basic_teach_clazz_unique` (`school_id`,`exam_id`,`clazz_name`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='基础教学班表';

-- ----------------------------
-- Table structure for basic_teach_course
-- ----------------------------
CREATE TABLE IF NOT EXISTS `basic_teach_course` (
  `id` bigint NOT NULL COMMENT '主键',
  `school_id` bigint DEFAULT NULL COMMENT '学校id',
  `org_id` bigint DEFAULT NULL COMMENT '机构id',
  `user_id` bigint DEFAULT NULL COMMENT '用户ID',
  `course_id` bigint DEFAULT NULL COMMENT '课程ID',
  `enable` tinyint DEFAULT '1' COMMENT '是否启用？1-是 ；0-否',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='老师和课程关联表';

-- ----------------------------
-- Table structure for basic_template
-- ----------------------------
CREATE TABLE IF NOT EXISTS `basic_template` (
  `id` bigint NOT NULL COMMENT '主键',
  `school_id` bigint NOT NULL COMMENT '学校id',
  `org_id` bigint DEFAULT NULL COMMENT '机构id',
  `name` varchar(30) NOT NULL COMMENT '模板名称',
  `classify` varchar(10) DEFAULT NULL COMMENT 'template_type值为VARIABLE、ORDINARY时，不为空。\n  SIGN-签到表，PACKAGE-卷袋贴，CHECK_IN-登记表',
  `display_range` mediumtext COMMENT '卷袋贴、签到表数据范围',
  `preview_path` varchar(500) DEFAULT NULL COMMENT '预览地址',
  `attachment_id` bigint DEFAULT NULL COMMENT '模板附件ID（保存到附件表ID）',
  `enable` tinyint NOT NULL COMMENT '0-禁用，1-启用',
  `diallel` tinyint(1) DEFAULT '1' COMMENT '考生信息是否双列显示',
  `text_desc` varchar(500) DEFAULT NULL COMMENT '文字说明',
  `remark` varchar(200) DEFAULT NULL COMMENT '备注',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  `add_blank_page` tinyint(1) DEFAULT '1' COMMENT '是否增加空白页',
  `line_height_times` double DEFAULT '1' COMMENT '行间距在初始值扩大倍数（默认1）',
  `font_size` varchar(10) DEFAULT NULL COMMENT '字体大小（small,medium,large）',
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='通用模板';

-- ----------------------------
-- Table structure for basic_template_org
-- ----------------------------
CREATE TABLE IF NOT EXISTS `basic_template_org` (
  `id` bigint NOT NULL COMMENT '主键',
  `school_id` bigint NOT NULL COMMENT '学校id',
  `type` varchar(20) NOT NULL COMMENT '模板类型：GENERIC-通卡模板，VARIABLE-变量印品模板，ORDINARY-普通印品模板，CARD_RULE-题卡规则，PRINT_CONFIG-印品配置',
  `rule_id` bigint NOT NULL COMMENT '通卡模板ID',
  `org_id` bigint NOT NULL COMMENT '组织机构ID',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='适用学院（题卡规则、模板适用学院）';

-- ----------------------------
-- Table structure for basic_user_course
-- ----------------------------
CREATE TABLE IF NOT EXISTS `basic_user_course` (
  `id` bigint NOT NULL COMMENT '主键',
  `school_id` bigint DEFAULT NULL COMMENT '学校id',
  `org_id` bigint DEFAULT NULL COMMENT '机构id',
  `user_id` bigint DEFAULT NULL COMMENT '用户ID',
  `course_id` bigint DEFAULT NULL COMMENT '课程ID',
  `clazz_id` bigint DEFAULT NULL COMMENT '授课班级',
  `enable` tinyint(1) DEFAULT NULL COMMENT '是否启用？1-是 ；0-否',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='用户和课程关联表';

-- ----------------------------
-- Table structure for basic_verify_code
-- ----------------------------
CREATE TABLE IF NOT EXISTS `basic_verify_code` (
  `id` bigint NOT NULL COMMENT '主键',
  `school_id` bigint DEFAULT NULL COMMENT '学校id',
  `org_id` bigint DEFAULT NULL COMMENT '机构id',
  `user_id` bigint NOT NULL COMMENT '用户ID',
  `mobile_number` varchar(11) NOT NULL COMMENT '手机号',
  `expire_time` bigint NOT NULL COMMENT '到期时间',
  `valid_period` int NOT NULL COMMENT '单位（分）',
  `verify_code` varchar(4) NOT NULL COMMENT '4位数字',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='短信验证码记录表';

-- ----------------------------
-- Table structure for client_print_data
-- ----------------------------
CREATE TABLE IF NOT EXISTS `client_print_data` (
  `id` bigint NOT NULL COMMENT '主键',
  `school_id` bigint DEFAULT NULL COMMENT '学校id',
  `org_id` bigint DEFAULT NULL COMMENT '机构id',
  `exam_detail_id` bigint DEFAULT NULL COMMENT '考场ID',
  `machine_code` varchar(45) DEFAULT NULL COMMENT '机器唯一码',
  `pages_a4` int DEFAULT NULL COMMENT 'A4页数量',
  `paper_pages` int DEFAULT NULL COMMENT '试卷数量',
  `card_pages` int DEFAULT NULL COMMENT '题卡数量',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='各机器打印数量';

-- ----------------------------
-- Table structure for client_status
-- ----------------------------
CREATE TABLE IF NOT EXISTS `client_status` (
  `id` bigint NOT NULL COMMENT '主键',
  `school_id` bigint DEFAULT NULL COMMENT '学校id',
  `org_id` bigint DEFAULT NULL COMMENT '机构id',
  `machine_code` varchar(100) NOT NULL COMMENT '机器唯一码',
  `exam_detail_id` bigint DEFAULT NULL COMMENT '考场ID',
  `print_plan_id` bigint DEFAULT NULL COMMENT '印刷计划ID',
  `course_id` bigint DEFAULT NULL COMMENT '课程ID（basic_course表ID）',
  `course_code` varchar(100) DEFAULT NULL COMMENT '课程代码',
  `course_name` varchar(200) DEFAULT NULL COMMENT '课程名称',
  `paper_number` varchar(100) NOT NULL COMMENT '试卷编号',
  `is_try` tinyint DEFAULT NULL COMMENT '是否打样：true-是，false-否',
  `try_time` bigint DEFAULT NULL COMMENT '打样时间',
  `is_pass` tinyint DEFAULT NULL COMMENT '是否合格：true-合格，false-不合格',
  `is_download` tinyint DEFAULT NULL COMMENT '是否缓存：true-是，false-否',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  PRIMARY KEY (`id`) USING BTREE,
  KEY `idx_1` (`school_id`,`exam_detail_id`,`course_code`,`paper_number`,`machine_code`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='机器打样记录';

-- ----------------------------
-- Table structure for client_upgrade
-- ----------------------------
CREATE TABLE IF NOT EXISTS `client_upgrade` (
  `platform` varchar(20) NOT NULL COMMENT '系统',
  `tool_type` VARCHAR(45) NOT NULL COMMENT '文件类型',
  `install_path` varchar(200) DEFAULT NULL COMMENT '安装包保存地址',
  `version` varchar(20) DEFAULT NULL COMMENT '版本名称',
  `build` varchar(45) DEFAULT NULL COMMENT '版本号，用于客户端比对版本高低',
  `support_min` varchar(45) DEFAULT NULL COMMENT '增量升级最低版本号，低于（含）此版本需要下载安装包重新安装',
  `upgrade_path` varchar(200) DEFAULT NULL COMMENT '升级包保存地址',
  `mainfest_path` varchar(200) DEFAULT NULL COMMENT 'mainfest.json文件保存路径',
  `install_upload_time` bigint DEFAULT NULL COMMENT '安装包更新时间',
  `upgrade_upload_time` bigint DEFAULT NULL COMMENT '升级包更新时间',
  PRIMARY KEY (`platform`, `tool_type`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci;

INSERT INTO `client_upgrade` (`platform`,`tool_type`, `install_path`,`install_upload_time`) select 'WINDOWS','TEACHCLOUD_CLIENT','{"md5":"b458068a408a11a82625b65e7e252eff","path":"知学知考-印刷扫描客户端_V3.4.0.exe","type":"oss","uploadType":"UPGRADE"}', unix_timestamp()*1000;

-- ----------------------------
-- Table structure for cloud_user_push_status
-- ----------------------------
CREATE TABLE IF NOT EXISTS `cloud_user_push_status` (
  `id` bigint NOT NULL,
  `school_id` bigint NOT NULL COMMENT '学校id',
  `user_id` bigint NOT NULL,
  `marker_status` varchar(100) NOT NULL COMMENT '评卷员推送状态(UserPushResultEnum):BIND_SUCCESS("绑定成功"),BIND_FAILED("绑定失败"),UNBIND_SUCCESS("解绑成功"),UNBIND_FAILED("解绑失败");',
  `marker_leader_status` varchar(100) NOT NULL COMMENT '科组长推送状态(UserPushResultEnum):BIND_SUCCESS("绑定成功"),BIND_FAILED("绑定失败"),UNBIND_SUCCESS("解绑成功"),UNBIND_FAILED("解绑失败");',
  `sync_status` varchar(100) NOT NULL COMMENT '同步状态(SyncStatusEnum)，ALREADY_PUSH("已经完成推送"),NEED_PUSH_AGAIN("需要重新推送");',
  `create_id` bigint DEFAULT NULL COMMENT '创建人\n',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  PRIMARY KEY (`id`) USING BTREE,
  KEY `idx_school_id_user_id` (`school_id`,`user_id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='云阅卷用户推送状态表';

-- ----------------------------
-- Table structure for exam_card
-- ----------------------------
CREATE TABLE IF NOT EXISTS `exam_card` (
  `id` bigint NOT NULL COMMENT '主键',
  `school_id` bigint NOT NULL COMMENT '学校id',
  `org_id` bigint DEFAULT NULL COMMENT '机构id',
  `course_id` bigint DEFAULT NULL COMMENT '课程ID（basic_course表ID）',
  `course_code` varchar(30) DEFAULT NULL COMMENT '课程编码',
  `course_name` varchar(100) DEFAULT NULL COMMENT '课程名称',
  `title` varchar(200) NOT NULL COMMENT '标题',
  `make_method` varchar(10) DEFAULT NULL COMMENT '题卡制作方式：SELECT-选择已有题卡，SELF-自助创建，CUST-客户制卡',
  `status` varchar(45) DEFAULT NULL COMMENT 'STAGE-暂存，SUBMIT-提交',
  `content` mediumtext COMMENT '题卡工具制作题卡内容',
  `stage_content` mediumtext COMMENT '暂存内容（提交后置空）',
  `html_content` mediumtext COMMENT 'html格式内容',
  `type` varchar(45) NOT NULL COMMENT '题卡类型：GENERIC-通卡，CUSTOM-自定义',
  `create_method` varchar(45) DEFAULT NULL COMMENT '通卡创建方式（UPLOAD-上传，CUSTOMIZE-自定义）',
  `attachment_id` bigint DEFAULT NULL COMMENT 'type=GENERIC时必传，上传附件ID',
  `enable` tinyint(1) DEFAULT '1' COMMENT '1正常，0禁用',
  `used` tinyint(1) DEFAULT '0' COMMENT '是否绑定命题任务，默认0',
  `remark` mediumtext COMMENT '备注',
  `card_rule_id` bigint DEFAULT NULL COMMENT '题卡规则Id',
  `backup_attachment` mediumtext COMMENT '备用题卡json',
  `jpg_attachment` text COMMENT '题卡转换成的jpg文件信息',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  `blank_card_path` varchar(500) DEFAULT NULL COMMENT '空白题卡pdf路径',
  `paper_id` bigint DEFAULT NULL COMMENT '题库试卷ID',
  `copy_card_id` bigint DEFAULT NULL COMMENT '复制源题卡ID',
  `page_size` VARCHAR(10) NULL COMMENT '题卡纸张大小（A3，8K）',
    `open_ab` TINYINT(1) NULL DEFAULT 0 COMMENT '是否AB卷（AB卷题卡）',
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='题卡';

-- ----------------------------
-- Table structure for exam_card_detail
-- ----------------------------
CREATE TABLE IF NOT EXISTS `exam_card_detail` (
  `id` bigint NOT NULL COMMENT '主键',
  `card_id` bigint NOT NULL COMMENT '题卡ID',
  `content` mediumtext COMMENT '题卡工具制作题卡内容',
  `html_content` mediumtext COMMENT 'html格式内容',
  `attachment_id` mediumtext COMMENT '备用题卡json',
  `cust_attachment_id` bigint DEFAULT NULL COMMENT '客服制卡上传附件ID',
  `jpg_attachment_info` text COMMENT '题卡转换成的jpg文件信息',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='题卡详情';

-- ----------------------------
-- Table structure for exam_detail
-- ----------------------------
CREATE TABLE IF NOT EXISTS `exam_detail` (
  `id` bigint NOT NULL COMMENT '主键',
  `school_id` bigint NOT NULL COMMENT '学校id',
  `org_id` bigint DEFAULT NULL COMMENT '机构id',
  `print_plan_id` bigint NOT NULL COMMENT '印刷计划ID',
  `exam_id` bigint DEFAULT NULL COMMENT '考试ID',
  `print_plan_name` varchar(100) DEFAULT NULL COMMENT '印刷计划名称',
  `exam_start_time` bigint DEFAULT NULL COMMENT '考试开始日期',
  `exam_end_time` bigint DEFAULT NULL COMMENT '考试结束日期',
  `exam_place` varchar(30) DEFAULT NULL COMMENT '考场',
  `total_subjects` int DEFAULT NULL COMMENT '总科次：考场总科次',
  `exam_room` varchar(30) DEFAULT NULL COMMENT '教室',
  `print_progress` double DEFAULT NULL COMMENT '打印进度',
  `print_user` varchar(30) DEFAULT NULL COMMENT '打印员：一个考场只能被一个打印员认领',
  `package_code` varchar(30) DEFAULT NULL COMMENT '考场代码：卷袋贴、签到表上条码',
  `paper_pages` int DEFAULT NULL COMMENT '试卷总计（张）',
  `card_pages` int DEFAULT NULL COMMENT '题卡总计（张）',
  `pages_a4` int DEFAULT NULL COMMENT 'A4印量小计（页）',
  `status` varchar(30) DEFAULT NULL COMMENT '状态：NEW: "新建",READY: "就绪",WAITING: "待印刷",PRINTING: "印刷中",FINISH: "已完成"',
  `validate` tinyint(1) DEFAULT '0' COMMENT '是否校验：true-是，false-否',
  `print_start_time` bigint DEFAULT NULL COMMENT '打印开始时间',
  `print_end_time` bigint DEFAULT NULL COMMENT '打印完成时间',
  `attachment_id` bigint DEFAULT NULL COMMENT '附件id',
  `card_attachment_id` bigint DEFAULT NULL COMMENT '题卡合并A3文件附件ID',
  `attachment_path` mediumtext COMMENT '印品附件路径',
  `exam_data_source` varchar(20) DEFAULT NULL COMMENT '数据来源（EXAM_TASK：卷库查询生成，FILE_IMPORT：考务文件导入）',
  `normal` tinyint DEFAULT '1' COMMENT '是否正常，true正常，false做废',
  `print_house_id` bigint DEFAULT NULL COMMENT '印刷室ID',
  `backup_count` double DEFAULT NULL COMMENT '试卷、题卡备份数量（0-1为按比例，大于等于1为按数量）',
  `task_id` bigint DEFAULT NULL COMMENT 'pdf生成异步任务ID（t_b_task）',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  PRIMARY KEY (`id`) USING BTREE,
  KEY `idx_print_plan_id` (`print_plan_id`) USING BTREE,
  KEY `exam_detail_school_id_IDX` (`school_id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='考务数据';

-- ----------------------------
-- Table structure for exam_detail_course
-- ----------------------------
CREATE TABLE IF NOT EXISTS `exam_detail_course` (
  `id` bigint NOT NULL COMMENT '主键',
  `school_id` bigint NOT NULL COMMENT '学校id',
  `exam_detail_id` bigint NOT NULL COMMENT '考试明细id',
  `course_id` bigint DEFAULT NULL COMMENT '课程ID（basic_course表ID）',
  `course_code` varchar(20) DEFAULT NULL COMMENT '课程编码',
  `course_name` varchar(50) DEFAULT NULL COMMENT '课程名称',
  `paper_number` varchar(100) NOT NULL COMMENT '页数',
  `course_paper_id` varchar(45) DEFAULT NULL COMMENT '试卷编号（交互）',
  `total_subjects` int DEFAULT NULL COMMENT '总科次：考场总科次',
  `paper_pages_a3` int DEFAULT NULL COMMENT '试卷单科次准印量A3（页）',
  `card_pages_a3` int DEFAULT NULL COMMENT '题卡单科次准印量A3（页）',
  `clazz_id` text COMMENT '班级id',
  `clazz_name` mediumtext COMMENT '班级（取学生表中的班级集合）',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  `serial_number` INT(2) NULL COMMENT '备用卷',
  `paper_type` varchar(30) DEFAULT NULL COMMENT '当前试卷类型',
  `attachment_id` mediumtext COMMENT '备用题卡json',
  `common_attachment_id` mediumtext COMMENT '模式2下考生公共题卡json',
  PRIMARY KEY (`id`) USING BTREE,
  KEY `detail_course_schoolId_idx` (`school_id`,`paper_number`,`paper_type`) USING BTREE,
  KEY `idx_exam_detail_id` (`exam_detail_id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='考务数据（考场关联科目）';

-- ----------------------------
-- Table structure for exam_detail_course_paper_type
-- ----------------------------
CREATE TABLE IF NOT EXISTS `exam_detail_course_paper_type` (
  `id` bigint NOT NULL,
  `exam_detail_course_id` bigint NOT NULL,
  `paper_type` varchar(5) NOT NULL COMMENT '卷型',
  PRIMARY KEY (`id`),
  KEY `idx_exam_detail_course_id` (`exam_detail_course_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='考务数据（考场关联科目对应的卷型）';

-- ----------------------------
-- Table structure for exam_paper_class_marker
-- ----------------------------
CREATE TABLE IF NOT EXISTS `exam_paper_class_marker` (
  `id` bigint NOT NULL,
  `exam_paper_structure_id` bigint DEFAULT NULL COMMENT '评卷参数id',
  `marker_id` bigint DEFAULT NULL COMMENT '评卷员id',
  `login_name` varchar(100) DEFAULT NULL COMMENT '评卷员登录名',
  `class_name` mediumtext COMMENT '班级',
  `create_id` bigint DEFAULT NULL,
  `create_time` bigint DEFAULT NULL,
  `update_id` bigint DEFAULT NULL,
  `update_time` bigint DEFAULT NULL,
  PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='分班阅（评卷员班级对应关系表）';

-- ----------------------------
-- Table structure for exam_paper_group
-- ----------------------------
CREATE TABLE IF NOT EXISTS `exam_paper_group` (
  `id` bigint NOT NULL COMMENT '分组id',
  `exam_paper_structure_id` bigint NOT NULL COMMENT '试卷结构id',
  `group_number` int NOT NULL COMMENT '分组序号',
  `double_rate` tinyint DEFAULT NULL COMMENT '双评比例（0关闭，1开启）',
  `arbitrate_threshold` int DEFAULT NULL COMMENT '仲裁阈值（开启双评必填）',
  `score_policy` varchar(45) DEFAULT NULL COMMENT '合分策略 开启双评是必填 AVG("平均分", 1), MAX("最高分", 2), MIN("最低分", 3)',
  `trial_count` int DEFAULT NULL COMMENT '试评数量',
  `mark_mode` varchar(45) DEFAULT NULL COMMENT '评卷模式 COMMON("普通"), TRACK("轨迹")',
  `question_info` text COMMENT '题目信息',
  `picture_config` text COMMENT '图片配置',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  PRIMARY KEY (`id`) USING BTREE,
  UNIQUE KEY `exam_paper_group_unique` (`exam_paper_structure_id`,`group_number`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='试卷分组表';

-- ----------------------------
-- Table structure for exam_paper_group_marker
-- ----------------------------
CREATE TABLE IF NOT EXISTS `exam_paper_group_marker` (
  `id` bigint NOT NULL COMMENT '评卷员-分组关系id',
  `group_id` bigint NOT NULL COMMENT '分组id',
  `marker_id` bigint NOT NULL COMMENT '评卷员id',
  `login_name` varchar(100) DEFAULT NULL COMMENT '评卷员登录名',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='评卷员-分组关系表';

-- ----------------------------
-- Table structure for exam_paper_structure
-- ----------------------------
CREATE TABLE IF NOT EXISTS `exam_paper_structure` (
  `id` bigint NOT NULL COMMENT '主键',
  `school_id` bigint DEFAULT NULL COMMENT '学校ID',
  `semester_id` bigint DEFAULT NULL COMMENT '学期ID',
  `semester_name` varchar(100) DEFAULT NULL COMMENT '学期',
  `exam_id` bigint NOT NULL COMMENT '考试id',
  `third_relate_id` bigint DEFAULT NULL COMMENT '云阅卷考试ID',
  `third_relate_name` varchar(255) DEFAULT NULL COMMENT '云阅卷考试名称',
  `paper_number` varchar(100) NOT NULL COMMENT '试卷编号',
  `course_code` varchar(50) DEFAULT NULL COMMENT '课程代码',
  `course_name` varchar(255) DEFAULT NULL COMMENT '课程名称',
  `sequence` varchar(5) DEFAULT NULL COMMENT '课程创建的任务序号（和exam_task保持一致）',
  `paper_type` varchar(20) DEFAULT NULL COMMENT '试卷类型',
  `status` varchar(700) DEFAULT NULL COMMENT '{\n    "structure":\n    {\n        "save": false,\n        "sync": false\n    },\n    "group":\n    {\n        "save": false,\n        "sync": false\n    },\n    "class":\n    {\n        "save": false,\n        "sync": false\n    },\n    "objective":\n    {\n        "save": false,\n        "sync": false\n    },\n    "markerLeader":\n    {\n        "save": false,\n        "sync": false\n    },\n    "marker":\n    {\n        "save": false,\n        "sync": false\n    },\n    "answerFile":\n    {\n        "save": false,\n        "sync": false\n    }\n}',
  `is_finish` tinyint(1) DEFAULT '0' COMMENT '是否完成。根据status中组合状态计算。用于当查询条件',
  `paper_answer` text COMMENT '试卷原卷和标答附件ID',
  `objective_structure` mediumtext COMMENT '客观题试卷结构JSON',
  `subjective_structure` mediumtext COMMENT '主观题试卷结构JSON',
  `paper_info_json` mediumtext COMMENT '整个试卷信息json（前端提供的全部信息）',
  `proposition_teacher_id` bigint DEFAULT NULL COMMENT '命题老师ID',
  `enable` tinyint(1) DEFAULT NULL COMMENT '是否启用（0-禁用，1-启用）',
  `mark_leader` text COMMENT '科组长，样例：[{markerId:1, loginName:"mt01"}]',
  `cloud_info_json` text COMMENT '云阅卷试卷结构',
  `structure_change` tinyint(1) DEFAULT NULL COMMENT '云阅卷试卷结构和知学知考结构不一样（默认false）',
  `object_answer_change` tinyint(1) DEFAULT '0' COMMENT '客观题答案有提交，需要重新统分。（1：有变动，需要统分，0：不统分）',
  `open_class_reading` tinyint(1) DEFAULT '0' COMMENT '是否开启分班阅',
  `create_id` bigint DEFAULT NULL,
  `create_time` bigint DEFAULT NULL,
  `update_id` bigint DEFAULT NULL,
  `update_time` bigint DEFAULT NULL,
  PRIMARY KEY (`id`) USING BTREE,
  UNIQUE KEY `exam_paper_structure_unique` (`school_id`,`exam_id`,`paper_number`,`paper_type`) USING BTREE,
  KEY `exam_paper_structure_status_IDX` (`status`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci;

-- ----------------------------
-- Table structure for exam_print_plan
-- ----------------------------
CREATE TABLE IF NOT EXISTS `exam_print_plan` (
  `id` bigint NOT NULL COMMENT '主键',
  `exam_id` bigint NOT NULL COMMENT '考试id',
  `school_id` bigint NOT NULL COMMENT '学校id',
  `org_id` bigint NOT NULL COMMENT '机构id',
  `category` varchar(10) DEFAULT NULL COMMENT '考试类型',
  `name` varchar(150) DEFAULT NULL COMMENT '名称',
  `exam_start_time` bigint DEFAULT NULL COMMENT '考试开始时间',
  `exam_end_time` bigint DEFAULT NULL COMMENT '考试结束时间',
  `print_content` varchar(30) NOT NULL COMMENT '{“PAPER”, "CARD"}',
  `backup_method` varchar(10) NOT NULL COMMENT 'PLACE-按考点，ROOM-按考场',
  `backup_count` double NOT NULL COMMENT '试卷、题卡备份数量（0-1为按比例，大于等于1为按数量）',
  `draw_rule` varchar(10) DEFAULT NULL COMMENT 'ONE-只抽一次，CIRCLE-可循环抽取',
  `variable_content` text NOT NULL COMMENT '[\n  	{\n  		"type": "SIGN",\n  		"usedId": 1,\n  		"backupMethod": "ROOM",\n  		"backupCount": 3\n  	},\n  	{\n  		"type": "PACKAG",\n  		"usedId": 3,\n  		"backupMethod": "STUDENT",\n  		"backupCount": 5\n  	}\n  ]',
  `ordinary_content` text NOT NULL COMMENT '[\n  	{\n  		"type": "CHECK_IN",\n  		"usedId": 1,\n  		"backupMethod": "ROOM",\n  		"backupCount": 3\n  	}\n  ]',
  `status` varchar(20) NOT NULL COMMENT '印刷计划状态',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='印刷计划管理';

-- ----------------------------
-- Table structure for exam_student
-- ----------------------------
CREATE TABLE IF NOT EXISTS `exam_student` (
  `id` bigint NOT NULL COMMENT '主键',
  `school_id` bigint NOT NULL COMMENT '学校id',
  `org_id` bigint DEFAULT NULL COMMENT '机构id',
  `exam_detail_course_id` bigint NOT NULL COMMENT '考试明细课程id',
  `basic_student_id` bigint DEFAULT NULL COMMENT '考生ID（basic_exam_student表ID）',
  `student_name` varchar(100) DEFAULT NULL COMMENT '考生姓名',
  `student_code` varchar(100) DEFAULT NULL COMMENT '考生代码',
  `exam_id` bigint DEFAULT NULL COMMENT '考试ID',
  `paper_number` varchar(100) NOT NULL COMMENT '试卷编号',
  `course_paper_id` varchar(45) DEFAULT NULL COMMENT '试卷编号（交互）',
  `ticket_number` varchar(30) DEFAULT NULL COMMENT '准考证号',
  `site_number` varchar(30) DEFAULT NULL COMMENT '座位号',
  `print_paper` tinyint DEFAULT NULL COMMENT '打印试卷',
  `print_card` tinyint DEFAULT NULL COMMENT '打印题卡',
  `required_fields` mediumtext COMMENT '基础字段',
  `extend_fields` text COMMENT '扩展字段',
  `paper_type` varchar(2) DEFAULT NULL COMMENT '试卷类型',
  `college_name` varchar(100) DEFAULT NULL COMMENT '学生学院名称',
  `major_name` varchar(100) DEFAULT NULL COMMENT '专业名称',
  `clazz_id` varchar(100) DEFAULT NULL COMMENT '班级Id',
  `clazz_name` varchar(100) DEFAULT NULL COMMENT '班级名称',
  `teach_clazz_id` bigint DEFAULT NULL COMMENT '教学班id',
  `teach_clazz_name` varchar(100) DEFAULT NULL COMMENT '教学班名称',
  `student_clazz_type` varchar(100) DEFAULT NULL COMMENT '主体班级类型',
  `campus_name` varchar(200) DEFAULT NULL COMMENT '校区',
  `teacher_id` bigint DEFAULT NULL COMMENT '任课老师ID',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  `attachment_id` bigint DEFAULT NULL COMMENT '附件id',
  PRIMARY KEY (`id`) USING BTREE,
  KEY `idx_exam_detail_course_id` (`exam_detail_course_id`) USING BTREE,
  KEY `idx_4` (`exam_id`,`paper_number`,`student_code`),
  KEY `idx_basic_student_id` (`basic_student_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='考生';

-- ----------------------------
-- Table structure for exam_task
-- ----------------------------
CREATE TABLE IF NOT EXISTS `exam_task` (
  `id` bigint NOT NULL COMMENT '主键',
  `exam_id` bigint NOT NULL COMMENT '考试id',
  `school_id` bigint NOT NULL COMMENT '学校id',
  `org_id` bigint DEFAULT NULL COMMENT '机构id',
  `course_id` bigint DEFAULT NULL COMMENT '课程ID（basic_course表ID）',
  `course_code` varchar(20) DEFAULT NULL COMMENT '课程代码',
  `course_name` varchar(50) DEFAULT NULL COMMENT '课程名称',
  `specialty` varchar(100) DEFAULT NULL COMMENT '适用专业',
  `paper_number` varchar(100) NOT NULL COMMENT '试卷编号',
  `open_ab` TINYINT(1) NULL DEFAULT 0 COMMENT '是否开启AB卷',
  `sequence` varchar(5) DEFAULT NULL COMMENT '课程创建的任务序号',
  `start_time` bigint DEFAULT NULL COMMENT '命题开始时间',
  `end_time` bigint DEFAULT NULL COMMENT '命题结束时间',
  `card_rule_id` bigint NOT NULL COMMENT '题卡规则',
  `user_id` bigint DEFAULT NULL COMMENT '命题老师',
  `transfer_id` bigint DEFAULT NULL COMMENT '转办人ID',
  `status` varchar(50) DEFAULT NULL COMMENT '任务状态，DRAFT：新建，SUBMIT：已提交，STAGE：待命题',
  `review` tinyint DEFAULT NULL COMMENT '是否审核，0-禁用，1-启用',
  `review_status` varchar(8) DEFAULT NULL COMMENT '审核状态：PASS-审核通过，NOT_PASS-审核不通过，默认为空-未审核',
  `enable` tinyint DEFAULT '1' COMMENT '是否启用，false：停用，true：启用',
  `exam_task_detail_enable` TINYINT(1) NULL DEFAULT 1 COMMENT '卷库启用禁用状态。是否启用，false：停用，true：启用',
  `batch_no` varchar(32) DEFAULT NULL COMMENT '批次id',
  `teaching_room_id` bigint DEFAULT NULL COMMENT '教学单位、教研室',
  `teacher_name` varchar(100) DEFAULT NULL COMMENT '拟卷教师',
  `lecturer_name` varchar(100) DEFAULT NULL COMMENT '主讲教师',
  `paper_name` varchar(100) DEFAULT NULL COMMENT '试卷名称',
  `cancel_remark` mediumtext COMMENT '作废原因',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '修改人',
  `update_time` bigint DEFAULT NULL COMMENT '修改时间',
  `flow_id` bigint DEFAULT NULL COMMENT '流程id',
  `approve_form_attachment_id` bigint DEFAULT NULL COMMENT '命题任务审批表附件id',
  `source` varchar(45) DEFAULT NULL COMMENT '命题任务创建方式（ASSIGN：命题任务指派，SELF：老师自主发起）',
  `exam_task_content` MEDIUMTEXT NULL COMMENT '入库申请请求参数',
  `paper_confirm_attachment_ids` TEXT NULL,
  `remark` VARCHAR(200) NULL,
  PRIMARY KEY (`id`) USING BTREE,
  KEY `task_schoolId_flowId_idx` (`school_id`,`flow_id`,`card_rule_id`) USING BTREE,
  KEY `idx_school_id_paper_number` (`exam_id`,`paper_number`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='命题任务';

-- ----------------------------
-- Table structure for exam_task_apply_temp
-- ----------------------------
CREATE TABLE IF NOT EXISTS `exam_task_apply_temp` (
  `id` bigint unsigned NOT NULL,
  `school_id` bigint NOT NULL COMMENT '学校Id',
  `org_id` bigint NOT NULL COMMENT '机构id',
  `exam_task_content` mediumtext COMMENT '命题任务json',
  `create_id` bigint DEFAULT NULL,
  `create_time` bigint DEFAULT NULL,
  `update_id` bigint DEFAULT NULL,
  `update_time` bigint DEFAULT NULL,
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='卷库查询-命题任务申请临时表';

-- ----------------------------
-- Table structure for exam_task_assign_paper_type
-- ----------------------------
CREATE TABLE IF NOT EXISTS `exam_task_assign_paper_type` (
  `exam_id` bigint NOT NULL COMMENT '考试ID',
  `paper_number` varchar(100) NOT NULL COMMENT '试卷编号',
  `exam_start_time` bigint NOT NULL COMMENT '考试时间（开始）',
  `exam_end_time` bigint NOT NULL COMMENT '考试时间（结束）',
  `paper_type` varchar(45) DEFAULT NULL COMMENT '卷型',
  `assign_mode` varchar(10) DEFAULT NULL COMMENT '分配方式：MANUAL-手动关联，AUTO-自动分配',
  PRIMARY KEY (`exam_id`,`paper_number`,`exam_start_time`,`exam_end_time`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci;

-- ----------------------------
-- Table structure for exam_task_detail
-- ----------------------------
CREATE TABLE IF NOT EXISTS `exam_task_detail` (
  `id` bigint NOT NULL COMMENT '主键',
  `exam_task_id` bigint NOT NULL COMMENT '命题任务ID',
  `serial_number` int(2) NULL COMMENT '备用卷序号（1,2,3,4…）',
  `paper_type` varchar(20) DEFAULT NULL COMMENT '试卷类型，多个以”/“分隔，如A/B/C/D',
  `paper_attachment_ids` text COMMENT '转换为A3格式的试卷附件\n[\\n  	{\\n  		"type": "A",\\n  		"attachmentId": 12,\\n  		"totalPages": 3\\n  	},\\n  	{\\n  		"type": "B",\\n  		"attachmentId": 13,\\n  		"totalPages": 2\\n  	}\\n  ]',
  `make_method` varchar(45) DEFAULT NULL COMMENT '题卡绑定方式',
  `paper_confirm_attachment_ids` text COMMENT '[\n  	{\n  		"attachmentId": 12\n  	},\n  	{\n  		"attachmentId": 13\n  	}\n  ]',
  `remark` varchar(200) DEFAULT NULL COMMENT '备注',
  `exposed` TINYINT(1) NULL DEFAULT 0 COMMENT '是否曝光',
  `exposed_exam_id` BIGINT(20) NULL COMMENT '曝光的考试ID',
  `exposed_paper_type` varchar(20) DEFAULT NULL COMMENT '曝光卷型：已打印卷型，多个以”/“分隔，如A/B/C/D',
  `relate_paper_type` varchar(5) DEFAULT NULL COMMENT '已曝光卷型',
  `unexposed_paper_type` varchar(20) DEFAULT NULL COMMENT '未曝光卷型',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  `enable` tinyint DEFAULT '1' COMMENT '0-禁用，1-启用',
  PRIMARY KEY (`id`) USING BTREE,
  KEY `exam_task_detail_idx` (`exam_task_id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='命题任务明细表（入库申请）';

-- ----------------------------
-- Table structure for exam_task_paper_data
-- ----------------------------
CREATE TABLE IF NOT EXISTS `exam_task_paper_data` (
  `exam_id` bigint NOT NULL COMMENT '考试ID',
  `paper_number` varchar(100) NOT NULL COMMENT '试卷编号',
  `paper_id` bigint NOT NULL COMMENT '题库试卷ID',
  `data_packet_path` varchar(200) DEFAULT NULL COMMENT '题库试卷数据包地址（zip）',
  `paper_json` mediumtext COMMENT '试卷结构内容（从paper.json解析）',
  `answer_json` mediumtext COMMENT '试卷答案内容（从answer.json解析）',
  `paper_pdf_attachment_id` bigint DEFAULT NULL COMMENT '试卷pdf文件地址（paper.pdf上传附件ID）',
  `answer_pdf_attachment_id` bigint DEFAULT NULL COMMENT '标答pdf文件地址（answer.pdf上传附件ID）',
  `uuid` varchar(200) DEFAULT NULL COMMENT '唯一标识',
  PRIMARY KEY (`exam_id`,`paper_number`,`paper_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='命题任务选择题库试卷相关文件表';

-- ----------------------------
-- Table structure for exam_task_paper_log
-- ----------------------------
CREATE TABLE IF NOT EXISTS `exam_task_paper_log` (
  `id` bigint NOT NULL COMMENT '主键',
  `exam_task_id` bigint NOT NULL COMMENT '命题任务ID',
  `paper_type` varchar(45) DEFAULT NULL COMMENT '试卷类型',
  `paper_attachment_ids` mediumtext COMMENT '修改后试卷',
  `card_id` bigint DEFAULT NULL COMMENT '修改后题卡ID',
  `review` tinyint(1) DEFAULT NULL COMMENT '是否审核：true-审核，false-未审核',
  `review_status` varchar(45) DEFAULT NULL COMMENT '审核状态：PASS-通过，NOT_PASS-不通过',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  PRIMARY KEY (`id`) USING BTREE,
  KEY `exam_task_paper_idx` (`exam_task_id`,`review`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='卷库修改审核临时表';

-- ----------------------------
-- Table structure for exam_task_print
-- ----------------------------
CREATE TABLE IF NOT EXISTS `exam_task_print` (
  `id` bigint NOT NULL,
  `school_id` bigint DEFAULT NULL,
  `org_id` bigint DEFAULT NULL,
  `print_plan_id` bigint DEFAULT NULL COMMENT '印刷计划ID',
  `course_code` varchar(45) DEFAULT NULL COMMENT '课程代码',
  `course_name` varchar(50) DEFAULT NULL COMMENT '课程名称',
  `paper_number` varchar(100) DEFAULT NULL COMMENT '试卷编号',
  `exam_start_time` bigint DEFAULT NULL COMMENT '考试开始时间',
  `exam_end_time` bigint DEFAULT NULL COMMENT '考试结束时间',
  `package_seq` varchar(30) DEFAULT NULL COMMENT '卷袋序号',
  `exam_place` varchar(100) DEFAULT NULL,
  `exam_room` varchar(100) DEFAULT NULL,
  `invigilator_teacher` varchar(45) DEFAULT NULL COMMENT '监考老师',
  `class_id` varchar(100) DEFAULT NULL COMMENT '班级ID集合',
  `class_name` varchar(500) DEFAULT NULL COMMENT '班级名称集合',
  `student_count` int DEFAULT NULL COMMENT '考生总人数',
  `campus_id` bigint DEFAULT NULL COMMENT '校区ID',
  `print_house_id` bigint DEFAULT NULL COMMENT '印刷室ID',
  `print_house_name` varchar(100) DEFAULT NULL COMMENT '印刷室名称',
  `exam_detail_id` bigint DEFAULT NULL COMMENT '考场ID',
  `extend_fields` text COMMENT '扩展字段',
  `create_id` bigint DEFAULT NULL,
  `create_time` bigint DEFAULT NULL,
  `update_id` bigint DEFAULT NULL,
  `update_time` bigint DEFAULT NULL,
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='以试卷作为起点生成考务数据';

-- ----------------------------
-- Table structure for exam_task_review_log
-- ----------------------------
CREATE TABLE IF NOT EXISTS `exam_task_review_log` (
  `id` bigint NOT NULL COMMENT '主键',
  `exam_task_id` bigint NOT NULL COMMENT '命题任务ID',
  `review_status` varchar(8) NOT NULL COMMENT 'PASS-通过，NOT_PASS-不通过',
  `reason` mediumtext COMMENT '审核意见',
  `operate_id` bigint DEFAULT NULL COMMENT '操作人id',
  `operate_time` bigint DEFAULT NULL COMMENT '操作时间',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='命题任务审核日志表';

-- ----------------------------
-- Table structure for exam_task_sync
-- ----------------------------
CREATE TABLE IF NOT EXISTS `exam_task_sync` (
  `id` bigint NOT NULL,
  `school_id` bigint NOT NULL,
  `exam_id` bigint NOT NULL COMMENT '考试ID',
  `course_code` varchar(50) NOT NULL COMMENT '课程代码',
  `paper_number` varchar(100) NOT NULL COMMENT '试着编号',
  `paper_type` varchar(50) DEFAULT NULL COMMENT '试卷类型',
  `total_paper_type` varchar(50) DEFAULT NULL COMMENT '所有卷型',
  `org_code` varchar(45) DEFAULT NULL COMMENT '机构代码',
  `third_relate_id` int DEFAULT NULL COMMENT '云阅卷考试ID',
  `sync_status` varchar(45) DEFAULT NULL COMMENT '同步状态',
  `error_msg` varchar(500) DEFAULT NULL COMMENT '同步失败原因',
  `sync_user_id` bigint DEFAULT NULL COMMENT '同步用户ID',
  `sync_start_time` bigint DEFAULT NULL COMMENT '同步开始时间 ',
  `sync_end_time` bigint DEFAULT NULL COMMENT '同步结束时间',
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci;

-- ----------------------------
-- Table structure for exam_task_temp
-- ----------------------------
CREATE TABLE IF NOT EXISTS `exam_task_temp` (
  `id` bigint NOT NULL COMMENT '主键',
  `school_id` bigint NOT NULL COMMENT '学校id',
  `org_id` bigint DEFAULT NULL COMMENT '机构id',
  `batch_no` varchar(32) NOT NULL COMMENT '本次导入数据临时ID',
  `course_id` bigint DEFAULT NULL COMMENT '课程ID',
  `course_code` varchar(20) NOT NULL COMMENT '课程代码',
  `course_name` varchar(50) NOT NULL COMMENT '课程名称',
  `teaching_room_name` varchar(100) DEFAULT NULL COMMENT '开课学院',
  `paper_number` varchar(100) DEFAULT NULL COMMENT '试卷编号',
  `specialty` varchar(100) DEFAULT NULL COMMENT '适用专业',
  `user_account` varchar(45) DEFAULT NULL COMMENT '命题老师账号',
  `user_name` varchar(45) DEFAULT NULL COMMENT '命题老师姓名',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='命题任务临时表';

-- ----------------------------
-- Table structure for exam_task_whole
-- ----------------------------
CREATE TABLE IF NOT EXISTS `exam_task_whole` (
  `id` bigint NOT NULL,
  `semester_id` bigint DEFAULT NULL COMMENT '学期ID',
  `exam_id` bigint DEFAULT NULL COMMENT '考试ID',
  `course_id` bigint DEFAULT NULL,
  `paper_number` varchar(100) DEFAULT NULL,
  `attachment_path` varchar(200) DEFAULT NULL COMMENT '文件地址',
  `status` varchar(45) DEFAULT NULL COMMENT '状态',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci;

-- ----------------------------
-- Table structure for grade_batch
-- ----------------------------
CREATE TABLE IF NOT EXISTS `grade_batch` (
  `id` bigint NOT NULL COMMENT '分析批次主键',
  `school_id` bigint NOT NULL COMMENT '学校id',
  `org_id` bigint NOT NULL COMMENT '机构id',
  `semester_id` bigint NOT NULL COMMENT '学期id',
  `exam_id` bigint NOT NULL COMMENT '知学知考考试id',
  `batch_code` varchar(32) NOT NULL COMMENT '批次编号（在学校下唯一）',
  `batch_name` varchar(32) NOT NULL COMMENT '批次名称（2020~2021学年上学期期末考试）',
  `batch_time` bigint NOT NULL COMMENT '分析批次时间',
  `status` varchar(32) NOT NULL COMMENT '批次状态\nSETTING_GRADE_PAPER("设置分析试卷"),PUSH_GRADE_BATCH("推送教研分析批次数据"),SETTING_GRADE_PAPER_PARAM("设置分析试卷参数"),READY_TO_CALCULATE("待计算"),FINISH_CALCULATE("完成计算")',
  `third_semester_id` bigint DEFAULT NULL COMMENT '教研分析学期id',
  `third_exam_id` bigint DEFAULT NULL COMMENT '第三方考试id',
  `create_id` bigint DEFAULT NULL,
  `create_time` bigint DEFAULT NULL,
  `update_id` bigint DEFAULT NULL,
  `update_time` bigint DEFAULT NULL,
  `source` varchar(1000) DEFAULT NULL COMMENT '原始数据',
  `progress` decimal(8,2) DEFAULT NULL COMMENT '进度',
  `report_file_path` varchar(500) DEFAULT NULL COMMENT '报告路径',
  `notify_time` bigint DEFAULT NULL COMMENT '回调时间',
  `result` varchar(30) DEFAULT NULL COMMENT '数据结果，SUCCESS：成功，ERROR：失败',
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='分析-批次表';

-- ----------------------------
-- Table structure for grade_batch_paper
-- ----------------------------
CREATE TABLE IF NOT EXISTS `grade_batch_paper` (
  `id` bigint NOT NULL,
  `school_id` bigint NOT NULL COMMENT '学校id',
  `org_id` bigint NOT NULL COMMENT '机构id',
  `exam_id` bigint NOT NULL COMMENT '知学知考考试id',
  `batch_id` bigint NOT NULL COMMENT '分析批次id',
  `grade_paper_id` bigint DEFAULT NULL COMMENT '试卷id',
  `paper_number` varchar(100) NOT NULL COMMENT '试卷编号',
  `paper_type` varchar(16) NOT NULL COMMENT '试卷类型',
  `status` varchar(32) NOT NULL COMMENT '分析试卷状态\nSETTING_GRADE_PAPER("设置分析试卷"),PUSH_GRADE_BATCH("推送教研分析批次数据"),SETTING_GRADE_PAPER_PARAM("设置分析试卷参数"),READY_TO_CALCULATE("待计算"),FINISH_CALCULATE("完成计算")',
  `teach_college_id` bigint NOT NULL COMMENT '当前开课学院id',
  `teach_college_name` varchar(100) NOT NULL COMMENT '当前开课学院名称',
  `paper_name` varchar(100) DEFAULT NULL COMMENT '试卷名称',
  `enable` tinyint DEFAULT NULL COMMENT '是否可用',
  `publish_status` tinyint DEFAULT NULL COMMENT '是否发布',
  `grade_course_code` varchar(100) DEFAULT NULL COMMENT '推送教研分析后回传的教研分析课程编号（考试下唯一）',
  `create_id` bigint DEFAULT NULL,
  `create_time` bigint DEFAULT NULL,
  `update_id` bigint DEFAULT NULL,
  `update_time` bigint DEFAULT NULL,
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='分析-批次试卷关联关系表';

-- ----------------------------
-- Table structure for grade_batch_student
-- ----------------------------
CREATE TABLE IF NOT EXISTS `grade_batch_student` (
  `id` bigint NOT NULL,
  `batch_id` bigint DEFAULT NULL COMMENT '分析-批次表ID',
  `course_code` varchar(50) DEFAULT NULL COMMENT '课程编码',
  `course_name` varchar(100) DEFAULT NULL COMMENT '课程名称',
  `paper_number` varchar(100) DEFAULT NULL COMMENT '试卷编号',
  `paper_type` varchar(16) DEFAULT NULL COMMENT '试卷类型',
  `student_code` varchar(45) DEFAULT NULL COMMENT '学号',
  `student_name` varchar(100) DEFAULT NULL COMMENT '学生姓名',
  `ticket_number` varchar(45) DEFAULT NULL COMMENT '准考证号',
  `teacher_name` varchar(100) DEFAULT NULL COMMENT '任课老师',
  `teacher_number` varchar(45) DEFAULT NULL COMMENT '任课老师工号',
  `create_id` bigint DEFAULT NULL,
  `create_time` bigint DEFAULT NULL,
  `update_id` bigint DEFAULT NULL,
  `update_time` bigint DEFAULT NULL,
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='考务数据（包含任课老师）';

-- ----------------------------
-- Table structure for grade_batch_student_clazz
-- ----------------------------
CREATE TABLE IF NOT EXISTS `grade_batch_student_clazz` (
  `id` bigint NOT NULL,
  `batch_id` bigint DEFAULT NULL COMMENT '分析-批次表ID',
  `course_code` varchar(50) DEFAULT NULL COMMENT '课程编码',
  `course_name` varchar(100) DEFAULT NULL COMMENT '课程名称',
  `paper_number` varchar(100) DEFAULT NULL COMMENT '试卷编号',
  `paper_type` varchar(16) DEFAULT NULL COMMENT '试卷类型',
  `clazz_id` varchar(45) DEFAULT NULL COMMENT '班级ID',
  `clazz_name` varchar(100) DEFAULT NULL COMMENT '班级名称',
  `teacher_name` varchar(100) DEFAULT NULL COMMENT '任课老师',
  `teacher_number` varchar(45) DEFAULT NULL COMMENT '任课老师工号',
  `create_id` bigint DEFAULT NULL,
  `create_time` bigint DEFAULT NULL,
  `update_id` bigint DEFAULT NULL,
  `update_time` bigint DEFAULT NULL,
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='考务数据班级（包含任课老师）';

-- ----------------------------
-- Table structure for grade_module_define
-- ----------------------------
CREATE TABLE IF NOT EXISTS `grade_module_define` (
  `id` bigint NOT NULL COMMENT '模块熟练度主键',
  `school_id` bigint NOT NULL COMMENT '学校id',
  `exam_id` bigint NOT NULL COMMENT '考试id',
  `paper_number` varchar(100) NOT NULL COMMENT '试卷编号',
  `paper_type` varchar(16) NOT NULL COMMENT '试卷类型',
  `paper_name` varchar(100) DEFAULT NULL COMMENT '试卷名称',
  `module_type` varchar(32) NOT NULL COMMENT '维度模块类型',
  `interpret` varchar(255) DEFAULT NULL COMMENT '解释',
  `remark` varchar(255) DEFAULT NULL COMMENT '备注',
  `define` varchar(255) DEFAULT NULL COMMENT '二级熟练度定义',
  `level` varchar(32) DEFAULT NULL COMMENT '等级',
  `min` varchar(32) DEFAULT NULL COMMENT '展示用最小边界',
  `max` varchar(32) DEFAULT NULL COMMENT '展示用最大边界',
  `scope` varchar(32) DEFAULT NULL COMMENT '取值范围（计算用）',
  `create_id` bigint DEFAULT NULL,
  `create_time` bigint DEFAULT NULL,
  `update_id` bigint DEFAULT NULL,
  `update_time` bigint DEFAULT NULL,
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='分析-模块二级考察点维度熟练度配置，及模块简要描述表';

-- ----------------------------
-- Table structure for grade_module_evaluation
-- ----------------------------
CREATE TABLE IF NOT EXISTS `grade_module_evaluation` (
  `id` bigint NOT NULL,
  `school_id` bigint NOT NULL COMMENT '学校id',
  `exam_id` bigint NOT NULL COMMENT '考试id',
  `paper_number` varchar(100) NOT NULL COMMENT '试卷编号',
  `paper_type` varchar(16) NOT NULL COMMENT '试卷类型',
  `paper_name` varchar(100) DEFAULT NULL COMMENT '试卷名称',
  `module_type` varchar(64) NOT NULL COMMENT '模块类型（知识、维度、素养）',
  `formula` varchar(64) NOT NULL COMMENT '计算公式规则',
  `scope` varchar(64) NOT NULL COMMENT '取值范围（计算用）',
  `level_code` varchar(32) NOT NULL COMMENT '等级',
  `level_name` varchar(32) NOT NULL COMMENT '水平等级层次',
  `result` text COMMENT '诊断结果',
  `advice` text COMMENT '学习建议',
  `create_id` bigint DEFAULT NULL,
  `create_time` bigint DEFAULT NULL,
  `update_id` bigint DEFAULT NULL,
  `update_time` bigint DEFAULT NULL,
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='校验-考察点模块评价建议表';

-- ----------------------------
-- Table structure for grade_paper
-- ----------------------------
CREATE TABLE IF NOT EXISTS `grade_paper` (
  `id` bigint NOT NULL COMMENT '试卷id',
  `school_id` bigint NOT NULL COMMENT '学校id',
  `exam_id` bigint NOT NULL COMMENT '知学知考考试id',
  `paper_number` varchar(100) NOT NULL COMMENT '试卷编号',
  `paper_type` varchar(16) NOT NULL COMMENT '试卷类型',
  `paper_name` varchar(100) NOT NULL COMMENT '试卷名称',
  `start_time` bigint DEFAULT NULL COMMENT '该试卷考试开始时间',
  `end_time` bigint DEFAULT NULL COMMENT '该试卷考试结束时间',
  `coefficient` decimal(12,4) DEFAULT NULL COMMENT '赋分系数（赋值分=卷面成绩 +（卷面满分-卷面成绩）/ 赋分系数）',
  `total_score` decimal(12,4) NOT NULL COMMENT '试卷满分',
  `pass_score` decimal(12,4) NOT NULL COMMENT '试卷及格分',
  `score_type` varchar(64) NOT NULL COMMENT '赋分类型\\n1.NEED_ASSIGN_SCORE("是","需要且按照公式赋分")\\n2.NO_NEED_ASSIGN_SCORE("否","不需要赋分，标准分即为赋分")\\n3.SPECIAL_ASSIGN_SCORE("特殊赋分","特殊赋分，按照excel表进行赋分")',
  `create_id` bigint DEFAULT NULL,
  `create_time` bigint DEFAULT NULL,
  `update_id` bigint DEFAULT NULL,
  `update_time` bigint DEFAULT NULL,
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='试卷基础信息表（赋分系数、试卷类型、满分、及格分等）';

-- ----------------------------
-- Table structure for grade_paper_dimension
-- ----------------------------
CREATE TABLE IF NOT EXISTS `grade_paper_dimension` (
  `id` bigint NOT NULL,
  `school_id` bigint NOT NULL COMMENT '学校id',
  `exam_id` bigint NOT NULL COMMENT '考试id',
  `paper_number` varchar(100) NOT NULL COMMENT '试卷编号',
  `paper_type` varchar(16) NOT NULL COMMENT '试卷类型',
  `paper_name` varchar(100) NOT NULL COMMENT '试卷名称',
  `dimension_type` varchar(32) NOT NULL COMMENT '维度类型 - 知识，能力，素养',
  `code_primary` varchar(32) NOT NULL COMMENT '一级维度编号',
  `name_primary` varchar(255) NOT NULL COMMENT '一级维度名称',
  `code_second` varchar(32) DEFAULT NULL COMMENT '二级维度编号',
  `name_second` varchar(255) DEFAULT NULL COMMENT '二级维度名称',
  `interpretation` varchar(255) DEFAULT NULL COMMENT '一级维度术语解释',
  `create_id` bigint DEFAULT NULL,
  `create_time` bigint DEFAULT NULL,
  `update_id` bigint DEFAULT NULL,
  `update_time` bigint DEFAULT NULL,
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='分析-考察点维度表';

-- ----------------------------
-- Table structure for grade_paper_struct
-- ----------------------------
CREATE TABLE IF NOT EXISTS `grade_paper_struct` (
  `id` bigint NOT NULL,
  `school_id` bigint NOT NULL COMMENT '学校id',
  `exam_id` bigint NOT NULL COMMENT '知学知考考试id',
  `paper_number` varchar(100) NOT NULL COMMENT '试卷编号',
  `paper_type` varchar(16) NOT NULL COMMENT '试卷类型',
  `paper_name` varchar(100) NOT NULL COMMENT '试卷名称',
  `question_name` varchar(200) NOT NULL COMMENT '大题名称',
  `number_type` varchar(32) NOT NULL COMMENT '题号类型（客观题、主观题）',
  `big_question_number` varchar(64) NOT NULL COMMENT '大题号',
  `small_question_number` varchar(64) NOT NULL COMMENT '小题号(每一课程试卷的唯一标识)',
  `big_topic_name` varchar(64) NOT NULL COMMENT '大题名称',
  `full_score` decimal(12,4) NOT NULL COMMENT '满分',
  `score_rules` varchar(32) DEFAULT NULL COMMENT '计分规则',
  `rules_desc` varchar(500) DEFAULT NULL COMMENT '规则说明',
  `knowledge_dimension` varchar(255) NOT NULL COMMENT '知识维度',
  `ability_dimension` varchar(255) DEFAULT NULL COMMENT '能力维度',
  `literacy_dimension` varchar(255) DEFAULT NULL COMMENT '素养维度',
  `create_id` bigint DEFAULT NULL,
  `create_time` bigint DEFAULT NULL,
  `update_id` bigint DEFAULT NULL,
  `update_time` bigint DEFAULT NULL,
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='分析-试卷结构（蓝图）表';

-- ----------------------------
-- Table structure for mark_arbitrate_history
-- ----------------------------
CREATE TABLE IF NOT EXISTS `mark_arbitrate_history` (
  `id` bigint NOT NULL COMMENT '自增主键',
  `exam_id` bigint NOT NULL COMMENT '考试ID',
  `paper_number` varchar(100) NOT NULL COMMENT '试卷编号',
  `group_number` int NOT NULL COMMENT '大题号',
  `student_code` varchar(64) NOT NULL COMMENT '准考证号',
  `secret_number` varchar(64) NOT NULL COMMENT '考生密号',
  `student_id` bigint NOT NULL COMMENT '考生ID',
  `status` varchar(16) NOT NULL COMMENT '状态',
  `update_user_id` bigint DEFAULT NULL COMMENT '处理人ID',
  `total_score` double DEFAULT NULL COMMENT '总分',
  `score_list` text COMMENT '给分明细',
  `create_time` bigint NOT NULL COMMENT '创建时间',
  `update_time` bigint DEFAULT NULL COMMENT '处理时间',
  PRIMARY KEY (`id`) USING BTREE,
  KEY `index1` (`exam_id`,`paper_number`,`group_number`,`status`) USING BTREE,
  KEY `index3` (`student_id`,`status`) USING BTREE,
  KEY `index2` (`update_user_id`,`status`,`update_time`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='仲裁记录表';

-- ----------------------------
-- Table structure for mark_document
-- ----------------------------
CREATE TABLE IF NOT EXISTS `mark_document` (
  `id` bigint NOT NULL,
  `exam_id` bigint NOT NULL COMMENT '考试ID',
  `course_id` bigint DEFAULT NULL COMMENT '课程ID（basic_course表ID）',
  `course_code` varchar(32) DEFAULT NULL COMMENT '课程代码',
  `course_name` varchar(100) DEFAULT NULL COMMENT '科目名称',
  `paper_number` varchar(100) DEFAULT NULL COMMENT '试卷编号',
  `type` varchar(64) NOT NULL COMMENT '类型',
  `name` varchar(64) DEFAULT NULL COMMENT '名称',
  `file_count` int DEFAULT NULL COMMENT '文件数量',
  `file_path` mediumtext COMMENT '文件地址',
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='归档文档表';

-- ----------------------------
-- Table structure for mark_group
-- ----------------------------
CREATE TABLE IF NOT EXISTS `mark_group` (
  `exam_id` bigint NOT NULL COMMENT '考试ID',
  `course_id` bigint DEFAULT NULL COMMENT '课程ID（basic_course表ID）',
  `course_code` varchar(100) DEFAULT NULL COMMENT '课程代码',
  `course_name` varchar(100) DEFAULT NULL COMMENT '课程名称',
  `paper_number` varchar(100) NOT NULL COMMENT '试卷编号',
  `number` int NOT NULL COMMENT '序号',
  `pic_list` text COMMENT '小图配置',
  `total_score` double NOT NULL COMMENT '满分',
  `double_enable` tinyint(1) DEFAULT '0' COMMENT '是否开启双评',
  `double_rate` double DEFAULT NULL COMMENT '双评比例',
  `arbitrate_threshold` double DEFAULT NULL COMMENT '仲裁阈值',
  `score_policy` varchar(16) DEFAULT NULL COMMENT '合分策略',
  `task_count` int NOT NULL COMMENT '任务总量',
  `marked_count` int NOT NULL COMMENT '已评数量',
  `left_count` int DEFAULT NULL COMMENT '剩余数量',
  PRIMARY KEY (`exam_id`,`paper_number`,`number`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='评卷分组表';

-- ----------------------------
-- Table structure for mark_group_student
-- ----------------------------
CREATE TABLE IF NOT EXISTS `mark_group_student` (
  `student_id` bigint NOT NULL COMMENT '考生ID',
  `group_number` int NOT NULL COMMENT '分组ID',
  `exam_id` bigint NOT NULL COMMENT '考试ID',
  `paper_number` varchar(100) NOT NULL COMMENT '科目代码',
  `status` varchar(16) NOT NULL COMMENT '状态',
  PRIMARY KEY (`student_id`,`group_number`) USING BTREE,
  KEY `index1` (`exam_id`,`paper_number`,`group_number`,`status`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='考生分组状态表';

-- ----------------------------
-- Table structure for mark_header_subjective_score
-- ----------------------------
CREATE TABLE IF NOT EXISTS `mark_header_subjective_score` (
  `student_id` bigint NOT NULL COMMENT '考生ID',
  `main_number` int NOT NULL COMMENT '大题号',
  `sub_number` varchar(32) NOT NULL COMMENT '小题号',
  `exam_id` bigint NOT NULL COMMENT '考试ID',
  `paper_number` varchar(100) NOT NULL COMMENT '课程代码',
  `group_number` int NOT NULL COMMENT '分组序号',
  `group_score` double NOT NULL COMMENT '分组得分',
  `main_score` double NOT NULL COMMENT '大题得分',
  `score` double NOT NULL COMMENT '小题得分',
  `uncalculate` tinyint NOT NULL COMMENT '是否合分',
  `user_id` bigint DEFAULT NULL COMMENT '科组长ID',
  PRIMARY KEY (`student_id`,`main_number`,`sub_number`) USING BTREE,
  KEY `index1` (`exam_id`,`paper_number`,`main_number`,`main_score`) USING BTREE,
  KEY `index11` (`exam_id`,`paper_number`,`main_number`,`main_score`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='主观题得分明细表';

-- ----------------------------
-- Table structure for mark_header_tag
-- ----------------------------
CREATE TABLE IF NOT EXISTS `mark_header_tag` (
  `id` bigint NOT NULL COMMENT '主键',
  `exam_id` bigint NOT NULL COMMENT '考试ID',
  `paper_number` varchar(100) NOT NULL COMMENT '试卷编号',
  `student_id` bigint NOT NULL COMMENT '考生ID',
  `group_number` int NOT NULL COMMENT '大题题号',
  `user_id` bigint NOT NULL COMMENT '用户ID',
  `tag_name` varchar(64) NOT NULL COMMENT '标记内容',
  `position_x` double NOT NULL COMMENT 'X轴位置',
  `position_y` double NOT NULL COMMENT 'Y轴位置',
  `offset_index` int NOT NULL COMMENT '裁切图序号',
  `offset_x` int NOT NULL COMMENT '裁切图X轴坐标',
  `offset_y` int NOT NULL COMMENT '裁切图Y轴坐标',
  `tag_type` varchar(64) DEFAULT NULL COMMENT '标记类型',
  PRIMARY KEY (`id`) USING BTREE,
  KEY `index1` (`student_id`,`group_number`) USING BTREE,
  KEY `index2` (`user_id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='组长特殊标记表';

-- ----------------------------
-- Table structure for mark_header_track
-- ----------------------------
CREATE TABLE IF NOT EXISTS `mark_header_track` (
  `student_id` bigint NOT NULL COMMENT '考生ID',
  `question_number` varchar(128) NOT NULL COMMENT '完整题号',
  `number` int NOT NULL COMMENT '序号',
  `exam_id` bigint NOT NULL COMMENT '考试ID',
  `paper_number` varchar(100) NOT NULL COMMENT '科目代码',
  `group_number` int NOT NULL COMMENT '大题题号',
  `user_id` bigint NOT NULL COMMENT '用户ID',
  `score` double NOT NULL COMMENT '给分',
  `position_x` double NOT NULL COMMENT 'X轴位置',
  `position_y` double NOT NULL COMMENT 'Y轴位置',
  `offset_index` int NOT NULL COMMENT '裁切图序号',
  `offset_x` int NOT NULL COMMENT '裁切图X轴坐标',
  `offset_y` int NOT NULL COMMENT '裁切图Y轴坐标',
  PRIMARY KEY (`student_id`,`question_number`,`number`) USING BTREE,
  KEY `index1` (`student_id`,`group_number`) USING BTREE,
  KEY `index2` (`user_id`) USING BTREE,
  KEY `index3` (`exam_id`,`paper_number`,`group_number`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='组长轨迹给分表';

-- ----------------------------
-- Table structure for mark_paper
-- ----------------------------
CREATE TABLE IF NOT EXISTS `mark_paper` (
  `id` bigint NOT NULL,
  `exam_id` bigint NOT NULL COMMENT '考试ID',
  `course_id` bigint DEFAULT NULL COMMENT '课程ID（basic_course表ID）',
  `course_code` varchar(32) DEFAULT NULL COMMENT '课程代码',
  `course_name` varchar(100) DEFAULT NULL COMMENT '科目名称',
  `paper_number` varchar(100) DEFAULT NULL COMMENT '试卷编号',
  `course_paper_id` varchar(32) DEFAULT NULL COMMENT '试卷编号（交互）',
  `objective_score` double NOT NULL COMMENT '客观题满分',
  `subjective_score` double NOT NULL COMMENT '主观题满分',
  `total_score` double NOT NULL COMMENT '全卷满分',
  `student_count` int DEFAULT NULL COMMENT '考生人数',
  `upload_count` int NOT NULL COMMENT '已上传人数',
  `absent_count` int DEFAULT NULL COMMENT '缺考人数',
  `remark` varchar(128) DEFAULT NULL COMMENT '备注',
  `sheet_config` text COMMENT '原图遮盖配置',
  `pass_score` double DEFAULT NULL COMMENT '及格分',
  `excellent_score` double DEFAULT NULL COMMENT '优秀分',
  `card_type` varchar(16) DEFAULT NULL COMMENT '题卡类型',
  `paper_file_path` varchar(500) DEFAULT NULL COMMENT '试卷文件类型',
  `answer_file_path` varchar(500) DEFAULT NULL COMMENT '标答文件类型',
  `auto_scroll` tinyint(1) DEFAULT NULL COMMENT '评卷是否跳转',
  `mark_mode` varchar(16) DEFAULT NULL COMMENT '评卷模式',
  `force_mode` tinyint(1) DEFAULT '1' COMMENT '强制评卷模式（不限为false）',
  `mark_start_time` bigint DEFAULT NULL COMMENT '评卷开始时间',
  `mark_end_time` bigint DEFAULT NULL COMMENT '评卷结束时间',
  `sheet_view` tinyint DEFAULT NULL COMMENT '是否原卷显示',
  `show_object_score` tinyint DEFAULT NULL COMMENT '评卷是否显示客观分',
  `question_status` tinyint(1) DEFAULT '0' COMMENT '试卷结构是否提交',
  `group_status` tinyint DEFAULT '0' COMMENT '评卷分组是否提交',
  `open_mark_class` tinyint DEFAULT '0' COMMENT '是否开启分班阅',
  `status` varchar(10) DEFAULT NULL COMMENT '评卷状态',
  `package_code` mediumtext COMMENT '签到表编号',
  `user_id` bigint DEFAULT NULL COMMENT '命题老师ID',
  `serial_number` INT(1) NULL COMMENT '备用卷',
  `paper_type` varchar(10) DEFAULT NULL COMMENT '试卷类型',
  `open_double_marking` bigint DEFAULT '0' COMMENT '是否开启双评：true-开启，false-关闭',
  PRIMARY KEY (`id`) USING BTREE,
    KEY `index2` (`exam_id`,`course_paper_id`,`status`),
    KEY `index1` (`exam_id`,`paper_number`,`paper_type`,`status`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='考试科目表';

-- ----------------------------
-- Table structure for mark_paper_package
-- ----------------------------
CREATE TABLE IF NOT EXISTS `mark_paper_package` (
  `exam_id` bigint NOT NULL COMMENT '考试ID',
  `course_id` bigint DEFAULT NULL COMMENT '课程ID（basic_course表ID）',
  `course_code` varchar(32) DEFAULT NULL COMMENT '课程代码',
  `course_name` varchar(100) DEFAULT NULL COMMENT '科目名称',
  `paper_number` varchar(100) DEFAULT NULL COMMENT '试卷编号',
  `course_paper_id` varchar(50) NOT NULL COMMENT '课程试卷号',
  `serial_number` INT(1) NULL COMMENT '备用卷序号',
  `paper_type` varchar(10) NOT NULL COMMENT '试卷类型',
  `package_code` varchar(10) NOT NULL COMMENT '卷袋号',
  PRIMARY KEY (`exam_id`,`course_paper_id`,`paper_type`,`package_code`),
  KEY `index1` (`exam_id`,`paper_number`) USING BTREE,
  KEY `index2` (`exam_id`,`course_paper_id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci;

-- ----------------------------
-- Table structure for mark_problem_history
-- ----------------------------
CREATE TABLE IF NOT EXISTS `mark_problem_history` (
  `id` bigint NOT NULL COMMENT '自增主键',
  `exam_id` bigint NOT NULL COMMENT '考试ID',
  `paper_number` varchar(100) NOT NULL COMMENT '试卷编号',
  `group_number` int NOT NULL COMMENT '大题号',
  `student_code` varchar(64) NOT NULL COMMENT '学号',
  `secret_number` varchar(64) NOT NULL COMMENT '考生密号',
  `type` varchar(10) NOT NULL COMMENT '问题类型',
  `student_id` bigint NOT NULL COMMENT '考生ID',
  `task_id` bigint NOT NULL COMMENT '评卷任务ID',
  `status` varchar(16) DEFAULT NULL COMMENT '状态',
  `update_user_id` bigint DEFAULT NULL COMMENT '处理人ID',
  `user_id` bigint DEFAULT NULL COMMENT '评卷员ID',
  `score_list` text COMMENT '给分明细',
  `create_time` bigint NOT NULL COMMENT '创建时间',
  `update_time` bigint DEFAULT NULL COMMENT '处理时间',
  `remark` varchar(100) DEFAULT NULL COMMENT '问题原因',
  PRIMARY KEY (`id`) USING BTREE,
  KEY `index2` (`update_user_id`,`status`,`update_time`) USING BTREE,
  KEY `index3` (`student_id`,`status`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='问题卷历史表';

-- ----------------------------
-- Table structure for mark_question
-- ----------------------------
CREATE TABLE IF NOT EXISTS `mark_question` (
  `id` bigint NOT NULL COMMENT '主键',
  `exam_id` bigint NOT NULL COMMENT '考试ID',
  `paper_number` varchar(100) NOT NULL COMMENT '试卷类型',
  `paper_type` varchar(100) DEFAULT NULL COMMENT '卷型',
  `objective` tinyint NOT NULL COMMENT '是否客观题',
  `group_number` int DEFAULT NULL COMMENT '分组序号',
  `main_number` int NOT NULL COMMENT '大题号',
  `sub_number` int NOT NULL COMMENT '小题号',
  `main_title` varchar(128) NOT NULL COMMENT '大题名称',
  `answer` varchar(16) DEFAULT NULL COMMENT '正确答案',
  `total_score` double DEFAULT NULL COMMENT '满分',
  `option_count` int DEFAULT NULL COMMENT '客观题答案数量',
  `interval_score` double DEFAULT NULL COMMENT '评卷间隔分',
  `objective_policy` varchar(16) DEFAULT NULL COMMENT '客观题判分策略',
  `question_type` varchar(2) DEFAULT NULL COMMENT '题型',
  `name` varchar(128) DEFAULT NULL,
  `paper_index` int DEFAULT NULL,
  `page_index` int DEFAULT NULL,
  `objective_policy_score` double DEFAULT NULL COMMENT '客观题判分策略计分',
  `create_id` bigint DEFAULT NULL,
  `create_time` bigint DEFAULT NULL,
  `update_id` bigint DEFAULT NULL,
  `update_time` bigint DEFAULT NULL,
  PRIMARY KEY (`id`) USING BTREE,
    INDEX `index_1`(`exam_id` ASC, `paper_number` ASC, `main_number` ASC, `sub_number` ASC) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='小题信息表';

-- ----------------------------
-- Table structure for mark_question_answer
-- ----------------------------
CREATE TABLE IF NOT EXISTS `mark_question_answer` (
    `id` BIGINT(20) NOT NULL,
    `exam_id` BIGINT(20) NULL COMMENT '考试ID',
    `paper_number` VARCHAR(100) NULL COMMENT '试卷编号',
    `paper_type` VARCHAR(10) NULL COMMENT '卷型',
    `main_number` INT NULL COMMENT '大题号',
    `sub_number` INT NULL COMMENT '小题号',
    `answer` VARCHAR(16) NULL COMMENT '客观题标答',
    `objective_policy` VARCHAR(16) NULL COMMENT '判分策略',
    `objective_policy_score` DOUBLE NULL COMMENT '客观题判分策略计分（漏选给分）',
    `create_id` BIGINT(20) NULL,
    `create_time` BIGINT(20) NULL,
    `update_id` BIGINT(20) NULL,
    `update_time` BIGINT(20) NULL,
    PRIMARY KEY (`id`))
    COMMENT = '客观题信息';

-- ----------------------------
-- Table structure for mark_reject_history
-- ----------------------------
CREATE TABLE IF NOT EXISTS `mark_reject_history`(
          `id` bigint NOT NULL,
          `exam_id` bigint NULL DEFAULT NULL COMMENT '考试ID',
          `paper_number` varchar(100) NULL DEFAULT NULL COMMENT '试卷编号',
          `group_number` int NULL DEFAULT NULL COMMENT '分组号',
          `task_id` bigint NULL DEFAULT NULL COMMENT '任务ID',
          `student_id` bigint NULL DEFAULT NULL COMMENT '考生ID',
          `basic_student_id` bigint NULL DEFAULT NULL COMMENT 'basic_exam_student表id',
          `student_code` varchar(255) NULL DEFAULT NULL COMMENT '学号',
          `secret_number` varchar(255) NULL DEFAULT NULL COMMENT '任务密号',
          `user_id` bigint NULL DEFAULT NULL COMMENT '评卷用户ID',
          `reject_reason` varchar(255) NULL DEFAULT NULL COMMENT '打回原因',
          `reject_score_list` varchar(255) NULL DEFAULT NULL COMMENT '打回后分数列表',
          `marker_score_list` varchar(255) NULL DEFAULT NULL COMMENT '打回前分数列表',
          `marker_score` double NULL DEFAULT NULL COMMENT '总分',
          `marker_time` bigint NULL DEFAULT NULL COMMENT '评卷时间',
          `reject_user_id` bigint NULL DEFAULT NULL COMMENT '打回用户ID',
          `reject_time` bigint NOT NULL COMMENT '打回时间',
          PRIMARY KEY (`id`) USING BTREE,
          INDEX `index2`(`task_id` ASC) USING BTREE,
          INDEX `index1`(`exam_id` ASC, `paper_number` ASC, `group_number` ASC) USING BTREE
) ENGINE = InnoDB AUTO_INCREMENT = 1 CHARACTER SET = utf8mb4 COLLATE = utf8mb4_general_ci ROW_FORMAT = DYNAMIC;

-- ----------------------------
-- Table structure for mark_special_tag
-- ----------------------------
CREATE TABLE IF NOT EXISTS `mark_special_tag` (
  `id` bigint NOT NULL COMMENT '主键',
  `task_id` bigint NOT NULL COMMENT '评卷任务ID',
  `tag_name` varchar(64) NOT NULL COMMENT '标记内容',
  `position_x` double NOT NULL COMMENT 'X轴位置',
  `position_y` double NOT NULL COMMENT 'Y轴位置',
  `offset_index` int NOT NULL COMMENT '裁切图序号',
  `offset_x` int NOT NULL COMMENT '裁切图X轴坐标',
  `offset_y` int NOT NULL COMMENT '裁切图Y轴坐标',
  `tag_type` varchar(64) DEFAULT NULL COMMENT '标记类型',
  PRIMARY KEY (`id`) USING BTREE,
  KEY `index1` (`task_id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='特殊标记表';

-- ----------------------------
-- Table structure for mark_student
-- ----------------------------
CREATE TABLE IF NOT EXISTS `mark_student` (
  `id` bigint NOT NULL COMMENT '主键(和exam_student表id一样)',
  `exam_id` bigint NOT NULL COMMENT '考试ID',
  `basic_student_id` bigint DEFAULT NULL COMMENT '考生ID（basic_exam_student表ID）',
  `course_id` bigint DEFAULT NULL COMMENT '课程ID（basic_course表ID）',
  `paper_number` varchar(100) DEFAULT NULL COMMENT '试卷类型',
  `course_paper_id` varchar(45) DEFAULT NULL COMMENT '试卷编号（交互）',
  `serial_number` INT(2) NULL COMMENT '备用卷序号',
  `paper_type` varchar(10) DEFAULT NULL COMMENT '试卷编号',
  `secret_number` varchar(64) NOT NULL COMMENT '密号',
  `student_code` varchar(64) DEFAULT NULL COMMENT '学号',
  `package_code` varchar(64) DEFAULT NULL COMMENT '试卷袋编号',
  `exam_place` varchar(32) DEFAULT NULL COMMENT '校区',
  `exam_room` varchar(32) DEFAULT NULL COMMENT '考场',
  `remark` varchar(128) DEFAULT NULL COMMENT '备注',
  `batch_code` varchar(32) DEFAULT NULL COMMENT '扫描批次号',
  `sheet_count` int NOT NULL COMMENT '原图数量',
  `sheet_path` mediumtext COMMENT '考生所有图片地址',
  `answers` text COMMENT '客观题识别结果',
  `is_upload` tinyint NOT NULL COMMENT '是否已上传',
  `is_absent` tinyint NOT NULL COMMENT '是否缺考',
  `is_manual_absent` tinyint NOT NULL COMMENT '是否人工指定缺考',
  `is_breach` tinyint NOT NULL COMMENT '是否违纪',
  `upload_time` bigint DEFAULT NULL COMMENT '上传时间',
  `check_time` bigint DEFAULT NULL COMMENT '检查时间',
  `check_user_id` bigint DEFAULT NULL COMMENT '检查人ID',
  `objective_score` double DEFAULT NULL COMMENT '客观总分',
  `objective_score_list` text COMMENT '客观得分明细',
  `subjective_status` varchar(16) NOT NULL COMMENT '主观题状态',
  `subjective_score` double DEFAULT NULL COMMENT '主观总分',
  `subjective_score_list` text COMMENT '主观得分明细',
  `scan_status` varchar(20) DEFAULT NULL COMMENT '扫描状态',
  `question_filled` bit(1) DEFAULT NULL,
  `assigned` bit(1) DEFAULT NULL,
  `assign_confirmed` bit(1) DEFAULT b'0' COMMENT '人工绑定确认',
  `absent_suspect` bit(1) DEFAULT NULL,
  `incomplete` bit(1) DEFAULT NULL,
  `breach_code` varchar(32) DEFAULT NULL,
  `omr_absent` bit(1) DEFAULT NULL COMMENT '识别缺考',
  `is_manual_breach` TINYINT(1) NULL DEFAULT 0 COMMENT '人工指定缺考',
  `omr_absent_checked` bit(1) DEFAULT NULL COMMENT '识别缺考检查',
  `omr_breach` TINYINT(1) NULL DEFAULT 0 COMMENT '识别违纪',
  `paper_type_check_status` varchar(32) NOT NULL DEFAULT 'NORMAL' COMMENT '卷型检查状态',
  `card_number` int DEFAULT NULL,
  `exam_start_time` bigint DEFAULT NULL COMMENT '考试开始时间',
  `exam_end_time` bigint DEFAULT NULL COMMENT '考试结束时间',
  `create_id` bigint DEFAULT NULL COMMENT '创建人ID',
  `invalid` bit(1) DEFAULT b'0' COMMENT '强制绑定',
  `miss_scan` tinyint(1) DEFAULT '0' COMMENT '是否漏扫',
  PRIMARY KEY (`id`) USING BTREE,
  UNIQUE KEY `index2` (`exam_id`,`secret_number`) USING BTREE,
  UNIQUE KEY `index1` (`exam_id`,`course_paper_id`,`student_code`),
  KEY `index3` (`exam_id`,`course_id`,`upload_time`) USING BTREE,
  KEY `index4` (`exam_id`,`scan_status`) USING BTREE,
  KEY `index5` (`basic_student_id`),
  KEY `index6` (`exam_id`,`paper_number`,`paper_type`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='考试考生库';

-- ----------------------------
-- Table structure for mark_subjective_score
-- ----------------------------
CREATE TABLE IF NOT EXISTS `mark_subjective_score` (
  `student_id` bigint NOT NULL COMMENT '考生ID',
  `main_number` int NOT NULL COMMENT '大题号',
  `sub_number` int NOT NULL COMMENT '小题号',
  `exam_id` bigint NOT NULL COMMENT '考试ID',
  `paper_number` varchar(100) NOT NULL COMMENT '课程代码',
  `group_number` int NOT NULL COMMENT '分组序号',
  `group_score` double NOT NULL COMMENT '分组得分',
  `main_score` double NOT NULL COMMENT '大题得分',
  `score` double NOT NULL COMMENT '小题得分',
  `uncalculate` tinyint(1) NOT NULL COMMENT '是否合分',
  `rejected` TINYINT(1) NULL DEFAULT 0 COMMENT '是否打回',
  PRIMARY KEY (`student_id`,`main_number`,`sub_number`) USING BTREE,
  KEY `index1` (`exam_id`,`paper_number`,`main_number`,`main_score`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='主观题得分明细表';

-- ----------------------------
-- Table structure for mark_task
-- ----------------------------
CREATE TABLE IF NOT EXISTS `mark_task` (
  `id` bigint NOT NULL COMMENT '主键',
  `exam_id` bigint NOT NULL COMMENT '考试ID',
  `course_id` bigint DEFAULT NULL COMMENT '课程ID（basic_course表ID）',
  `course_code` varchar(32) DEFAULT NULL COMMENT '课程代码',
  `course_name` varchar(100) DEFAULT NULL COMMENT '课程名称',
  `paper_number` varchar(100) NOT NULL COMMENT '试卷编号',
  `group_number` int NOT NULL COMMENT '大题号',
  `student_id` bigint NOT NULL COMMENT '考生ID',
  `basic_student_id` BIGINT(20) NULL COMMENT '考生管理ID',
  `student_code` varchar(64) NOT NULL COMMENT '学号',
  `secret_number` varchar(64) NOT NULL COMMENT '考生密号',
  `task_number` int NOT NULL COMMENT '多评任务序号',
  `status` varchar(16) NOT NULL COMMENT '状态',
  `user_id` bigint DEFAULT NULL COMMENT '评卷员ID',
  `marker_time` bigint DEFAULT NULL COMMENT '评卷时间',
  `marker_score` double DEFAULT NULL COMMENT '评卷总分',
  `marker_score_list` text COMMENT '评卷给分明细',
  `marker_spent` int DEFAULT NULL COMMENT '评卷时长',
  `header_id` bigint DEFAULT NULL COMMENT '科组长ID',
  `header_time` bigint DEFAULT NULL COMMENT '科组长评卷时间',
  `header_score` double DEFAULT NULL COMMENT '科组长总分',
  `header_score_list` text COMMENT '科组长给分明细',
  `reject_reason` VARCHAR(128) NULL COMMENT '打回原因',
  PRIMARY KEY (`id`) USING BTREE,
  UNIQUE KEY `index2` (`student_id`,`group_number`,`task_number`) USING BTREE,
  KEY `index3` (`user_id`,`status`,`marker_time`) USING BTREE,
  KEY `index1` (`exam_id`,`paper_number`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='评卷任务表';

-- ----------------------------
-- Table structure for mark_track
-- ----------------------------
CREATE TABLE IF NOT EXISTS `mark_track` (
  `id` bigint NOT NULL,
  `task_id` bigint NOT NULL COMMENT '评卷任务ID',
  `question_number` varchar(128) NOT NULL COMMENT '完整题号',
  `number` int NOT NULL COMMENT '序号',
  `student_id` bigint NOT NULL COMMENT '考生ID',
  `exam_id` bigint NOT NULL COMMENT '考试ID',
  `paper_number` varchar(100) NOT NULL COMMENT '试卷编号',
  `group_number` int NOT NULL COMMENT '大题题号',
  `user_id` bigint NOT NULL COMMENT '评卷员ID',
  `type` varchar(10) DEFAULT NULL COMMENT '标记类型（SCORE：分数，REMARK：备注）',
  `score` double DEFAULT NULL COMMENT '给分',
  `remark` varchar(20) DEFAULT NULL COMMENT '备注',
  `position_x` double NOT NULL COMMENT 'X轴位置',
  `position_y` double NOT NULL COMMENT 'Y轴位置',
  `offset_index` int NOT NULL COMMENT '裁切图序号',
  `offset_x` int NOT NULL COMMENT '裁切图X轴坐标',
  `offset_y` int NOT NULL COMMENT '裁切图Y轴坐标',
  PRIMARY KEY (`id`) USING BTREE,
  KEY `index2` (`user_id`) USING BTREE,
  KEY `index3` (`exam_id`,`paper_number`,`group_number`) USING BTREE,
  KEY `index1` (`exam_id`,`task_id`,`student_id`,`group_number`) USING BTREE,
  KEY `index4` (`task_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='轨迹给分表';

-- ----------------------------
-- Table structure for mark_user_class
-- ----------------------------
CREATE TABLE IF NOT EXISTS `mark_user_class` (
  `Id` bigint NOT NULL,
  `exam_id` bigint DEFAULT NULL COMMENT '\n考试ID',
  `paper_number` varchar(100) DEFAULT NULL COMMENT '试卷编号',
  `user_id` bigint unsigned DEFAULT NULL COMMENT '评卷员ID',
  `class_name` varchar(100) DEFAULT NULL COMMENT '班级名称',
  PRIMARY KEY (`Id`) USING BTREE,
  KEY `idx_1` (`exam_id`,`paper_number`,`class_name`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='评卷员班级';

-- ----------------------------
-- Table structure for mark_user_group
-- ----------------------------
CREATE TABLE IF NOT EXISTS `mark_user_group` (
  `id` bigint NOT NULL COMMENT '主键',
  `exam_id` bigint NOT NULL COMMENT '考试ID',
  `paper_number` varchar(100) NOT NULL COMMENT '试卷编号',
  `group_number` int NOT NULL COMMENT '分组编号',
  `user_id` bigint NOT NULL COMMENT '用户ID',
  `mode` varchar(16) DEFAULT NULL COMMENT '强制评卷模式',
  `top_count` int DEFAULT NULL COMMENT '评卷数上限',
  `finish_count` int DEFAULT NULL COMMENT '完成数量',
  `header_finish_count` int DEFAULT NULL COMMENT '管理员直接打分数量',
  `valid_count` int DEFAULT NULL COMMENT '有效数量',
  `avg_score` double DEFAULT NULL COMMENT '平均分',
  `avg_speed` double DEFAULT NULL COMMENT '平均时长',
  `stdev_score` double DEFAULT NULL COMMENT '标准差',
  `max_score` double DEFAULT NULL COMMENT '最高分',
  `min_score` double DEFAULT NULL COMMENT '最低分',
  `reject_count` INT NULL COMMENT '打回次数',
  PRIMARY KEY (`id`) USING BTREE,
  KEY `index2` (`exam_id`,`paper_number`,`group_number`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='评卷员表';

-- ----------------------------
-- Table structure for obe_course_dimension
-- ----------------------------
CREATE TABLE IF NOT EXISTS `obe_course_dimension` (
  `id` bigint NOT NULL COMMENT 'id',
  `obe_course_outline_id` bigint NOT NULL COMMENT '课程大纲id',
  `school_id` bigint NOT NULL COMMENT '学校id',
  `source` varchar(20) NOT NULL COMMENT '来源',
  `code` varchar(32) NOT NULL COMMENT '知识点编号',
  `name` varchar(255) NOT NULL COMMENT '知识点名称',
  `number` int NOT NULL COMMENT '排序',
  `parent_id` bigint NOT NULL COMMENT '父id（顶级知识点父id为0）',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  PRIMARY KEY (`id`),
  UNIQUE KEY `course_dimension_unique` (`obe_course_outline_id`,`source`,`code`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='课程知识点表';

-- ----------------------------
-- Table structure for obe_course_evaluation
-- ----------------------------
CREATE TABLE IF NOT EXISTS `obe_course_evaluation` (
  `id` bigint NOT NULL COMMENT 'id',
  `obe_course_outline_id` bigint NOT NULL COMMENT '教学课程id',
  `school_id` bigint NOT NULL COMMENT '学校id',
  `type` varchar(20) NOT NULL COMMENT '类型（默认DEFAULT、自定义CUSTOM）',
  `evaluation` varchar(255) NOT NULL COMMENT '评价方式',
  `evaluation_desc` varchar(255) DEFAULT NULL COMMENT '评价方式描述',
  `create_id` bigint DEFAULT NULL COMMENT '创建人id',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人id',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  PRIMARY KEY (`id`),
  UNIQUE KEY `course_evaluation_unique` (`obe_course_outline_id`,`evaluation`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='课程大纲评价方式表';

-- ----------------------------
-- Table structure for obe_course_outline
-- ----------------------------
CREATE TABLE IF NOT EXISTS `obe_course_outline` (
  `id` bigint NOT NULL COMMENT '课程大纲表id',
  `culture_program_id` bigint NOT NULL COMMENT '培养方案id',
  `school_id` bigint NOT NULL COMMENT '学校id',
  `name` varchar(255) NOT NULL COMMENT '课程大纲名称',
  `semester_id` bigint NOT NULL COMMENT '修读学期id',
  `course_id` bigint NOT NULL,
  `evaluation_mode` varchar(32) NOT NULL COMMENT '考核方式',
  `course_type` varchar(32) NOT NULL COMMENT '课程类别',
  `user_id` bigint NOT NULL COMMENT '课程大纲责任人',
  `weight_setting` tinyint(1) DEFAULT NULL COMMENT '权重设置',
  `weight_setting_sign` bigint DEFAULT NULL COMMENT '课程权重设置标识',
  `dimension_sign` bigint DEFAULT NULL COMMENT '知识点标识',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  PRIMARY KEY (`id`),
  UNIQUE KEY `obe_course_outline_unique` (`culture_program_id`,`course_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='课程大纲表';

-- ----------------------------
-- Table structure for obe_course_outline_audit_report
-- ----------------------------
CREATE TABLE `obe_course_outline_audit_report`  (
    `id` bigint NOT NULL,
    `course_outline_id` bigint NOT NULL,
    `weight_setting_sign` bigint NULL DEFAULT NULL COMMENT '课程权重设置标识',
    `course_basic_info` mediumtext CHARACTER SET utf8mb4 COLLATE utf8mb4_general_ci NULL COMMENT '课程基础信息',
    `course_target_info` mediumtext CHARACTER SET utf8mb4 COLLATE utf8mb4_general_ci NULL COMMENT '课程目标情况',
    `rationality_evaluation` mediumtext CHARACTER SET utf8mb4 COLLATE utf8mb4_general_ci NULL COMMENT '合理性评价',
    PRIMARY KEY (`id`) USING BTREE
) ENGINE = InnoDB CHARACTER SET = utf8mb4 COLLATE = utf8mb4_general_ci COMMENT = '课程目标达成情况评价依据合理性审核表';


-- ----------------------------
-- Table structure for obe_course_requirement_dimension
-- ----------------------------
CREATE TABLE IF NOT EXISTS `obe_course_requirement_dimension` (
  `id` bigint NOT NULL COMMENT 'id',
  `obe_course_outline_id` bigint NOT NULL COMMENT '课程大纲id',
  `obe_culture_program_requirement_id` bigint NOT NULL COMMENT '指标id',
  `obe_course_dimension_id` bigint NOT NULL COMMENT '知识点id',
  `school_id` bigint NOT NULL COMMENT '学校id',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  PRIMARY KEY (`id`),
  UNIQUE KEY `obe_course_requirement_dimension_unique` (`obe_course_outline_id`,`obe_culture_program_requirement_id`,`obe_course_dimension_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='课程大纲指标与知识点关联表';

-- ----------------------------
-- Table structure for obe_course_requirement_matrix
-- ----------------------------
CREATE TABLE IF NOT EXISTS `obe_course_requirement_matrix` (
  `id` bigint NOT NULL,
  `culture_program_id` bigint NOT NULL COMMENT '培养方案ID',
  `obe_course_id` bigint NOT NULL COMMENT '专业认证课程ID',
  `obe_requirement_id` bigint NOT NULL COMMENT '专业认证毕业要求ID',
  `obe_sub_requirement_id` bigint DEFAULT NULL COMMENT '子节点名称',
  `scale` double DEFAULT NULL COMMENT '指标值',
  `intensity` varchar(1) DEFAULT NULL COMMENT '强度',
  `create_id` bigint DEFAULT NULL COMMENT '创建人ID',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人ID',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='培养方案毕业要求支撑矩阵';

-- ----------------------------
-- Table structure for obe_course_requirement_report
-- ----------------------------
CREATE TABLE IF NOT EXISTS `obe_course_requirement_report` (
  `id` bigint NOT NULL COMMENT '主键',
  `culture_program_id` bigint NOT NULL COMMENT '培养方案id',
  `requirement_radar` mediumtext NOT NULL COMMENT '毕业要求雷达报表',
  `requirement_detail` mediumtext NOT NULL COMMENT '毕业要求详情报表',
  `requirement_degree` double NOT NULL COMMENT '毕业要求达成值',
  PRIMARY KEY (`id`),
  UNIQUE KEY `obe_course_requirement_report_unique` (`culture_program_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='课程毕业要求报表';

-- ----------------------------
-- Table structure for obe_course_target
-- ----------------------------
CREATE TABLE IF NOT EXISTS `obe_course_target` (
  `id` bigint NOT NULL COMMENT 'id',
  `obe_course_outline_id` bigint NOT NULL COMMENT '教学课程id',
  `obe_culture_program_requirement_id` bigint DEFAULT NULL COMMENT '培养方案毕业要求指标点id',
  `school_id` bigint NOT NULL COMMENT '学校id',
  `target_name` varchar(50) DEFAULT NULL COMMENT '课程目标',
  `degree_requirement` varchar(255) DEFAULT NULL COMMENT '毕业要求',
  `total_weight` decimal(6,2) DEFAULT NULL COMMENT '课程目标整体权重',
  `expect_value` double DEFAULT NULL COMMENT '预期值',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  PRIMARY KEY (`id`),
  UNIQUE KEY `course_target_unique` (`obe_course_outline_id`,`target_name`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='课程大纲目标表';

-- ----------------------------
-- Table structure for obe_course_weight
-- ----------------------------
CREATE TABLE IF NOT EXISTS `obe_course_weight` (
  `id` bigint NOT NULL COMMENT 'id',
  `obe_course_outline_id` bigint NOT NULL COMMENT '课程大纲id',
  `school_id` bigint NOT NULL COMMENT '学校id',
  `target_id` bigint NOT NULL COMMENT '课程目标表id',
  `evaluation_id` bigint NOT NULL COMMENT '课程评价表id',
  `enable` tinyint(1) NOT NULL DEFAULT '1' COMMENT '使用状态 1:使用 0:未使用',
  `weight` decimal(6,2) DEFAULT NULL COMMENT '权重',
  `target_score` decimal(6,2) DEFAULT NULL COMMENT '目标分值',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  PRIMARY KEY (`id`),
  UNIQUE KEY `course_weight_unique` (`obe_course_outline_id`,`target_id`,`evaluation_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='课程大纲权重设置表';

-- ----------------------------
-- Table structure for obe_culture_program
-- ----------------------------
CREATE TABLE IF NOT EXISTS `obe_culture_program` (
  `id` bigint NOT NULL,
  `school_id` bigint DEFAULT NULL COMMENT '学校ID',
  `professional_id` bigint DEFAULT NULL COMMENT '专业ID',
  `grade` varchar(100) DEFAULT NULL COMMENT '年级',
  `name` varchar(100) DEFAULT NULL COMMENT '培养方案名称',
  `target_count` int DEFAULT '0' COMMENT '培养目标数量',
  `requirement_count` int DEFAULT '0' COMMENT '毕业要求数量',
  `course_count` int DEFAULT '0' COMMENT '课程体系数量',
  `description` mediumtext COMMENT '总体描述',
  `expect_value` double DEFAULT NULL COMMENT '期望值',
  `create_id` bigint DEFAULT NULL COMMENT '创建人ID',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人ID',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  PRIMARY KEY (`id`),
  KEY `idx_1` (`professional_id`,`name`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='培养方案';

-- ----------------------------
-- Table structure for obe_culture_program_course
-- ----------------------------
CREATE TABLE IF NOT EXISTS `obe_culture_program_course` (
  `id` bigint NOT NULL,
  `culture_program_id` bigint NOT NULL COMMENT '培养方案ID',
  `course_id` bigint DEFAULT NULL COMMENT '课程ID',
  `course_code` varchar(45) DEFAULT NULL COMMENT '课程代码',
  `course_name` varchar(100) DEFAULT NULL COMMENT '课程名称',
  `sort_num` int DEFAULT NULL COMMENT '排序值',
  `create_id` bigint DEFAULT NULL COMMENT '创建人ID',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人ID',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  PRIMARY KEY (`id`),
  KEY `uq_professional_id` (`culture_program_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='培养方案课程体系';

-- ----------------------------
-- Table structure for obe_culture_program_requirement
-- ----------------------------
CREATE TABLE IF NOT EXISTS `obe_culture_program_requirement` (
  `id` bigint NOT NULL,
  `culture_program_id` bigint NOT NULL COMMENT '培养方案ID',
  `code` varchar(45) DEFAULT NULL COMMENT '毕业要求code',
  `name` varchar(50) NOT NULL COMMENT '毕业要求',
  `content` mediumtext COMMENT '内容',
  `parent_id` bigint DEFAULT NULL COMMENT '上级毕业要求',
  `sort_num` int DEFAULT NULL COMMENT '排序值',
  `expect_value` double DEFAULT NULL COMMENT '预期值',
  `create_id` bigint DEFAULT NULL COMMENT '创建人ID',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人ID',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
   PRIMARY KEY (`id`) USING BTREE,
   UNIQUE INDEX `obe_culture_program_requirement_unique`(`culture_program_id` ASC, `name` ASC) USING BTREE,
   INDEX `uq_professional_id`(`culture_program_id` ASC) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='培养方案毕业要求';

-- ----------------------------
-- Table structure for obe_culture_program_target
-- ----------------------------
CREATE TABLE IF NOT EXISTS `obe_culture_program_target` (
  `id` bigint NOT NULL,
  `culture_program_id` bigint DEFAULT NULL COMMENT '培养方案ID',
  `name` varchar(100) DEFAULT NULL COMMENT '培养目标名称',
  `detail` varchar(2000) DEFAULT NULL COMMENT '目标拆解详情',
  `create_id` bigint DEFAULT NULL,
  `create_time` bigint DEFAULT NULL,
  `update_id` bigint DEFAULT NULL,
  `update_time` bigint DEFAULT NULL,
  PRIMARY KEY (`id`),
  KEY `idx_1` (`culture_program_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='培养方案培养目标';

-- ----------------------------
-- Table structure for obe_requirement_target_matrix
-- ----------------------------
CREATE TABLE IF NOT EXISTS `obe_requirement_target_matrix` (
  `id` bigint NOT NULL,
  `culture_program_id` bigint DEFAULT NULL COMMENT '培养方案ID',
  `obe_requirement_id` bigint DEFAULT NULL COMMENT '毕业要求ID',
  `obe_target_id` bigint DEFAULT NULL COMMENT '培养目标ID',
  `enable` tinyint(1) DEFAULT NULL COMMENT '是否包含',
  `create_id` bigint DEFAULT NULL,
  `create_time` bigint DEFAULT NULL,
  `update_id` bigint DEFAULT NULL,
  `update_time` bigint DEFAULT NULL,
  PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='培养目标与毕业要求关系矩阵';

-- ----------------------------
-- Table structure for qrtz_blob_triggers
-- ----------------------------
CREATE TABLE IF NOT EXISTS `qrtz_blob_triggers` (
  `SCHED_NAME` varchar(120) NOT NULL,
  `TRIGGER_NAME` varchar(200) NOT NULL,
  `TRIGGER_GROUP` varchar(200) NOT NULL,
  `BLOB_DATA` blob,
  PRIMARY KEY (`SCHED_NAME`,`TRIGGER_NAME`,`TRIGGER_GROUP`) USING BTREE,
  KEY `SCHED_NAME` (`SCHED_NAME`,`TRIGGER_NAME`,`TRIGGER_GROUP`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci;

-- ----------------------------
-- Table structure for qrtz_calendars
-- ----------------------------
CREATE TABLE IF NOT EXISTS `qrtz_calendars` (
  `SCHED_NAME` varchar(120) NOT NULL,
  `CALENDAR_NAME` varchar(200) NOT NULL,
  `CALENDAR` blob NOT NULL,
  PRIMARY KEY (`SCHED_NAME`,`CALENDAR_NAME`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci;

-- ----------------------------
-- Table structure for qrtz_cron_triggers
-- ----------------------------
CREATE TABLE IF NOT EXISTS `qrtz_cron_triggers` (
  `SCHED_NAME` varchar(120) NOT NULL,
  `TRIGGER_NAME` varchar(200) NOT NULL,
  `TRIGGER_GROUP` varchar(200) NOT NULL,
  `CRON_EXPRESSION` varchar(120) NOT NULL,
  `TIME_ZONE_ID` varchar(80) DEFAULT NULL,
  PRIMARY KEY (`SCHED_NAME`,`TRIGGER_NAME`,`TRIGGER_GROUP`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci;

-- ----------------------------
-- Table structure for qrtz_fired_triggers
-- ----------------------------
CREATE TABLE IF NOT EXISTS `qrtz_fired_triggers` (
  `SCHED_NAME` varchar(120) NOT NULL,
  `ENTRY_ID` varchar(95) NOT NULL,
  `TRIGGER_NAME` varchar(200) NOT NULL,
  `TRIGGER_GROUP` varchar(200) NOT NULL,
  `INSTANCE_NAME` varchar(200) NOT NULL,
  `FIRED_TIME` bigint NOT NULL,
  `SCHED_TIME` bigint NOT NULL,
  `PRIORITY` int NOT NULL,
  `STATE` varchar(16) NOT NULL,
  `JOB_NAME` varchar(200) DEFAULT NULL,
  `JOB_GROUP` varchar(200) DEFAULT NULL,
  `IS_NONCONCURRENT` varchar(1) DEFAULT NULL,
  `REQUESTS_RECOVERY` varchar(1) DEFAULT NULL,
  PRIMARY KEY (`SCHED_NAME`,`ENTRY_ID`) USING BTREE,
  KEY `IDX_QRTZ_FT_TRIG_INST_NAME` (`SCHED_NAME`,`INSTANCE_NAME`) USING BTREE,
  KEY `IDX_QRTZ_FT_INST_JOB_REQ_RCVRY` (`SCHED_NAME`,`INSTANCE_NAME`,`REQUESTS_RECOVERY`) USING BTREE,
  KEY `IDX_QRTZ_FT_J_G` (`SCHED_NAME`,`JOB_NAME`,`JOB_GROUP`) USING BTREE,
  KEY `IDX_QRTZ_FT_JG` (`SCHED_NAME`,`JOB_GROUP`) USING BTREE,
  KEY `IDX_QRTZ_FT_T_G` (`SCHED_NAME`,`TRIGGER_NAME`,`TRIGGER_GROUP`) USING BTREE,
  KEY `IDX_QRTZ_FT_TG` (`SCHED_NAME`,`TRIGGER_GROUP`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci;

-- ----------------------------
-- Table structure for qrtz_job_details
-- ----------------------------
CREATE TABLE IF NOT EXISTS `qrtz_job_details` (
  `SCHED_NAME` varchar(120) NOT NULL,
  `JOB_NAME` varchar(200) NOT NULL,
  `JOB_GROUP` varchar(200) NOT NULL,
  `DESCRIPTION` varchar(250) DEFAULT NULL,
  `JOB_CLASS_NAME` varchar(250) NOT NULL,
  `IS_DURABLE` varchar(1) NOT NULL,
  `IS_NONCONCURRENT` varchar(1) NOT NULL,
  `IS_UPDATE_DATA` varchar(1) NOT NULL,
  `REQUESTS_RECOVERY` varchar(1) NOT NULL,
  `JOB_DATA` blob,
  PRIMARY KEY (`SCHED_NAME`,`JOB_NAME`,`JOB_GROUP`) USING BTREE,
  KEY `IDX_QRTZ_J_REQ_RECOVERY` (`SCHED_NAME`,`REQUESTS_RECOVERY`) USING BTREE,
  KEY `IDX_QRTZ_J_GRP` (`SCHED_NAME`,`JOB_GROUP`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci;

-- ----------------------------
-- Table structure for qrtz_locks
-- ----------------------------
CREATE TABLE IF NOT EXISTS `qrtz_locks` (
  `SCHED_NAME` varchar(120) NOT NULL,
  `LOCK_NAME` varchar(40) NOT NULL,
  PRIMARY KEY (`SCHED_NAME`,`LOCK_NAME`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci;

-- ----------------------------
-- Table structure for qrtz_paused_trigger_grps
-- ----------------------------
CREATE TABLE IF NOT EXISTS `qrtz_paused_trigger_grps` (
  `SCHED_NAME` varchar(120) NOT NULL,
  `TRIGGER_GROUP` varchar(200) NOT NULL,
  PRIMARY KEY (`SCHED_NAME`,`TRIGGER_GROUP`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci;

-- ----------------------------
-- Table structure for qrtz_scheduler_state
-- ----------------------------
CREATE TABLE IF NOT EXISTS `qrtz_scheduler_state` (
  `SCHED_NAME` varchar(120) NOT NULL,
  `INSTANCE_NAME` varchar(200) NOT NULL,
  `LAST_CHECKIN_TIME` bigint NOT NULL,
  `CHECKIN_INTERVAL` bigint NOT NULL,
  PRIMARY KEY (`SCHED_NAME`,`INSTANCE_NAME`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci;

-- ----------------------------
-- Table structure for qrtz_simple_triggers
-- ----------------------------
CREATE TABLE IF NOT EXISTS `qrtz_simple_triggers` (
  `SCHED_NAME` varchar(120) NOT NULL,
  `TRIGGER_NAME` varchar(200) NOT NULL,
  `TRIGGER_GROUP` varchar(200) NOT NULL,
  `REPEAT_COUNT` bigint NOT NULL,
  `REPEAT_INTERVAL` bigint NOT NULL,
  `TIMES_TRIGGERED` bigint NOT NULL,
  PRIMARY KEY (`SCHED_NAME`,`TRIGGER_NAME`,`TRIGGER_GROUP`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci;

-- ----------------------------
-- Table structure for qrtz_simprop_triggers
-- ----------------------------
CREATE TABLE IF NOT EXISTS `qrtz_simprop_triggers` (
  `SCHED_NAME` varchar(120) NOT NULL,
  `TRIGGER_NAME` varchar(200) NOT NULL,
  `TRIGGER_GROUP` varchar(200) NOT NULL,
  `STR_PROP_1` varchar(512) DEFAULT NULL,
  `STR_PROP_2` varchar(512) DEFAULT NULL,
  `STR_PROP_3` varchar(512) DEFAULT NULL,
  `INT_PROP_1` int DEFAULT NULL,
  `INT_PROP_2` int DEFAULT NULL,
  `LONG_PROP_1` bigint DEFAULT NULL,
  `LONG_PROP_2` bigint DEFAULT NULL,
  `DEC_PROP_1` decimal(13,4) DEFAULT NULL,
  `DEC_PROP_2` decimal(13,4) DEFAULT NULL,
  `BOOL_PROP_1` varchar(1) DEFAULT NULL,
  `BOOL_PROP_2` varchar(1) DEFAULT NULL,
  PRIMARY KEY (`SCHED_NAME`,`TRIGGER_NAME`,`TRIGGER_GROUP`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci;

-- ----------------------------
-- Table structure for qrtz_triggers
-- ----------------------------
CREATE TABLE IF NOT EXISTS `qrtz_triggers` (
  `SCHED_NAME` varchar(120) NOT NULL,
  `TRIGGER_NAME` varchar(200) NOT NULL,
  `TRIGGER_GROUP` varchar(200) NOT NULL,
  `JOB_NAME` varchar(200) NOT NULL,
  `JOB_GROUP` varchar(200) NOT NULL,
  `DESCRIPTION` varchar(250) DEFAULT NULL,
  `NEXT_FIRE_TIME` bigint DEFAULT NULL,
  `PREV_FIRE_TIME` bigint DEFAULT NULL,
  `PRIORITY` int DEFAULT NULL,
  `TRIGGER_STATE` varchar(16) NOT NULL,
  `TRIGGER_TYPE` varchar(8) NOT NULL,
  `START_TIME` bigint NOT NULL,
  `END_TIME` bigint DEFAULT NULL,
  `CALENDAR_NAME` varchar(200) DEFAULT NULL,
  `MISFIRE_INSTR` smallint DEFAULT NULL,
  `JOB_DATA` blob,
  PRIMARY KEY (`SCHED_NAME`,`TRIGGER_NAME`,`TRIGGER_GROUP`) USING BTREE,
  KEY `IDX_QRTZ_T_J` (`SCHED_NAME`,`JOB_NAME`,`JOB_GROUP`) USING BTREE,
  KEY `IDX_QRTZ_T_JG` (`SCHED_NAME`,`JOB_GROUP`) USING BTREE,
  KEY `IDX_QRTZ_T_C` (`SCHED_NAME`,`CALENDAR_NAME`) USING BTREE,
  KEY `IDX_QRTZ_T_G` (`SCHED_NAME`,`TRIGGER_GROUP`) USING BTREE,
  KEY `IDX_QRTZ_T_STATE` (`SCHED_NAME`,`TRIGGER_STATE`) USING BTREE,
  KEY `IDX_QRTZ_T_N_STATE` (`SCHED_NAME`,`TRIGGER_NAME`,`TRIGGER_GROUP`,`TRIGGER_STATE`) USING BTREE,
  KEY `IDX_QRTZ_T_N_G_STATE` (`SCHED_NAME`,`TRIGGER_GROUP`,`TRIGGER_STATE`) USING BTREE,
  KEY `IDX_QRTZ_T_NEXT_FIRE_TIME` (`SCHED_NAME`,`NEXT_FIRE_TIME`) USING BTREE,
  KEY `IDX_QRTZ_T_NFT_ST` (`SCHED_NAME`,`TRIGGER_STATE`,`NEXT_FIRE_TIME`) USING BTREE,
  KEY `IDX_QRTZ_T_NFT_MISFIRE` (`SCHED_NAME`,`MISFIRE_INSTR`,`NEXT_FIRE_TIME`) USING BTREE,
  KEY `IDX_QRTZ_T_NFT_ST_MISFIRE` (`SCHED_NAME`,`MISFIRE_INSTR`,`NEXT_FIRE_TIME`,`TRIGGER_STATE`) USING BTREE,
  KEY `IDX_QRTZ_T_NFT_ST_MISFIRE_GRP` (`SCHED_NAME`,`MISFIRE_INSTR`,`NEXT_FIRE_TIME`,`TRIGGER_GROUP`,`TRIGGER_STATE`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci;

-- ----------------------------
-- Table structure for scan_answer_card
-- ----------------------------
CREATE TABLE IF NOT EXISTS `scan_answer_card` (
  `id` bigint NOT NULL,
  `exam_id` bigint NOT NULL COMMENT '考试ID',
  `paper_number` varchar(100) DEFAULT NULL COMMENT '科目代码',
  `course_paper_id` varchar(50) NOT NULL COMMENT '试卷编号（交互）',
  `serial_number` INT(2) NULL COMMENT '备用卷序号',
  `paper_type` varchar(100) NOT NULL COMMENT '试卷编号',
  `card_id` bigint DEFAULT NULL COMMENT '知学知考题卡ID（自定义卡格式为空）',
  `number` int NOT NULL COMMENT '序号',
  `source` varchar(16) NOT NULL COMMENT '来源',
  `uri` varchar(500) DEFAULT NULL COMMENT '卡格式文件路径',
  `md5` varchar(255) DEFAULT NULL COMMENT '文件md5',
  `parameter` varchar(255) DEFAULT NULL COMMENT '其他属性',
  `remark` varchar(255) DEFAULT NULL COMMENT '备注',
  `update_time` bigint DEFAULT NULL COMMENT '修改时间',
  `need_adapte` tinyint DEFAULT NULL COMMENT '是否需要适配',
  `paper_count` int DEFAULT NULL,
  `single_page` tinyint(1) DEFAULT NULL,
  `slice_config` text COMMENT '裁切配置',
  `dpi` int DEFAULT NULL COMMENT 'dpi',
  `code` varchar(64) DEFAULT NULL COMMENT '混扫代码',
  `adapte_md5` varchar(32) DEFAULT NULL COMMENT '适配后文件md5',
  `adapte_uri` varchar(500) DEFAULT NULL,
  `content` MEDIUMTEXT NULL COMMENT '题卡内容（模式4导入）',
  PRIMARY KEY (`id`) USING BTREE,
  UNIQUE KEY `idx_1` (`exam_id`,`number`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='题卡卡格式表';

-- ----------------------------
-- Table structure for scan_answer_card_question
-- ----------------------------
CREATE TABLE IF NOT EXISTS `scan_answer_card_question`  (
      `id` bigint NOT NULL COMMENT '主键',
      `exam_id` bigint NOT NULL COMMENT '考试ID',
      `paper_number` varchar(100) NOT NULL COMMENT '试卷类型',
      `course_paper_id` varchar(32) NULL DEFAULT NULL COMMENT '试卷编号（交互）',
      `serial_number` int NULL DEFAULT NULL COMMENT '备用卷',
      `card_number` int NULL DEFAULT NULL COMMENT '卡格式序号',
      `main_number` int NOT NULL COMMENT '大题号',
      `sub_number` int NOT NULL COMMENT '小题号',
      `option_count` int NOT NULL COMMENT '选项个数',
      `question_type` varchar(2) NULL DEFAULT NULL COMMENT '题型',
      `paper_index` int NULL DEFAULT NULL,
      `page_index` int NULL DEFAULT NULL,
      `create_id` bigint NULL DEFAULT NULL,
      `create_time` bigint NULL DEFAULT NULL,
      `update_id` bigint NULL DEFAULT NULL,
      `update_time` bigint NULL DEFAULT NULL,
      PRIMARY KEY (`id`) USING BTREE
) ENGINE = InnoDB CHARACTER SET = utf8mb4 COLLATE = utf8mb4_general_ci COMMENT = '卡格式题型';

-- ----------------------------
-- Table structure for scan_answer_card_subject
-- ----------------------------
CREATE TABLE IF NOT EXISTS `scan_answer_card_subject` (
  `id` bigint NOT NULL COMMENT '主键',
  `exam_id` bigint NOT NULL COMMENT '考试ID',
  `card_number` int NOT NULL COMMENT '卡格式号',
  `paper_number` varchar(255) NOT NULL COMMENT '试卷编号',
  `course_paper_id` varchar(50) NOT NULL COMMENT '试卷编号（交互）',
  PRIMARY KEY (`id`) USING BTREE,
  UNIQUE KEY `index1` (`exam_id`,`card_number`,`paper_number`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='卡格式科目关联关系';

-- ----------------------------
-- Table structure for scan_batch
-- ----------------------------
CREATE TABLE IF NOT EXISTS `scan_batch` (
  `id` bigint NOT NULL AUTO_INCREMENT,
  `exam_id` bigint NOT NULL COMMENT '考试ID',
  `user_id` bigint NOT NULL COMMENT '扫描员ID',
  `scan_count` int NOT NULL COMMENT '扫描数量',
  `assigned_count` int NOT NULL,
  `status` varchar(16) NOT NULL COMMENT '状态',
  `creator_id` bigint DEFAULT NULL,
  `updater_id` bigint DEFAULT NULL,
  `create_time` bigint DEFAULT NULL,
  `update_time` bigint DEFAULT NULL,
  `course_paper_id` varchar(64) DEFAULT NULL COMMENT '试卷编号',
  PRIMARY KEY (`id`) USING BTREE,
  KEY `exam_verify_status` (`exam_id`) USING BTREE
) ENGINE=InnoDB AUTO_INCREMENT=1110 DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci;

-- ----------------------------
-- Table structure for scan_batch_paper
-- ----------------------------
CREATE TABLE IF NOT EXISTS `scan_batch_paper` (
  `batch_id` bigint NOT NULL COMMENT '批次ID',
  `student_id` bigint NOT NULL COMMENT '考生ID',
  `paper_index` int NOT NULL COMMENT '试卷页数',
  `paper_id` bigint NOT NULL COMMENT '试卷ID',
  `card_number` int NOT NULL COMMENT '题卡编号',
  `assigned` bit(1) NOT NULL,
  PRIMARY KEY (`batch_id`,`student_id`,`paper_index`) USING BTREE,
  UNIQUE KEY `paper_id` (`paper_id`) USING BTREE,
  UNIQUE KEY `paper_id_2` (`paper_id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci;

-- ----------------------------
-- Table structure for scan_file_property
-- ----------------------------
CREATE TABLE IF NOT EXISTS `scan_file_property` (
  `path` varchar(255) NOT NULL COMMENT '文件路径',
  `md5` varchar(32) NOT NULL COMMENT '文件md5',
  `exam_id` bigint NOT NULL COMMENT '考试ID',
  `file_size` bigint NOT NULL COMMENT '文件大小',
  `create_time` bigint DEFAULT NULL,
  `update_time` bigint DEFAULT NULL,
  PRIMARY KEY (`path`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci;

-- ----------------------------
-- Table structure for scan_log
-- ----------------------------
CREATE TABLE IF NOT EXISTS `scan_log` (
  `school_code` varchar(20) NOT NULL COMMENT '学校代码',
  `device_code` varchar(50) NOT NULL COMMENT '设备号',
  `file_name` varchar(45) NOT NULL,
  `type` varchar(10) DEFAULT NULL COMMENT '错误文件类型，崩溃异常文件：COLLAPSE，正常文件：NORMAL',
  `md5` varchar(45) NOT NULL COMMENT '扫描端日志',
  `path` varchar(500) DEFAULT NULL,
  `login_name` varchar(45) DEFAULT NULL COMMENT '扫描账号',
  `upload_time` bigint DEFAULT NULL,
  PRIMARY KEY (`school_code`,`device_code`,`file_name`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci;

-- ----------------------------
-- Table structure for scan_omr_task
-- ----------------------------
CREATE TABLE IF NOT EXISTS `scan_omr_task` (
  `id` bigint NOT NULL,
  `exam_id` bigint NOT NULL COMMENT '考试ID',
  `conditions` varchar(100) NOT NULL COMMENT '条件',
  `student_id` bigint NOT NULL COMMENT '考生ID',
  `paper_index` int NOT NULL COMMENT '试卷页数',
  `paper_id` bigint NOT NULL COMMENT '试卷ID',
  `card_number` int NOT NULL COMMENT '题卡编号',
  `status` varchar(16) NOT NULL COMMENT '状态',
  `user_id` bigint DEFAULT NULL,
  `pages` text NOT NULL,
  `creator_id` bigint DEFAULT NULL,
  `updater_id` bigint DEFAULT NULL,
  `create_time` bigint DEFAULT NULL,
  `update_time` bigint DEFAULT NULL,
  PRIMARY KEY (`id`) USING BTREE,
  KEY `exam_status` (`exam_id`,`status`,`student_id`) USING BTREE,
  KEY `group_paper` (`conditions`,`paper_id`) USING BTREE,
  KEY `index1` (`student_id`,`status`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci;

-- ----------------------------
-- Table structure for scan_package
-- ----------------------------
CREATE TABLE IF NOT EXISTS `scan_package` (
  `id` bigint NOT NULL,
  `exam_id` bigint NOT NULL COMMENT '考试ID',
  `course_paper_id` varchar(30) DEFAULT NULL COMMENT '试卷编号',
  `package_code` varchar(20) DEFAULT NULL COMMENT '卷袋编号',
  `package_no` int DEFAULT NULL COMMENT '页码',
  `path` varchar(255) NOT NULL COMMENT '签到表图片路径',
  `md5` varchar(32) NOT NULL COMMENT '文件md5',
  `assigned` bit(1) DEFAULT NULL,
  `creator_id` bigint DEFAULT NULL,
  `updater_id` bigint DEFAULT NULL,
  `create_time` bigint DEFAULT NULL,
  `update_time` bigint DEFAULT NULL,
  PRIMARY KEY (`id`) USING BTREE,
  KEY `exam` (`exam_id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci;

-- ----------------------------
-- Table structure for scan_paper
-- ----------------------------
CREATE TABLE IF NOT EXISTS `scan_paper` (
  `id` bigint NOT NULL,
  `exam_id` bigint NOT NULL COMMENT '考试ID',
  `card_number` int NOT NULL COMMENT '卡格式编号',
  `number` int NOT NULL COMMENT '题卡编号（对应paper_index）',
  `page_count` int NOT NULL COMMENT '数量',
  `mismatch` bit(1) NOT NULL,
  `assigned` bit(1) NOT NULL,
  `question_filled` bit(1) NOT NULL,
  `checked` varchar(45) DEFAULT NULL COMMENT '已检查',
  `creator_id` bigint DEFAULT NULL,
  `updater_id` bigint DEFAULT NULL,
  `create_time` bigint DEFAULT NULL,
  `update_time` bigint DEFAULT NULL,
  `assign_confirmed` bit(1) DEFAULT NULL COMMENT '人工绑定确认',
  `invalid` bit(1) DEFAULT b'0' COMMENT '强制绑定',
  PRIMARY KEY (`id`) USING BTREE,
  KEY `mismatch` (`exam_id`,`mismatch`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci;

-- ----------------------------
-- Table structure for scan_paper_page
-- ----------------------------
CREATE TABLE IF NOT EXISTS `scan_paper_page` (
  `id` bigint DEFAULT NULL,
  `paper_id` bigint NOT NULL,
  `page_index` int NOT NULL,
  `absent` text,
  `breach` text,
  `question` text,
  `selective` text,
  `sheet_path` varchar(1000) NOT NULL,
  `recog_data` longtext,
  `paper_type` TEXT NULL COMMENT '卷型',
  `creator_id` bigint DEFAULT NULL,
  `updater_id` bigint DEFAULT NULL,
  `create_time` bigint DEFAULT NULL,
  `update_time` bigint DEFAULT NULL,
  PRIMARY KEY (`paper_id`,`page_index`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci;

-- ----------------------------
-- Table structure for scan_refix_batch
-- ----------------------------
CREATE TABLE IF NOT EXISTS `scan_refix_batch` (
  `id` bigint NOT NULL,
  `exam_id` bigint NOT NULL,
  `card_number` int NOT NULL,
  `conditions` text NOT NULL,
  `update_slice` bit(1) NOT NULL,
  `total_count` int NOT NULL,
  `sucess_count` int NOT NULL,
  `finished` bit(1) NOT NULL,
  `creator_id` bigint DEFAULT NULL,
  `updater_id` bigint DEFAULT NULL,
  `create_time` bigint DEFAULT NULL,
  `update_time` bigint DEFAULT NULL,
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci;

-- ----------------------------
-- Table structure for scan_student_paper
-- ----------------------------
CREATE TABLE IF NOT EXISTS `scan_student_paper` (
  `id` bigint NOT NULL,
  `student_id` bigint NOT NULL,scan_student
  `paper_index` int NOT NULL,
  `paper_id` bigint NOT NULL,
  `create_time` bigint DEFAULT NULL,
  `update_time` bigint DEFAULT NULL,
  PRIMARY KEY (`id`) USING BTREE,
  UNIQUE KEY `paper_id` (`paper_id`) USING BTREE,
  UNIQUE KEY `index1` (`student_id`,`paper_index`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci;

-- ----------------------------
-- Table structure for sys_config
-- ----------------------------
CREATE TABLE IF NOT EXISTS `sys_config` (
  `id` bigint NOT NULL COMMENT '主键',
  `school_id` bigint DEFAULT NULL COMMENT '学校id',
  `org_id` bigint DEFAULT NULL COMMENT '机构id',
  `config_key` varchar(100) NOT NULL COMMENT '参数键名',
  `config_name` varchar(100) NOT NULL COMMENT '参数名称',
  `config_value` MEDIUMTEXT DEFAULT NULL COMMENT '参数键值',
  `remark` varchar(100) DEFAULT NULL COMMENT '备注',
  `enable` tinyint DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
  `sort` int DEFAULT '1' COMMENT '排序',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='参数表';

-- ----------------------------
-- Records of sys_config
-- ----------------------------
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (1, NULL, NULL, 'sys.user.initPassword', '初始密码', 'Qmth87863577', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (2, NULL, NULL, 'sys.warning.days', '预警天数', '3', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (3, NULL, NULL, 'sys.code.enable', '是否启用短信验证码', 'true', 'true-启用，false-禁用', 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (4, NULL, NULL, 'sys.message.enable', '是否启用短信消息提示', 'true', 'true-启用，false-禁用', 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (5, NULL, NULL, 'sys.message.resendCount', '失败短信重试次数', '3', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (6, NULL, NULL, 'sys.sync.enable', '是否开启数据同步云阅卷', 'true', 'true-启用，false-禁用', 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (8, NULL, NULL, 'sys.pdf.size.list', '全局pdf格式清单', '[A3, A4, A5, B3, B4, B5, 8K]', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (9, NULL, NULL, 'admin.logo.url', '超级管理员logo', 'http://qmth-test.oss-cn-shenzhen.aliyuncs.com/frontend/wx_logo.png', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (10, NULL, NULL, 'attachment.type', '附件类型', '[.xlsx,.xls,.doc,.docx,.pdf,.jpg,.jpeg,.png,.html,.zip,.mp3,.wav,.dll,.exe,.ftl,.bpmn,.xml]', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (11, NULL, NULL, 'attachment.length', '附件名称长度', '100', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (12, NULL, NULL, 'attachment.size', '附件大小 ', '200', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (13, NULL, NULL, 'auto.create.pdf.reset.max.count', '自动重试生成pdf失败最大次数', '5', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (14, NULL, NULL, 'thread.pool.core.size', '自定义线程池大小', '500', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (15, NULL, NULL, 'custom.thread.pool.core.size', '是否自定义线程池大小', 'true', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (16, NULL, NULL, 'session.active', 'session会话时长', '4h', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (18, NULL, NULL, 'sms.normal.code', '万能短信验证码', 'qmth', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (19, NULL, NULL, 'code.expired.time', '短信验证码有效时间', '2', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (20, NULL, NULL, 'code.send.interval', '短信验证码发送间隔', '60', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (21, NULL, NULL, 'teachcloud.print.open.callback.pwd', '分布式印刷回调密码', 'qmth87863577-offline', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (24, NULL, NULL, 'sms.sign.name', '短信签名', '知学知考', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (25, NULL, NULL, 'sms.tpl.code', '登录短信验证码', 'SMS_147416565', NULL, 1, 2, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (26, NULL, NULL, 'sms.audit.pass.code', '命题任务审核通过', 'SMS_216425141', NULL, 1, 3, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (27, NULL, NULL, 'sms.audit.not.pass.code', '命题任务审核不通过', 'SMS_216275156', NULL, 1, 4, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (28, NULL, NULL, 'sms.exam.task.create.code', '命题任务待办生成通知', 'SMS_217436292', NULL, 1, 5, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (29, NULL, NULL, 'sms.exam.task.will.expire.code', '命题任务待办到期预警通知', 'SMS_217436295', NULL, 1, 6, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (30, NULL, NULL, 'sms.exam.task.overdue.code', '命题任务待办逾期通知', 'SMS_217426313', NULL, 1, 7, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (31, NULL, NULL, 'sms.allocation.will.expire.code', '命题分配待办到期预警通知', 'SMS_217406305', NULL, 1, 8, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (32, NULL, NULL, 'sms.allocation.overdue.code', '命题分配待办逾期通知', 'SMS_217406308', NULL, 1, 9, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (33, NULL, NULL, 'sms.audit.create.code', '审核待办生成通知', 'SMS_237216000', NULL, 1, 10, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (34, NULL, NULL, 'sms.audit.review.code', '审核待办修改申请通知', 'SMS_237216002', NULL, 1, 11, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (35, NULL, NULL, 'sms.audit.will.expire.code', '审核待办到期预警通知', 'SMS_217436302', NULL, 1, 12, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (36, NULL, NULL, 'sms.audit.overdue.code', '审核待办逾期通知', 'SMS_217416271', NULL, 1, 13, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (37, NULL, NULL, 'sms.audit.reject.code', '审核驳回通知', 'SMS_237206063', NULL, 1, 14, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (38, NULL, NULL, 'sms.upload.structure.code', '试卷结构推送通知', 'SMS_237201068', NULL, 1, 15, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (39, NULL, NULL, 'sms.audit.copy.user.code', '用户', 'SMS_237206065', NULL, 1, 16, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (40, NULL, NULL, 'sys.txt.charset', 'txt文件编码', 'UTF-8', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (41, NULL, NULL, 'package.barcode.position', '签到表条码坐标', '[0.7,0,0.3,0.1]', NULL, 1, 17, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (42, NULL, NULL, 'create.pdf.job.db.limit', '创建pdf查询条数', 'limit 0,100', NULL, 1, 18, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (43, NULL, NULL, 'sys.card.size.list', '全局题卡格式清单', '[A3, 8K]', NULL, 1, 2, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `school_id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES ('50', NULL, NULL, 'sms.sync.data.code', '数据同步失败通知', 'SMS_475870952', NULL, 1, 19, 1, '1733878447209', NULL, '1733878447249');

-- ----------------------------
-- Table structure for sys_org
-- ----------------------------
CREATE TABLE IF NOT EXISTS `sys_org` (
  `id` bigint NOT NULL COMMENT '主键',
  `school_id` bigint NOT NULL COMMENT '学校id',
  `type` varchar(50) DEFAULT NULL COMMENT '类型，SCHOOL：学校，COLLEGE：学院，FACULTY：院系，TEACHING_ROOM：教研室，PRINTING_HOUSE：印刷厂',
  `code` varchar(20) DEFAULT NULL COMMENT '机构代码',
  `name` varchar(100) NOT NULL COMMENT '机构名称',
  `parent_id` bigint DEFAULT NULL COMMENT '上级机构id',
  `enable` tinyint NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '修改人',
  `update_time` bigint DEFAULT NULL COMMENT '修改时间',
  `historic_name` varchar(100) DEFAULT NULL COMMENT '历史机构名称',
  PRIMARY KEY (`id`) USING BTREE,
  UNIQUE KEY `org_schoolId_name_idx` (`school_id`,`name`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='学校组织架构';

-- ----------------------------
-- Table structure for sys_privilege
-- ----------------------------
CREATE TABLE IF NOT EXISTS `sys_privilege` (
  `id` bigint NOT NULL COMMENT '主键',
  `name` varchar(100) NOT NULL COMMENT '菜单名称',
  `url` varchar(100) NOT NULL COMMENT 'URL地址',
  `type` varchar(50) NOT NULL COMMENT '类型，MENU：菜单，BUTTON：按钮，LINK：链接，URL：接口地址，LIST：列表，CONDITION：查询条件，TAB：选项卡',
  `parent_id` bigint DEFAULT NULL COMMENT '上级菜单',
  `sequence` int NOT NULL COMMENT '序号',
  `property` varchar(30) DEFAULT NULL COMMENT '属性，NO_AUTH：无需鉴权，AUTH：鉴权，SYS：系统公用',
  `related` text COMMENT '关联属性，type为URL且property为AUTH才有',
  `enable` tinyint NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
  `default_auth` tinyint DEFAULT '1' COMMENT '默认权限，0：否，1：是',
  `front_display` tinyint(1) DEFAULT '1' COMMENT '是否前端展示',
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='菜单权限表';

-- ----------------------------
-- Records of sys_privilege
-- ----------------------------
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1, '基础配置', 'base', 'MENU', NULL, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2, '用户管理', 'user', 'MENU', 1, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3, '配置管理', 'rule', 'MENU', 1, 2, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (4, '组织架构', 'OrganizationManage', 'MENU', 2, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (5, '用户管理', 'UserManage', 'MENU', 2, 2, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (6, '角色管理', 'RoleManage', 'MENU', 2, 3, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (7, '权限管理', 'MenuManage', 'MENU', 129, 6, NULL, NULL, 0, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (8, '通用规则', 'CommonRule', 'MENU', 3, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (9, '题卡规则管理', 'CardRuleManage', 'MENU', 3, 2, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (11, '印品管理', 'PrintTemplateManage', 'MENU', 3, 4, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (13, '课程管理', 'CourseManage', 'MENU', 356, 7, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (14, '考试中心', 'exam', 'MENU', NULL, 5, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (38, '印刷管理', 'print', 'MENU', 14, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (39, '命题任务管理', 'ExamTaskManage', 'MENU', 652, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (40, '入库申请', 'TaskApplyManage', 'MENU', 652, 2, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (41, '入库审核', 'TaskReviewManage', 'MENU', 652, 3, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (42, '卷库查询', 'TaskPaperManage', 'MENU', 652, 4, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (43, '印刷计划管理', 'PrintPlanManage', 'MENU', 38, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (44, '考务数据导入', 'BusinessDataExport', 'MENU', 38, 2, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (45, '考务明细查询', 'BusinessDataDetail', 'MENU', 38, 3, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (47, '印刷任务管理', 'PrintTaskManage', 'MENU', 38, 5, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (48, '查询', '/api/admin/exam/print/list', 'URL', 43, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (49, '按类型查询模板', '/api/admin/exam/print/get_template', 'URL', 43, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (50, '新增/修改', '/api/admin/exam/print/save', 'URL', 43, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (51, '删除', '/api/admin/exam/print/remove', 'URL', 43, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (52, '查询', '/api/admin/exam/print/data_list', 'URL', 44, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (53, '汇总数据查询', '/api/admin/exam/print/data_total', 'URL', 44, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (54, '导入', '/api/admin/exam/print/data_import', 'URL', 44, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (55, '查看明细', '/api/admin/exam/print/data_detail_list', 'URL', 44, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (56, '查询', '/api/admin/exam/print/relate_list', 'URL', 43, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (57, '关联/更换试卷', '/api/admin/exam/print/relate_update', 'URL', 43, 2, 'AUTH', '58', 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (58, '查询所有试卷编号/关联卷型', '/api/admin/exam/print/relate_get_paper_numbers', 'URL', 43, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (59, '查询', '/api/admin/exam/print/task_list', 'URL', 47, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (60, '提交印刷', '/api/admin/exam/print/task_submit', 'URL', 47, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (61, '撤回提交', '/api/admin/exam/print/task_cancel', 'URL', 47, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (62, '汇总数据查询', '/api/admin/exam/print/task_total_data', 'URL', 47, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (63, '查询', '/api/admin/exam/task/list', 'URL', 39, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (64, '更改/指派命题老师', '/api/admin/exam/task/assign_user', 'URL', 39, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (65, '启用/禁用', '/api/admin/exam/task/enable', 'URL', 39, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (66, '新建', '/api/admin/exam/task/save', 'URL', 39, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (67, '批量新建-文件导入', '/api/admin/exam/task/import', 'URL', 39, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (68, '批量新建', '/api/admin/exam/task/save_batch', 'URL', 39, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (69, '审核历史', '/api/admin/exam/task/review_list', 'URL', 39, 7, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (70, '查询', '/api/admin/exam/task/apply_list', 'URL', 40, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (71, '提交/暂存', '/api/admin/exam/task/apply_save', 'URL', 40, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (72, '撤回/重新申请', '/api/admin/exam/task/apply_status', 'URL', 40, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (73, '查询未审核', '/api/admin/exam/task/review_list_unaudited', 'URL', 41, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (74, '审核', '/api/admin/exam/task/review_save', 'URL', 41, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (75, '批量审核', '/api/admin/exam/task/review_save_batch', 'URL', 41, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (76, '查询', '/api/admin/exam/task/paper_list', 'URL', 42, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (77, '查询', '/api/admin/sys/user/list', 'URL', 5, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (78, '新增/修改', '/api/admin/sys/user/save', 'URL', 5, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (79, '启用/禁用', '/api/admin/sys/user/enable', 'URL', 5, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (80, '重置密码', '/api/admin/sys/user/reset_password', 'URL', 5, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (81, '导入', '/api/admin/sys/user/import', 'URL', 5, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (82, '绑定角色', '/api/admin/sys/user/bind_roles', 'URL', 5, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (83, '查询', '/api/admin/sys/role/list', 'URL', 6, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (84, '删除', '/api/admin/sys/role/remove', 'URL', 6, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (85, '新增/修改', '/api/admin/sys/role/save', 'URL', 6, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (86, '启用/禁用', '/api/admin/sys/role/enable', 'URL', 6, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (87, '用户已绑定角色列表', '/api/admin/sys/role/get_user_roles', 'URL', 5, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (88, '查询', '/api/admin/sys/privilege/list', 'URL', 7, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (89, '新增/修改', '/api/admin/sys/privilege/save', 'URL', 7, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (90, '删除', '/api/admin/sys/privilege/remove', 'URL', 7, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (91, '角色已绑定权限列表', '/api/admin/sys/privilege/get_role_privileges', 'URL', 6, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (92, '组织机构-查询', '/api/admin/sys/org/list', 'URL', 4, 1, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (93, '组织机构-新增/修改', '/api/admin/sys/org/save', 'URL', 4, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (94, '组织机构-删除', '/api/admin/sys/org/remove', 'URL', 4, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (95, '组织机构-启用/禁用', '/api/admin/sys/org/enable', 'URL', 4, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (96, '查询', '/api/admin/basic/course/list', 'URL', 13, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (97, '新增/修改', '/api/admin/basic/course/save', 'URL', 13, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (98, '删除', '/api/admin/basic/course/remove', 'URL', 13, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (99, '查询', '/api/admin/basic/exam_rule/list', 'URL', 8, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (100, '新增/修改', '/api/admin/basic/exam_rule/save', 'URL', 8, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (101, '查询', '/api/admin/basic/card_rule/list', 'URL', 9, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (102, '查询单个', '/api/admin/basic/card_rule/get_one', 'URL', 9, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (103, '新增/修改', '/api/admin/basic/card_rule/save', 'URL', 9, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (104, '删除', '/api/admin/basic/card_rule/delete', 'URL', 9, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (105, '查询-汇总数据', '/api/admin/exam/print/relate_total', 'URL', 43, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (112, '系统设置', 'data', 'MENU', 1, 5, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (113, '任务管理', 'DataTaskManage', 'MENU', 112, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (114, '查询', '/api/admin/data/task/query', 'URL', 113, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (115, 'PDF生成管理', 'PdfBuildManage', 'MENU', 112, 2, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (116, '查询', 'Select', 'BUTTON', 115, 1, 'AUTH', '127', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (117, '命题待办接口', '/api/admin/work/exam/task/ready', 'URL', 16, 1, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (118, '命题待办计数接口', '/api/admin/work/exam/task/ready_count', 'URL', 16, 2, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (119, '命题分配待办接口', '/api/admin/work/exam/task/new', 'URL', 16, 3, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (120, '命题分配待办计数接口', '/api/admin/work/exam/task/new_count', 'URL', 16, 4, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (121, '审核待办接口', '/api/admin/work/exam/task/submit', 'URL', 16, 5, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (122, '审核待办计数接口', '/api/admin/work/exam/task/submit_count', 'URL', 16, 6, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (123, '查询', '/api/admin/basic/template/list', 'URL', 11, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (124, '新增/修改', '/api/admin/basic/template/save', 'URL', 11, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (125, '启用/禁用', '/api/admin/basic/template/enable', 'URL', 11, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (126, '查询条件', 'Condition', 'CONDITION', 115, 1, 'AUTH', '127', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (127, '查询', '/api/admin/data/task/page_pdf_create', 'URL', 115, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (128, '列表', 'List', 'LIST', 115, 2, 'AUTH', '127', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (129, '公共接口', 'common', 'MENU', NULL, 12, NULL, NULL, 0, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (130, '用户登录相关', 'login', 'MENU', 129, 1, NULL, NULL, 0, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (131, '用户登录', '/api/admin/common/login', 'URL', 130, 1, 'NO_AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (132, '查询用户权限', '/api/admin/common/get_menu', 'URL', 130, 2, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (133, '发送验证码', '/api/admin/common/get_verify_code', 'URL', 130, 3, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (134, '修改密码', '/api/admin/sys/user/update_password', 'URL', 130, 4, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (135, '用户登出', '/api/admin/common/logout', 'URL', 130, 5, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (136, '模糊查询', 'query', 'MENU', 129, 2, NULL, NULL, 0, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (137, '模糊查询-印刷计划', '/api/admin/exam/print/query', 'URL', 136, 1, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (138, '题卡规则', '/api/admin/basic/card_rule/query', 'URL', 136, 2, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (139, '模糊查询-课程', '/api/admin/basic/course/query', 'URL', 136, 3, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (140, '模糊查询-试卷编号', '/api/admin/exam/task/paper_number_query', 'URL', 136, 4, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (141, '模糊查询-命题老师', '/api/admin/exam/task/user_query', 'URL', 136, 5, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (142, '模糊查询-考点', '/api/admin/exam/print/data_place_query', 'URL', 136, 6, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (143, '模糊查询-考场', '/api/admin/exam/print/data_room_query', 'URL', 136, 7, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (144, '文件相关', 'file', 'MENU', 129, 3, NULL, NULL, 0, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (145, '文件上传', '/api/admin/common/file/upload', 'URL', 144, 1, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (146, '文件查询', '/api/admin/common/file/get_one', 'URL', 144, 2, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (147, '文件下载', '/api/admin/common/file/download', 'URL', 144, 3, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (148, '文件预览', '/api/admin/common/file/preview', 'URL', 144, 4, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (149, '系统相关', 'sys', 'MENU', 129, 4, NULL, NULL, 0, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (150, '查询枚举类型', '/api/admin/common/get_enums', 'URL', 149, 1, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (151, '查询学校列表', '/api/admin/common/school/list', 'URL', 149, 2, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (152, '查询系统参数', '/api/admin/common/sys_config/get_one', 'URL', 149, 3, 'NO_AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (153, '根据学校代码查询学校信息', '/api/admin/common/school/query_by_school_code', 'URL', 149, 4, 'NO_AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (154, '其他', 'other', 'MENU', 129, 5, NULL, NULL, 0, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (155, '用户导入（异步）', '/api/admin/sys/user/async/import', 'URL', 154, 1, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (156, '用户导出', '/api/admin/sys/user/export', 'URL', 154, 2, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (157, '模板下载', '/api/admin/exam/print/template_download', 'URL', 44, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (158, '结果导出', '/api/admin/exam/print/data_export', 'URL', 44, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (159, '查询', '/api/admin/exam/print/data_detail', 'URL', 45, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (160, '重新提交印刷', '/api/admin/exam/print/task_resubmit', 'URL', 47, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (161, '查看PDF', '/api/admin/exam/print/task_view_pdf', 'URL', 47, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (162, '批量下载PDF', '/api/admin/exam/print/task_download_pdf', 'URL', 47, 7, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (163, '导出审核样本', '/api/admin/exam/task/review_export', 'URL', 41, 4, 'AUTH', NULL, 0, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (164, '下载试卷', '/api/admin/exam/task/paper_download', 'URL', 42, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (165, '试卷启用/禁用', '/api/admin/exam/task/paper_enable', 'URL', 42, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (166, '查询已审核', '/api/admin/exam/task/review_list_audited', 'URL', 41, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (167, '查询可选择题卡', '/api/admin/exam/card/select_card_list', 'URL', 40, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (168, '查询申请详情', '/api/admin/exam/task/apply_get_one', 'URL', 40, 5, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (169, '根据课程查询命题老师', '/api/admin/sys/user/user_list', 'URL', 39, 7, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (170, '客服制卡', 'customer', 'MENU', NULL, 9, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (171, '客服制卡', 'customerManage', 'MENU', 170, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (172, '题卡审核', 'CustomerCard', 'MENU', 171, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (173, '客服制卡-查询', '/api/admin/exam/card/cust_list', 'URL', 172, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (174, '导入模板下载', '/api/admin/common/download_import_template', 'URL', 149, 15, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (175, '题卡管理', 'card', 'MENU', NULL, 10, NULL, NULL, 0, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (176, '查询题卡规则', '/api/admin/basic/card_rule/get_one', 'URL', 175, 1, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (177, '查询题卡详情', '/api/admin/exam/card/get_one', 'URL', 175, 2, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (178, '保存/提交', '/api/admin/exam/card/save', 'URL', 175, 3, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (179, '查询考务规则', '/api/admin/basic/exam_rule/list', 'URL', 149, 5, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (180, '查询可设置权限列表', '/api/admin/sys/privilege/list_auth', 'URL', 6, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (181, '超管中心', 'admin', 'MENU', 129, 7, NULL, NULL, 0, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (182, '查询客服', '/api/admin/sys/user/list_customer', 'URL', 181, 1, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (183, '新增客服', '/api/admin/sys/user/save_customer', 'URL', 181, 2, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (184, '查询用户可绑定角色', '/api/admin/sys/role/list_to_user', 'URL', 5, 7, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (185, '客服制卡-批量下载文件', '/api/admin/exam/card/download_files', 'URL', 172, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (186, '查看印品', '/api/admin/exam/print/template_view', 'URL', 47, 8, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (187, '客户端登录', '/api/admin/client/user/login', 'URL', NULL, 1, 'NO_AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (188, '学生明细', '/api/admin/exam/print/get_student_detail', 'URL', 45, 2, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (189, '修改', '/api/admin/exam/task/paper_update', 'URL', 42, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (190, '重新生成pdf', '/api/admin/data/task/reset_create_pdf', 'URL', 115, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (191, '批量下载试卷PDF、题卡', '/api/admin/exam/task/paper_card_download_pdf', 'URL', 42, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (192, '预览', '/api/admin/basic/template/read_content', 'URL', 11, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (193, '补考任务查询', '/api/admin/exam/print/list_makeup_task', 'URL', 43, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (194, '补考任务保存', '/api/admin/exam/print/save_makeup_task', 'URL', 43, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (195, '获取服务器时间', '/api/admin/common/get_system_time', 'URL', 129, 2, 'NO_AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (198, '数据同步', '/api/admin/exam/print/sync_data_cloud', 'URL', 47, 9, 'NO_AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (199, '打印端', 'client', 'MENU', NULL, 11, NULL, '200,201,202,203,204,205,206,207,208,209,210,211,212,213,214,215,216,241,242,973,974,975', 1, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (200, '试卷打样-查询列表', '/api/admin/client/paper_try/list', 'URL', 199, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (201, '试卷打样-查看/试印/重印', '/api/admin/client/paper_try/print', 'URL', 199, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (202, '试卷打样-批量试印', '/api/admin/client/paper_try/print_batch', 'URL', 199, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (203, '试卷打样-标记合格状态', '/api/admin/client/paper_try/tag_pass', 'URL', 199, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (204, '查询列表', '/api/admin/client/print/task_list', 'URL', 199, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (205, '导出', '/api/admin/client/print/task_list_export', 'URL', 199, 7, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (206, '汇总数据查询', '/api/admin/client/print/task_total_data', 'URL', 199, 8, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (207, '查看', '/api/admin/client/print/preview', 'URL', 199, 9, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (208, '印刷/缓存数据', '/api/admin/client/print/get_print_data', 'URL', 199, 10, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (209, '批量缓存', '/api/admin/client/print/get_print_data_batch', 'URL', 199, 11, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (210, '缓存后更新状态', '/api/admin/client/print/update_download', 'URL', 199, 12, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (211, '校验', '/api/admin/client/print/validate_data', 'URL', 199, 13, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (212, '更新打印进度', '/api/admin/client/print/update_progress', 'URL', 199, 14, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (213, '重打-查询考生列表', '/api/admin/client/print/list_student', 'URL', 199, 15, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (214, '重打-内容查询', '/api/admin/client/print/get_reprint_data', 'URL', 199, 16, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (215, '统计查询-查询列表', '/api/admin/client/print/statistics_list', 'URL', 199, 16, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (216, '统计查询-汇总数据查询', '/api/admin/client/print/statistics_total_data', 'URL', 199, 16, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (217, '查询', 'Select', 'BUTTON', 5, 1, 'AUTH', '77', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (218, '新增', 'Add', 'BUTTON', 5, 2, 'AUTH', '78', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (219, '编辑', 'Edit', 'LINK', 5, 1, 'AUTH', '78', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (220, '启用/禁用', 'Enable', 'LINK', 5, 2, 'AUTH', '79', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (221, '列表', 'List', 'LIST', 5, 1, 'AUTH', '77', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (222, '查询条件', 'Condition', 'CONDITION', 5, 1, 'AUTH', '184,87', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (223, '重置密码', 'Reset', 'LINK', 5, 3, 'AUTH', '80', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (224, '新增', 'Add', 'BUTTON', 4, 1, 'AUTH', '93', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (225, '子部门', 'Add', 'LINK', 4, 1, 'AUTH', '93', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (226, '编辑', 'Edit', 'LINK', 4, 2, 'AUTH', '93', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (227, '删除', 'Delete', 'LINK', 4, 3, 'AUTH', '94', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (228, '树形列表', 'List', 'LIST', 4, 1, 'AUTH', '92', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (229, '查询', 'Select', 'BUTTON', 6, 1, 'AUTH', '83,88', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (230, '新增', 'Add', 'BUTTON', 6, 2, 'AUTH', '85,89', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (231, '编辑', 'Edit', 'LINK', 6, 1, 'AUTH', '83,91', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (232, '删除', 'Delete', 'LINK', 6, 2, 'AUTH', '84,90', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (233, '列表', 'List', 'LIST', 6, 1, 'AUTH', '83', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (234, '查询条件', 'Condition', 'CONDITION', 4, 1, 'AUTH', '92', 0, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (235, '查询条件', 'Condition', 'CONDITION', 6, 1, 'AUTH', '83', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (236, '考务规则配置', 'Add', 'BUTTON', 8, 1, 'AUTH', '99, 100', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (237, '查询', 'Select', 'BUTTON', 9, 1, 'AUTH', '101', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (238, '新增', 'Add', 'BUTTON', 9, 2, 'AUTH', '103', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (239, '查询条件', 'Condition', 'CONDITION', 9, 1, 'AUTH', '101', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (240, '列表', 'List', 'LIST', 9, 1, 'AUTH', '101', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (241, '印刷-单文件打印', '/api/admin/client/print/single', 'URL', 199, 17, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (242, '查询条件-课程', '/api/admin/client/course/list', 'URL', 199, 18, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (243, '导入', 'Import', 'BUTTON', 5, 3, 'AUTH', '81', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (249, '查询', 'Select', 'BUTTON', 11, 1, 'AUTH', '123', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (250, '新增', 'Add', 'BUTTON', 11, 2, 'AUTH', '124', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (251, '预览', 'Preview', 'LINK', 11, 1, 'AUTH', '192', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (252, '编辑', 'Edit', 'LINK', 11, 2, 'AUTH', '124,763', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (253, '启用/禁用', 'Enable', 'LINK', 11, 3, 'AUTH', '125', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (254, '查询条件', 'Condition', 'CONDITION', 11, 1, 'AUTH', '123', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (255, '列表', 'List', 'LIST', 11, 1, 'AUTH', '123', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (263, '查询', 'Select', 'BUTTON', 13, 1, 'AUTH', '96,462', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (268, '编辑', 'Edit', 'LINK', 13, 1, 'AUTH', '97', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (269, '删除', 'Delete', 'LINK', 13, 2, 'AUTH', '368', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (270, '查询条件', 'Condition', 'CONDITION', 13, 2, 'AUTH', '96', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (271, '列表', 'List', 'LIST', 13, 1, 'AUTH', '96', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (272, '查询', 'Select', 'BUTTON', 39, 1, 'AUTH', '63', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (273, '批量新增', 'BatchAdd', 'BUTTON', 39, 2, 'AUTH', '67,68', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (274, '新增', 'Add', 'BUTTON', 39, 3, 'AUTH', '66,169,441,442,445,482,839', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (275, '预览', 'Preview', 'LINK', 39, 1, 'AUTH', '68', 0, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (277, '启用/禁用', 'Cancel', 'LINK', 39, 3, 'AUTH', '65', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (278, '查询条件', 'Condition', 'CONDITION', 39, 1, 'AUTH', '63', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (279, '列表', 'List', 'LIST', 39, 1, 'AUTH', '63', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (280, '查询', 'Select', 'BUTTON', 40, 1, 'AUTH', '70', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (281, '预览', 'Preview', 'LINK', 40, 1, 'AUTH', '168,477,568,417,420,421,441,442,640,641', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (282, '撤销申请', 'End', 'LINK', 40, 2, 'AUTH', '72', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (283, '立即申请', 'Edit', 'LINK', 40, 3, 'AUTH', '71,167,286,287,424,477,417,420,421,441,442,640,641,636', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (284, '查询条件', 'Condition', 'CONDITION', 40, 1, 'AUTH', '70', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (285, '列表', 'List', 'LIST', 40, 1, 'AUTH', '70', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (286, '题库试卷列表', '/api/admin/exam/task/page_tiku_paper', 'URL', 40, 12, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (287, '题库试卷包', '/api/admin/exam/task/get_tiku_paper_data', 'URL', 40, 13, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (288, '查询（未审核）', 'NotReviewSelect', 'BUTTON', 41, 1, 'AUTH', '73', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (289, '导出审核样本（未审核）', 'NotReviewExport', 'BUTTON', 41, 2, 'AUTH', '163', 0, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (290, '批量不通过（未审核）', 'NotReviewBatchNotPass', 'BUTTON', 41, 3, 'AUTH', '75', 0, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (291, '批量通过（未审核）', 'NotReviewBatchPass', 'BUTTON', 41, 4, 'AUTH', '75', 0, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (292, '审核（未审核）', 'NotReviewEdit', 'LINK', 41, 1, 'AUTH', '74,417,420,421,427,441,442,477,417,640,641,642', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (293, '查询条件（未审核）', 'NotReviewCondition', 'CONDITION', 41, 1, 'AUTH', '73', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (294, '列表（未审核）', 'NotReviewList', 'LIST', 41, 1, 'AUTH', '73', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (295, '查询（已审核）', 'ReviewSelect', 'BUTTON', 41, 1, 'AUTH', '166', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (296, '查看详情（已审核）', 'ReviewPreview', 'LINK', 41, 1, 'AUTH', '74,477,417,420,421,427,441,442,640,641,642', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (297, '查询条件（已审核）', 'ReviewCondition', 'CONDITION', 41, 1, 'AUTH', '166', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (298, '列表（已审核）', 'ReviewList', 'LIST', 41, 1, 'AUTH', '166', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (299, '查询', 'Select', 'BUTTON', 42, 1, 'AUTH', '76', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (300, '批量下载试卷题卡', 'BatchDownload', 'BUTTON', 42, 2, 'AUTH', '191', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (301, '预览', 'Preview', 'LINK', 42, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (302, '启用/禁用', 'Enable', 'LINK', 42, 2, 'AUTH', '165', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (303, '下载', 'Download', 'LINK', 42, 3, 'AUTH', '164', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (304, '查询条件', 'Condition', 'CONDITION', 42, 1, 'AUTH', '76', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (305, '列表', 'List', 'LIST', 42, 1, 'AUTH', '76', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (306, '从题库选择试卷', 'SelectTikuPaper', 'BUTTON', 40, 13, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (308, '编辑', 'Edit', 'LINK', 42, 4, 'AUTH', '189', 0, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (309, '查询', 'Select', 'BUTTON', 43, 1, 'AUTH', '48,49', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (310, '新增', 'Add', 'BUTTON', 43, 2, 'AUTH', '50,49,193,194,621', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (311, '查看', 'Preview', 'LINK', 43, 1, 'AUTH', '56,57,58,105,825,826', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (312, '编辑', 'Edit', 'LINK', 43, 2, 'AUTH', '50', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (313, '删除', 'Delete', 'LINK', 43, 3, 'AUTH', '51', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (314, '查询条件', 'Condition', 'CONDITION', 43, 1, 'AUTH', '48', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (315, '列表', 'List', 'LIST', 43, 1, 'AUTH', '48,49', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (316, '查询', 'Select', 'BUTTON', 44, 1, 'AUTH', '52,53', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (317, '模版下载', 'TempleteDownload', 'BUTTON', 44, 2, 'AUTH', '157', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (318, '导出', 'Export', 'BUTTON', 44, 3, 'AUTH', '158', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (319, '导入', 'Import', 'BUTTON', 44, 4, 'AUTH', '54', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (320, '查看详情', 'Preview', 'LINK', 44, 1, 'AUTH', '55,327,328,329', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (321, '查询条件', 'Condition', 'CONDITION', 44, 1, 'AUTH', '52', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (322, '列表', 'List', 'LIST', 44, 1, 'AUTH', '52,53', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (323, '查询', 'Select', 'BUTTON', 45, 1, 'AUTH', '159', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (324, '预览', 'Preview', 'LINK', 45, 1, 'AUTH', '188', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (325, '查询条件', 'Condition', 'CONDITION', 45, 1, 'AUTH', '159', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (326, '列表', 'List', 'LIST', 45, 1, 'AUTH', '159', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (327, '新增考生', '/api/admin/exam/print/add_exam_student', 'URL', 44, 8, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (328, '查询考场下试卷列表', '/api/admin/exam/print/list_exam_detail_course', 'URL', 44, 9, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (329, '删除考生', '/api/admin/exam/print/delete_exam_student', 'URL', 44, 10, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (330, '重新生成pdf', 'CreatePdf', 'LINK', 47, 6, 'AUTH', '190', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (331, '查询', 'Select', 'BUTTON', 47, 1, 'AUTH', '59,62', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (332, '查看pdf', 'Download', 'LINK', 47, 1, 'AUTH', '161,162', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (333, '查询条件', 'Condition', 'CONDITION', 47, 1, 'AUTH', '59', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (334, '列表', 'List', 'LIST', 47, 1, 'AUTH', '59', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (335, '撤回提交', 'End', 'LINK', 47, 2, 'AUTH', '61', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (336, '批量下载PDF', 'BatchDownload', 'BUTTON', 47, 2, 'AUTH', '336', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (337, '查询', 'Select', 'BUTTON', 113, 1, 'AUTH', '114', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (338, '导出日志', 'Export', 'LINK', 113, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (339, '下载文件', 'Download', 'LINK', 113, 2, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (340, '查询条件', 'Condition', 'CONDITION', 113, 1, 'AUTH', '114', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (341, '列表', 'List', 'LIST', 113, 1, 'AUTH', '114', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (342, '预览', 'Preview', 'LINK', 9, 1, 'AUTH', '102', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (343, '编辑', 'Edit', 'LINK', 9, 2, 'AUTH', '103', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (345, '删除', 'Delete', 'LINK', 9, 3, 'AUTH', '104', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (346, '题卡审核-未审核查询', 'NotReviewSelect', 'BUTTON', 172, 1, 'AUTH', '173', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (347, '题卡审核-未审核批量下载试卷文件', 'NotReviewBatchDownload', 'BUTTON', 172, 2, 'AUTH', '185', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (348, '题卡审核-未审核查询条件', 'NotReviewCondition', 'CONDITION', 172, 1, 'AUTH', '173', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (349, '题卡审核-未审核列表', 'NotReviewList', 'LIST', 172, 1, 'AUTH', '173', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (350, '题卡审核-未审核预览', 'NotReviewPreview', 'LINK', 172, 1, 'AUTH', '173', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (351, '题卡审核-未审核设计题卡', 'NotReviewDesign', 'LINK', 172, 2, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (352, '题卡审核-已审核查询', 'ReviewSelect', 'BUTTON', 172, 1, 'AUTH', '173', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (353, '题卡审核-已审核查询条件', 'ReviewCondition', 'CONDITION', 172, 1, 'AUTH', '173', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (354, '题卡审核-已审核列表', 'ReviewList', 'LIST', 172, 1, 'AUTH', '173', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (355, '题卡审核-已审核预览', 'ReviewPreview', 'LINK', 172, 1, 'AUTH', '173', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (356, '字典管理', 'dictionary', 'MENU', 1, 4, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (357, '学生学院管理', 'CollegeManage', 'MENU', 356, 3, NULL, NULL, 0, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (358, '学生管理', 'StudentManage', 'MENU', 356, 6, NULL, NULL, 0, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (359, '查询', '/api/admin/basic/college/query', 'URL', 357, 1, 'AUTH', NULL, 0, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (360, '新增/编辑', '/api/admin/basic/college/save', 'URL', 357, 2, 'AUTH', NULL, 0, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (361, '删除（逻辑）', '/api/admin/basic/college/delete', 'URL', 357, 3, 'AUTH', NULL, 0, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (362, '查询', '/api/admin/basic/student/query', 'URL', 358, 1, 'AUTH', NULL, 0, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (363, '增/编辑', '/api/admin/basic/student/save', 'URL', 358, 2, 'AUTH', NULL, 0, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (364, '批量删除（逻辑）', '/api/admin/basic/student/delete_batch', 'URL', 358, 3, 'AUTH', NULL, 0, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (365, '批量导入（异步）', '/api/admin/basic/student/data_import', 'URL', 358, 4, 'AUTH', NULL, 0, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (366, '查询', '/api/admin/basic/course/query', 'URL', 13, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (367, '新增/编辑', '/api/admin/basic/course/save', 'URL', 13, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (368, '批量删除', '/api/admin/basic/course/delete_batch', 'URL', 13, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (369, '批量导入', '/api/admin/basic/course/data_import', 'URL', 13, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (370, '获取系统编码', '/api/admin/common/get_code', 'URL', 149, 5, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (371, '根据类型查询机构', '/api/admin/sys/org/find_by_type', 'URL', 4, 5, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (372, '查询', 'Select', 'BUTTON', 357, 1, 'AUTH', '359', 0, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (373, '列表', 'List', 'LIST', 357, 1, 'AUTH', '359', 0, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (374, '新增', 'Add', 'BUTTON', 357, 1, 'AUTH', '360,370', 0, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (375, '编辑', 'Edit', 'LINK', 357, 1, 'AUTH', '360', 0, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (376, '删除', 'Delete', 'LINK', 357, 1, 'AUTH', '361', 0, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (377, '查询', 'Select', 'BUTTON', 358, 1, 'AUTH', '362,462', 0, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (378, '列表', 'List', 'LIST', 358, 1, 'AUTH', '362', 0, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (379, '新增', 'Add', 'BUTTON', 358, 1, 'AUTH', '359,363,462', 0, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (380, '编辑', 'Edit', 'LINK', 358, 1, 'AUTH', '359,363', 0, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (381, '删除', 'Delete', 'LINK', 358, 1, 'AUTH', '364', 0, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (382, '导入', 'Import', 'BUTTON', 358, 2, 'AUTH', '365', 0, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (383, '启用/禁用', 'Enable', 'LINK', 9, 4, 'AUTH', '384', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (384, '启用/禁用', '/api/admin/basic/card_rule/enable', 'URL', 9, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (385, '新增', 'Add', 'BUTTON', 13, 1, 'AUTH', '367,462', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (388, '导入', 'Import', 'BUTTON', 13, 2, 'AUTH', '369', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (389, '机构列表', 'List', 'LIST', 13, 1, 'AUTH', NULL, 0, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (390, '流程管理', 'flow', 'MENU', 1, 3, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (391, '查询条件', 'Condition', 'CONDITION', 357, 1, 'AUTH', '359', 0, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (400, '查询条件', 'Condition', 'CONDITION', 358, 1, 'AUTH', '377', 0, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (402, '流程管理', 'FlowManage', 'MENU', 390, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (403, '流程审批记录管理', 'ApproveRecordManage', 'MENU', 390, 2, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (404, '查询', 'Select', 'BUTTON', 403, 1, 'AUTH', '418', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (405, '查询条件', 'Condition', 'CONDITION', 403, 1, 'AUTH', '418', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (406, '列表', 'List', 'LIST', 403, 1, 'AUTH', '418', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (407, '查询', 'Select', 'BUTTON', 402, 1, 'AUTH', '416', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (408, '新建流程', 'Register', 'BUTTON', 402, 2, 'AUTH', '184,414', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (409, '删除', 'Delete', 'LINK', 402, 2, 'AUTH', '415', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (410, '查询条件', 'Condition', 'CONDITION', 402, 1, 'AUTH', '416', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (411, '列表', 'List', 'LIST', 402, 1, 'AUTH', '416', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (413, '重新生成pdf', 'CreatePdf', 'LINK', 115, 3, 'AUTH', '190', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (414, '保存和发布流程', '/api/admin/custom/flow/save', 'URL', 402, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (415, '流程逻辑删除', '/api/admin/custom/flow/enable', 'URL', 402, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (416, '流程列表', '/api/admin/custom/flow/list', 'URL', 402, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (417, '审批流程', '/api/admin/custom/flow/task/approve', 'URL', 402, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (418, '流程审批记录列表', '/api/admin/custom/flow/approve/list', 'URL', 402, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (419, '流程审批记录逻辑删除', '/api/admin/custom/flow/approve/enable', 'URL', 402, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (420, '流程终止', '/api/admin/custom/flow/end', 'URL', 402, 7, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (421, '获取所有流程节点', '/api/admin/custom/flow/task/all', 'URL', 402, 8, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (422, '删除', 'Delete', 'LINK', 403, 1, 'AUTH', '419', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (427, '发布印刷任务-印刷室列表', '/api/admin/exam/task/list_task_print_house', 'URL', 42, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (437, '查询模板', 'List', 'LIST', 43, 1, 'AUTH', '49', 0, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (439, '题卡标题查询', '/api/admin/exam/task/find_card_title', 'URL', 40, 7, 'AUTH', NULL, 0, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (440, '题卡标题查询', 'List', 'LIST', 40, 1, 'AUTH', '439', 0, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (441, '根据流程类型获取流程节点', '/api/admin/custom/flow/get_flow_info_by_type', 'URL', 402, 9, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (442, '流程节点转他人审批', '/api/admin/custom/flow/task/approver/exchange', 'URL', 402, 10, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (446, '做废/恢复', '/api/admin/exam/print/task_normal', 'URL', 47, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (447, '作废/恢复', 'Normal', 'LINK', 47, 3, 'AUTH', '446', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (448, '指派/更改', 'Assign', 'LINK', 39, 4, 'AUTH', '64', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (450, '班级管理', 'ClazzManage', 'MENU', 356, 5, NULL, NULL, 0, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (451, '查询', '/api/admin/basic/clazz/query', 'URL', 450, 1, 'AUTH', NULL, 0, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (452, '新增/编辑', '/api/admin/basic/clazz/save', 'URL', 450, 2, 'AUTH', NULL, 0, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (453, '批量删除（逻辑）', '/api/admin/basic/clazz/delete_batch', 'URL', 450, 3, 'AUTH', NULL, 0, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (455, '查询', 'Select', 'BUTTON', 450, 1, 'AUTH', '451', 0, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (456, '列表', 'List', 'LIST', 450, 1, 'AUTH', '451', 0, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (457, '新增', 'Add', 'BUTTON', 450, 1, 'AUTH', '452', 0, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (458, '编辑', 'Edit', 'LINK', 450, 1, 'AUTH', '452', 0, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (459, '删除', 'Delete', 'LINK', 450, 1, 'AUTH', '453', 0, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (461, '查询条件', 'Condition', 'CONDITION', 450, 1, 'AUTH', '451', 0, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (462, '班级数据源查询', '/api/admin/basic/clazz/datasource', 'URL', 450, 5, 'AUTH', NULL, 0, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (463, '新建任务', 'ExamTaskApplyManage', 'BUTTON', 40, 8, 'AUTH', '49,167,174,286,287,371,464,465,427,642,417,420,421,441,442,640,641,642', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (464, '新建命题申请', '/api/admin/exam/task/submit_task_apply', 'URL', 40, 8, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (465, '新建命题申请-班级列表', '/api/admin/exam/task/list_task_apply_class', 'URL', 39, 9, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (466, '命题任务-审批-获取试卷审批表数据', '/api/admin/exam/task/find_approve_form_json', 'URL', 39, 10, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (467, '命题任务-审批-获取试卷审批表pdf', '/api/admin/exam/task/find_approve_form_pdf', 'URL', 39, 11, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (468, '下载审批表', 'Download', 'LINK', 40, 5, 'AUTH', '466,467', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (469, '命题计划统计', 'Statistics', 'MENU', 14, 2, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (470, '命题计划统计表', 'StatisticsManage', 'MENU', 469, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (471, '查询', 'Select', 'BUTTON', 470, 1, 'AUTH', '476', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (472, '模板下载', 'Download', 'BUTTON', 470, 2, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (473, '导入', 'Import', 'BUTTON', 470, 3, 'AUTH', '475', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (474, '列表', 'List', 'LIST', 470, 1, 'AUTH', '476', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (475, '导入', '/api/admin/statistics/import', 'URL', 470, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (476, '列表', '/api/admin/statistics/list', 'URL', 470, 2, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (477, '命题任务-考试对象查询', '/api/admin/exam/task/find_exam_object', 'URL', 39, 12, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (478, '刷新数据', 'Freshen', 'BUTTON', 470, 4, 'AUTH', '480', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (479, '删除数据', 'Delete', 'BUTTON', 470, 5, 'AUTH', '481', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (480, '刷新数据', '/api/admin/statistics/freshen', 'URL', 470, 3, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (481, '删除数据', '/api/admin/statistics/delete', 'URL', 470, 4, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (483, '评卷管理', 'stmms', 'MENU', NULL, 6, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (486, '评卷管理', 'MarkTaskManage', 'MENU', 483, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (487, '成绩查询', 'ScoreManage', 'MENU', 561, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (488, '班级详情', 'ClassScoreDetail', 'LINK', 487, 1, 'AUTH', '490,543,955,959,1139', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (489, '成绩报告', 'ScoreReport', 'LINK', 487, 2, 'AUTH', '494,544', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (490, '列表查询', '/api/admin/mark/archive/student/list', 'URL', 488, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (491, '评卷入口', 'MarkEntrance', 'MENU', 486, 1, NULL, '492,493,541', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (492, '评卷员登录', '/api/admin/exam/sso/marker_login', 'URL', 491, 3, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (493, '科组长登录', '/api/admin/exam/sso/marker_leader_login', 'URL', 491, 4, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (494, '报告数据', '/api/admin/mark/archive/score/report', 'URL', 489, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (495, '分班阅（开关）', 'OpenClassReading', 'BUTTON', 897, 3, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (498, '列表', 'List', 'LIST', 487, 1, 'AUTH', '542,959,1139', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (499, '查询条件', 'Condition', 'CONDITION', 487, 1, 'AUTH', '542,959,1139', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (500, '查询', 'Select', 'BUTTON', 487, 1, 'AUTH', '542,959,1139', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (506, '归档管理', 'ScoreArchiveManage', 'MENU', NULL, 7, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (507, '菜单管理', 'MenuManage', 'MENU', 1, 7, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (508, '自定义菜单', 'CustomMenu', 'MENU', 507, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (509, '同步', 'SyncUser', 'BUTTON', 5, 1, 'AUTH', '532', 0, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (510, '学期管理', 'SemesterManage', 'MENU', 356, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (511, '新增', 'Add', 'BUTTON', 510, 1, 'AUTH', '534', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (512, '编辑', 'Edit', 'LINK', 510, 1, 'AUTH', '534', 0, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (513, '删除', 'Delete', 'LINK', 510, 2, 'AUTH', '535', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (514, '列表', 'List', 'LIST', 510, 1, 'AUTH', '533', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (515, '专业管理', 'ProfessionalManage', 'MENU', 356, 4, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (516, '新增', 'Add', 'BUTTON', 515, 1, 'AUTH', '537', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (517, '编辑', 'Edit', 'LINK', 515, 1, 'AUTH', '537', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (518, '删除', 'Delete', 'LINK', 515, 2, 'AUTH', '538', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (519, '列表', 'List', 'LIST', 515, 1, 'AUTH', '536,540', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (520, '查询条件', 'Condition', 'CONDITION', 515, 1, 'AUTH', '536,540', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (521, '同步管理', 'SyncManage', 'MENU', 112, 2, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (522, '查询', 'Select', 'BUTTON', 515, 1, 'AUTH', '536,540', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (523, '查询', 'Select', 'BUTTON', 521, 1, 'AUTH', '557', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (524, '查询条件', 'Condition', 'CONDITION', 521, 1, 'AUTH', '557', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (525, '列表', 'List', 'LIST', 521, 1, 'AUTH', '557', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (526, '导出日志', 'Export', 'LINK', 521, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (527, '查询自定义菜单权限', '/api/admin/set/menu/custom/list', 'URL', 508, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (528, '学校已绑定自定义菜单权限列表', '/api/admin/set/menu/custom/get_school_custom_privileges', 'URL', 508, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (529, '学校新增/修改自定义菜单权限', '/api/admin/set/menu/custom/save', 'URL', 508, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (530, '查询', 'Select', 'BUTTON', 508, 1, 'AUTH', '527,528', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (531, '保存', 'Save', 'BUTTON', 508, 2, 'AUTH', '529', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (532, '同步', '/api/admin/sys/user/push', 'URL', 5, 8, 'AUTH', NULL, 0, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (533, '查询', '/api/admin/basic/semester/query', 'URL', 510, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (534, '新增/编辑', '/api/admin/basic/semester/save', 'URL', 510, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (535, '删除', '/api/admin/basic/semester/delete', 'URL', 510, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (536, '查询', '/api/admin/basic/professional/page', 'URL', 515, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (537, '新增/编辑', '/api/admin/basic/professional/save', 'URL', 515, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (538, '删除', '/api/admin/basic/professional/remove', 'URL', 515, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (539, '共用接口-查询字典', '/api/admin/common/get_dictionary', 'URL', 149, 7, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (540, '查询（不分页）', '/api/admin/basic/professional/list', 'URL', 515, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (541, '获取用户阅卷角色', '/api/admin/common/get_open_role', 'URL', 491, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (542, '查询接口', '/api/admin/mark/archive/score/list', 'URL', 487, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (543, '成绩导出接口', '/api/admin/mark/archive/score/export', 'URL', 488, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (544, '报告下载', '/api/admin/mark/archive/score/report/download', 'URL', 489, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (545, '成绩下载', 'DownloadScore', 'BUTTON', 487, 2, 'AUTH', '546', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (546, '成绩下载', '/api/admin/mark/archive/score/download', 'URL', 487, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (547, '阅卷试卷上传待办计数接口', '/api/admin/work/structure/task/ready_count', 'URL', 16, 7, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (548, '阅卷试卷上传待办接口', '/api/admin/work/structure/task/ready', 'URL', 16, 8, 'SYS', '490', 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (549, '短信日志', 'SmsManage', 'MENU', 112, 3, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (550, '查询', 'Select', 'BUTTON', 549, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (551, '查询', '/api/admin/basic/message/list', 'URL', 550, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (552, '查询短信类型', '/api/admin/basic/message/list_message_types', 'URL', 550, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (553, '短信失败重发', '/api/admin/basic/message/resend', 'URL', 550, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (554, '查询条件', 'Condition', 'CONDITION', 549, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (555, '列表', 'List', 'LIST', 549, 1, 'AUTH', '551,552', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (556, '短信失败重发', 'Send', 'LINK', 549, 1, 'AUTH', '553', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (557, '查询', '/api/admin/data/sync/query', 'URL', 521, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (558, '模糊查询', '/api/admin/basic/semester/list', 'URL', 510, 1, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (559, '启用/禁用', 'Enable', 'LINK', 510, 1, 'AUTH', '560', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (560, '启用/禁用', '/api/admin/basic/semester/enable', 'URL', 510, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (561, '归档管理', 'ScoreArchivePrimary', 'MENU', 506, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (562, '提交印刷', 'Submit', 'LINK', 47, 5, 'AUTH', '60', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (564, '考试试卷文件上传接口', '/api/admin/common/file/exam_paper_upload', 'URL', 144, 5, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (566, '审核-印刷员驳回后命题老师修改', '/api/admin/exam/task/apply_resave', 'URL', 40, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (567, '查看', 'Preview', 'LINK', 403, 1, 'AUTH', '168,568,640,641,477', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (568, '查询任务详情', '/api/admin/exam/task/exam_task_get_one', 'URL', 40, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (569, '根据条件删除（物理）', '/api/admin/basic/student/delete_by_condition', 'URL', 358, 5, 'AUTH', NULL, 0, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (570, '批量删除', 'Delete', 'BUTTON', 358, 1, 'AUTH', '569', 0, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (571, '批量删除', 'Delete', 'BUTTON', 450, 1, 'AUTH', '453', 0, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (572, '批量删除', 'Delete', 'BUTTON', 515, 1, 'AUTH', '538', 0, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (573, '根据查询条件批量删除', '/api/admin/basic/course/delete_by_query', 'URL', 13, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (574, '批量删除', 'Delete', 'BUTTON', 13, 5, 'AUTH', '573', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (575, '课程管理', 'basicInfo', 'MENU', 650, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (576, '课程管理', 'CourseSimpleManage', 'MENU', 575, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (577, '文档管理', 'CourseDocumentManage', 'MENU', 561, 2, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (578, '列表', 'List', 'LIST', 577, 1, 'AUTH', '583', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (579, '查询条件', 'Condition', 'CONDITION', 577, 2, 'AUTH', '583', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (580, '查询', 'Select', 'BUTTON', 577, 3, 'AUTH', '583', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (581, '新建文件', 'Add', 'BUTTON', 577, 4, 'AUTH', '584', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (582, '查看详情', 'Detail', 'LINK', 577, 5, 'AUTH', '466,467,494,544,585,586,587,588,3059', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (583, '文档管理列表', '/api/admin/mark/archive/document/list', 'URL', 577, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (584, '文档新建', '/api/admin/mark/archive/document/save', 'URL', 577, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (585, '文档详情', '/api/admin/mark/archive/document/detail', 'URL', 577, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (586, '文档上传', '/api/admin/mark/archive/document/import', 'URL', 577, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (587, '文档删除', '/api/admin/mark/archive/document/delete', 'URL', 577, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (588, '文档下载', '/api/admin/mark/archive/document/download', 'URL', 577, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (598, '教学学生-查询', '/api/admin/teach/student/page', 'URL', 597, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (599, '教学学生-创建教学学生', '/api/admin/teach/student/create', 'URL', 597, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (600, '教学学生-批量删除', '/api/admin/teach/student/delete', 'URL', 597, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (601, '教学学生-批量导入', '/api/admin/teach/student/import', 'URL', 597, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (602, '教学学生-列表', 'List', 'LIST', 597, 1, 'AUTH', '598', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (603, '教学学生-查询', 'Select', 'BUTTON', 597, 2, 'AUTH', '598', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (604, '教学学生-新增', 'Add', 'BUTTON', 597, 3, 'AUTH', '599', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (605, '教学学生-批量导入', 'Import', 'BUTTON', 597, 4, 'AUTH', '601', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (606, '教学学生-删除', 'Delete', 'LINK', 597, 5, 'AUTH', '600', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (607, '教学学生-批量删除', 'DeleteBatch', 'BUTTON', 597, 6, 'AUTH', '600', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (609, '考试管理', 'ExamManage', 'MENU', 356, 2, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (610, '查询', 'Select', 'BUTTON', 609, 1, 'AUTH', '616', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (611, '新增', 'Add', 'BUTTON', 609, 3, 'AUTH', '617', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (612, '启用/禁用', 'Enable', 'LINK', 609, 4, 'AUTH', '618', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (613, '删除', 'Delete', 'LINK', 609, 5, 'AUTH', '619', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (614, '设置', 'Set', 'LINK', 609, 6, 'AUTH', '620,621,622,623,49', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (615, '查询条件', 'Condition', 'CONDITION', 609, 2, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (616, '查询', '/api/admin/basic/exam/page', 'URL', 609, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (617, '新增/修改', '/api/admin/basic/exam/save', 'URL', 609, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (618, '启用/禁用', '/api/admin/basic/exam/enable', 'URL', 609, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (619, '删除', '/api/admin/basic/exam/delete', 'URL', 609, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (620, '设置', '/api/admin/basic/exam/set_in_userd', 'URL', 609, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (621, '查询', '/api/admin/basic/print_config/page', 'URL', 614, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (622, '新增/修改', '/api/admin/basic/print_config/save', 'URL', 614, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (623, '启用/禁用', '/api/admin/basic/print_config/enable', 'URL', 614, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (624, '题卡管理', 'CardManage', 'MENU', 3, 3, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (625, '查询', 'Select', 'BUTTON', 624, 1, 'AUTH', '627', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (626, '查询条件', 'Condition', 'CONDITION', 624, 2, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (627, '查询', '/api/admin/exam/card/page', 'URL', 624, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (628, '编辑', 'Edit', 'LINK', 609, 7, 'AUTH', '617', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (629, '新增', 'Add', 'BUTTON', 624, 3, 'AUTH', '631', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (630, '编辑题卡/编辑信息', 'Edit', 'LINK', 624, 4, 'AUTH', '631', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (631, '新增/修改', '/api/admin/exam/card/save_generic', 'URL', 624, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (632, '列表', 'List', 'LIST', 609, 7, 'AUTH', '616', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (633, '列表', 'List', 'LIST', 624, 5, 'AUTH', '627', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (634, '发送验证码（绑定手机号）', '/api/admin/common/get_verify_code_for_bind', 'URL', 130, 3, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (635, '删除', 'Delete', 'LINK', 624, 6, 'AUTH', '636', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (636, '删除', '/api/admin/exam/card/delete_generic', 'URL', 624, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (637, '模糊查询', '/api/admin/basic/exam/query', 'URL', 609, 4, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (638, '编辑', 'Edit', 'LINK', 402, 1, 'AUTH', '639', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (639, '编辑流程', '/api/admin/custom/flow/edit', 'URL', 402, 14, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (640, '获取当前流程节点信息', '/api/admin/custom/flow/task/info', 'URL', 402, 15, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (641, '流程信息', '/api/admin/custom/flow/view', 'URL', 402, 16, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (642, '根据考试ID和机构ID查询印品', '/api/admin/basic/print_config/get_by_exam_id_and_org_id', 'URL', 614, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (643, '重命名', 'Rename', 'LINK', 402, 3, 'AUTH', '643', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (644, '重命名自定义流程名称', '/api/admin/custom/flow/rename', 'URL', 402, 17, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (647, '成员管理', 'Manager', 'LINK', 6, 3, 'AUTH', '648,649', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (648, '新增/修改', '/api/admin/sys/role/group/save', 'URL', 647, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (649, '查询', '/api/admin/sys/role/group/list', 'URL', 647, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (650, '课程管理', 'baseInfo', 'MENU', NULL, 2, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (651, '电子交卷', 'electronic', 'MENU', NULL, 4, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (652, '电子交卷', 'electronicSubmit', 'MENU', 651, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (653, '下载管理', 'DownloadManage', 'MENU', 112, 4, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (654, '查询', 'Select', 'BUTTON', 653, 1, 'AUTH', '659', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (655, '查询条件', 'Condition', 'CONDITION', 653, 2, 'AUTH', '659', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (656, '下载文件', 'Download', 'LINK', 653, 3, 'AUTH', '660', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (657, '批量下载文件', 'Download', 'BUTTON', 653, 4, 'AUTH', '661', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (658, '列表', 'List', 'LIST', 653, 5, 'AUTH', '659', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (659, '查询', '/api/admin/data/download/page', 'URL', 653, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (660, '下载文件', '/api/admin/data/download/download_one', 'URL', 653, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (661, '批量下载文件', '/api/admin/data/download/download_batch', 'URL', 653, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (662, '教学学生-从课程班级批量添加学生', '/api/admin/teach/student/create_batch', 'URL', 597, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (663, '教学学生-从课程班级添加', 'SaveBatch', 'BUTTON', 597, 7, 'AUTH', '662', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (664, '重新同步', 'Push', 'LINK', 521, 1, 'AUTH', '665', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (665, '重新同步', '/api/admin/data/sync/resync', 'URL', 521, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (672, '获取题卡图片', '/api/admin/exam/card/find_jpg_file', 'URL', 488, 8, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (675, '授权配置', 'AuthManage', 'MENU', 1, 6, NULL, NULL, 1, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (676, '授权配置-导出硬件信息按钮', 'Export', 'BUTTON', 675, 1, 'AUTH', '677,678,679,794', 1, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (677, '导出硬件信息', '/api/admin/auth/export/device/info', 'URL', 676, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (678, '离线激活', '/api/admin/auth/offline/activation', 'URL', 676, 2, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (679, '查询激活信息', '/api/admin/auth/select', 'URL', 676, 3, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (680, '教研分析', 'analyze', 'MENU', NULL, 8, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (681, '分析批次管理', 'AnalysisBatchManage', 'MENU', 680, 2, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (682, '查询', '/api/admin/grade/batch/page', 'URL', 681, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (683, '新建', '/api/admin/grade/batch/save', 'URL', 681, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (684, '删除', '/api/admin/grade/batch/delete', 'URL', 681, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (685, '推送', '/api/admin/grade/batch/push', 'URL', 681, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (686, '查询', 'Select', 'BUTTON', 681, 1, 'AUTH', '682', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (687, '列表', 'List', 'LIST', 681, 2, 'AUTH', '682', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (688, '新建', 'Add', 'BUTTON', 681, 3, 'AUTH', '683', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (689, '批量删除', 'Delete', 'BUTTON', 681, 4, 'AUTH', '684', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (690, '删除', 'Delete', 'LINK', 681, 5, 'AUTH', '684', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (691, '推送', 'Push', 'LINK', 681, 6, 'AUTH', '685', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (692, '添加分析课程', 'Window', 'LINK', 681, 7, 'AUTH', '693,694,695,696,704,705', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (693, '查询', '/api/admin/grade/batch/paper/page', 'URL', 692, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (694, '新建', '/api/admin/grade/batch/paper/save', 'URL', 692, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (695, '删除', '/api/admin/grade/batch/paper/delete', 'URL', 692, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (696, '查询可分析试卷', '/api/admin/grade/batch/paper/able', 'URL', 692, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (697, '课程查询', 'Select', 'BUTTON', 692, 1, 'AUTH', '693', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (698, '课程列表', 'List', 'LIST', 692, 2, 'AUTH', '693', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (699, '课程添加分析课程', 'Add', 'BUTTON', 692, 3, 'AUTH', '694', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (700, '课程批量删除', 'Delete', 'BUTTON', 692, 4, 'AUTH', '695', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (701, '课程删除', 'Delete', 'BUTTON', 692, 5, 'AUTH', '695', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (702, '可分析课程查询', 'Select', 'BUTTON', 692, 6, 'AUTH', '696', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (703, '可分析课程列表', 'List', 'LIST', 692, 7, 'AUTH', '696', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (704, '推送', '/api/admin/grade/batch/paper/push', 'URL', 692, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (705, '计算', '/api/admin/grade/batch/paper/calculate', 'URL', 692, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (706, '推送', 'Push', 'LINK', 692, 8, 'AUTH', '704', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (707, '计算', 'Calculate', 'LINK', 692, 9, 'AUTH', '705', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (708, '蓝图设置', 'DataInitManage', 'MENU', 680, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (709, '成绩分析课程数据初始化', '/api/admin/grade/paper/initialize', 'URL', 708, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (710, '查询', 'Select', 'BUTTON', 708, 1, 'AUTH', '709', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (711, '列表', 'List', 'LIST', 708, 2, 'AUTH', '709', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (712, '成绩分析基础配置', 'Window', 'LINK', 708, 3, 'AUTH', '717,718,719,720,726,727,728,729,735,736,737,738,755', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (717, '查询', '/api/admin/grade/paper/dimension/list', 'URL', 712, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (718, '新建维度', '/api/admin/grade/paper/dimension/save', 'URL', 712, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (719, '模板导出', '/api/admin/grade/paper/dimension/export', 'URL', 712, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (720, '导入', '/api/admin/grade/paper/dimension/import', 'URL', 712, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (721, '查询', 'Select', 'BUTTON', 712, 1, 'AUTH', '717', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (722, '列表', 'List', 'LIST', 712, 2, 'AUTH', '717', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (723, '新建', 'Add', 'BUTTON', 712, 3, 'AUTH', '718', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (724, '导入', 'Import', 'BUTTON', 712, 4, 'AUTH', '720', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (725, '模板导出', 'Export', 'BUTTON', 712, 5, 'AUTH', '719', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (726, '查询', '/api/admin/grade/paper/struct/list', 'URL', 712, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (727, '新建试卷结构', '/api/admin/grade/paper/struct/save', 'URL', 712, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (728, '导入', '/api/admin/grade/paper/struct/import', 'URL', 712, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (729, '模板导出', '/api/admin/grade/paper/struct/export', 'URL', 712, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (730, '查询', 'Select', 'BUTTON', 712, 1, 'AUTH', '726', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (731, '列表', 'List', 'LIST', 712, 2, 'AUTH', '726', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (732, '导入', 'Import', 'BUTTON', 712, 3, 'AUTH', '728', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (734, '模板导出', 'Export', 'BUTTON', 712, 4, 'AUTH', '729', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (735, '查询定义', '/api/admin/grade/paper/paper_define', 'URL', 712, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (736, '新建定义', '/api/admin/grade/paper/save_define', 'URL', 712, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (737, '查询', '/api/admin/grade/module/evaluation/list', 'URL', 712, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (738, '新建模块评价', '/api/admin/grade/module/evaluation/save', 'URL', 712, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (739, '查看报告', 'AnalysisReportView', 'MENU', 680, 3, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (740, '教研分析单点登录', '/api/admin/exam/sso/analysis_login', 'URL', 149, 1, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (741, '考务数据导出', 'Export', 'LINK', 681, 8, 'AUTH', '743', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (742, '考务数据导入', 'Import', 'LINK', 681, 9, 'AUTH', '744', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (743, '考务数据导出', '/api/admin/grade/batch/download', 'URL', 681, 10, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (744, '考务数据导入', '/api/admin/grade/batch/upload', 'URL', 681, 11, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (745, '教研待办接口', '/api/admin/work/grade/task/ready', 'URL', 16, 9, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (746, '教研待办计数接口', '/api/admin/work/grade/task/ready_count', 'URL', 16, 10, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (747, '开始计算', 'StartCalc', 'LINK', 681, 12, 'AUTH', '748', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (748, '开始计算', '/api/admin/grade/batch/start_calc', 'URL', 681, 13, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (749, '查询条件', 'Condition', 'CONDITION', 708, 4, 'AUTH', '709', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (750, '查询条件', 'Condition', 'CONDITION', 681, 14, 'AUTH', '682', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (751, '课程发布', 'Publish', 'LINK', 681, 6, 'AUTH', '752', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (752, '课程发布', '/api/admin/grade/batch/paper/publish', 'URL', 681, 7, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (755, '更新试卷机构', '/api/admin/grade/paper/struct/change_paper_structure', 'URL', 712, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (756, '查询已审核数量', '/api/admin/exam/task/review_list_audited_count', 'URL', 41, 6, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (757, '批量导入', '/api/admin/sys/org/import', 'URL', 4, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (758, '导入', 'Import', 'BUTTON', 4, 4, 'AUTH', '757', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (759, '考生对象查询', '/api/admin/exam/task/find_exam_task_student_object', 'URL', 40, 9, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (760, '增加考试对象', 'ExamTaskStudentObject', 'BUTTON', 40, 9, 'AUTH', '759', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (761, '导入考生对象', '/api/admin/exam/task/exam_task_exam_student_import', 'URL', 40, 10, 'AUTH', NULL, 0, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (762, '批量导入考生', 'ExamTaskStudentObjectImport', 'BUTTON', 40, 10, 'AUTH', '761', 0, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (763, '查询详情', '/api/admin/basic/template/get_one', 'URL', 11, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (764, '复制', '/api/admin/exam/card/copy', 'URL', 175, 4, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (765, '查询开课学院', '/api/admin/grade/batch/paper/find_teach_college', 'URL', 692, 7, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (766, '查询条件', 'Condition', 'CONDITION', 470, 1, 'AUTH', '476', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (767, '同步机构数据', '/api/admin/auth/sync/org', 'URL', 676, 4, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (768, '题库单点登录', '/api/admin/exam/sso/question_library_login', 'URL', 149, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (769, '题库管理', 'questionLibrary', 'MENU', NULL, 3, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (770, '题库管理', 'QuestionLibraryManage', 'MENU', 769, 1, NULL, '768', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (771, '用户验证查询', '/api/admin/set/user/select', 'URL', 508, 4, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (772, '用户验证保存', '/api/admin/set/user/save', 'URL', 508, 5, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (773, '试卷规格配置保存', '/api/admin/set/paper/save', 'URL', 508, 6, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (774, '试卷规格配置查询', '/api/admin/set/paper/select', 'URL', 508, 7, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (775, '系统试卷规格配置查询', '/api/admin/set/paper/sys/select', 'URL', 508, 8, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (776, '同步配置保存', '/api/admin/set/sync/save', 'URL', 508, 9, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (777, '同步配置查询', '/api/admin/set/sync/select', 'URL', 508, 10, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (778, '数据还原', '/api/admin/set/backup', 'URL', 508, 11, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (779, '查询自定义角色权限', '/api/admin/set/role/custom/list', 'URL', 508, 12, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (780, '学校新增/修改自定义角色权限', '/api/admin/set/role/custom/save', 'URL', 508, 13, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (781, '学校已绑定自定义角色权限列表', '/api/admin/set/role/custom/get_school_custom_roles', 'URL', 508, 14, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (782, '操作日志', 'ActionLogManage', 'MENU', 112, 5, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (783, '查询', '/api/admin/basic/operation_log/query', 'URL', 782, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (784, '查询条件', 'Condition', 'CONDITION', 782, 1, 'AUTH', '783', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (785, '查询', 'Select', 'BUTTON', 782, 2, 'AUTH', '783', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (786, '课程目标达成度模板配置查询', '/api/admin/set/course/degree/select', 'URL', 508, 15, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (787, '课程目标达成度模板配置保存', '/api/admin/set/course/degree/save', 'URL', 508, 16, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (788, '删除', 'Delete', 'LINK', 11, 4, 'AUTH', '789', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (789, '删除', '/api/admin/basic/template/remove', 'URL', 11, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (790, '查询异步结果', '/api/admin/data/task/query_result', 'URL', 113, 3, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (794, '在线激活', '/api/admin/auth/online/activation', 'URL', 676, 4, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (795, '学校管理', 'SchoolManage', 'MENU', 1, 7, NULL, NULL, 1, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (796, '查询', 'Select', 'BUTTON', 795, 1, 'AUTH', '798', 1, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (797, '查询条件', 'Condition', 'CONDITION', 795, 2, 'AUTH', '798', 1, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (798, '查询', '/api/admin/commom/school/list', 'URL', 795, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (799, '机构同步', 'Sync', 'BUTTON', 795, 3, 'AUTH', '800', 1, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (800, '机构同步', '/api/admin/auth/sync/org', 'URL', 795, 2, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (801, '编辑', 'Edit', 'LINK', 795, 1, 'AUTH', '802', 1, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (802, '编辑', '/api/admin/commom/school/update', 'URL', 795, 3, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (803, '列表', 'List', 'LIST', 782, 3, 'AUTH', '783', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (804, '删除', 'Delete', 'LINK', 39, 5, 'AUTH', '805', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (805, '删除', '/api/admin/exam/task/delete', 'URL', 39, 13, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (806, '删除', 'Delete', 'LINK', 40, 6, 'AUTH', '807', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (807, '删除', '/api/admin/exam/task/apply_delete', 'URL', 40, 11, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (808, '删除', 'Delete', 'LINK', 44, 2, 'AUTH', '809', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (809, '删除', '/api/admin/exam/print/data_delete', 'URL', 44, 7, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (810, '导出', 'Export', 'BUTTON', 4, 5, 'AUTH', '811', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (811, '导出', '/api/admin/sys/org/export', 'URL', 4, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (812, '导出', 'Export', 'BUTTON', 5, 4, 'AUTH', '813', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (813, '导出', '/api/admin/sys/user/export', 'URL', 5, 10, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (814, '导出', 'Export', 'BUTTON', 358, 3, 'AUTH', '815', 0, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (815, '导出', '/api/admin/basic/student/export', 'URL', 358, 6, 'AUTH', NULL, 0, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (816, '导出', 'Export', 'BUTTON', 13, 6, 'AUTH', '817', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (817, '导出', '/api/admin/basic/course/export', 'URL', 13, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (825, '命题任务公共卷型查询接口', '/api/admin/exam/print/list_relate_common_types', 'URL', 43, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (826, '批量关联接口', '/api/admin/exam/print/relate_update_batch', 'URL', 43, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (827, '超管-角色管理查询', '/api/admin/sys/role/list_to_admin', 'URL', 5, 8, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (828, '生成图片/预览图片', 'ConvertImage', 'LINK', 624, 7, 'AUTH', '829', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (829, '生成图片', '/api/admin/exam/card/convert_image', 'URL', 624, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (830, '教学课程-分页查询', '/api/admin/teach/course/page', 'URL', 576, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (831, '教学课程-手动添加', '/api/admin/teach/course/manual_create', 'URL', 576, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (832, '教学课程-课程表选择并添加', '/api/admin/teach/course/select_create', 'URL', 576, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (833, '教学课程-批量导入', '/api/admin/teach/course/import', 'URL', 576, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (834, '教学课程-批量删除', '/api/admin/teach/course/delete', 'URL', 576, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (835, '教学课程-列表', 'List', 'LIST', 576, 1, 'AUTH', '830', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (836, '查询', 'Select', 'BUTTON', 576, 2, 'AUTH', '830', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (837, '删除', 'Delete', 'LINK', 576, 3, 'AUTH', '834', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (838, '查询条件', 'Condition', 'CONDITION', 576, 4, 'AUTH', '830', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (839, '基础课程-根据学院查询课程', '/api/admin/basic/course/list_by_org', 'URL', 13, 7, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (840, '新增', 'AddCourse', 'BUTTON', 576, 5, 'AUTH', '831,832,833,839', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (841, '管理', 'Link', 'LINK', 576, 6, 'AUTH', '77,842,843,844,845,846,847,848,849,850,851,852,853,866,1115,3018,1117,1118,1119,1120,1121,1122,1123,1124,1125,1126,1128,1129', 1, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (842, '教学班-分页查询', '/api/admin/teach/clazz/page', 'URL', 841, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (843, '教学班-编辑', '/api/admin/teach/clazz/edit', 'URL', 841, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (844, '教学班-删除', '/api/admin/teach/clazz/delete', 'URL', 841, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (845, '教师团队-分页查询', '/api/admin/teach/course/teacher_team_page', 'URL', 841, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (846, '教师团队-手动添加', '/api/admin/teach/course/manual_create_teacher', 'URL', 841, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (847, '教师团队-教师库添加', '/api/admin/teach/course/select_create_teacher', 'URL', 841, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (848, '教师团队-批量导入教师', '/api/admin/teach/course/import_teacher', 'URL', 841, 7, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (849, '教学学生-创建教学学生', '/api/admin/teach/student/create', 'URL', 841, 8, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (850, '教学学生-查询', '/api/admin/teach/student/page', 'URL', 841, 9, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (851, '教学学生-从课程班级批量添加学生', '/api/admin/teach/student/create_batch', 'URL', 841, 10, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (852, '教学学生-批量导入', '/api/admin/teach/student/import', 'URL', 841, 11, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (853, '教学学生-批量删除', '/api/admin/teach/student/delete', 'URL', 841, 12, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (854, '指定用户添加课程', 'AssginCourseUser', 'BUTTON', '576', '6', 'AUTH', NULL,'1', '0', '1');
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (856, '查看', 'Preview', 'LINK', 624, 8, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (860, '共用接口-查询学期', '/api/admin/basic/condition/list_semester', 'URL', 149, 8, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (861, '共用接口-查询考试', '/api/admin/basic/condition/list_exam', 'URL', 149, 9, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (862, '共用接口-查询计划', '/api/admin/basic/condition/list_print_plan', 'URL', 149, 10, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (863, '共用接口-查询课程', '/api/admin/basic/condition/list_course', 'URL', 149, 11, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (864, '共用接口-查询试卷编号', '/api/admin/basic/condition/list_paper_number', 'URL', 149, 12, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (865, '共用接口-查询班级', '/api/admin/basic/condition/list_clazz', 'URL', 149, 13, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (866, '教师团队-根据机构查询', '/api/admin/teach/course/find_teacher_by_org', 'URL', 841, 13, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (867, '下载', 'Download', 'LINK', 624, 9, 'AUTH', '868', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (868, '下载', '/api/admin/exam/card/download_card', 'URL', 624, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (869, '试卷题卡预览日志记录接口', '/api/admin/basic/operation_log/save_preview_log', 'URL', 149, 14, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (870, '启用/禁用', 'Enable', 'LINK', 624, 10, 'AUTH', '871', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (871, '启用/禁用', '/api/admin/exam/card/enable', 'URL', 624, 7, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (872, '批量撤回', 'BatchEnd', 'BUTTON', 47, 3, 'AUTH', '873', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (873, '批量撤回', '/api/admin/exam/print/task_batch_cancel', 'URL', 47, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (874, '共用接口-查询学院', '/api/admin/basic/condition/list_college', 'URL', 149, 14, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (875, '我教课程选择', 'MyTeachCourse', 'BUTTON', 40, 10, 'AUTH', '759', 0, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (876, '考生库选择', 'StudentLibrary', 'BUTTON', 40, 11, 'AUTH', '759', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (877, '考生库选择（是否可选学生）', 'SelectStudent', 'BUTTON', 40, 12, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (878, '作废', 'Cancel', 'LINK', 42, 5, 'AUTH', '879', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (879, '作废', '/api/admin/exam/task/cancel', 'URL', 42, 8, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (880, '重启流程', '/api/admin/exam/task/restart', 'URL', 42, 9, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (881, '打回', 'Restart', 'LINK', 42, 6, 'AUTH', '880', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (882, '是否开启AB卷', 'OpenAb', 'BUTTON', 40, 13, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (884, '列表', 'List', 'LIST', 491, 1, 'AUTH', '887', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (885, '查询', 'Select', 'BUTTON', 491, 2, 'AUTH', '887', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (886, '点击评卷', 'Marking', 'LINK', 491, 3, 'AUTH', '888,889,890,891,892,893,894,895,896', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (887, '查询', '/api/admin/mark/entrance/list', 'URL', 491, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (888, '评卷页面', '/api/mark/getSetting', 'URL', 491, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (889, '获取分组列表', '/api/mark/getGroup', 'URL', 491, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (890, '获取评卷状态', '/api/mark/getStatus', 'URL', 491, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (891, '获取任务', '/api/mark/getTask', 'URL', 491, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (892, '保存任务', '/api/mark/saveTask', 'URL', 491, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (893, '获取评卷历史', '/api/mark/getHistory', 'URL', 491, 7, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (894, '更新个人评卷配置', '/api/mark/updateSetting', 'URL', 491, 8, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (895, '选择评卷分组', '/api/mark/subjectSelect', 'URL', 491, 9, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (896, '清理评卷员的评卷任务缓存', '/api/mark/clear', 'URL', 491, 10, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (897, '评卷设置', 'MarkSetting', 'MENU', 486, 2, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (898, '列表', 'List', 'LIST', 897, 1, 'AUTH', '900', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (899, '查询', 'Select', 'BUTTON', 897, 2, 'AUTH', '900', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (900, '查询', '/api/admin/mark/setting/list', 'URL', 897, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (901, '评卷参数设置', 'MarkParamSetting', 'LINK', 897, 3, 'AUTH', '672,902,903,904,905,906,907,908,909,910,911,912,913,962,963,1132,1133', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (902, '结构查询', '/api/admin/mark/question/list', 'URL', 897, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (903, '结构提交', '/api/admin/mark/question/save', 'URL', 897, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (904, '评卷任务查询', '/api/admin/mark/group/list', 'URL', 897, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (905, '评卷任务新增', '/api/admin/mark/group/save', 'URL', 897, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (906, '评卷任务修改', '/api/admin/mark/group/update', 'URL', 897, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (907, '评卷任务删除', '/api/admin/mark/group/delete', 'URL', 897, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (908, '分班阅查询', '/api/admin/mark/class/list', 'URL', 897, 7, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (909, '分班阅提交', '/api/admin/mark/class/save', 'URL', 897, 8, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (910, '客观题标答查询', '/api/admin/mark/question/objective/list', 'URL', 897, 9, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (911, '客观题标答提交', '/api/admin/mark/question/objective/save', 'URL', 897, 10, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (912, '主观题标答查询', '/api/admin/mark/question/subjective/get', 'URL', 897, 11, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (913, '主观题标答提交', '/api/admin/mark/question/subjective/upload', 'URL', 897, 12, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (914, '评卷设置', 'MarkConfig', 'LINK', 897, 4, 'AUTH', '915,916', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (915, '数据获取', '/api/admin/mark/setting/paper/get', 'URL', 897, 13, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (916, '数据保存', '/api/admin/mark/setting/paper/save', 'URL', 897, 14, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (917, '评卷管理', 'MarkManage', 'MENU', 486, 3, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (918, '列表', 'List', 'LIST', 917, 1, 'AUTH', '925', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (919, '查询', 'Select', 'BUTTON', 917, 2, 'AUTH', '925', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (920, '查询条件', 'Condition', 'CONDITION', 917, 3, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (921, '查询条件', 'Condition', 'CONDITION', 491, 3, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (922, '查询条件', 'Condition', 'CONDITION', 897, 3, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (923, '导出评卷工作量', 'Export', 'BUTTON', 917, 4, 'AUTH', '926', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (924, '结束评卷', 'BatchFinish', 'BUTTON', 917, 5, 'AUTH', '927', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (925, '列表获取', '/api/admin/mark/setting/summary', 'URL', 917, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (926, '导出评卷员工作量', '/api/admin/mark/setting/export_marker', 'URL', 917, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (927, '结束评卷', '/api/admin/mark/paper/finish', 'URL', 917, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (928, '进度列表查询', '/api/admin/mark/group/summary', 'URL', 917, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (929, '评卷员管理列表查询', '/api/admin/mark/marker/list', 'URL', 917, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (930, '重置', '/api/admin/mark/marker/reset', 'URL', 917, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (931, '解绑', '/api/admin/mark/marker/delete', 'URL', 917, 7, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (932, '回收', '/api/admin/mark/marker/release', 'URL', 917, 8, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (933, '设置评卷数', '/api/admin/mark/marker/setTaskCount', 'URL', 917, 9, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (934, '评卷质量查询', '/api/admin/mark/quality/list', 'URL', 917, 10, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (935, '问题试卷列表查询', '/api/admin/mark/problem/list', 'URL', 917, 11, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (936, '问题试卷重置', '/api/admin/mark/problem/reset', 'URL', 917, 12, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (937, '仲裁管理列表查询', '/api/admin/mark/arbitrate/list', 'URL', 917, 13, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (938, '仲裁-获取任务', '/api/admin/mark/arbitrate/getTask', 'URL', 917, 14, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (939, '仲裁-左边的列表', '/api/admin/mark/arbitrate/getArbitrationList', 'URL', 917, 15, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (940, '仲裁-清理锁卷缓存', '/api/admin/mark/arbitrate/clear', 'URL', 917, 16, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (941, '仲裁-获取评卷状态', '/api/admin/mark/arbitrate/getStatus', 'URL', 917, 17, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (942, '仲裁-获取配置信息', '/api/admin/mark/arbitrate/getSetting', 'URL', 917, 18, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (943, '仲裁-保存任务', '/api/admin/mark/arbitrate/saveTask', 'URL', 917, 19, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (944, '查看详情', 'Detail', 'LINK', 917, 6, 'AUTH', '928,929,932,933,934,935,936,937,938,939,940,941,942,943,964,965,966,967,968,1008,1183,1186', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (945, '结束评卷', 'Finish', 'LINK', 917, 7, 'AUTH', '927', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (946, '成绩检查', 'ScoreCheck', 'MENU', 486, 4, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (947, '列表', 'List', 'LIST', 946, 1, 'AUTH', '952', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (948, '查询条件', 'Condition', 'CONDITION', 946, 2, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (949, '查询', 'Select', 'BUTTON', 946, 3, 'AUTH', '952', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (950, '查看详情', 'Detail', 'LINK', 946, 4, 'AUTH', '953,954,955,969,1139', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (951, '查看签到表', 'PackageDetail', 'LINK', 946, 5, 'AUTH', '961', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (952, '列表获取', '/api/admin/mark/setting/scoreList', 'URL', 946, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (953, '详情列表', '/api/admin/mark/student/score', 'URL', 946, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (954, '获取配置', '/api/admin/mark/track/getSetting', 'URL', 946, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (955, '获取任务', '/api/admin/mark/track/getTask', 'URL', 946, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (956, '主观题检查-获取配置', '/api/admin/mark/inspected/getSetting', 'URL', 946, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (957, '主观题检查-获取任务', '/api/admin/mark/inspected/subjective/getTask', 'URL', 946, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (958, '主观题检查-保存任务', '/api/admin/mark/inspected/subjective/saveTask', 'URL', 946, 7, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (959, '客观题检查-获取任务', '/api/admin/mark/inspected/objective/getTask', 'URL', 946, 8, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (960, '客观题检查-保存任务', '/api/admin/mark/inspected/objective/saveTask', 'URL', 946, 9, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (961, '签到表列表', '/api/admin/mark/paper/package/list', 'URL', 946, 10, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (962, '评卷区设置', '/api/admin/mark/group/update_picture_config', 'URL', 897, 13, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (963, '开启/关闭分班阅', '/api/admin/mark/group/update_open_mark_class', 'URL', 897, 14, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (964, '班级阅卷进度', '/api/admin/mark/group/class/summary', 'URL', 917, 20, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (965, '绑定评卷员', '/api/admin/mark/marker/add', 'URL', 917, 21, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (966, '评卷质量重新计算', '/api/admin/mark/quality/update', 'URL', 917, 22, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (967, '评卷质量给分曲线', '/api/admin/mark/quality/chart', 'URL', 917, 23, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (968, '绑定评卷员-已绑定评卷员集合', '/api/admin/mark/marker/list_bind_marker', 'URL', 917, 20, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (969, '客观题统分', '/api/admin/mark/student/objective/calculate', 'URL', 946, 11, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (970, '扫描端', 'scan', 'MENU', NULL, 11, NULL, '971,972,973,974,975,976,977,978,979,980,981,982,983,984,985,986,987,988,989,990,991,992,993,994,995,996,997,998,999,1000,1001,1002,1003,1004,1005,1007,1009,1010,1148,3072,3073,3074,3075', 1, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (971, '心跳接口', '/api/scan/server/heartbeat', 'URL', 970, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (972, '登出接口', '/api/scan/logout', 'URL', 970, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (973, '获取学期、考试列表', '/api/admin/basic/condition/scan/semester/list', 'URL', 970, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (974, '获取课程、试卷编号列表', '/api/admin/basic/condition/scan/course/list', 'URL', 970, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (975, '获取校区、考场', '/api/admin/basic/condition/scan/campus/list', 'URL', 970, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (976, '扫描汇总-答题卡扫描、签到表扫描', '/api/scan/exam/card/info', 'URL', 970, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (977, '扫描汇总-数据校对', '/api/scan/exam/check/info', 'URL', 970, 7, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (978, '答题卡卡格式列表', '/api/scan/answer/card/list', 'URL', 970, 8, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (979, '创建/修改题卡卡格式', '/api/scan/answer/card/save', 'URL', 970, 9, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (980, '删除答题卡卡格式', '/api/scan/answer/card/delete', 'URL', 970, 10, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (981, '答题卡适配卡格式上传', '/api/scan/answer/card/adapte/upload', 'URL', 970, 11, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (982, '答题卡卡格式查询', '/api/scan/answer/card/get', 'URL', 970, 12, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (983, '答题卡扫描批次创建', '/api/scan/answer/batch/create', 'URL', 970, 13, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (984, '答题卡扫描批次完成', '/api/scan/answer/batch/finish', 'URL', 970, 14, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (985, '答题卡扫描结果保存', '/api/scan/answer/batch/save', 'URL', 970, 15, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (986, '获取考生信息', '/api/scan/student/find', 'URL', 970, 16, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (987, '答题卡扫描原图上传', '/api/scan/answer/sheet/upload', 'URL', 970, 17, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (988, '答题卡条码识别坐标获取', '/api/scan/package/position/get', 'URL', 970, 18, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (989, '签到表扫描图片上传', '/api/scan/package/image/upload', 'URL', 970, 19, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (990, '签到表扫描结果保存', '/api/scan/package/save', 'URL', 970, 20, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (991, '签到表扫描结果列表', '/api/scan/package/image/list', 'URL', 970, 21, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (992, '签到表编号查询', '/api/scan/package/find', 'URL', 970, 22, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (993, '签到表详情', '/api/scan/package/image/get', 'URL', 970, 23, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (994, '查询答题卡扫描详情', '/api/scan/answer/query', 'URL', 970, 24, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (995, '查询答题卡扫描概要', '/api/scan/answer/summary', 'URL', 970, 25, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (996, '修改答题卡识别结果', '/api/scan/answer/omr/edit', 'URL', 970, 26, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (997, '更新缺考状态', '/api/scan/student/absent/update', 'URL', 970, 27, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (998, '修改答题卡扫描图片绑定考生', '/api/scan/answer/paper/migrate', 'URL', 970, 28, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (999, '任务确认', '/api/scan/student/confirm', 'URL', 970, 29, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1000, '任务待处理列表', '/api/scan/task/omr/get', 'URL', 970, 30, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1001, '任务提交', '/api/scan/task/omr/save', 'URL', 970, 31, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1002, '任务状态', '/api/scan/task/omr/status', 'URL', 970, 32, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1003, '任务释放', '/api/scan/task/omr/release', 'URL', 970, 33, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1004, '任务历史', '/api/scan/task/omr/history', 'URL', 970, 34, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1005, '答题卡二次识别', '/api/scan/answer/refix', 'URL', 970, 35, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1006, '查询条件', 'Condition', 'CONDITION', 970, 1, 'AUTH', '971,972,973,974,975,976,977,978,979,980,981,982,983,984,985,986,987,988,989,990,991,992,993,994,995,996,997,998,999,1000,1001,1002,1003,1004,1005,1007,1009,1010,1148,3072,3073,3074', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1007, '未扫描-导出', '/api/scan/student/unexist/export', 'URL', 970, 36, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1008, '仲裁-评卷历史', '/api/admin/mark/arbitrate/getHistory', 'URL', 917, 20, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1009, '任务列表', '/api/scan/task/omr/list', 'URL', 970, 37, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1010, '人工绑定确认', '/api/scan/student/assign/confirm', 'URL', 970, 38, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1011, '主观题检查-确认任务', '/api/admin/mark/inspected/subjective/confirmTask', 'URL', 946, 14, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1100, '考生管理', 'BasicExamStudentManage', 'MENU', 356, 8, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1101, '分页查询', '/api/admin/basic/exam_student/page', 'URL', 1100, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1102, '新增/编辑', '/api/admin/basic/exam_student/save', 'URL', 1100, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1103, '导入', '/api/admin/basic/exam_student/import', 'URL', 1100, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1104, '导出', '/api/admin/basic/exam_student/export', 'URL', 1100, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1105, '批量删除', '/api/admin/basic/exam_student/delete', 'URL', 1100, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1106, '查询', 'Select', 'BUTTON', 1100, 1, 'AUTH', '1101', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1107, '查询条件', 'Condition', 'CONDITION', 1100, 3, 'AUTH', '1101', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1108, '列表', 'List', 'LIST', 1100, 2, 'AUTH', '1101', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1109, '新增', 'Add', 'BUTTON', 1100, 4, 'AUTH', '1102', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1110, '编辑', 'Edit', 'LINK', 1100, 5, 'AUTH', '1102', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1111, '导入学生', 'Import', 'BUTTON', 1100, 6, 'AUTH', '157,1103', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1112, '导出学生', 'Export', 'BUTTON', 1100, 7, 'AUTH', '1104', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1113, '删除', 'Delete', 'LINK', 1100, 8, 'AUTH', '1105,1144', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1114, '批量删除', 'DeleteBatch', 'BUTTON', 1100, 9, 'AUTH', '1105,1144', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1115, '教学课程-查询课程权重设置状态', '/api/admin/teach/course/find_course_weight_setting_status', 'URL', 841, 14, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1116, '课程目标管理-列表查询', '/api/admin/teach/course_target/list', 'URL', 841, 15, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1117, '课程目标管理-同步题库知识点', '/api/admin/teach/course_target/dimension_sync_tiku', 'URL', 841, 16, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1118, '课程目标管理-导入课程知识点', '/api/admin/teach/course_target/dimension_import', 'URL', 841, 17, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1119, '课程目标管理-课程知识点树查询', '/api/admin/teach/course_target/dimension_tree', 'URL', 841, 18, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1120, '课程目标管理-新增/编辑', '/api/admin/teach/course_target/save', 'URL', 841, 19, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1121, '课程目标管理-删除', '/api/admin/teach/course_target/delete', 'URL', 841, 20, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1122, '评价方式管理-列表查询', '/api/admin/teach/course_evaluation/list', 'URL', 841, 21, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1123, '评价方式管理-新增', '/api/admin/teach/course_evaluation/save', 'URL', 841, 22, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1124, '评价方式管理-删除', '/api/admin/teach/course_evaluation/delete', 'URL', 841, 23, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1125, '权重设置-查询', '/api/admin/teach/course_weight/find', 'URL', 841, 24, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1126, '权重设置-保存', '/api/admin/teach/course_weight/save', 'URL', 841, 25, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1127, '共用接口-查询教学班字典', '/api/admin/basic/condition/list_basic_teach_clazz', 'URL', 149, 16, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1128, '教师团队管理-分配班级列表查询', '/api/admin/teach/course/find_clazz_list', 'URL', 841, 26, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1129, '教师团队管理-分配班级', '/api/admin/teach/course/assign_clazz', 'URL', 841, 27, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1130, '阅卷数据批量导入（同步）', '/api/admin/mark/setting/mark_student_import', 'URL', 917, 23, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1131, '阅卷数据导入', 'MarkStudentImport', 'BUTTON', 917, 8, 'AUTH', '1130', 0, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1132, '客观题模板导出', '/api/admin/mark/question/objective/export', 'URL', 897, 15, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1133, '客观题标答导入', '/api/admin/mark/question/objective/import', 'URL', 897, 16, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1134, '编辑试卷结构', 'EditPaperStruct', 'LINK', 897, 17, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1135, '客观题导入', '/api/admin/mark/setting/objective_struct/import', 'URL', 897, 17, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1136, '主观题导入', '/api/admin/mark/setting/subjective_struct/import', 'URL', 897, 18, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1137, '客观题导入', 'ObjectiveStructImport', 'BUTTON', 897, 6, 'AUTH', '1135', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1138, '主观题导入', 'SubjectiveStructImport', 'BUTTON', 897, 7, 'AUTH', '1136', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1139, '轨迹图查询题卡内容', '/api/admin/mark/track/getCard', 'URL', 946, 12, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1140, '状态', 'Status', 'LINK', 1100, 10, 'AUTH', '1141', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1141, '状态', '/api/admin/basic/exam_student/status', 'URL', 1100, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1142, '任课老师导入', 'ImportTeacher', 'BUTTON', 1100, 11, 'AUTH', '1143', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1143, '任课老师导入', '/api/admin/basic/exam_student/import_teacher', 'URL', 1100, 7, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1144, '删除考生时查询扫描阅卷状态', '/api/admin/basic/exam_student/mark_status', 'URL', 1100, 8, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1145, '合并PDF查看', '/api/admin/exam/print/get_whole_pdf', 'URL', 47, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1146, '合并PDF生成', 'CreateWholePdf', 'BUTTON', 47, 5, 'AUTH', '1145,1147', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1147, '合并PDF生成', '/api/admin/exam/print/create_whole_pdf', 'URL', 47, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1148, '更新漏扫状态', '/api/scan/student/missScan/update', 'URL', 970, 39, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1149, '评卷设置（批量）', 'BatchMarkConfig', 'BUTTON', 897, 8, 'AUTH', '1150', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1150, '评卷设置保存', '/api/admin/mark/setting/paper/batch_save', 'URL', 897, 19, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1151, '导入数据包', 'CardJsonImport', 'BUTTON', 897, 9, 'AUTH', '1152', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1152, '导入数据包', '/api/admin/mark/setting/card_json/import', 'URL', 897, 20, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1170, '共用接口-OBE专业', '/api/admin/basic/condition/list_professional', 'URL', 149, 17, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1171, '共用接口-OBE培养方案', '/api/admin/basic/condition/list_culture_program', 'URL', 149, 18, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1172, '共用接口-OBE课程', '/api/admin/basic/condition/list_culture_program_course', 'URL', 149, 18, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1173, '共用接口-OBE学期', '/api/admin/basic/condition/list_outline_semester', 'URL', 149, 19, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1174, '重置', 'MarkTaskReset', 'LINK', 917, 9, 'AUTH', '930', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1175, '解绑', 'MarkTaskMarkerDelete', 'LINK', 917, 9, 'AUTH', '931', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1176, '批量复核-主观题', 'BatchInspectedSubjective', 'BUTTON', 946, 6, 'AUTH', '956,957,958,1011', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1177, '批量复核-客观题', 'BatchObjectiveSubjective', 'BUTTON', 946, 7, 'AUTH', '959,960', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1178, '成绩导出', 'ScoreExport', 'BUTTON', 946, 8, 'AUTH', '2105', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1179, '主观题复核', 'InspectedSubjective', 'LINK', 946, 9, 'AUTH', '888,894,956,957,958,1011', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1180, '客观题复核', 'ObjectiveSubjective', 'LINK', 946, 10, 'AUTH', '959,960', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1181, '客观题统分', 'ObjectiveCalculate', 'BUTTON', 946, 9, 'AUTH', '1182', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1182, '客观题统分', '/api/admin/mark/student/objective/calculate_batch', 'URL', 946, 13, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1183, '任务管理列表查询', '/api/admin/mark/task/list', 'URL', 917, 24, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1184, '打回', 'MarkTaskReject', 'LINK', 917, 10, 'AUTH', '1185,1187', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1185, '打回', '/api/admin/mark/task/reject', 'URL', 917, 25, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1186, '打回历史列表查询', '/api/admin/mark/reject/list', 'URL', 917, 26, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1187, '打回-查询轨迹图', '/api/admin/mark/task/get_mark_track', 'URL', 917, 27, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1999, '达成度统计', 'DegreeManage', 'MENU', 2000, 2, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2000, '达成度分析', 'TargetDegreeManage', 'MENU', NULL, 13, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2001, '成绩管理', 'TargetScoreManage', 'MENU', 2100, 1, NULL, '2003,2004,2005,3053', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2002, '课程目标达成度统计', 'TargetReportManage', 'MENU', 1999, 2, NULL, '2021,2022,2023', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2003, '列表', 'List', 'LIST', 2001, 1, 'AUTH', '2007', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2004, '查询条件', 'Condition', 'CONDITION', 2001, 2, 'AUTH', '2007', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2005, '查询', 'Select', 'BUTTON', 2001, 3, 'AUTH', '2007', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2006, '管理成绩', 'Score', 'LINK', 2001, 1, 'AUTH', '2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020,2029,2030,2031,2032,2033,2034,3018,2111,2114', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2007, '列表', '/api/admin/course/degree/score/list', 'URL', 2001, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2008, '平时成绩-下载模版', '/api/admin/course/degree/usual_score/template_download', 'URL', 2006, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2009, '平时成绩-导入模版', '/api/admin/course/degree/usual_score/import', 'URL', 2006, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2010, '平时成绩列表', '/api/admin/course/degree/usual_score/list', 'URL', 2006, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2011, '平时成绩编辑', '/api/admin/course/degree/usual_score/edit', 'URL', 2006, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2012, '平时成绩保存', '/api/admin/course/degree/usual_score/save', 'URL', 2006, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2013, '平时成绩启用/禁用', '/api/admin/course/degree/usual_score/enable', 'URL', 2006, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2014, '期末成绩-下载模版', '/api/admin/course/degree/final_score/template_download', 'URL', 2006, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2015, '期末成绩-导入模版', '/api/admin/course/degree/final_score/import', 'URL', 2006, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2016, '期末成绩列表', '/api/admin/course/degree/final_score/list', 'URL', 2006, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2017, '期末成绩编辑', '/api/admin/course/degree/final_score/edit', 'URL', 2006, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2018, '期末成绩保存', '/api/admin/course/degree/final_score/save', 'URL', 2006, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2019, '期末成绩启用/禁用', '/api/admin/course/degree/final_score/enable', 'URL', 2006, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2020, '保存试卷蓝图', '/api/admin/course/degree/final_score/paper_struct/save', 'URL', 2006, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2021, '列表', 'List', 'LIST', 2002, 1, 'AUTH', '2024', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2022, '查询条件', 'Condition', 'CONDITION', 2002, 2, 'AUTH', '2024', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2023, '查询', 'Select', 'BUTTON', 2002, 3, 'AUTH', '2024', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2024, '列表', '/api/admin/course/degree/report/list', 'URL', 2002, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2025, '查看详情', 'View', 'LINK', 2002, 1, 'AUTH', '2026,2027,2028,2031,2110', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2026, '查看报告', '/api/admin/course/degree/report/view', 'URL', 2025, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2027, '导出报告', '/api/admin/course/degree/report/export', 'URL', 2025, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2028, '保存报告', '/api/admin/course/degree/report/save', 'URL', 2025, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2029, '获取试卷蓝图信息', '/api/admin/course/degree/final_score/paper_struct/query', 'URL', 2006, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2030, '期末成绩-同步', '/api/admin/course/degree/final_score/sync', 'URL', 2006, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2031, '报告数据发生变化', '/api/admin/course/degree/report/change', 'URL', 2006, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2032, '试卷结构-导入模版', '/api/admin/course/degree/final_score/paper_struct/import', 'URL', 2006, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2033, '同步试卷蓝图结构', '/api/admin/course/degree/final_score/paper_struct_dimension/sync', 'URL', 2006, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2034, '同步期末成绩选择试卷', '/api/admin/course/degree/final_score/choose_paper', 'URL', 2006, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2035, '培养方案管理', 'CultureProgramManage', 'MENU', 2000, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2036, '培养方案管理', 'CultureProgram', 'MENU', 2035, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2037, '列表', 'List', 'LIST', 2036, 1, 'AUTH', '2042', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2038, '查询条件', 'Condition', 'CONDITION', 2036, 2, 'AUTH', '2042,540', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2039, '查询', 'Select', 'BUTTON', 2036, 3, 'AUTH', '2042', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2040, '新增', 'Add', 'BUTTON', 2036, 4, 'AUTH', '2043', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2041, '查看详情', 'CultureProgramDetail', 'LINK', 2036, 5, 'AUTH', '2044,2045,2046,2047,2048,2049,2050,2051,2052,2053,2054,2055,2058,2059,2060,2066,2067,2068,2069', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2042, '查询', '/api/admin/obe/culture/program/page', 'URL', 2036, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2043, '新增', '/api/admin/obe/culture/program/save', 'URL', 2036, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2044, '课程体系-选择课程', '/api/admin/obe/culture/program/course/query_list', 'URL', 2036, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2045, '课程体系-查询列表', '/api/admin/obe/culture/program/course/list', 'URL', 2036, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2046, '课程体系-新增', '/api/admin/obe/culture/program/course/save', 'URL', 2036, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2047, '课程体系-排序', '/api/admin/obe/culture/program/course/sort', 'URL', 2036, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2048, '课程体系-删除', '/api/admin/obe/culture/program/course/remove', 'URL', 2036, 7, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2049, '毕业要求-查询', '/api/admin/obe/culture/program/requirement/list', 'URL', 2036, 8, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2050, '毕业要求-新增/编辑', '/api/admin/obe/culture/program/requirement/save', 'URL', 2036, 9, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2051, '培养目标-列表', '/api/admin/obe/culture/program/target/list', 'URL', 2036, 10, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2052, '毕业要求-删除', '/api/admin/obe/culture/program/requirement/remove', 'URL', 2036, 11, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2053, '课程支撑毕业要求达成矩阵-查询-查询', '/api/admin/obe/culture/program/course/matrix/get', 'URL', 2036, 12, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2054, '课程支撑毕业要求达成矩阵-提交-提交', '/api/admin/obe/culture/program/course/matrix/save', 'URL', 2036, 13, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2055, '培养目标-新增/编辑', '/api/admin/obe/culture/program/target/save', 'URL', 2036, 14, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2056, '是否可编辑（课程）', 'canEditCourse', 'LINK', 2036, 6, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2057, '是否可编辑（毕业要求）', 'canEditRequirement', 'LINK', 2036, 7, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2058, '培养目标-删除', '/api/admin/obe/culture/program/target/remove', 'URL', 2036, 15, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2059, '培养目标与毕业要求关系矩阵-查询', '/api/admin/obe/culture/program/target/matrix/get', 'URL', 2036, 16, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2060, '培养目标与毕业要求关系矩阵-提交', '/api/admin/obe/culture/program/target/matrix/save', 'URL', 2036, 17, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2061, '编辑', 'Edit', 'LINK', 2036, 8, 'AUTH', '2043', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2062, '复制', 'Copy', 'LINK', 2036, 9, 'AUTH', '2063', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2063, '复制', '/api/admin/obe/culture/program/copy', 'URL', 2036, 18, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2064, '删除', 'Delete', 'LINK', 2036, 10, 'AUTH', '2065', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2065, '删除', '/api/admin/obe/culture/program/remove', 'URL', 2036, 19, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2066, '保存总体描述', '/api/admin/obe/culture/program/description/save', 'URL', 2036, 20, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2067, '设置期望值', '/api/admin/obe/culture/program/requirement/expect/save', 'URL', 2036, 21, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2068, '基本信息查询', '/api/admin/obe/culture/program/get', 'URL', 2036, 22, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2069, '毕业要求-新增指标点', '/api/admin/obe/culture/program/requirement/node/add', 'URL', 2036, 23, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2070, '毕业要求达成度统计', 'GraduationRequirement', 'MENU', 1999, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2071, '列表', 'List', 'LIST', 2070, 1, 'AUTH', '2075', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2072, '查询条件', 'Condition', 'CONDITION', 2070, 2, 'AUTH', '2075', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2073, '查询', 'Select', 'BUTTON', 2070, 3, 'AUTH', '2075', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2074, '查看详情', 'Detail', 'LINK', 2070, 4, 'AUTH', '2076,2077', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2075, '毕业要求达成度报表列表', '/api/admin/obe/requirements/list', 'URL', 2070, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2076, '毕业要求达成度雷达图', '/api/admin/obe/requirements/radar/report', 'URL', 2070, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2077, '毕业要求达成情况详细报表', '/api/admin/obe/requirements/detail/report', 'URL', 2070, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2080, '学生毕业要求达成度', 'StudentRequirement', 'MENU', 1999, 3, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2081, '列表', 'List', 'LIST', 2080, 1, 'AUTH', '2085', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2082, '查询条件', 'Condition', 'CONDITION', 2080, 2, 'AUTH', '2085', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2083, '查询', 'Select', 'BUTTON', 2080, 3, 'AUTH', '2085', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2084, '查看详情', 'Detail', 'LINK', 2080, 4, 'AUTH', '2086', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2085, '查询', '/api/admin/obe/student_requirement/page', 'URL', 2080, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2086, '查看详情', '/api/admin/obe/student_requirement/detail', 'URL', 2080, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2087, '统计管理', 'count', 'MENU', 1, 6, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2088, '印刷任务统计', 'PrintCount', 'MENU', 2087, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2089, '扫描任务统计', 'ScanCount', 'MENU', 2087, 2, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2090, '查询', 'Select', 'BUTTON', 2088, 1, 'AUTH', '2093', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2091, '查询条件', 'Condition', 'CONDITION', 2088, 1, 'AUTH', '2093', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2092, '列表', 'List', 'LIST', 2088, 1, 'AUTH', '2093', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2093, '印刷任务统计查询', '/api/admin/statistics/print/count', 'URL', 2088, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2094, '导出印刷进度', 'Export', 'BUTTON', 2088, 2, 'AUTH', '2095', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2095, '印刷任务统计导出', '/api/admin/statistics/print/count/export', 'URL', 2088, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2096, '查询', 'Select', 'BUTTON', 2089, 1, 'AUTH', '2099', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2097, '查询条件', 'Condition', 'CONDITION', 2089, 1, 'AUTH', '2099', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2098, '列表', 'List', 'LIST', 2089, 1, 'AUTH', '2099', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2099, '扫描任务统计查询', '/api/admin/statistics/scan/count', 'URL', 2089, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2100, '基础数据', 'BasicDataManage', 'MENU', 2000, 3, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2101, '重新计算', 'Calculate', 'LINK', 2070, 5, 'AUTH', '2102', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2102, '毕业要求达成情况重新计算', '/api/admin/obe/requirements/again/calculate', 'URL', 2070, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2103, '重新计算', 'Calculate', 'BUTTON', 2080, 4, 'AUTH', '2104', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2104, '学生毕业要求达成度重新计算', '/api/admin/obe/student_requirement/again/calculate', 'URL', 2080, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2105, '成绩导出', '/api/admin/mark/student/score/export', 'URL', 946, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2106, '导出扫描进度', 'Export', 'BUTTON', 2089, 2, 'AUTH', '2107', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2107, '扫描任务统计导出', '/api/admin/statistics/scan/count/export', 'URL', 2089, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2108, '重新计算', 'Calculate', 'LINK', 2002, 2, 'AUTH', '2109', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2109, '重新计算接口', '/api/admin/course/degree/report/again/calculate', 'URL', 2108, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2110, '导出平时成绩报告', '/api/admin/course/degree/report/usual_score/export', 'URL', 2025, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2111, '平时成绩-获取作业集合', '/api/admin/course/degree/usual_score/template_download/list', 'URL', 2006, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2114, '试卷清除信息', '/api/admin/course/degree/final_score/paper_struct/clear', 'URL', 2006, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3000, '课程大纲管理', 'CourseOutlineManager', 'MENU', 2035, 2, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3001, '课程考核设置', 'CourseAssessmentSetting', 'MENU', 2035, 3, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3002, '课程大纲-查询', '/api/admin/obe/course_outline/page', 'URL', 3000, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3003, '课程大纲-保存', '/api/admin/obe/course_outline/save', 'URL', 3000, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3004, '课程大纲-删除', '/api/admin/obe/course_outline/delete', 'URL', 3000, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3005, '课程大纲-查询课程权重设置状态', '/api/admin/obe/course_outline/find_course_weight_setting_status', 'URL', 3000, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3006, '课程大纲-查询可选择的培养方案', '/api/admin/obe/course_outline/can_choose_culture_program', 'URL', 3000, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3007, '课程大纲-查询可选择的课程', '/api/admin/obe/course_outline/can_choose_course', 'URL', 3000, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3010, '列表', 'List', 'LIST', 3000, 1, 'AUTH', '3002', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3011, '查询条件', 'Condition', 'CONDITION', 3000, 2, 'AUTH', '3002', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3012, '查询', 'Select', 'BUTTON', 3000, 3, 'AUTH', '3002', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3013, '新增课程大纲', 'Save', 'BUTTON', 3000, 4, 'AUTH', '3003', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3014, '编辑', 'Edit', 'LINK', 3000, 5, 'AUTH', '3003,3006,3007', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3015, '删除', 'Delete', 'LINK', 3000, 6, 'AUTH', '3004', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3016, '查看详情', 'Detail', 'LINK', 3000, 7, 'AUTH', '3005,3017,3018,3019,3020,3021,3022,3023,3024,3025', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3017, '课程大纲-课程基本信息', '/api/admin/obe/course_outline/course_basic_info', 'URL', 3016, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3018, '课程目标管理-列表查询', '/api/admin/obe/course_target/list', 'URL', 3016, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3019, '课程目标管理-新增/编辑', '/api/admin/obe/course_target/save', 'URL', 3016, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3020, '课程目标管理-课程知识点树查询', '/api/admin/obe/course_target/dimension_tree', 'URL', 3016, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3021, '课程目标管理-导入课程知识点', '/api/admin/obe/course_target/dimension_import', 'URL', 3016, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3022, '课程目标管理-指标知识点关系查询', '/api/admin/obe/course_target/requirement_dimension', 'URL', 3016, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3023, '课程目标管理-删除', '/api/admin/obe/course_target/delete', 'URL', 3016, 7, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3024, '课程目标管理-预期值设置', '/api/admin/obe/course_target/setting_expect_value', 'URL', 3016, 8, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3025, '根据培养方案、课程查询指标点', '/api/admin/obe/course_target/sub_requirement/list', 'URL', 3016, 9, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3030, '课程考核设置-查询', '/api/admin/obe/course_outline/assessment_setting_page', 'URL', 3001, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3031, '课程考核设置-删除', '/api/admin/obe/course_outline/assessment_setting_delete', 'URL', 3001, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3040, '列表', 'List', 'LIST', 3001, 1, 'AUTH', '3030', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3041, '查询条件', 'Condition', 'CONDITION', 3001, 2, 'AUTH', '3030', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3042, '查询', 'Select', 'BUTTON', 3001, 3, 'AUTH', '3030', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3043, '删除', 'Delete', 'LINK', 3001, 4, 'AUTH', '3031', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3044, '编辑', 'Edit', 'LINK', 3001, 5, 'AUTH', '3050,3051,3052,3053,3054', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3050, '评价方式管理-列表查询', '/api/admin/obe/course_evaluation/list', 'URL', 3044, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3051, '评价方式管理-新增/编辑', '/api/admin/obe/course_evaluation/save', 'URL', 3044, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3052, '评价方式管理-删除', '/api/admin/obe/course_evaluation/delete', 'URL', 3044, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3053, '课程权重管理-列表查询', '/api/admin/obe/course_weight/find', 'URL', 3044, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3054, '课程权重管理-新增/编辑', '/api/admin/obe/course_weight/save', 'URL', 3044, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3055, '首页-评卷待办数量', '/api/admin/work/mark/task/ready_count', 'URL', 16, 11, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3056, '首页-评卷待办列表', '/api/admin/work/mark/task/ready', 'URL', 16, 12, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3057, '总量分析', '/api/admin/mark/archive/score/list/export', 'URL', 487, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3058, '总量分析', 'aggregateAnalysis', 'BUTTON', 487, 3, 'AUTH', '3057', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3059, '文档管理-批量下载', '/api/admin/mark/archive/document/download_batch', 'URL', 577, 7, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3060, '课程大纲-审核', '/api/admin/obe/course_outline/audit_report', 'URL', 3000, 7, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3061, '课程大纲-审核及合理性', '/api/admin/obe/course_outline/audit_rationality_report', 'URL', 3000, 8, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3062, '课程大纲-审核报告保存', '/api/admin/obe/course_outline/audit/save', 'URL', 3000, 9, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3063, '审核', 'Audit', 'LINK', 3000, 8, 'AUTH', '3060,3062', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3064, '审核&合理性', 'AuditAndRationality', 'LINK', 3000, 9, 'AUTH', '3061,3062', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3070, '学号配置保存', '/api/admin/set/student_number_config/save', 'URL', 508, 17, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3071, '学号配置查询', '/api/admin/set/student_number_config/select', 'URL', 508, 18, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3072, '查询学号字母映射关系', '/api/scan/config/student_number_letter_relation', 'URL', 970, 40, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3073, '卷型检查任务-状态数量', '/api/scan/student/paper_type_check/status/count', 'URL', 970, 41, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3074, '卷型检查任务-任务确认', '/api/scan/student/paper_type_check/confirm', 'URL', 970, 42, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3075, '修改违纪状态', '/api/scan/student/breach/update', 'URL', '970', '43', 'AUTH', NULL, '1', '1', '1');
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3076, '数据同步参数查询', '/api/admin/set/data/sync/select', 'URL', '508', 19, 'AUTH', NULL,'1', '0', '1');
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3077, '数据同步参数保存', '/api/admin/set/data/sync/save', 'URL', '508', 20, 'AUTH', NULL,'1', '0', '1');
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3078, '机器人参数查询', '/api/admin/set/ai/robot/select', 'URL', '508', 21, 'AUTH',NULL, '1', '0', '1');
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3079, '机器人参数保存', '/api/admin/set/ai/robot/save', 'URL', '508', 22, 'AUTH', NULL,'1', '0', '1');

-- ----------------------------
-- Table structure for sys_role
-- ----------------------------
CREATE TABLE IF NOT EXISTS `sys_role` (
  `id` bigint NOT NULL COMMENT '主键',
  `school_id` bigint DEFAULT NULL COMMENT '学校id',
  `org_id` bigint DEFAULT NULL COMMENT '机构id',
  `name` varchar(50) NOT NULL COMMENT '角色名称',
  `enable` tinyint NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
  `type` varchar(20) DEFAULT NULL COMMENT '角色类别，ADMIN：超级管理员，SCHOOL_ADMIN：管理员：EXAM_TEACHER：考务老师，QUESTION_TEACHER：命题老师，CUSTOMER：客服人员，PRINTER：印刷人员，CUSTOM：自定义',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '修改人',
  `update_time` bigint DEFAULT NULL COMMENT '修改时间',
  `default_role` tinyint NOT NULL DEFAULT '0' COMMENT '是否系统内置角色，1：是，0：不是',
  `source` varchar(100) DEFAULT NULL COMMENT '角色来源，ANALYSIS：教研分析',
  `interpret` varchar(1000) DEFAULT NULL COMMENT '角色说明',
  PRIMARY KEY (`id`) USING BTREE,
  UNIQUE KEY `role_schoolId_name_idx` (`school_id`,`name`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='角色表';

-- ----------------------------
-- Records of sys_role
-- ----------------------------
INSERT INTO `sys_role` (`id`, `school_id`, `org_id`, `name`, `enable`, `type`, `create_id`, `create_time`, `update_id`, `update_time`, `default_role`, `source`, `interpret`) VALUES (1, NULL, NULL, '系统管理员', 1, 'ADMIN', NULL, NULL, NULL, NULL, 1, NULL, NULL);
INSERT INTO `sys_role` (`id`, `school_id`, `org_id`, `name`, `enable`, `type`, `create_id`, `create_time`, `update_id`, `update_time`, `default_role`, `source`, `interpret`) VALUES (2, NULL, NULL, '学校管理员', 1, 'SCHOOL_ADMIN', NULL, NULL, 1, 1721631507483, 1, 'SYS', '系统内置');
INSERT INTO `sys_role` (`id`, `school_id`, `org_id`, `name`, `enable`, `type`, `create_id`, `create_time`, `update_id`, `update_time`, `default_role`, `source`, `interpret`) VALUES (3, NULL, NULL, '老师', 1, 'SCHOOL_TEACHER', NULL, NULL, 1, 1721631571742, 1, 'SYS', '系统内置');
INSERT INTO `sys_role` (`id`, `school_id`, `org_id`, `name`, `enable`, `type`, `create_id`, `create_time`, `update_id`, `update_time`, `default_role`, `source`, `interpret`) VALUES (4, NULL, NULL, '学院秘书', 1, 'ASSISTANT', NULL, NULL, 1, 1721631586836, 1, 'SYS', '系统内置');
INSERT INTO `sys_role` (`id`, `school_id`, `org_id`, `name`, `enable`, `type`, `create_id`, `create_time`, `update_id`, `update_time`, `default_role`, `source`, `interpret`) VALUES (5, NULL, NULL, '印刷员', 1, 'PRINT', NULL, NULL, 1, 1721631596260, 1, 'SYS', '系统内置');
INSERT INTO `sys_role` (`id`, `school_id`, `org_id`, `name`, `enable`, `type`, `create_id`, `create_time`, `update_id`, `update_time`, `default_role`, `source`, `interpret`) VALUES (6, NULL, NULL, '扫描员', 1, 'SCANNER', NULL, NULL, 1, 1721631606876, 1, 'SYS', '系统内置');
INSERT INTO `sys_role` (`id`, `school_id`, `org_id`, `name`, `enable`, `type`, `create_id`, `create_time`, `update_id`, `update_time`, `default_role`, `source`, `interpret`) VALUES (7, NULL, NULL, '评卷员', 1, 'MARKER', NULL, NULL, 1, 1721631616100, 1, 'CLOUD_MARK', '系统内置');
INSERT INTO `sys_role` (`id`, `school_id`, `org_id`, `name`, `enable`, `type`, `create_id`, `create_time`, `update_id`, `update_time`, `default_role`, `source`, `interpret`) VALUES (8, NULL, NULL, '学科老师', 1, 'SUBJECT_TEACHER', NULL, NULL, 1, 1721631623216, 1, 'QUESTION_LIBRARY', '系统内置');
INSERT INTO `sys_role` (`id`, `school_id`, `org_id`, `name`, `enable`, `type`, `create_id`, `create_time`, `update_id`, `update_time`, `default_role`, `source`, `interpret`) VALUES (9, NULL, NULL, '命题老师', 1, 'ASSIGN_TEACHER', NULL, NULL, 1, 1721631643998, 1, 'QUESTION_LIBRARY', '系统内置');
INSERT INTO `sys_role` (`id`, `school_id`, `org_id`, `name`, `enable`, `type`, `create_id`, `create_time`, `update_id`, `update_time`, `default_role`, `source`, `interpret`) VALUES (10, NULL, NULL, '教务处老师', 1, 'OFFICE_TEACHER', NULL, NULL, 1, 1721631691339, 1, 'ANALYSIS', '系统内置');
INSERT INTO `sys_role` (`id`, `school_id`, `org_id`, `name`, `enable`, `type`, `create_id`, `create_time`, `update_id`, `update_time`, `default_role`, `source`, `interpret`) VALUES (11, NULL, NULL, '学院院长', 1, 'PRESIDENT', NULL, NULL, 1, 1721631700536, 1, 'ANALYSIS', '系统内置');
INSERT INTO `sys_role` (`id`, `school_id`, `org_id`, `name`, `enable`, `type`, `create_id`, `create_time`, `update_id`, `update_time`, `default_role`, `source`, `interpret`) VALUES (12, NULL, NULL, '任课老师', 1, 'TEACHER', NULL, NULL, 1, 1721631709250, 1, 'ANALYSIS', '系统内置');

-- ----------------------------
-- Table structure for sys_role_group_member
-- ----------------------------
CREATE TABLE IF NOT EXISTS `sys_role_group_member` (
  `id` bigint NOT NULL COMMENT '主键',
  `role_id` bigint NOT NULL COMMENT '角色id',
  `user_id` bigint NOT NULL COMMENT '用户id',
  `member_id` bigint NOT NULL COMMENT '成员id',
  `enable` tinyint NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
  PRIMARY KEY (`id`) USING BTREE,
  UNIQUE KEY `role_group_roleId_memberId_idx` (`role_id`,`member_id`,`enable`) USING BTREE,
  KEY `role_group_member_idx` (`role_id`,`user_id`,`member_id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='角色组成员表';

-- ----------------------------
-- Table structure for sys_role_privilege
-- ----------------------------
CREATE TABLE IF NOT EXISTS `sys_role_privilege` (
  `id` bigint NOT NULL COMMENT '主键',
  `role_id` bigint NOT NULL COMMENT '角色id',
  `privilege_id` bigint NOT NULL COMMENT '菜单id',
  `enable` tinyint DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
  `school_id` bigint DEFAULT NULL COMMENT '内置角色，按学校自定义的权限',
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='角色和菜单关联表';

-- ----------------------------
-- Records of sys_role_privilege
-- ----------------------------
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1, 1, 1, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (2, 1, 2, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (3, 1, 4, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (4, 1, 5, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (5, 1, 6, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (6, 1, 92, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (7, 1, 93, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (8, 1, 94, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (9, 1, 95, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (10, 1, 224, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (11, 1, 225, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (12, 1, 226, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (13, 1, 227, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (14, 1, 228, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (15, 1, 234, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (16, 1, 77, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (17, 1, 78, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (18, 1, 79, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (19, 1, 80, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (20, 1, 81, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (21, 1, 82, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (22, 1, 87, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (23, 1, 184, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (24, 1, 217, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (25, 1, 218, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (26, 1, 219, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (27, 1, 220, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (28, 1, 221, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (29, 1, 222, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (30, 1, 223, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (31, 1, 243, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (32, 1, 83, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (33, 1, 84, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (34, 1, 85, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (35, 1, 86, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (36, 1, 91, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (37, 1, 180, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (38, 1, 229, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (39, 1, 230, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (40, 1, 231, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (41, 1, 232, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (42, 1, 233, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (43, 1, 235, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (44, 1, 390, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (45, 1, 402, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (46, 1, 407, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (47, 1, 408, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (48, 1, 409, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (49, 1, 410, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (50, 1, 411, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (53, 1, 527, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (54, 1, 528, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (55, 1, 529, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (56, 1, 638, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (57, 1, 643, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (58, 1, 675, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (59, 1, 676, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (60, 1, 677, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (61, 1, 678, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (62, 1, 679, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (81, 1, 767, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (88, 1, 771, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (89, 1, 772, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (90, 1, 773, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (91, 1, 774, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (92, 1, 775, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (93, 1, 776, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (94, 1, 777, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (95, 1, 778, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (96, 1, 779, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (97, 1, 780, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (98, 1, 781, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1778352691320963073, 8, 768, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1778352691325157378, 8, 769, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1778352691329351681, 8, 770, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1778352766998790145, 7, 896, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1778352767002984449, 7, 483, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1778352767002984450, 7, 486, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1778352767007178754, 7, 491, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1778352767007178755, 7, 492, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1778352767007178756, 7, 493, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1778352767011373057, 7, 884, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1778352767011373058, 7, 885, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1778352767011373059, 7, 886, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1778352767015567362, 7, 887, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1778352767015567363, 7, 888, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1778352767019761666, 7, 921, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1778352767019761667, 7, 889, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1778352767019761668, 7, 890, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1778352767023955969, 7, 891, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1778352767023955970, 7, 892, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1778352767023955971, 7, 893, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1778352767028150273, 7, 541, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1778352767028150274, 7, 894, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1778352767032344577, 7, 895, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1784807531827937282, 11, 739, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1784807531836325890, 11, 680, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1808322839683252225, 5, 199, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1808322839687446530, 5, 200, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1808322839687446531, 5, 201, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1808322839691640833, 5, 202, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1808322839691640834, 5, 203, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1808322839691640835, 5, 204, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1808322839695835138, 5, 205, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1808322839695835139, 5, 973, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1808322839700029441, 5, 206, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1808322839700029442, 5, 974, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1808322839700029443, 5, 207, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1808322839704223746, 5, 975, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1808322839704223747, 5, 208, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1808322839704223748, 5, 209, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1808322839708418049, 5, 241, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1808322839708418050, 5, 210, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1808322839712612353, 5, 242, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1808322839712612354, 5, 211, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1808322839712612355, 5, 212, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1808322839716806657, 5, 213, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1808322839716806658, 5, 214, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1808322839716806659, 5, 215, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1808322839721000962, 5, 216, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1809123859401261057, 9, 768, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1809123859409649666, 9, 769, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1809123859409649667, 9, 770, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1810153446033833985, 6, 970, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1810153446038028289, 6, 971, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1810153446042222593, 6, 972, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1810153446046416897, 6, 973, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1810153446050611202, 6, 974, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1810153446050611203, 6, 975, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1810153446054805505, 6, 976, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1810153446058999810, 6, 977, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1810153446058999811, 6, 978, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1810153446063194114, 6, 979, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1810153446063194115, 6, 980, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1810153446067388417, 6, 981, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1810153446071582722, 6, 982, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1810153446071582723, 6, 983, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1810153446075777025, 6, 984, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1810153446075777026, 6, 985, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1810153446079971330, 6, 986, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1810153446079971331, 6, 987, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1810153446084165633, 6, 988, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1810153446088359937, 6, 989, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1810153446088359938, 6, 990, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1810153446092554242, 6, 991, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1810153446092554243, 6, 992, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1810153446096748545, 6, 993, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1810153446100942850, 6, 994, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1810153446100942851, 6, 995, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1810153446105137153, 6, 996, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1810153446105137154, 6, 997, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1810153446109331458, 6, 998, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1810153446109331459, 6, 999, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1810153446113525761, 6, 1000, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1810153446113525762, 6, 1001, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1810153446117720066, 6, 1002, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1810153446117720067, 6, 1003, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1810153446121914370, 6, 1004, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1810153446126108674, 6, 1005, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1810153446126108675, 6, 1006, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1810153446130302978, 6, 1007, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1810153446130302979, 6, 1009, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1810153446134497281, 6, 1010, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1810153446134497282, 6, 1148, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1810153446134497283, 6, 3072, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1810153446134497284, 6, 3073, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1810153446134497285, 6, 3074, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1810153446134497286, 6, 3075, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882722553858, 2, 2048, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882735136770, 2, 1, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882739331073, 2, 2049, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882739331074, 2, 2, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882739331075, 2, 2050, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882743525377, 2, 3, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882743525378, 2, 2051, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882743525379, 2, 4, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882747719681, 2, 2052, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882747719682, 2, 5, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882747719683, 2, 2053, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882747719684, 2, 6, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882751913986, 2, 2054, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882751913987, 2, 2055, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882751913988, 2, 8, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882751913989, 2, 2056, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882751913990, 2, 9, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882756108290, 2, 2057, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882756108291, 2, 2058, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882756108292, 2, 11, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882756108293, 2, 2059, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882756108294, 2, 2060, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882760302593, 2, 13, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882760302594, 2, 2061, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882760302595, 2, 14, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882760302596, 2, 2062, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882760302597, 2, 2063, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882764496898, 2, 2064, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882764496899, 2, 2065, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882764496900, 2, 2066, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882764496901, 2, 2067, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882768691201, 2, 2068, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882768691202, 2, 2069, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882768691203, 2, 2070, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882768691204, 2, 2071, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882768691205, 2, 2072, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882772885506, 2, 2073, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882772885507, 2, 2074, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882772885508, 2, 2075, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882772885509, 2, 2076, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882772885510, 2, 2077, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882777079809, 2, 2080, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882777079810, 2, 2081, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882777079811, 2, 2082, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882777079812, 2, 2083, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882781274114, 2, 2084, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882781274115, 2, 2085, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882781274116, 2, 38, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882781274117, 2, 2086, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882781274118, 2, 2087, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882785468418, 2, 39, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882785468419, 2, 2088, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882785468420, 2, 40, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882785468421, 2, 2089, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882785468422, 2, 41, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882789662721, 2, 2090, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882789662722, 2, 42, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882789662723, 2, 2091, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882789662724, 2, 43, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882789662725, 2, 2092, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882793857025, 2, 44, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882793857026, 2, 45, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882793857027, 2, 2093, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882793857028, 2, 2094, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882798051329, 2, 47, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882798051330, 2, 2095, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882798051331, 2, 2096, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882798051332, 2, 48, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882802245633, 2, 2097, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882802245634, 2, 49, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882802245635, 2, 2098, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882802245636, 2, 50, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882802245637, 2, 51, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882806439937, 2, 2099, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882806439938, 2, 2100, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882806439939, 2, 52, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882806439940, 2, 2101, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882806439941, 2, 53, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882810634241, 2, 2102, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882810634242, 2, 54, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882810634243, 2, 2103, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882810634244, 2, 55, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882814828546, 2, 2104, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882814828547, 2, 56, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882814828548, 2, 2105, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882814828549, 2, 57, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882814828550, 2, 2106, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882819022849, 2, 58, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882819022850, 2, 2107, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882819022851, 2, 59, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882819022852, 2, 2108, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882819022853, 2, 60, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882823217153, 2, 2109, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882823217154, 2, 61, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882823217155, 2, 62, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882823217156, 2, 63, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882827411458, 2, 64, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882827411459, 2, 65, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882827411460, 2, 66, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882827411461, 2, 67, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882827411462, 2, 68, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882831605761, 2, 70, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882831605762, 2, 71, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882831605763, 2, 72, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882831605764, 2, 73, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882835800065, 2, 74, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882835800066, 2, 76, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882835800067, 2, 77, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882835800068, 2, 78, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882835800069, 2, 79, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882839994369, 2, 80, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882839994370, 2, 81, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882839994371, 2, 83, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882839994372, 2, 84, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882844188674, 2, 85, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882844188675, 2, 87, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882844188676, 2, 88, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882848382978, 2, 89, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882848382979, 2, 90, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882848382980, 2, 91, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882848382981, 2, 92, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882848382982, 2, 93, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882852577281, 2, 94, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882852577282, 2, 96, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882852577283, 2, 97, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882852577284, 2, 99, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882852577285, 2, 100, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882856771586, 2, 101, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882856771587, 2, 102, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882856771588, 2, 103, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882860965890, 2, 104, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882860965891, 2, 105, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882860965892, 2, 112, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882860965893, 2, 113, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882865160193, 2, 114, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882865160194, 2, 115, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882865160195, 2, 116, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882865160196, 2, 123, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882869354498, 2, 124, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882869354499, 2, 125, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882869354500, 2, 126, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882869354501, 2, 127, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882873548801, 2, 128, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882873548802, 2, 157, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882873548803, 2, 158, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882873548804, 2, 159, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882873548805, 2, 161, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882877743105, 2, 162, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882877743106, 2, 164, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882877743107, 2, 165, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882877743108, 2, 166, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882881937410, 2, 167, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882881937411, 2, 168, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882881937412, 2, 169, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882881937413, 2, 174, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882881937414, 2, 184, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882886131714, 2, 188, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882886131715, 2, 190, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882886131716, 2, 191, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882886131717, 2, 192, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882886131718, 2, 193, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882890326017, 2, 194, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882890326018, 2, 199, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882890326019, 2, 200, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882890326020, 2, 201, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882894520322, 2, 202, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882894520323, 2, 203, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882894520324, 2, 204, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882894520325, 2, 205, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882894520326, 2, 206, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882898714625, 2, 207, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882898714626, 2, 208, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882898714627, 2, 209, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882898714628, 2, 210, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882898714629, 2, 211, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882902908930, 2, 212, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882902908931, 2, 213, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882902908932, 2, 214, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882902908933, 2, 215, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882902908934, 2, 216, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882907103234, 2, 217, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882907103235, 2, 218, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882907103236, 2, 219, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882907103237, 2, 220, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882911297538, 2, 221, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882911297539, 2, 222, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882911297540, 2, 223, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882911297541, 2, 224, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882911297542, 2, 225, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882915491842, 2, 226, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882915491843, 2, 227, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882915491844, 2, 228, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882915491845, 2, 229, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882915491846, 2, 230, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882919686145, 2, 231, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882919686146, 2, 232, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882919686147, 2, 233, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882919686148, 2, 235, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882923880449, 2, 236, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882923880450, 2, 237, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882923880451, 2, 238, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882923880452, 2, 239, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882923880453, 2, 240, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882928074754, 2, 241, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882928074755, 2, 242, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882928074756, 2, 243, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882928074757, 2, 249, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882928074758, 2, 250, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882932269058, 2, 251, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882932269059, 2, 252, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882932269060, 2, 253, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882932269061, 2, 254, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882936463361, 2, 255, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882936463362, 2, 263, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882936463363, 2, 268, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882936463364, 2, 269, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882936463365, 2, 270, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882940657666, 2, 271, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882940657667, 2, 272, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882940657668, 2, 273, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882940657669, 2, 274, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882940657670, 2, 275, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882944851970, 2, 277, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882944851971, 2, 278, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882944851972, 2, 279, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882944851973, 2, 280, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882949046273, 2, 281, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882949046274, 2, 282, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882949046275, 2, 283, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882949046276, 2, 284, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882949046277, 2, 285, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882953240578, 2, 286, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882953240579, 2, 287, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882953240580, 2, 288, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882953240581, 2, 292, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882953240582, 2, 293, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882957434882, 2, 294, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882957434883, 2, 295, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882957434884, 2, 296, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882957434885, 2, 297, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882961629186, 2, 298, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882961629187, 2, 299, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882961629188, 2, 300, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882961629189, 2, 301, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882961629190, 2, 302, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882965823489, 2, 303, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882965823490, 2, 304, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882965823491, 2, 305, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882965823492, 2, 306, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882965823493, 2, 309, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882970017794, 2, 310, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882970017795, 2, 311, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882970017796, 2, 312, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882970017797, 2, 313, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882970017798, 2, 314, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882974212098, 2, 315, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882974212099, 2, 316, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882974212100, 2, 317, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882974212101, 2, 318, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882974212102, 2, 319, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882978406402, 2, 320, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882978406403, 2, 321, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882978406404, 2, 322, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882978406405, 2, 323, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882982600705, 2, 324, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882982600706, 2, 325, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882982600707, 2, 326, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882982600708, 2, 327, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882982600709, 2, 328, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882986795010, 2, 329, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882986795011, 2, 330, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882986795012, 2, 331, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882986795013, 2, 332, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882986795014, 2, 333, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882990989314, 2, 334, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882990989315, 2, 335, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882990989316, 2, 336, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882990989317, 2, 337, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882995183618, 2, 338, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882995183619, 2, 339, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882995183620, 2, 340, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882995183621, 2, 341, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882995183622, 2, 342, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882999377922, 2, 343, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882999377923, 2, 345, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882999377924, 2, 356, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391882999377925, 2, 367, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883003572226, 2, 368, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883003572227, 2, 369, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883003572228, 2, 371, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883003572229, 2, 383, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883003572230, 2, 384, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883007766529, 2, 385, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883007766530, 2, 388, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883007766531, 2, 390, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883007766532, 2, 402, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883011960834, 2, 403, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883011960835, 2, 404, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883011960836, 2, 405, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883011960837, 2, 406, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883011960838, 2, 407, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883016155137, 2, 408, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883016155138, 2, 409, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883016155139, 2, 410, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883016155140, 2, 411, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883016155141, 2, 413, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883020349442, 2, 414, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883020349443, 2, 415, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883020349444, 2, 416, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883020349445, 2, 417, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883024543745, 2, 418, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883024543746, 2, 419, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883024543747, 2, 420, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883024543748, 2, 421, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883024543749, 2, 422, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883028738049, 2, 424, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883028738050, 2, 427, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883028738051, 2, 441, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883028738052, 2, 442, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883028738053, 2, 445, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883032932354, 2, 446, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883032932355, 2, 447, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883032932356, 2, 448, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883032932357, 2, 462, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883037126658, 2, 463, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883037126659, 2, 464, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883037126660, 2, 465, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883037126661, 2, 466, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883037126662, 2, 467, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883041320962, 2, 468, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883041320963, 2, 477, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883041320964, 2, 482, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883041320965, 2, 483, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883041320966, 2, 486, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883045515266, 2, 487, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883045515267, 2, 488, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883045515268, 2, 489, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883045515269, 2, 490, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883049709570, 2, 491, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883049709571, 2, 492, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883049709572, 2, 493, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883049709573, 2, 494, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883049709574, 2, 495, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883053903873, 2, 498, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883053903874, 2, 499, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883053903875, 2, 500, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883053903876, 2, 506, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883058098178, 2, 510, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883058098179, 2, 511, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883058098180, 2, 513, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883058098181, 2, 514, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883058098182, 2, 515, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883062292482, 2, 516, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883062292483, 2, 517, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883062292484, 2, 518, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883062292485, 2, 519, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883062292486, 2, 520, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883066486786, 2, 521, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883066486787, 2, 522, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883066486788, 2, 523, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883066486789, 2, 524, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883070681089, 2, 525, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883070681090, 2, 526, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883070681091, 2, 533, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883070681092, 2, 534, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883070681093, 2, 535, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883074875393, 2, 536, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883074875394, 2, 537, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883074875395, 2, 538, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883074875396, 2, 540, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883079069698, 2, 541, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883079069699, 2, 542, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883079069700, 2, 543, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883079069701, 2, 544, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883079069702, 2, 545, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883083264002, 2, 546, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883083264003, 2, 549, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883083264004, 2, 550, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883083264005, 2, 551, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883087458306, 2, 552, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883087458307, 2, 553, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883087458308, 2, 554, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883087458309, 2, 555, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883087458310, 2, 556, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883091652610, 2, 557, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883091652611, 2, 559, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883091652612, 2, 560, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883091652613, 2, 561, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883095846914, 2, 562, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883095846915, 2, 567, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883095846916, 2, 568, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883095846917, 2, 573, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883095846918, 2, 574, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883100041217, 2, 575, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883100041218, 2, 576, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883100041219, 2, 577, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883100041220, 2, 578, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883100041221, 2, 579, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883104235521, 2, 580, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883104235522, 2, 581, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883104235523, 2, 582, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883104235524, 2, 583, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883108429826, 2, 584, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883108429827, 2, 585, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883108429828, 2, 586, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883108429829, 2, 587, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883108429830, 2, 588, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883112624130, 2, 609, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883112624131, 2, 610, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883112624132, 2, 611, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883112624133, 2, 612, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883112624134, 2, 613, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883116818433, 2, 614, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883116818434, 2, 615, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883116818435, 2, 616, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883116818436, 2, 617, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883121012737, 2, 618, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883121012738, 2, 619, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883121012739, 2, 620, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883121012740, 2, 621, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883125207042, 2, 622, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883125207043, 2, 623, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883125207044, 2, 624, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883125207045, 2, 625, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883125207046, 2, 626, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883129401345, 2, 627, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883129401346, 2, 628, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883129401347, 2, 629, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883129401348, 2, 630, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883133595650, 2, 631, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883133595651, 2, 632, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883133595652, 2, 633, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883133595653, 2, 635, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883133595654, 2, 636, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883137789954, 2, 638, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883137789955, 2, 639, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883137789956, 2, 640, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883137789957, 2, 641, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883137789958, 2, 642, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883141984258, 2, 643, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883141984259, 2, 647, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883141984260, 2, 648, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883141984261, 2, 649, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883146178562, 2, 650, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883146178563, 2, 651, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883146178564, 2, 652, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883146178565, 2, 653, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883150372865, 2, 654, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883150372866, 2, 655, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883150372867, 2, 656, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883150372868, 2, 657, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883150372869, 2, 658, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883154567169, 2, 659, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883154567170, 2, 660, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883154567171, 2, 661, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883154567172, 2, 664, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883154567173, 2, 665, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883158761474, 2, 672, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883158761475, 2, 680, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883158761476, 2, 681, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883158761477, 2, 682, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883162955778, 2, 683, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883162955779, 2, 684, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883162955780, 2, 685, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883162955781, 2, 686, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883162955782, 2, 687, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883167150082, 2, 688, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883167150083, 2, 689, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883167150084, 2, 690, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883167150085, 2, 691, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883171344386, 2, 692, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883171344387, 2, 693, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883171344388, 2, 694, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883171344389, 2, 695, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883171344390, 2, 696, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883175538689, 2, 704, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883175538690, 2, 705, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883175538691, 2, 708, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883175538692, 2, 709, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883175538693, 2, 710, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883179732994, 2, 711, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883179732995, 2, 712, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883179732996, 2, 717, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883179732997, 2, 718, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883183927298, 2, 719, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883183927299, 2, 720, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883183927300, 2, 726, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883183927301, 2, 727, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883183927302, 2, 728, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883188121602, 2, 729, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883188121603, 2, 735, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883188121604, 2, 736, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883188121605, 2, 737, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883188121606, 2, 738, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883192315906, 2, 739, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883192315907, 2, 741, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883192315908, 2, 742, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883192315909, 2, 743, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883196510210, 2, 744, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883196510211, 2, 747, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883196510212, 2, 748, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883196510213, 2, 749, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883196510214, 2, 750, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883200704514, 2, 751, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883200704515, 2, 752, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883200704516, 2, 755, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883200704517, 2, 757, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883204898818, 2, 758, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883204898819, 2, 759, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883204898820, 2, 760, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883204898821, 2, 763, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883204898822, 2, 782, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883209093121, 2, 783, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883209093122, 2, 784, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883209093123, 2, 785, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883209093124, 2, 788, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883209093125, 2, 789, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883213287425, 2, 803, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883213287426, 2, 804, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883213287427, 2, 805, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883213287428, 2, 806, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883217481729, 2, 807, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883217481730, 2, 808, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883217481731, 2, 809, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883217481732, 2, 810, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883217481733, 2, 811, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883221676034, 2, 812, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883221676035, 2, 813, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883221676036, 2, 816, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883221676037, 2, 817, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883225870337, 2, 825, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883225870338, 2, 826, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883225870339, 2, 828, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883225870340, 2, 829, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883225870341, 2, 830, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883230064641, 2, 831, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883230064642, 2, 832, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883230064643, 2, 833, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883230064644, 2, 834, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883234258946, 2, 835, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883234258947, 2, 836, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883234258948, 2, 837, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883234258949, 2, 838, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883234258950, 2, 839, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883238453250, 2, 840, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883238453251, 2, 841, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883238453252, 2, 842, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883238453253, 2, 843, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883238453254, 2, 844, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883242647554, 2, 845, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883242647555, 2, 846, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883242647556, 2, 847, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883242647557, 2, 848, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883246841858, 2, 849, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883246841859, 2, 850, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883246841860, 2, 851, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883246841861, 2, 852, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883246841862, 2, 853, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883251036162, 2, 856, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883251036163, 2, 866, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883251036164, 2, 867, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883251036165, 2, 868, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883251036166, 2, 870, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883255230466, 2, 871, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883255230467, 2, 872, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883255230468, 2, 873, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883255230469, 2, 876, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883259424770, 2, 878, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883259424771, 2, 879, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883259424772, 2, 880, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883259424773, 2, 881, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883259424774, 2, 884, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883263619074, 2, 885, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883263619075, 2, 886, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883263619076, 2, 887, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883263619077, 2, 888, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883267813378, 2, 889, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883267813379, 2, 890, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883267813380, 2, 891, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883267813381, 2, 892, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883267813382, 2, 893, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883272007682, 2, 894, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883272007683, 2, 895, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883272007684, 2, 896, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883272007685, 2, 897, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883272007686, 2, 898, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883276201986, 2, 899, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883276201987, 2, 900, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883276201988, 2, 901, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883276201989, 2, 902, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883280396290, 2, 903, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883280396291, 2, 904, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883280396292, 2, 905, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883280396293, 2, 906, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883280396294, 2, 907, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883284590593, 2, 908, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883284590594, 2, 909, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883284590595, 2, 910, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883284590596, 2, 911, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883288784897, 2, 912, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883288784898, 2, 913, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883288784899, 2, 914, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883288784900, 2, 915, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883288784901, 2, 916, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883292979202, 2, 917, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883292979203, 2, 918, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883292979204, 2, 919, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883292979205, 2, 920, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883297173505, 2, 921, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883297173506, 2, 922, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883297173507, 2, 923, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883297173508, 2, 924, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883301367809, 2, 925, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883301367810, 2, 926, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883301367811, 2, 927, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883301367812, 2, 928, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883301367813, 2, 929, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883305562114, 2, 930, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883305562115, 2, 931, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883305562116, 2, 932, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883305562117, 2, 933, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883309756417, 2, 934, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883309756418, 2, 935, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883309756419, 2, 936, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883309756420, 2, 937, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883309756421, 2, 938, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883313950721, 2, 939, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883313950722, 2, 940, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883313950723, 2, 941, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883313950724, 2, 942, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883318145026, 2, 943, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883318145027, 2, 944, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883318145028, 2, 945, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883318145029, 2, 946, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883318145030, 2, 947, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883322339329, 2, 948, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883322339330, 2, 949, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883322339331, 2, 950, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883322339332, 2, 951, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883322339333, 2, 3000, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883326533634, 2, 952, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883326533635, 2, 3001, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883326533636, 2, 953, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883326533637, 2, 3002, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883330727937, 2, 954, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883330727938, 2, 3003, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883330727939, 2, 955, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883330727940, 2, 956, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883330727941, 2, 3004, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883334922241, 2, 3005, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883334922242, 2, 957, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883334922243, 2, 3006, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883334922244, 2, 958, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883334922245, 2, 3007, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883339116545, 2, 959, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883339116546, 2, 960, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883339116547, 2, 961, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883339116548, 2, 3010, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883343310849, 2, 962, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883343310850, 2, 3011, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883343310851, 2, 963, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883343310852, 2, 3012, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883343310853, 2, 964, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883347505153, 2, 3013, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883347505154, 2, 965, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883347505155, 2, 3014, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883347505156, 2, 966, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883347505157, 2, 3015, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883351699457, 2, 967, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883351699458, 2, 3016, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883351699459, 2, 968, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883351699460, 2, 3017, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883355893762, 2, 969, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883355893763, 2, 970, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883355893764, 2, 3018, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883355893765, 2, 3019, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883355893766, 2, 971, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883360088066, 2, 3020, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883360088067, 2, 972, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883360088068, 2, 3021, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883360088069, 2, 973, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883360088070, 2, 3022, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883364282369, 2, 974, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883364282370, 2, 3023, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883364282371, 2, 975, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883364282372, 2, 3024, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883368476674, 2, 976, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883368476675, 2, 3025, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883368476676, 2, 977, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883368476677, 2, 978, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883368476678, 2, 979, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883372670977, 2, 980, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883372670978, 2, 981, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883372670979, 2, 982, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883372670980, 2, 3030, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883376865282, 2, 983, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883376865283, 2, 3031, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883376865284, 2, 984, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883376865285, 2, 985, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883376865286, 2, 986, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883381059586, 2, 987, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883381059587, 2, 988, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883381059588, 2, 989, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883381059589, 2, 990, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883381059590, 2, 991, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883385253889, 2, 3040, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883385253890, 2, 992, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883385253891, 2, 3041, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883385253892, 2, 993, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883385253893, 2, 3042, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883389448193, 2, 994, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883389448194, 2, 3043, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883389448195, 2, 995, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883389448196, 2, 3044, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883393642497, 2, 996, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883393642498, 2, 997, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883393642499, 2, 998, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883393642500, 2, 999, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883393642501, 2, 1000, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883397836802, 2, 1001, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883397836803, 2, 3050, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883397836804, 2, 1002, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883397836805, 2, 3051, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883397836806, 2, 1003, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883402031105, 2, 3052, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883402031106, 2, 1004, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883402031107, 2, 3053, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883402031108, 2, 1005, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883406225409, 2, 1006, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883406225410, 2, 3054, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883406225411, 2, 1007, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883406225412, 2, 1008, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883406225413, 2, 1009, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883410419713, 2, 1010, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883410419714, 2, 1100, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883410419715, 2, 1101, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883410419716, 2, 1102, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883410419717, 2, 1103, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883414614018, 2, 1104, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883414614019, 2, 1105, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883414614020, 2, 1106, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883414614021, 2, 1107, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883418808322, 2, 1108, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883418808323, 2, 1109, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883418808324, 2, 1110, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883418808325, 2, 1111, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883418808326, 2, 1112, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883423002625, 2, 1113, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883423002626, 2, 1114, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883423002627, 2, 1115, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883423002628, 2, 1117, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883427196930, 2, 1118, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883427196931, 2, 1119, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883427196932, 2, 1120, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883427196933, 2, 1121, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883427196934, 2, 1122, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883431391234, 2, 1123, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883431391235, 2, 1124, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883431391236, 2, 1125, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883431391237, 2, 1126, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883435585537, 2, 1128, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883435585538, 2, 1129, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883435585539, 2, 1132, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883435585540, 2, 1133, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883435585541, 2, 1134, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883439779842, 2, 1135, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883439779843, 2, 1136, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883439779844, 2, 1137, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883439779845, 2, 1138, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883439779846, 2, 1139, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883443974146, 2, 1140, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883443974147, 2, 1141, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883443974148, 2, 1142, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883443974149, 2, 1143, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883448168449, 2, 1144, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883448168450, 2, 1145, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883448168451, 2, 1146, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883448168452, 2, 1147, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883448168453, 2, 1148, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883452362753, 2, 1149, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883452362754, 2, 1150, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883452362755, 2, 1174, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883452362756, 2, 1175, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883456557058, 2, 1176, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883456557059, 2, 1177, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883456557060, 2, 1178, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883456557061, 2, 1179, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883456557062, 2, 1180, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883460751361, 2, 1181, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883460751362, 2, 1182, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883460751363, 2, 1999, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883460751364, 2, 2000, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883460751365, 2, 2001, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883464945665, 2, 2002, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883464945666, 2, 2003, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883464945667, 2, 2004, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883464945668, 2, 2005, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883464945669, 2, 2006, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883469139969, 2, 2007, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883469139970, 2, 2008, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883469139971, 2, 2009, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883469139972, 2, 2010, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883473334273, 2, 2011, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883473334274, 2, 2012, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883473334275, 2, 2013, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883473334276, 2, 2014, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883473334277, 2, 2015, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883477528578, 2, 2016, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883477528579, 2, 2017, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883477528580, 2, 2018, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883477528581, 2, 2019, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883481722881, 2, 2020, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883481722882, 2, 2021, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883481722883, 2, 2022, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883481722884, 2, 2023, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883481722885, 2, 2024, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883485917185, 2, 2025, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883485917186, 2, 2026, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883485917187, 2, 2027, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883485917188, 2, 2028, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883485917189, 2, 2029, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883490111490, 2, 2030, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883490111491, 2, 2031, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883490111492, 2, 2032, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883490111493, 2, 2033, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883494305793, 2, 2034, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883494305794, 2, 2035, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883494305795, 2, 2036, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883494305796, 2, 2037, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883494305797, 2, 2038, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883498500098, 2, 2039, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883498500099, 2, 2040, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883498500100, 2, 2041, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883498500101, 2, 2042, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883502694401, 2, 2043, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883502694402, 2, 2044, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883502694403, 2, 2045, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883502694404, 2, 2046, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1813391883502694405, 2, 2047, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498671378433, 3, 2048, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498704932866, 3, 2049, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498704932867, 3, 2050, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498704932868, 3, 2051, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498709127170, 3, 2052, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498709127171, 3, 2053, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498713321474, 3, 2054, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498713321475, 3, 2055, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498717515777, 3, 2058, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498717515778, 3, 2059, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498717515779, 3, 2060, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498721710082, 3, 2066, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498721710083, 3, 2067, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498721710084, 3, 2068, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498725904386, 3, 2069, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498725904387, 3, 2070, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498725904388, 3, 2071, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498730098689, 3, 2072, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498730098690, 3, 2073, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498730098691, 3, 2074, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498730098692, 3, 2075, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498734292994, 3, 2076, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498734292995, 3, 2077, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498734292996, 3, 2080, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498738487298, 3, 2081, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498738487299, 3, 2082, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498738487300, 3, 2083, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498742681602, 3, 2084, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498742681603, 3, 2085, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498746875905, 3, 2086, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498746875906, 3, 40, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498746875907, 3, 49, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498751070210, 3, 2100, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498751070211, 3, 2101, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498751070212, 3, 2102, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498755264513, 3, 2103, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498755264514, 3, 2104, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498755264515, 3, 2105, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498759458818, 3, 70, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498759458819, 3, 71, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498759458820, 3, 72, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498763653122, 3, 77, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498763653123, 3, 1115, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498763653124, 3, 1117, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498767847425, 3, 1118, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498767847426, 3, 1119, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498767847427, 3, 1120, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498767847428, 3, 1121, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498772041729, 3, 1122, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498772041730, 3, 1123, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498772041731, 3, 1124, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498776236033, 3, 1125, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498776236034, 3, 1126, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498776236035, 3, 1128, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498780430338, 3, 1129, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498780430339, 3, 1132, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498780430340, 3, 1133, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498784624641, 3, 1139, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498784624642, 3, 1148, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498788818946, 3, 1174, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498793013249, 3, 1175, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498797207554, 3, 1176, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498797207555, 3, 1177, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498801401858, 3, 1178, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498805596161, 3, 1179, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498809790466, 3, 1180, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498809790467, 3, 1181, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498818179073, 3, 1182, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498826567682, 3, 167, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498830761986, 3, 168, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498830761987, 3, 174, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498839150594, 3, 280, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498843344898, 3, 281, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498847539202, 3, 282, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498851733505, 3, 283, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498851733506, 3, 284, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498851733507, 3, 285, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498855927810, 3, 286, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498855927811, 3, 287, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498855927812, 3, 306, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498860122113, 3, 371, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498860122114, 3, 417, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498860122115, 3, 420, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498864316417, 3, 421, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498864316418, 3, 424, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498864316419, 3, 427, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498868510722, 3, 441, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498868510723, 3, 442, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498868510724, 3, 463, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498868510725, 3, 464, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498872705025, 3, 465, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498872705026, 3, 466, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498872705027, 3, 467, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498876899329, 3, 468, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498876899330, 3, 477, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498876899331, 3, 483, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498881093633, 3, 486, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498881093634, 3, 487, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498881093635, 3, 488, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498885287938, 3, 489, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498885287939, 3, 490, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498885287940, 3, 491, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498885287941, 3, 492, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498889482242, 3, 493, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498889482243, 3, 494, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498893676545, 3, 495, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498893676546, 3, 498, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498893676547, 3, 499, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498893676548, 3, 500, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498897870849, 3, 506, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498897870850, 3, 540, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498897870851, 3, 541, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498902065154, 3, 542, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498902065155, 3, 543, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498902065156, 3, 544, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498906259457, 3, 561, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498906259458, 3, 568, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498910453761, 3, 575, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498910453762, 3, 576, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498910453763, 3, 577, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498914648066, 3, 578, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498914648067, 3, 579, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498914648068, 3, 580, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498918842370, 3, 581, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498918842371, 3, 582, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498918842372, 3, 583, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498923036674, 3, 584, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498923036675, 3, 585, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498923036676, 3, 586, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498927230978, 3, 587, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498927230979, 3, 588, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498927230980, 3, 636, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498927230981, 3, 640, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498931425282, 3, 641, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498931425283, 3, 642, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498931425284, 3, 650, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498935619586, 3, 651, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498935619587, 3, 652, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498935619588, 3, 672, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498939813890, 3, 680, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498939813891, 3, 708, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498939813892, 3, 709, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498944008193, 3, 710, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498944008194, 3, 711, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498944008195, 3, 712, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498948202497, 3, 717, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498948202498, 3, 718, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498948202499, 3, 719, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498952396801, 3, 720, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498952396802, 3, 726, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498952396803, 3, 727, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498952396804, 3, 728, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498956591106, 3, 729, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498956591107, 3, 735, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498956591108, 3, 736, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498960785409, 3, 737, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498960785410, 3, 738, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498960785411, 3, 749, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498964979713, 3, 755, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498964979714, 3, 759, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498964979715, 3, 760, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498964979716, 3, 806, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498969174018, 3, 807, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498969174019, 3, 830, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498969174020, 3, 831, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498973368321, 3, 832, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498973368322, 3, 833, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498977562625, 3, 834, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498977562626, 3, 835, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498977562627, 3, 836, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498981756929, 3, 837, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498981756930, 3, 838, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498981756931, 3, 839, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498985951234, 3, 840, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498985951235, 3, 841, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498990145538, 3, 842, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498990145539, 3, 843, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498990145540, 3, 844, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498994339842, 3, 845, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498994339843, 3, 846, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498994339844, 3, 847, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498998534145, 3, 848, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280498998534146, 3, 849, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499002728450, 3, 850, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499006922754, 3, 851, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499006922755, 3, 852, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499006922756, 3, 853, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499011117058, 3, 866, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499011117059, 3, 876, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499015311361, 3, 884, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499019505665, 3, 885, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499019505666, 3, 886, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499023699969, 3, 887, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499023699970, 3, 888, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499027894274, 3, 889, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499032088577, 3, 890, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499044671489, 3, 891, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499048865793, 3, 892, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499053060098, 3, 893, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499053060099, 3, 894, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499053060100, 3, 895, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499057254401, 3, 896, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499057254402, 3, 897, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499057254403, 3, 898, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499061448705, 3, 899, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499065643010, 3, 900, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499065643011, 3, 901, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499065643012, 3, 902, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499074031617, 3, 903, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499078225922, 3, 904, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499078225923, 3, 905, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499082420225, 3, 906, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499082420226, 3, 907, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499086614529, 3, 908, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499086614530, 3, 909, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499090808833, 3, 910, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499090808834, 3, 911, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499090808835, 3, 912, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499095003137, 3, 913, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499095003138, 3, 917, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499095003139, 3, 918, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499095003140, 3, 919, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499099197442, 3, 920, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499099197443, 3, 921, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499099197444, 3, 922, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499103391745, 3, 923, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499103391746, 3, 924, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499103391747, 3, 925, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499107586050, 3, 926, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499107586051, 3, 927, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499107586052, 3, 928, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499107586053, 3, 929, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499111780354, 3, 930, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499111780355, 3, 931, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499111780356, 3, 932, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499115974657, 3, 933, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499115974658, 3, 934, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499115974659, 3, 935, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499115974660, 3, 936, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499120168961, 3, 937, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499120168962, 3, 938, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499120168963, 3, 939, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499124363265, 3, 940, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499124363266, 3, 941, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499124363267, 3, 942, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499128557569, 3, 943, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499128557570, 3, 944, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499128557571, 3, 945, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499128557572, 3, 946, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499132751873, 3, 947, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499132751874, 3, 948, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499132751875, 3, 949, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499136946177, 3, 950, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499136946178, 3, 951, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499136946179, 3, 3000, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499141140481, 3, 952, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499141140482, 3, 3001, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499141140483, 3, 953, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499145334786, 3, 3002, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499149529090, 3, 954, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499149529091, 3, 955, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499153723393, 3, 3003, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499153723394, 3, 956, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499153723395, 3, 3004, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499153723396, 3, 957, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499162112001, 3, 3005, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499162112002, 3, 958, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499162112003, 3, 3006, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499166306306, 3, 959, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499166306307, 3, 3007, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499166306308, 3, 960, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499166306309, 3, 961, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499166306310, 3, 3010, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499166306311, 3, 962, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499170500609, 3, 3011, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499170500610, 3, 963, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499170500611, 3, 3012, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499174694914, 3, 964, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499174694915, 3, 3013, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499178889217, 3, 965, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499178889218, 3, 3014, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499178889219, 3, 966, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499178889220, 3, 3015, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499183083521, 3, 967, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499183083522, 3, 3016, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499183083523, 3, 968, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499187277825, 3, 969, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499187277826, 3, 3017, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499187277827, 3, 970, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499191472129, 3, 3018, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499191472130, 3, 971, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499191472131, 3, 3019, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499191472132, 3, 972, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499199860737, 3, 3020, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499199860738, 3, 973, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499199860739, 3, 3021, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499199860740, 3, 974, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499199860741, 3, 3022, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499199860742, 3, 1999, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499204055042, 3, 975, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499204055043, 3, 3023, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499204055044, 3, 2000, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499208249345, 3, 976, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499212443649, 3, 3024, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499212443650, 3, 2001, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499216637953, 3, 977, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499216637954, 3, 3025, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499220832258, 3, 2002, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499220832259, 3, 978, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499225026561, 3, 2003, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499229220865, 3, 979, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499229220866, 3, 2004, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499237609473, 3, 980, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499241803777, 3, 2005, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499241803778, 3, 981, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499250192386, 3, 2006, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499266969601, 3, 982, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499279552514, 3, 3030, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499283746817, 3, 2007, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499283746818, 3, 983, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499287941122, 3, 3031, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499287941123, 3, 2008, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499292135425, 3, 984, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499292135426, 3, 2009, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499296329730, 3, 985, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499304718338, 3, 2010, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499308912641, 3, 986, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499313106945, 3, 2011, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499313106946, 3, 987, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499317301250, 3, 2012, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499317301251, 3, 988, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499317301252, 3, 2013, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499321495553, 3, 989, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499321495554, 3, 2014, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499321495555, 3, 990, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499325689857, 3, 2015, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499325689858, 3, 991, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499329884162, 3, 3040, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499329884163, 3, 2016, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499329884164, 3, 992, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499334078466, 3, 3041, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499334078467, 3, 2017, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499338272769, 3, 993, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499338272770, 3, 3042, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499338272771, 3, 2018, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499342467074, 3, 994, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499342467075, 3, 3043, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499342467076, 3, 2019, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499346661378, 3, 995, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499346661379, 3, 3044, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499346661380, 3, 2020, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499350855682, 3, 996, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499350855683, 3, 2021, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499350855684, 3, 997, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499355049985, 3, 2022, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499355049986, 3, 998, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499355049987, 3, 2023, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499355049988, 3, 999, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499359244289, 3, 2024, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499359244290, 3, 1000, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499359244291, 3, 2025, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499363438593, 3, 1001, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499363438594, 3, 2026, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499367632898, 3, 3050, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499367632899, 3, 1002, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499367632900, 3, 2027, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499371827201, 3, 3051, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499371827202, 3, 1003, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499371827203, 3, 2028, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499376021506, 3, 3052, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499376021507, 3, 1004, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499376021508, 3, 2029, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499380215810, 3, 3053, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499380215811, 3, 1005, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499380215812, 3, 1006, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499384410114, 3, 2030, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499384410115, 3, 3054, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499384410116, 3, 2031, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499388604418, 3, 1007, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499388604419, 3, 2032, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499388604420, 3, 1008, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499392798721, 3, 2033, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499392798722, 3, 1009, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499392798723, 3, 2034, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499401187329, 3, 1010, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499401187330, 3, 2035, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499401187331, 3, 2036, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499401187332, 3, 2037, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499405381633, 3, 2038, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499405381634, 3, 2039, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499409575937, 3, 2041, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499413770241, 3, 2042, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499413770242, 3, 2044, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499417964546, 3, 2045, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499417964547, 3, 2046, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280499417964548, 3, 2047, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280562001174530, 4, 650, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280562026340354, 4, 651, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280562026340355, 4, 652, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280562030534658, 4, 272, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280562030534659, 4, 273, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280562030534660, 4, 274, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280562030534661, 4, 275, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280562034728962, 4, 277, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280562034728963, 4, 278, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280562034728964, 4, 279, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280562038923265, 4, 804, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280562038923266, 4, 165, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280562038923267, 4, 805, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280562038923268, 4, 39, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280562043117569, 4, 169, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280562043117570, 4, 42, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280562043117571, 4, 299, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280562047311874, 4, 300, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280562047311875, 4, 301, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280562047311876, 4, 302, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280562047311877, 4, 304, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280562051506177, 4, 305, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280562051506178, 4, 441, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280562051506179, 4, 442, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280562055700482, 4, 445, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280562055700483, 4, 830, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280562055700484, 4, 575, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280562059894786, 4, 191, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280562059894787, 4, 831, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280562059894788, 4, 63, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280562064089089, 4, 576, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280562064089090, 4, 448, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280562064089091, 4, 832, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280562064089092, 4, 64, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280562068283393, 4, 833, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280562068283394, 4, 65, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280562068283395, 4, 66, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280562068283396, 4, 834, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280562072477698, 4, 835, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280562072477699, 4, 67, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280562072477700, 4, 836, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280562076672002, 4, 68, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280562080866305, 4, 837, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280562080866306, 4, 838, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280562085060610, 4, 839, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280562085060611, 4, 840, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280562085060612, 4, 841, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280562085060613, 4, 842, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280562089254913, 4, 3018, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280562089254914, 4, 843, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280562089254915, 4, 844, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280562093449217, 4, 76, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280562093449218, 4, 77, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280562093449219, 4, 845, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280562093449220, 4, 846, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280562097643522, 4, 847, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280562097643523, 4, 848, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280562097643524, 4, 849, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280562101837826, 4, 850, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280562101837827, 4, 851, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280562101837828, 4, 852, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280562101837829, 4, 853, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280562106032129, 4, 1115, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280562106032130, 4, 1117, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280562106032131, 4, 1118, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280562106032132, 4, 1119, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280562110226433, 4, 1120, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280562110226434, 4, 1121, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280562110226435, 4, 482, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280562114420738, 4, 866, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280562114420739, 4, 1122, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280562114420740, 4, 1123, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280562114420741, 4, 1124, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280562118615041, 4, 1125, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280562118615042, 4, 1126, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280562118615043, 4, 1128, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815280562122809346, 4, 1129, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815281000125587457, 10, 739, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815281000133976066, 10, 680, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815281075232989185, 12, 739, 1, NULL);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`, `school_id`) VALUES (1815281075241377793, 12, 680, 1, NULL);

-- ----------------------------
-- Table structure for sys_tools
-- ----------------------------
CREATE TABLE IF NOT EXISTS `sys_tools` (
     `type` VARCHAR(20) NOT NULL COMMENT '类型',
     `path` MEDIUMTEXT NOT NULL COMMENT '文件地址信息',
     `create_id` BIGINT(20) NULL,
     `create_time` BIGINT(20) NULL,
     PRIMARY KEY (`type`))
    COMMENT = '工具管理';

-- ----------------------------
-- Table structure for sys_user
-- ----------------------------
CREATE TABLE IF NOT EXISTS `sys_user` (
  `id` bigint NOT NULL COMMENT '主键',
  `school_id` bigint DEFAULT NULL COMMENT '学校id',
  `login_name` varchar(100) CHARACTER SET utf8mb4 COLLATE utf8mb4_bin NOT NULL COMMENT '用户名',
  `real_name` varchar(100) NOT NULL COMMENT '姓名',
  `code` varchar(100) CHARACTER SET utf8mb4 COLLATE utf8mb4_bin DEFAULT NULL COMMENT '工号',
  `password` varchar(50) NOT NULL COMMENT '密码',
  `mobile_number` varchar(30) DEFAULT NULL COMMENT '手机号',
  `org_id` bigint DEFAULT NULL COMMENT '机构id',
  `enable` tinyint NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
  `pwd_count` tinyint DEFAULT '0' COMMENT '密码修改次数，默认为0',
  `pwd_update_time` bigint DEFAULT NULL COMMENT '密码修改时间',
  `remark` varchar(200) DEFAULT NULL COMMENT '备注',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '修改人',
  `update_time` bigint DEFAULT NULL COMMENT '修改时间',
  `historic_name` varchar(100) DEFAULT NULL COMMENT '历史用户名称',
  `ui_setting` text COMMENT '阅卷参数',
  `allow_manual_card` tinyint(1) DEFAULT '0' COMMENT '是否允许新增卡格式',
  `allow_course_scan` tinyint(1) DEFAULT '1' COMMENT '是否允许识别课程条码',
  PRIMARY KEY (`id`) USING BTREE,
  UNIQUE KEY `user_schoolId_loginName_idx` (`school_id`,`login_name`) USING BTREE,
  UNIQUE KEY `uq_school_id_mobile_number` (`school_id`,`mobile_number`) USING BTREE,
    KEY `org_id` (`org_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='用户表';

INSERT INTO `sys_user` (`id`, `school_id`, `login_name`, `real_name`, `code`, `password`, `mobile_number`, `org_id`, `enable`, `pwd_count`, `pwd_update_time`, `remark`, `create_id`, `create_time`, `update_id`, `update_time`, `historic_name`, `ui_setting`, `allow_manual_card`, `allow_course_scan`) VALUES (1, NULL, 'sysadmin', 'sysadmin', NULL, 'UW10aDg3ODYzNTc3', NULL, NULL, 1, 2, 1700550554413, NULL, NULL, NULL, NULL, 1700550554425, NULL, NULL, 0, 1);

-- ----------------------------
-- Table structure for sys_user_role
-- ----------------------------
CREATE TABLE IF NOT EXISTS `sys_user_role` (
  `id` bigint NOT NULL COMMENT '主键',
  `user_id` bigint NOT NULL COMMENT '用户id',
  `role_id` bigint NOT NULL COMMENT '角色id',
  `privilege_id` bigint DEFAULT NULL COMMENT '权限id',
  `enable` tinyint NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='用户和角色关联表';

-- ----------------------------
-- Records of sys_user_role
-- ----------------------------
INSERT INTO `sys_user_role` (`id`, `user_id`, `role_id`, `privilege_id`, `enable`) VALUES (1, 1, 1, NULL, 1);

-- ----------------------------
-- Table structure for t_b_session
-- ----------------------------
CREATE TABLE IF NOT EXISTS `t_b_session` (
  `id` varchar(100) NOT NULL COMMENT '主键',
  `school_id` bigint DEFAULT NULL COMMENT '学校id',
  `org_id` bigint DEFAULT NULL COMMENT '机构id',
  `identity` varchar(100) NOT NULL COMMENT '用户标识',
  `type` varchar(200) NOT NULL COMMENT '用户类型',
  `source` varchar(50) NOT NULL COMMENT '访问来源',
  `platform` varchar(50) NOT NULL COMMENT '设备分类',
  `device_id` varchar(100) NOT NULL COMMENT '设备标识',
  `address` varchar(100) NOT NULL COMMENT '登录IP地址',
  `access_token` varchar(50) NOT NULL COMMENT '访问令牌',
  `last_access_time` bigint DEFAULT NULL COMMENT '最近访问时间',
  `last_access_ip` varchar(100) DEFAULT NULL COMMENT '最近访问IP地址',
  `update_time` bigint DEFAULT NULL COMMENT '令牌更新时间',
  `expire_time` bigint DEFAULT NULL COMMENT '令牌强制失效时间',
  `app_source` varchar(50) DEFAULT NULL COMMENT '应用来源',
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='会话表';

-- ----------------------------
-- Table structure for t_b_sync_task
-- ----------------------------
CREATE TABLE IF NOT EXISTS `t_b_sync_task` (
  `id` bigint NOT NULL COMMENT '主键',
  `school_id` bigint DEFAULT NULL COMMENT '学校id',
  `semester_id` bigint DEFAULT NULL COMMENT '学期',
  `exam_id` bigint DEFAULT NULL COMMENT '考试',
  `course_id` bigint DEFAULT NULL COMMENT '课程ID（basic_course表ID）',
  `course_code` varchar(50) DEFAULT NULL COMMENT '课程代码',
  `course_name` varchar(100) DEFAULT NULL COMMENT '课程名称',
  `paper_number` varchar(100) DEFAULT NULL COMMENT '试卷编号',
  `paper_type` varchar(10) DEFAULT NULL COMMENT '试卷类型',
  `org_id` bigint DEFAULT NULL COMMENT '机构id',
  `type` varchar(50) NOT NULL COMMENT '推送类型，USER_PUSH：用户推送',
  `exam_time` varchar(20) DEFAULT NULL COMMENT '考试时间',
  `object_id` bigint DEFAULT NULL COMMENT '印刷计划id',
  `status` varchar(30) NOT NULL COMMENT '任务状态，INIT：未开始，RUNNING：进行中，FINISH：已完成',
  `summary` text COMMENT '实时摘要信息',
  `result` varchar(30) DEFAULT NULL COMMENT '数据结果，SUCCESS：成功，ERROR：失败',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '修改人',
  `update_time` bigint DEFAULT NULL COMMENT '修改时间',
  `remark` mediumtext COMMENT '备注',
  `reset_count` int DEFAULT '0' COMMENT '重试次数',
  `error_message` varchar(500) DEFAULT NULL COMMENT '人工错误原因',
  `report_file_path` varchar(500) DEFAULT NULL COMMENT '报告路径',
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='云阅卷同步任务表';

-- ----------------------------
-- Table structure for t_b_task
-- ----------------------------
CREATE TABLE IF NOT EXISTS `t_b_task` (
  `id` bigint NOT NULL COMMENT '主键',
  `semester_id` bigint DEFAULT NULL COMMENT '学期',
  `exam_id` bigint DEFAULT NULL COMMENT '考试',
  `course_id` bigint DEFAULT NULL COMMENT '课程ID',
  `course_code` varchar(50) DEFAULT NULL COMMENT '课程代码',
  `course_name` varchar(100) DEFAULT NULL COMMENT '课程名称',
  `paper_number` varchar(100) DEFAULT NULL COMMENT '试卷编号',
  `type` varchar(50) NOT NULL COMMENT '任务类型，USER_IMPORT：用户导入，\nQUESTION_MISSION_BATCH_CREATE：批量新建命题任务，\nSAMPLE_EXPORT：导出审核样本，\nPAPER_DOWNLOAD：卷库下载，\nEXAMINATION_IMPORT：考务数据导入，\nEXAMINATION_EXPORT：考务数据导出，\nPRINT_PDF_DOWNLOAD：批量下载pdf，\nCREATE_PDF：生成pdf，\nCOURSE_IMPORT：课程导入',
  `entity_id` bigint DEFAULT NULL COMMENT '关联业务对象id',
  `status` varchar(30) NOT NULL COMMENT '任务状态，INIT：未开始，RUNNING：进行中，FINISH：已完成',
  `summary` text COMMENT '实时摘要信息',
  `progress` double DEFAULT NULL COMMENT '执行进度',
  `result` varchar(30) DEFAULT NULL COMMENT '数据结果，SUCCESS：成功，ERROR：失败',
  `enable` tinyint NOT NULL DEFAULT '1' COMMENT '是否启用，false：停用，true：启用',
  `import_file_name` varchar(100) DEFAULT NULL COMMENT '导入文件名',
  `import_file_path` varchar(500) DEFAULT NULL COMMENT '导入文件路径',
  `result_file_path` varchar(500) DEFAULT NULL COMMENT '导出文件路径',
  `error_file_path` varchar(500) DEFAULT NULL COMMENT '错误数据文件路径',
  `report_file_path` varchar(500) DEFAULT NULL COMMENT '报告路径',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `school_id` bigint DEFAULT NULL COMMENT '学校id',
  `org_id` bigint DEFAULT NULL COMMENT '机构id',
  `print_plan_id` bigint DEFAULT NULL COMMENT '印刷计划id',
  `remark` mediumtext COMMENT '备注',
  `obj_name` varchar(500) DEFAULT NULL COMMENT '实体名称',
  `reset_count` int DEFAULT '0' COMMENT '重试次数',
  `version` int DEFAULT '0' COMMENT '更新版本号',
  `error_message` varchar(500) DEFAULT NULL COMMENT '人工错误原因',
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='导入导出任务表';

-- ----------------------------
-- Table structure for t_b_task_pdf
-- ----------------------------
CREATE TABLE IF NOT EXISTS `t_b_task_pdf` (
  `id` bigint NOT NULL COMMENT '主键（与exam_detail表id一致）',
  `school_id` bigint NOT NULL COMMENT '学校id',
  `semester_id` bigint DEFAULT NULL COMMENT '学期ID',
  `exam_id` bigint DEFAULT NULL COMMENT '考试ID',
  `print_plan_id` bigint NOT NULL COMMENT '印刷计划id',
  `course_name_code` mediumtext COMMENT '课程名称',
  `paper_number` mediumtext COMMENT '试卷编号',
  `create_type` varchar(45) DEFAULT NULL COMMENT '生成pdf类型：ALL("全部"),PAPER("试卷"),CARD_A3("题卡"),SIGN("签到表"),PACKAGE("卷袋贴"),CHECK_IN("登记表")',
  `status` varchar(30) NOT NULL COMMENT '任务状态，INIT：未开始，RUNNING：进行中，FINISH：已完成',
  `summary` mediumtext COMMENT '实时摘要信息',
  `result` varchar(30) DEFAULT NULL COMMENT '数据结果，SUCCESS：成功，ERROR：失败',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  `remark` mediumtext COMMENT '备注',
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='pdf生成日志表';

-- ----------------------------
-- Table structure for t_c_final_score
-- ----------------------------
CREATE TABLE IF NOT EXISTS `t_c_final_score` (
  `id` bigint NOT NULL COMMENT '主键',
  `culture_program_id` bigint DEFAULT NULL COMMENT '培养方案id',
  `exam_id` bigint DEFAULT NULL COMMENT '考试id',
  `course_id` bigint DEFAULT NULL COMMENT '课程id',
  `course_code` varchar(100) DEFAULT NULL COMMENT '课程编码',
  `course_name` varchar(100) DEFAULT NULL COMMENT '课程名称',
  `paper_number` varchar(100) DEFAULT NULL COMMENT '试卷编码',
  `name` varchar(100) NOT NULL COMMENT '考生姓名',
  `student_code` varchar(100) NOT NULL COMMENT '学号',
  `score` double NULL COMMENT '分数',
  `score_detail` mediumtext NULL COMMENT '成绩明细',
  `enable` tinyint NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
  `source` varchar(50) DEFAULT NULL COMMENT '来源，EXCEL_IMPORT：excel导入，SYNC：数据同步',
  `teach_class_name` varchar(100) NOT NULL COMMENT '教学班',
  `create_id` bigint DEFAULT NULL COMMENT '创建人id',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人id',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  PRIMARY KEY (`id`),
  UNIQUE KEY `t_c_final_score_unique` (`culture_program_id`,`course_id`,`student_code`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='期末考试成绩表';

-- ----------------------------
-- Table structure for t_c_paper_struct
-- ----------------------------
CREATE TABLE IF NOT EXISTS `t_c_paper_struct` (
  `id` bigint NOT NULL COMMENT '主键',
  `culture_program_id` bigint DEFAULT NULL COMMENT '培养方案id',
  `exam_id` bigint DEFAULT NULL COMMENT '考试id',
  `course_id` bigint DEFAULT NULL COMMENT '课程id',
  `course_code` varchar(100) NOT NULL COMMENT '课程编码',
  `course_name` varchar(100) DEFAULT NULL COMMENT '课程名称',
  `paper_number` varchar(100) DEFAULT NULL COMMENT '试卷编码',
  `paper_struct` mediumtext COMMENT '试卷结构',
  `paper_struct_dimension` mediumtext COMMENT '试卷知识点结构',
  `total_score` double DEFAULT NULL COMMENT '全卷满分',
  `pass_score` double DEFAULT NULL COMMENT '及格分',
  `enable` tinyint NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
  `dimension_sign` bigint DEFAULT NULL COMMENT '知识点变化id',
  `create_id` bigint DEFAULT NULL COMMENT '创建人id',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人id',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  PRIMARY KEY (`id`),
  UNIQUE KEY `t_c_paper_struct_unique` (`culture_program_id`,`course_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='试卷蓝图表';

-- ----------------------------
-- Table structure for t_c_statistics
-- ----------------------------
CREATE TABLE IF NOT EXISTS `t_c_statistics` (
  `id` bigint NOT NULL COMMENT '主键',
  `exam_id` bigint NOT NULL COMMENT '考试id',
  `exam_task_detail_ids` mediumtext COMMENT '命题任务明细id',
  `exam_detail_course_ids` mediumtext COMMENT '考务数据课程id',
  `college_id` bigint NOT NULL COMMENT '学院id',
  `college_name` varchar(100) NOT NULL COMMENT '学院名称',
  `course_name` varchar(100) NOT NULL COMMENT '课程名称',
  `course_code` varchar(100) NOT NULL COMMENT '课程代码',
  `teaching_room_id` bigint NOT NULL COMMENT '教研室id',
  `teaching_room_name` varchar(100) NOT NULL COMMENT '教研室名称',
  `teacher_name` mediumtext NOT NULL COMMENT '任课老师名称',
  `clazz_ids` varchar(100) NOT NULL COMMENT '班级ids',
  `clazz_name` varchar(100) NOT NULL COMMENT '班级名称',
  `paper_number` varchar(100) DEFAULT NULL COMMENT '试卷编号',
  `print_plan_id` bigint DEFAULT NULL COMMENT '印刷计划id',
  `batch_no` varchar(100) NOT NULL COMMENT '批次号',
  `create_id` bigint DEFAULT NULL COMMENT '创建人id',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人id',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  `print_sum` varchar(100) DEFAULT NULL COMMENT '总印份数',
  `status` varchar(50) DEFAULT NULL COMMENT '完成状态，FINISH：已完成，UN_FINISH：未完成',
  `exam_detail_id` bigint DEFAULT NULL COMMENT '命题明细id',
  `clazz_content` mediumtext COMMENT '班级详细信息内容',
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='命题统计表';

-- ----------------------------
-- Table structure for t_c_statistics_temp
-- ----------------------------
CREATE TABLE IF NOT EXISTS `t_c_statistics_temp` (
  `id` bigint NOT NULL COMMENT '主键',
  `exam_id` bigint NOT NULL COMMENT '考试id',
  `exam_task_detail_id` bigint DEFAULT NULL COMMENT '命题任务明细id',
  `exam_detail_course_id` bigint DEFAULT NULL COMMENT '考务数据课程id',
  `college_id` bigint NOT NULL COMMENT '学院id',
  `college_name` varchar(100) NOT NULL COMMENT '学院名称',
  `course_name` varchar(100) NOT NULL COMMENT '课程名称',
  `course_code` varchar(100) NOT NULL COMMENT '课程代码',
  `teaching_room_id` bigint NOT NULL COMMENT '教研室id',
  `teaching_room_name` varchar(100) NOT NULL COMMENT '教研室名称',
  `teacher_name` mediumtext NOT NULL COMMENT '任课老师名称',
  `clazz_ids` varchar(100) NOT NULL COMMENT '班级ids',
  `clazz_name` varchar(100) NOT NULL COMMENT '班级名称',
  `paper_number` varchar(100) DEFAULT NULL COMMENT '试卷编号',
  `print_plan_id` bigint DEFAULT NULL COMMENT '印刷计划id',
  `batch_no` varchar(100) NOT NULL COMMENT '批次号',
  `create_id` bigint DEFAULT NULL COMMENT '创建人id',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人id',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  `data_source` varchar(50) DEFAULT NULL COMMENT '数据来源，IMPORT：导入，JOIN：关联查询的数据',
  `exam_detail_id` bigint DEFAULT NULL COMMENT '命题明细id',
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='命题统计表';

-- ----------------------------
-- Table structure for t_c_usual_score
-- ----------------------------
CREATE TABLE IF NOT EXISTS `t_c_usual_score` (
    `id` bigint NOT NULL COMMENT '主键',
    `culture_program_id` bigint NULL DEFAULT NULL COMMENT '培养方案id',
    `exam_id` bigint NULL DEFAULT NULL COMMENT '考试id',
    `course_id` bigint NULL DEFAULT NULL COMMENT '课程id',
    `course_code` varchar(100) NULL DEFAULT NULL COMMENT '课程编码',
    `course_name` varchar(100) NULL DEFAULT NULL COMMENT '课程名称',
    `paper_number` varchar(100) NULL DEFAULT NULL COMMENT '试卷编码',
    `name` varchar(100) NOT NULL COMMENT '考生姓名',
    `student_code` varchar(100) NOT NULL COMMENT '学号',
    `score` mediumtext NULL COMMENT '平常作业分数',
    `enable` tinyint NOT NULL DEFAULT 1 COMMENT '是否启用，0：停用，1：启用',
    `source` varchar(50) NULL DEFAULT NULL COMMENT '来源，EXCEL_IMPORT：excel导入，SYNC：数据同步',
    `teach_class_name` varchar(100) NOT NULL COMMENT '教学班',
    `create_id` bigint NULL DEFAULT NULL COMMENT '创建人id',
    `create_time` bigint NULL DEFAULT NULL COMMENT '创建时间',
    `update_id` bigint NULL DEFAULT NULL COMMENT '更新人id',
    `update_time` bigint NULL DEFAULT NULL COMMENT '更新时间',
  PRIMARY KEY (`id`),
  UNIQUE KEY `t_c_usual_score_unique` (`culture_program_id`,`course_id`,`student_code`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='平常作业成绩表';

-- ----------------------------
-- Table structure for t_f_custom_flow
-- ----------------------------
CREATE TABLE IF NOT EXISTS `t_f_custom_flow` (
  `id` bigint NOT NULL COMMENT '主键',
  `school_id` bigint NOT NULL COMMENT '学校id',
  `org_id` bigint DEFAULT NULL COMMENT '机构id',
  `name` varchar(100) NOT NULL COMMENT '流程名称',
  `model_type` varchar(50) NOT NULL COMMENT '模式类型，USER_FIXED：指定人员,APPROVE_SET：发起人指定',
  `type` varchar(100) NOT NULL COMMENT '流程类型，ELECTRON_FLOW：电子交卷审核，QUESTION_ELECTRON_FLOW：题库试题提交审核，PAPER_ELECTRON_FLOW：题库试卷审核',
  `enable` tinyint DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
  `publish` tinyint NOT NULL DEFAULT '1' COMMENT '是否发布，0：否，1：是',
  `object_data` mediumtext NOT NULL COMMENT '自定义流程源数据',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  `version` bigint NOT NULL DEFAULT '1' COMMENT '版本号',
  `flow_key` varchar(100) DEFAULT NULL COMMENT '流程key',
  `act_flow_id` varchar(100) DEFAULT NULL COMMENT '流程版本id',
  `flow_process_var` mediumtext COMMENT '流程过程变量数据',
  PRIMARY KEY (`id`) USING BTREE,
  UNIQUE KEY `custom_flow_schoolId_type_idx` (`school_id`,`type`,`enable`,`publish`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='自定义流程表';

-- ----------------------------
-- Table structure for t_f_custom_flow_entity
-- ----------------------------
CREATE TABLE IF NOT EXISTS `t_f_custom_flow_entity` (
  `id` bigint NOT NULL COMMENT '主键',
  `t_f_custom_flow_id` bigint NOT NULL COMMENT '自定义流程id',
  `flow_id` bigint DEFAULT NULL COMMENT '流程id',
  `flow_process_var` mediumtext NOT NULL COMMENT '流程过程变量',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  PRIMARY KEY (`id`) USING BTREE,
  KEY `flow_entity_flowId_idx` (`flow_id`) USING BTREE,
  KEY `flow_entity_custom_flowId_idx` (`t_f_custom_flow_id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='自定义流程实体数据';

-- ----------------------------
-- Table structure for t_f_flow_approve
-- ----------------------------
CREATE TABLE IF NOT EXISTS `t_f_flow_approve` (
  `id` bigint NOT NULL COMMENT '主键',
  `school_id` bigint NOT NULL COMMENT '学校id',
  `org_id` bigint DEFAULT NULL COMMENT '机构id',
  `flow_id` bigint NOT NULL COMMENT '流程id',
  `approve_id` bigint DEFAULT NULL COMMENT '审核id',
  `status` varchar(50) NOT NULL COMMENT '状态，START：已开始，AUDITING：审核中，REJECT：已驳回，END：已终止，FINISH：已结束',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  `setup` tinyint NOT NULL COMMENT '计划步骤',
  `approve_setup` int DEFAULT NULL COMMENT '审批步骤',
  `approve_max_setup` int DEFAULT NULL COMMENT '最大审批步骤',
  `transfer_assignee` mediumtext COMMENT '流程转办',
  `all_assignee` mediumtext COMMENT '会签审批人',
  `reject_link` mediumtext COMMENT '驳回路线',
  `enable` tinyint NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
  PRIMARY KEY (`id`) USING BTREE,
  KEY `flow_approve_idx` (`school_id`,`flow_id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='流程审核表';

-- ----------------------------
-- Table structure for t_f_flow_join
-- ----------------------------
CREATE TABLE IF NOT EXISTS `t_f_flow_join` (
  `id` bigint NOT NULL COMMENT '主键',
  `flow_entity_id` bigint NOT NULL COMMENT '自定义流程实体id',
  `object_table` varchar(100) NOT NULL COMMENT '实体表',
  `object_id` bigint NOT NULL COMMENT '实体id',
  `flow_id` bigint NOT NULL COMMENT '流程id',
  `level` int NOT NULL COMMENT '流程等级',
  `type` varchar(50) NOT NULL COMMENT '类型，NEW：新增，RESTART：重新提交',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  PRIMARY KEY (`id`) USING BTREE,
  KEY `task_flowId_idx` (`flow_id`) USING BTREE,
  KEY `task_objectId_idx` (`object_id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='实体和流程关联表';

-- ----------------------------
-- Table structure for t_f_flow_log
-- ----------------------------
CREATE TABLE IF NOT EXISTS `t_f_flow_log` (
  `id` bigint NOT NULL COMMENT '主键',
  `school_id` bigint NOT NULL COMMENT '学校id',
  `org_id` bigint DEFAULT NULL COMMENT '机构id',
  `flow_id` bigint NOT NULL COMMENT '流程id',
  `object_table` varchar(100) DEFAULT NULL COMMENT '实体表',
  `object_id` bigint NOT NULL COMMENT '实体id',
  `approve_id` bigint DEFAULT NULL COMMENT '审核人id',
  `approve_setup` tinyint DEFAULT NULL COMMENT '审核步骤',
  `approve_remark` mediumtext COMMENT '审核意见',
  `enable` tinyint NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  `approve_operation` varchar(50) NOT NULL COMMENT '审批操作，SUBMIT：提交，APPROVE：审批，REJECT：驳回，END：终止',
  `pend_approve_id` mediumtext COMMENT '待审核人id',
  `paper_attachment_id` mediumtext COMMENT '试卷附件',
  `approve_multi_setup` int DEFAULT NULL COMMENT '审批步骤',
  `approve_user_approve_type` varchar(30) DEFAULT NULL COMMENT '审批用户类别，ORDER：依次审批，ALL：会签（所有人必须审批），SOME：或签（一名审批人同意或拒绝即可）',
  `task_id` bigint DEFAULT NULL COMMENT '流程节点id',
  PRIMARY KEY (`id`) USING BTREE,
  KEY `flow_log_idx` (`school_id`,`flow_id`) USING BTREE,
  KEY `flow_log_objectId_idx` (`object_id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='流程流水表';

-- ----------------------------
-- Table structure for t_g_error
-- ----------------------------
CREATE TABLE IF NOT EXISTS `t_g_error` (
  `id` bigint NOT NULL COMMENT '主键',
  `school_id` bigint DEFAULT NULL COMMENT '学校id',
  `org_id` bigint DEFAULT NULL COMMENT '机构id',
  `obj_id` bigint DEFAULT NULL COMMENT '实体id',
  `summary` mediumtext COMMENT '实时摘要信息',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='全局异常错误信息表';

-- ----------------------------
-- Table structure for t_m_mq_message
-- ----------------------------
CREATE TABLE IF NOT EXISTS `t_m_mq_message` (
  `id` varchar(50) NOT NULL COMMENT '主键',
  `school_id` bigint DEFAULT NULL COMMENT '学校id',
  `org_id` bigint DEFAULT NULL COMMENT '机构id',
  `topic` varchar(100) NOT NULL COMMENT '消息topic',
  `tag` varchar(100) NOT NULL COMMENT '消息tag',
  `body` mediumtext COMMENT '消息内容',
  `type` varchar(50) NOT NULL COMMENT '消息类型',
  `obj_id` varchar(50) NOT NULL COMMENT '关联业务id',
  `obj_name` varchar(100) DEFAULT NULL COMMENT '关联业务名称',
  `ack` tinyint NOT NULL COMMENT '消息ack',
  `sequence` int DEFAULT NULL COMMENT '消息序号',
  `properties` varchar(1000) DEFAULT NULL COMMENT '扩展类型',
  `create_time` bigint NOT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  `timestamp` bigint NOT NULL COMMENT '时间戳',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='mq消息';

-- ----------------------------
-- Table structure for t_r_basic_info
-- ----------------------------
CREATE TABLE IF NOT EXISTS `t_r_basic_info` (
  `id` bigint NOT NULL COMMENT '主键',
  `culture_program_id` bigint DEFAULT NULL COMMENT '培养方案id',
  `exam_id` bigint DEFAULT NULL COMMENT '考试id',
  `course_id` bigint DEFAULT NULL COMMENT '课程id',
  `course_code` varchar(100) NOT NULL COMMENT '课程编码',
  `course_name` varchar(100) DEFAULT NULL COMMENT '课程名称',
  `course_en_name` varchar(100) DEFAULT NULL COMMENT '课程英文名称',
  `paper_number` varchar(100) DEFAULT NULL COMMENT '试卷编码',
  `course_type` varchar(100) DEFAULT NULL COMMENT '课程性质',
  `credit` varchar(50) DEFAULT NULL COMMENT '学分',
  `period` varchar(50) DEFAULT NULL COMMENT '学时',
  `college` varchar(200) DEFAULT NULL COMMENT '开课学院',
  `profession` varchar(200) DEFAULT NULL COMMENT '开课专业',
  `evaluation_mode` varchar(100) DEFAULT NULL COMMENT '考核方式',
  `open_time` varchar(100) DEFAULT NULL COMMENT '开课时间',
  `teaching_object` varchar(1000) DEFAULT NULL COMMENT '授课对象',
  `selection_count` int DEFAULT NULL COMMENT '选课人数',
  `participant_count` int DEFAULT NULL COMMENT '参评人数',
  `course_degree` double DEFAULT NULL COMMENT '课程目标达成度期望值',
  `teacher` varchar(100) DEFAULT NULL COMMENT '任课老师',
  `director` varchar(100) DEFAULT NULL COMMENT '评价责任人',
  `participant` varchar(100) DEFAULT NULL COMMENT '评价参与人',
  `course_suggest` mediumtext COMMENT '课程持续改进',
  `enable` tinyint NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
  `teach_class_name` varchar(100) NOT NULL COMMENT '教学班',
  `create_id` bigint DEFAULT NULL COMMENT '创建人id',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人id',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  `course_evaluation_spread` mediumtext COMMENT '课程目标考核分布',
  `course_evaluation_result` mediumtext COMMENT '课程考核成绩评价结果',
  `course_evaluation_result_detail` mediumtext COMMENT '课程考核成绩评价明细结果',
  PRIMARY KEY (`id`),
  UNIQUE KEY `t_r_basic_info_unique` (`culture_program_id`, `course_id`, `exam_id`, `paper_number`, `teach_class_name`)
    ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='报告基本情况表';

-- ----------------------------
-- Table structure for t_r_exam_student
-- ----------------------------
CREATE TABLE IF NOT EXISTS `t_r_exam_student` (
  `id` bigint NOT NULL COMMENT '主键',
  `r_basic_info_id` bigint NOT NULL COMMENT '报告id',
  `name` varchar(100) DEFAULT NULL COMMENT '考生姓名',
  `student_code` varchar(100) DEFAULT NULL COMMENT '学号',
  `administrative_class` varchar(100) DEFAULT NULL COMMENT '行政班级',
  `score` double DEFAULT NULL COMMENT '学生成绩',
  `result_detail` mediumtext COMMENT '考生课程考核成绩评价明细结果',
  PRIMARY KEY (`id`),
  KEY `idx_1` (`r_basic_info_id`,`student_code`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='报告考生表';

-- ----------------------------
-- Table structure for t_s_auth
-- ----------------------------
CREATE TABLE IF NOT EXISTS `t_s_auth` (
  `id` bigint NOT NULL COMMENT '主键',
  `school_id` bigint DEFAULT NULL COMMENT '学校id',
  `access_key` varchar(255) DEFAULT NULL COMMENT 'accessKey',
  `access_secret` varchar(255) DEFAULT NULL COMMENT 'accessSecret',
  `description` text COMMENT '离线授权证书',
  `type` varchar(16) NOT NULL COMMENT '授权类型',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  `expire_time` bigint DEFAULT NULL COMMENT '过期时间',
  `file` mediumblob COMMENT '文件数据',
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='激活授权配置表';

-- ----------------------------
-- Table structure for t_s_job_log
-- ----------------------------
CREATE TABLE `t_s_job_log` (
       `id` bigint NOT NULL COMMENT '主键',
       `type` varchar(100) NOT NULL COMMENT '类型，CREATE_PDF_JOB：创建pdf',
       `obj_id` bigint DEFAULT NULL COMMENT '实体id',
       `obj_name` varchar(1000) DEFAULT NULL COMMENT '实体名称',
       `job_exec_time` varchar(200) DEFAULT NULL COMMENT '执行时间',
       `create_id` bigint NOT NULL COMMENT '创建人id',
       `create_time` bigint NOT NULL COMMENT '创建时间',
       PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='任务日志表';

-- ----------------------------
-- Table structure for t_school_privilege
-- ----------------------------
CREATE TABLE IF NOT EXISTS `t_school_privilege` (
  `id` bigint NOT NULL COMMENT '主键',
  `school_id` bigint NOT NULL COMMENT '学校id',
  `privilege_id` bigint NOT NULL COMMENT '菜单权限id',
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='学校菜单权限表';

-- ----------------------------
-- Table structure for t_school_role
-- ----------------------------
CREATE TABLE IF NOT EXISTS `t_school_role` (
  `id` bigint NOT NULL COMMENT '主键',
  `school_id` bigint NOT NULL COMMENT '学校id',
  `role_id` bigint NOT NULL COMMENT '菜单权限id',
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='学校角色权限表';

-- ----------------------------
-- Table structure for t_sync_data
-- ----------------------------
CREATE TABLE IF NOT EXISTS `t_sync_data` (
       `school_id` BIGINT(20) NOT NULL COMMENT '学校ID',
       `semester_id` BIGINT(20) NOT NULL COMMENT '学期ID',
       `exam_id` BIGINT(20) NOT NULL COMMENT '考试ID',
       `data_type` VARCHAR(45) NOT NULL COMMENT '同步数据类型（A：机构同步，B：用户同步，C：课程同步，D：考生同步，E：命题任务同步）',
       `start_time` BIGINT(20) NULL COMMENT '同步开始时间',
       `end_time` BIGINT(20) NULL COMMENT '同步结束时间',
       `last_sync_time` BIGINT(20) NULL COMMENT '最后一次同步时间',
       `status` TINYINT(1) NULL DEFAULT 0 COMMENT '状态（0：未同步/已完成，1：同步中）',
       `enable` TINYINT(1) NULL DEFAULT 1 COMMENT '启用/禁用',
       `cron` VARCHAR(45) NULL COMMENT 'cron表达式',
       `phone_number` VARCHAR(100) NULL COMMENT '手机号（多个用逗号分隔）',
       `detail` VARCHAR(200) NULL COMMENT '同步明细',
       `create_time` BIGINT(20) NULL COMMENT '创建时间',
       `update_time` BIGINT(20) NULL COMMENT '更新时间',
       PRIMARY KEY (`school_id`, `exam_id`))
    COMMENT = '数据同步设置';

-- ----------------------------
-- Table structure for t_sync_data_log
-- ----------------------------
CREATE TABLE IF NOT EXISTS `t_sync_data_log` (
       `id` BIGINT(20) NOT NULL,
       `school_id` BIGINT(20) NOT NULL COMMENT '学校ID',
       `exam_id` BIGINT(20) NOT NULL COMMENT '考试ID',
       `data_type` VARCHAR(45) NOT NULL COMMENT '同步数据类型',
       `start_time` BIGINT(20) NULL COMMENT '开始同步时间',
       `end_time` BIGINT(20) NULL COMMENT '结束同步时间',
       `count` INT NULL COMMENT '同步数量',
        `success_count` INT(11) NULL DEFAULT NULL COMMENT '成功数量',
       `error_msg` MEDIUMTEXT NULL COMMENT '失败原因',
       PRIMARY KEY (`id`))
    COMMENT = '数据同步日志';

-- ----------------------------
-- Table structure for t_sync_data_student
-- ----------------------------
CREATE TABLE IF NOT EXISTS `t_sync_data_student`  (
        `id` bigint NOT NULL COMMENT 'id',
        `school_id` bigint NOT NULL COMMENT '学校id',
        `exam_id` bigint NOT NULL COMMENT '考试id',
        `course_code` varchar(100) NULL DEFAULT NULL COMMENT '课程编号',
        `course_name` varchar(200) NULL DEFAULT NULL COMMENT '课程名称',
        `course_college_name` varchar(200) NULL DEFAULT NULL COMMENT '课程开课学院',
        `paper_number` varchar(100) NULL DEFAULT NULL COMMENT '试卷编号',
        `student_name` varchar(50) NOT NULL COMMENT '姓名',
        `student_code` varchar(50) NOT NULL COMMENT '学号',
        `site_number` varchar(10) NULL DEFAULT NULL COMMENT '座位号',
        `student_college_name` varchar(100) NULL DEFAULT NULL COMMENT '学院',
        `major_name` varchar(100) NULL DEFAULT NULL COMMENT '专业',
        `teach_class_name` varchar(100) NULL DEFAULT NULL,
        `class_name` varchar(100) NULL DEFAULT NULL,
        `exam_start_time` bigint NULL DEFAULT NULL COMMENT '考试开始时间',
        `exam_end_time` bigint NULL DEFAULT NULL COMMENT '考试结束时间',
        `exam_place` varchar(50) NULL DEFAULT NULL COMMENT '考点',
        `exam_room` varchar(50) NULL DEFAULT NULL COMMENT '考场',
        `teacher_code` varchar(50) NULL COMMENT '命题老师工号',
        `teacher_name` varchar(50) NULL COMMENT '命题老师姓名',
        `update_status` TINYINT(1) NULL DEFAULT 0 COMMENT '是否已同步到考生表中',
        `jxb_id` VARCHAR(100) NULL COMMENT '教学班ID',
        `jxbzc` VARCHAR(2000) NULL COMMENT '教学班组成',
        `xnm` VARCHAR(45) NULL COMMENT '学年名',
        `xqm` VARCHAR(45) NULL COMMENT '学期名',
        `exam_date` VARCHAR(45) NULL COMMENT '考试日期',
        `exam_time` VARCHAR(45) NULL COMMENT '考试时间',
        PRIMARY KEY (`id`) USING BTREE
) ENGINE = InnoDB CHARACTER SET = utf8mb4 COLLATE = utf8mb4_general_ci COMMENT = '数据同步考生' ROW_FORMAT = Dynamic;

-- ----------------------------
-- Table structure for t_sync_exam_log
-- ----------------------------
CREATE TABLE IF NOT EXISTS `t_sync_exam_log` (
  `id` bigint NOT NULL,
  `school_id` bigint DEFAULT NULL COMMENT '学校ID',
  `semester_id` bigint DEFAULT NULL COMMENT '学期ID',
  `exam_id` bigint DEFAULT NULL COMMENT '云阅卷考试ID',
  `third_relate_id` int DEFAULT NULL COMMENT '云阅卷考试ID',
  `exam_name` varchar(100) DEFAULT NULL COMMENT '云阅卷考试名称',
  `status` varchar(20) DEFAULT NULL COMMENT '任务状态，INIT：未开始，RUNNING：进行中，FINISH：已完成',
  `result` varchar(20) DEFAULT NULL COMMENT '数据结果，SUCCESS：成功，ERROR：失败',
  `start_time` bigint DEFAULT NULL COMMENT '同步开始时间',
  `end_time` bigint DEFAULT NULL COMMENT '结束同步时间',
  `summary` mediumtext,
  `total_courses` int DEFAULT NULL COMMENT '总课程数',
  `total_exam_tasks` int DEFAULT NULL COMMENT '总命题任务数',
  `total_students` int DEFAULT NULL COMMENT '总考生数',
  `success_students` int DEFAULT NULL COMMENT '成功人数',
  `fail_students` int DEFAULT NULL COMMENT '失败人数',
  `error_file_path` varchar(500) DEFAULT NULL COMMENT '错误日志文件地址',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='同步考试日志表';

-- ----------------------------
-- Table structure for t_sync_exam_student_score
-- ----------------------------
CREATE TABLE IF NOT EXISTS `t_sync_exam_student_score` (
  `id` bigint NOT NULL COMMENT '主键',
  `school_id` bigint DEFAULT NULL COMMENT '学校id',
  `org_id` bigint DEFAULT NULL COMMENT '机构id',
  `semester_id` bigint DEFAULT NULL COMMENT '学期id',
  `major_name` varchar(100) DEFAULT NULL COMMENT '专业id',
  `clazz_id` bigint DEFAULT NULL COMMENT '班级id',
  `exam_id` bigint DEFAULT NULL COMMENT '云阅卷考试id',
  `third_relate_id` int DEFAULT NULL COMMENT '云阅卷考试ID',
  `exam_code` varchar(100) DEFAULT NULL COMMENT '云阅卷考试标识',
  `exam_number` varchar(100) DEFAULT NULL COMMENT '准考证号',
  `student_code` varchar(100) DEFAULT NULL COMMENT '学号',
  `name` varchar(100) NOT NULL COMMENT '姓名',
  `paper_number` varchar(100) DEFAULT NULL COMMENT '试卷编号',
  `subject_code` varchar(100) DEFAULT NULL COMMENT '科目代码',
  `subject_name` varchar(100) DEFAULT NULL COMMENT '科目名称',
  `paper_type` varchar(50) DEFAULT NULL COMMENT '试卷类型',
  `college` varchar(100) DEFAULT NULL COMMENT '学院',
  `clazz_name` varchar(100) DEFAULT NULL COMMENT '班级',
  `teacher` varchar(100) DEFAULT NULL COMMENT '教师',
  `status` tinyint DEFAULT NULL COMMENT '考生状态，1-正常，2-缺考(包含未上传)，3-违纪',
  `sheet_urls` mediumtext COMMENT '原卷url',
  `local_sheet_urls` mediumtext COMMENT '从云阅卷下载下来的原图url',
  `total_score` double DEFAULT NULL COMMENT '总分',
  `objective_score` double DEFAULT NULL COMMENT '客观题分数',
  `subjective_score` double DEFAULT NULL COMMENT '主观题分数',
  `sync_data` mediumtext COMMENT '同步数据json',
  `trajectory_urls` varchar(500) DEFAULT NULL COMMENT '轨迹图地址',
  `create_id` bigint DEFAULT NULL COMMENT '创建人id',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  PRIMARY KEY (`id`) USING BTREE,
  KEY `score_schoolId_idx` (`school_id`,`semester_id`,`org_id`,`major_name`,`clazz_id`,`student_code`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='同步考生成绩表';

-- ----------------------------
-- Table structure for t_sync_stmms_exam
-- ----------------------------
CREATE TABLE IF NOT EXISTS `t_sync_stmms_exam` (
  `id` bigint NOT NULL,
  `school_id` bigint DEFAULT NULL COMMENT '学校ID',
  `org_code` varchar(45) DEFAULT NULL COMMENT '机构代码',
  `exam_id` int DEFAULT NULL COMMENT '云阅卷ID',
  `exam_name` varchar(45) DEFAULT NULL COMMENT '云阅卷考试名称',
  `create_id` bigint DEFAULT NULL,
  `create_time` bigint DEFAULT NULL,
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='云阅卷考试表';

-- ----------------------------
-- Table structure for teach_clazz
-- ----------------------------
CREATE TABLE IF NOT EXISTS `teach_clazz` (
  `id` bigint NOT NULL COMMENT '教学班表主键',
  `school_id` bigint NOT NULL COMMENT '学校id',
  `teach_course_id` bigint NOT NULL,
  `basic_teach_clazz_id` bigint NOT NULL,
  `user_id` bigint DEFAULT NULL COMMENT '用户id',
  `clazz_name` varchar(50) NOT NULL COMMENT '教学课程名称',
  `belong_teach_course_id` bigint DEFAULT NULL COMMENT '分配班级时属于的教学课程id',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='教学班表';

-- ----------------------------
-- Table structure for teach_course
-- ----------------------------
CREATE TABLE IF NOT EXISTS `teach_course` (
  `id` bigint NOT NULL,
  `school_id` bigint NOT NULL COMMENT '学校id',
  `exam_id` bigint NOT NULL COMMENT '考试id',
  `course_id` bigint DEFAULT NULL COMMENT '课程ID（basic_course表ID）',
  `course_code` varchar(20) DEFAULT NULL COMMENT '课程代码',
  `user_id` bigint NOT NULL COMMENT '任课教师id',
  `enable` tinyint(1) DEFAULT NULL COMMENT '是否删除',
  `weight_setting` tinyint(1) DEFAULT NULL COMMENT '权重设置',
  `weight_setting_sign` bigint DEFAULT NULL COMMENT '课程权重设置标识',
  `dimension_sign` bigint DEFAULT NULL COMMENT '知识点标识',
  `default_rate` decimal(6,2) DEFAULT NULL COMMENT '默认占比',
  `custom_rate` decimal(6,2) DEFAULT NULL COMMENT '自定义占比',
  `create_id` bigint DEFAULT NULL COMMENT '创建人id',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人id',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  PRIMARY KEY (`id`) USING BTREE,
  UNIQUE KEY `teach_course_unique` (`school_id`,`exam_id`,`course_code`,`user_id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='教学课程表';

-- ----------------------------
-- Table structure for teach_student
-- ----------------------------
CREATE TABLE IF NOT EXISTS `teach_student` (
  `id` bigint NOT NULL COMMENT '教学学生关系表主键',
  `school_id` bigint NOT NULL COMMENT '学校id',
  `student_id` bigint NOT NULL COMMENT '学生id',
  `teach_clazz_id` bigint NOT NULL COMMENT '教学班级id',
  `teach_course_id` bigint NOT NULL COMMENT '教学课程id（冗余）',
  `user_id` bigint NOT NULL COMMENT '用户id（冗余）',
  `create_id` bigint DEFAULT NULL COMMENT '创建人',
  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
  `update_id` bigint DEFAULT NULL COMMENT '更新人',
  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
  PRIMARY KEY (`id`) USING BTREE,
  UNIQUE KEY `teach_student_unique` (`school_id`,`student_id`,`teach_clazz_id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci COMMENT='教学学生关系表';

-- ----------------------------
-- Table structure for tmp_help_topic
-- ----------------------------
CREATE TABLE IF NOT EXISTS `tmp_help_topic` (
  `help_topic_id` bigint NOT NULL AUTO_INCREMENT,
  PRIMARY KEY (`help_topic_id`)
) ENGINE=InnoDB AUTO_INCREMENT=51 DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_general_ci;

-- ----------------------------
-- Records of tmp_help_topic
-- ----------------------------
INSERT INTO `tmp_help_topic` (`help_topic_id`) VALUES (1);
INSERT INTO `tmp_help_topic` (`help_topic_id`) VALUES (2);
INSERT INTO `tmp_help_topic` (`help_topic_id`) VALUES (3);
INSERT INTO `tmp_help_topic` (`help_topic_id`) VALUES (4);
INSERT INTO `tmp_help_topic` (`help_topic_id`) VALUES (5);
INSERT INTO `tmp_help_topic` (`help_topic_id`) VALUES (6);
INSERT INTO `tmp_help_topic` (`help_topic_id`) VALUES (7);
INSERT INTO `tmp_help_topic` (`help_topic_id`) VALUES (8);
INSERT INTO `tmp_help_topic` (`help_topic_id`) VALUES (9);
INSERT INTO `tmp_help_topic` (`help_topic_id`) VALUES (10);
INSERT INTO `tmp_help_topic` (`help_topic_id`) VALUES (11);
INSERT INTO `tmp_help_topic` (`help_topic_id`) VALUES (12);
INSERT INTO `tmp_help_topic` (`help_topic_id`) VALUES (13);
INSERT INTO `tmp_help_topic` (`help_topic_id`) VALUES (14);
INSERT INTO `tmp_help_topic` (`help_topic_id`) VALUES (15);
INSERT INTO `tmp_help_topic` (`help_topic_id`) VALUES (16);
INSERT INTO `tmp_help_topic` (`help_topic_id`) VALUES (17);
INSERT INTO `tmp_help_topic` (`help_topic_id`) VALUES (18);
INSERT INTO `tmp_help_topic` (`help_topic_id`) VALUES (19);
INSERT INTO `tmp_help_topic` (`help_topic_id`) VALUES (20);
INSERT INTO `tmp_help_topic` (`help_topic_id`) VALUES (21);
INSERT INTO `tmp_help_topic` (`help_topic_id`) VALUES (22);
INSERT INTO `tmp_help_topic` (`help_topic_id`) VALUES (23);
INSERT INTO `tmp_help_topic` (`help_topic_id`) VALUES (24);
INSERT INTO `tmp_help_topic` (`help_topic_id`) VALUES (25);
INSERT INTO `tmp_help_topic` (`help_topic_id`) VALUES (26);
INSERT INTO `tmp_help_topic` (`help_topic_id`) VALUES (27);
INSERT INTO `tmp_help_topic` (`help_topic_id`) VALUES (28);
INSERT INTO `tmp_help_topic` (`help_topic_id`) VALUES (29);
INSERT INTO `tmp_help_topic` (`help_topic_id`) VALUES (30);
INSERT INTO `tmp_help_topic` (`help_topic_id`) VALUES (31);
INSERT INTO `tmp_help_topic` (`help_topic_id`) VALUES (32);
INSERT INTO `tmp_help_topic` (`help_topic_id`) VALUES (33);
INSERT INTO `tmp_help_topic` (`help_topic_id`) VALUES (34);
INSERT INTO `tmp_help_topic` (`help_topic_id`) VALUES (35);
INSERT INTO `tmp_help_topic` (`help_topic_id`) VALUES (36);
INSERT INTO `tmp_help_topic` (`help_topic_id`) VALUES (37);
INSERT INTO `tmp_help_topic` (`help_topic_id`) VALUES (38);
INSERT INTO `tmp_help_topic` (`help_topic_id`) VALUES (39);
INSERT INTO `tmp_help_topic` (`help_topic_id`) VALUES (40);
INSERT INTO `tmp_help_topic` (`help_topic_id`) VALUES (41);
INSERT INTO `tmp_help_topic` (`help_topic_id`) VALUES (42);
INSERT INTO `tmp_help_topic` (`help_topic_id`) VALUES (43);
INSERT INTO `tmp_help_topic` (`help_topic_id`) VALUES (44);
INSERT INTO `tmp_help_topic` (`help_topic_id`) VALUES (45);
INSERT INTO `tmp_help_topic` (`help_topic_id`) VALUES (46);
INSERT INTO `tmp_help_topic` (`help_topic_id`) VALUES (47);
INSERT INTO `tmp_help_topic` (`help_topic_id`) VALUES (48);
INSERT INTO `tmp_help_topic` (`help_topic_id`) VALUES (49);
INSERT INTO `tmp_help_topic` (`help_topic_id`) VALUES (50);

-- ----------------------------
-- Function structure for getRootChildId
-- ----------------------------
DROP FUNCTION IF EXISTS `getRootChildId`;
delimiter ;;
CREATE FUNCTION `getRootChildId`(rootId bigint)
    RETURNS mediumtext CHARSET utf8mb4
  DETERMINISTIC
BEGIN
    DECLARE ptemp MEDIUMTEXT;
    DECLARE ctemp MEDIUMTEXT;
    SET ptemp = '#';
    SET ctemp = cast(rootId as CHAR);
    WHILE ctemp IS NOT NULL DO
      SET ptemp = concat(ptemp, ',', ctemp);
SELECT group_concat(id) INTO ctemp FROM sys_org WHERE FIND_IN_SET(parent_id, ctemp) > 0;
END WHILE;
RETURN ptemp;
END
;;
delimiter ;